#!/usr/bin/env python
########
# Get Shannon radii, calculate goldschmidt tolerance factor
# 2015-01-30 Tam Mayeshiba
# 2015-11-04 TTM use correct coordination numbers
# 2015-12-18 TTM separate from radii and pv_combo, and separate evaluation
########
import numpy as np
from descriptors.radii import shannon_entry
from descriptors.pv_combo import pv_combo
import descriptors.goldschmidt as dg
import descriptors.shannon_radii as sr
import descriptors 
def evaluate_pure_perovskite_combinations(aelems="",belems="",celems="", verbose=0):
    """Evaluate combinations for pure perovskites
        Args:
            aelems <str>: comma-separated string of A-site element names
                            Send empty string to use all.
            belems <str>: comma-separated string of B-site element names
            celems <str>: comma-separated string of O-site element names
            verbose <int>: 1 - verbose
                           0 - silent
        Returns:
            list of pv_combo items of appropriate combinations
    """
    combolist=list()
    sdict = dict(sr.shannon_radii_dict)
    skeys = sdict.keys()

    if aelems == "":
        alist=list(skeys)
    else:
        afilter = aelems.split(",")
        alist=list()
        for aitem in afilter:
            if aitem in skeys:
                alist.append(aitem)
    if belems == "":
        blist=list(skeys)
    else:
        bfilter = belems.split(",")
        blist=list()
        for bitem in bfilter:
            if bitem in skeys:
                blist.append(bitem)
    if celems == "":
        clist=list(skeys)
    else:
        cfilter = celems.split(",")
        clist=list()
        for citem in cfilter:
            if citem in skeys:
                clist.append(citem)

    for atry in alist:
        aentries = sr.get_entries_from_shannon_dict(atry,"",9,12,"",1)
        for aentry in aentries:
            for btry in blist:
                bentries = sr.get_entries_from_shannon_dict(btry,"",6,6,"")
                for bentry in bentries:
                    for ctry in clist:
                        centries = sr.get_entries_from_shannon_dict(ctry,"",6,6,"")
                        for centry in centries:
                            my_combo = pv_combo()
                            my_combo.initialize_pure_from_shannon(aentry, bentry, centry)
                            if my_combo.is_possible_pure_perovskite(verbose):
                                combolist.append(my_combo)
    if (verbose == 1):
        for comboitem in combolist:
            print comboitem.get_gold_oneline()
    return combolist





    
def filter_by_goldschmidt(combolist, goldlow=0.9, goldhigh=1.0, verbose=1):
    """Filter list of pv_combo items based on Goldschmidt tolerance factors
        Args:
            combolist <list of pv_combo>: List of pv_combo items
        Returns:
            list of pv_combo items with the appropriate Goldschmidt 
                tolerance factors
    """
    filteredlist=list()
    for comboitem in combolist:
        cgt = comboitem.goldschmidt_tolerance
        if (cgt >= goldlow) and (cgt <= goldhigh):
            filteredlist.append(comboitem)
    if (verbose == 1):
        print "Goldschmidt tolerance factor filtered list:"
        for comboitem in filteredlist:
            print comboitem.get_gold_oneline()
    return filteredlist

def sort_by_elements(combolist, verbose=0):
    """Sort list of pv_combo by element name string.
    """
    sortedlist=list()
    for comboitem in combolist:
        elemstr = comboitem.get_elements()
        sortedlist.append((elemstr,comboitem))
    sortedlist.sort()
    newlist=list()
    for sorteditem in sortedlist:
        newlist.append(sorteditem[1])
    if (verbose == 1):
        print "Sorted by element:"
        for comboitem in newlist:
            print comboitem.get_gold_oneline()
    return newlist

def filter_by_oxidation_state(combolist, aox="", box="", verbose=0):
    """Filter list by oxidation states.
    """
    aox = int(aox)
    box = int(box)
    filteredlist=list()
    for comboitem in combolist:
        if (comboitem.aentry.oxstate == aox):
            if (comboitem.bentry.oxstate == box):
                filteredlist.append(comboitem)
    if (verbose == 1):
        print "Filtered by oxidation state:"
        for comboitem in filteredlist:
            print comboitem.get_gold_oneline()
    return filteredlist

def filter_out_radioactive(combolist, verbose=0):
    """Filter out radioactive elements.
        Radioactive elements listed at periodictable.com/Elements/Radioactive
    """
    radiolist=list(["Tc","Pm","Po","At","Rn","Fr",
                    "Ra","Ac","Th","Pa","U","Np",
                    "Pu","Am","Cm","Bk","Cf","Es",
                    "Fm","Md","No","Lr","Rf","Db",
                    "Sg","Bh","Hs","Mt","Ds","Rg",
                    "Cn","Uut","Uuq","Uup","Uuh","Uus",
                    "Uuo"])
    filteredlist=list()
    for comboitem in combolist:
        remove=0
        if comboitem.aentry.element in radiolist:
            remove = remove + 1
        if comboitem.bentry.element in radiolist:
            remove = remove + 1
        if comboitem.centry.element in radiolist:
            remove = remove + 1
        if not (comboitem.aprimeentry == ""):
            if comboitem.aprimeentry.element in radiolist:
                remove = remove + 1
        if not (comboitem.bprimeentry == ""):
            if comboitem.bprimeentry.element in radiolist:
                remove = remove + 1
        if (remove == 0):
            filteredlist.append(comboitem)
    if (verbose == 1):
        print "List with radioactive elements removed:"
        for comboitem in filteredlist:
            print comboitem.get_gold_oneline()
    return filteredlist

if __name__ == "__main__":
    pvlist=evaluate_pure_perovskite_combinations("","","O",1)
    pvlist = sort_by_elements(pvlist, 1)
    pvlist = filter_out_radioactive(pvlist, 1)
    pvlist = filter_by_goldschmidt(pvlist, 0.65, 1.05, 1)
    pvlist = filter_by_oxidation_state(pvlist, 3, 3, 1)
