#!/usr/bin/env python
########
# Get Shannon radii, calculate goldschmidt tolerance factor
# 2015-01-30 Tam Mayeshiba
# 2015-11-04 TTM use correct coordination numbers
# 2015-12-18 TTM separate from goldschmidt and radii
########
import numpy as np
import descriptors.shannon_radii as sr
import descriptors.goldschmidt as dg
import os

class pv_combo():
    """Perovskite combination class
        self.aentry <shannon entry>: shannon radius entry for A-site
        self.aprimeentry <shannon entry>: shannon radius entry for A-site dopant
        self.bentry <shannon entry>: shannon radius entry for B-site
        self.bprimeentry <shannon entry>: shannon radius entry for B-site dopant
        self.centry <shannon entry>: shannon radius entry for O-site
        self.goldschmidt_tolerance <float>: goldschmidt tolerance ratio
        self.afrac <float>: fraction of A-site, default 1.0
        self.bfrac <float>
        self.cfrac <float>: fraction of O-site, default 3.0
        self.aprimefrac <float>
        self.bprimefrac <float>
        self.bmagmom <float>: magnetic moment on B-site cation, no B-site dopants only
        self.calcdir <str>: calculation directory if any
    """
    def __init__(self):
        self.aentry=""
        self.bentry=""
        self.centry=""
        self.aprimeentry=""
        self.bprimeentry=""
        self.afrac=0.0
        self.bfrac=0.0
        self.cfrac=0.0
        self.aprimefrac=0.0
        self.bprimefrac=0.0
        self.goldschmidt_tolerance=0.0
        self.bmagmom=""
        self.calcdir=""
        return

    def initialize_pure_from_shannon(self, aentry, bentry, centry, verbose=0):
        """Initialize a pure perovskite
        """
        self.aentry = aentry
        self.bentry = bentry
        self.centry = centry
        self.afrac=1.0
        self.bfrac=1.0
        self.cfrac=3.0
        self.goldschmidt_tolerance = dg.calculate_goldschmidt_from_pv_combo(self, verbose)
        return

    def initialize_from_tam_directory(self, dirname, dobulkdescriptors=1, verbose=1):
        """Initialize a perovskite from one of my directories.
            Note that doped cells are always A0.75 A'0.25 B O3 with no B'
            A' is 2+
            And, all cells are 3+3+ perovskites
        """
        import simplegrep as sg
        self.calcdir=dirname
        elem_lines_raw = sg.grep_me("TITEL", os.path.join(dirname, "OUTCAR"))
        elem_lines = elem_lines_raw.split("\n")
        elem_list=list()
        for eline in elem_lines:
            if len(eline.strip()) == 0:
                pass
            else:
                if (verbose == 1):
                    print eline
                elem = eline.split()[3].split("_")[0]
                elem_list.append(elem)
        self.bfrac=1.0
        self.cfrac=3.0
        if "dop" in dirname: #TTM hardcode
            aidx=0
            aprimeidx=1
            bidx=2
            cidx=3
            self.afrac=0.75
            self.aprimefrac=0.25
        else:
            aidx=0
            aprimeidx=-1
            bidx=1
            cidx=2
            self.afrac=1.0
        self.aentry = sr.match_single_entry_from_element_highest_coordnum_match(elem_list[aidx],3,12,"",0)
        self.centry = sr.match_single_entry_from_element_highest_coordnum_match(elem_list[cidx],-2,6,"",0)
        if aprimeidx == 1:
            self.aprimeentry = sr.match_single_entry_from_element_highest_coordnum_match(elem_list[aprimeidx],2,12,"",0)

        magline = sg.grep_me("mag", os.path.join(dirname, "OSZICAR")).split("\n")[0].split()
        if (verbose == 1):
            print magline
        self.bmagmom=float(magline[9])/8.0 # hardcode for number of formula units in supercell
        self.bentry = sr.match_single_entry_from_element_highest_coordnum_match(elem_list[bidx],3,6,self.bmagmom,0)
        if (dobulkdescriptors == 1):
            self.set_bulk_descriptors(verbose)
        if (verbose == 1):
            print self.calcdir
            print self.get_gold_oneline()
        return

    def set_bulk_descriptors(self, verbose=1):
        """
        """
        self.goldschmidt_tolerance = dg.calculate_goldschmidt_from_pv_combo(self, verbose)
        return
    
    def get_bulk_descriptors(self, verbose=1):
        """
        """
        raise NotImplementedError
        return

    def get_elements(self):
        dispstr=""
        dispstr=dispstr + self.aentry.element
        if not (self.afrac == 1.0):
            dispstr = dispstr + "%s%s%s" % (self.afrac, self.aprimeentry.element, self.aprimefrac)
        dispstr = dispstr + self.bentry.element
        if not (self.bfrac == 1.0):
            dispstr = dispstr + "%s%s%s" % (self.bfrac, self.bprimeentry.element, self.bprimefrac)
        dispstr = dispstr + "%s%s" % (self.centry.element, self.cfrac)
        return dispstr

    def get_names(self):
        dispstr=""
        dispstr = dispstr + "%s%s " % (self.aentry.name, self.aentry.spinstate)
        if not (self.afrac == 1.0):
            dispstr = dispstr + "%s%s " % (self.aprimeentry.name, self.aprimeentry.spinstate)
        dispstr = dispstr + "%s%s " % (self.bentry.name, self.bentry.spinstate)
        if not (self.bfrac == 1.0):
            dispstr = dispstr + "%s%s " % (self.bprimeentry.name, self.bprimeentry.spinstate)
        dispstr = dispstr + "%s%s" % (self.centry.name, self.centry.spinstate)
        return dispstr

    def get_coordnumbers(self):
        dispstr=""
        dispstr = dispstr + "%1.0f " % self.aentry.coordnumber
        if not (self.afrac == 1.0):
            dispstr = dispstr + "%1.0f " % self.aprimeentry.coordnumber
        dispstr = dispstr + "%1.0f " % self.bentry.coordnumber
        if not (self.bfrac == 1.0):
            dispstr = dispstr + "%1.0f " % self.bprimeentry.coordnumber
        dispstr = dispstr + "%1.0f" % self.centry.coordnumber
        return dispstr

    def get_gold_oneline(self):
        return "%s, %s, CNs %s, Gt %3.3f" % (self.get_elements(),
                self.get_names(), self.get_coordnumbers(), 
                self.goldschmidt_tolerance)

    def is_possible_pure_perovskite(self, verbose=1):
        """Determine whether the perovskite is possible, based on
            oxidation states and coordination numbers
            Known perovskites are for example LaMnO3 (3+3+,2-);
                BaTiO3 (2+4+,2-); KMnF3 (1+2+,1-)
            Assume a pure perovskite is ABO3
        """
        aox=self.aentry.oxstate
        box=self.bentry.oxstate
        cox=self.centry.oxstate
        acn=self.aentry.coordnumber
        bcn=self.bentry.coordnumber
        ccn=self.centry.coordnumber
        #Assume pure perovskite is ABO3
        if (verbose == 1):
            print self.get_gold_oneline()
        if not (aox + box + 3*cox == 0):
            if (verbose == 1):
                print "Oxidation states do not add up to zero."
            return False
        if not (acn >= 9):
            if (verbose == 1):
                print "A-site coordination number %i is not >= 9" % acn
            return False
        if not (bcn == 6):
            if (verbose == 1):
                print "B-site coordination number %i is not 6" % bcn
            return False
        if not (ccn == 6):
            if (verbose == 1):
                print "O-site coordination number %i is not 6 (2 A-site, 4 B-site)" % ccn
            return False
        if (verbose == 1):
            print "Combination is a possible pure perovskite based on oxidation state and coordination number."
        return True
    
    def is_possible_perovskite_by_goldschmidt(self, verbose=0):
        """Determine whether the perovskite is possible, based on
            goldschmidt tolerance factor
        """
        if (self.goldschmidt_tolerance == 0.0):
            try:
                self.goldschmidt_tolerance = dg.calculate_goldschmidt_from_pv_combo(self, verbose)
            except:
                print "Error getting goldschmidt tolerance factor."
                return None
        category = dg.get_goldschmidt_category(self.goldschmidt_tolerance)
        if (category == "not perovskite"):
            return False
        return True

