#!/usr/bin/env python
########
# Get Shannon radii, calculate goldschmidt tolerance factor
# 2015-01-30 Tam Mayeshiba
# 2015-11-04 TTM use correct coordination numbers
# 2015-12-18 TTM make general
########
import numpy as np
import os
shannon_radii_location = os.path.join(os.getenv("HOME"),"pvhelper",
            "descriptors","shannon_radii.csv")
spins_location = os.path.join(os.getenv("HOME"),"pvhelper",
            "descriptors","shannon_radii_spins.csv")

class shannon_entry():
    """Originally a class for shannon radii, but could also be general radii.
    """
    def __init__(self, csvline=""):
        self.record=""
        self.name=""
        self.oxstate=""
        self.elecconfig=""
        self.coordnumber=""
        self.spinstate=""
        self.crystalrad=""
        self.ionicrad=""
        self.notes=""
        self.zoverir=""
        self.element=""
        if not (csvline==""):
            csplit=csvline.split(',')
            self.record      = csplit[0].strip()
            self.name        = csplit[1].strip()
            self.oxstate     = float(csplit[2].strip())
            self.elecconfig  = csplit[3].strip()
            self.coordnumber = float(csplit[4].strip())
            self.spinstate   = csplit[5].strip()
            self.crystalrad  = float(csplit[6].strip())
            self.ionicrad    = float(csplit[7].strip())
            self.notes       = csplit[8].strip()
            self.zoverir     = csplit[9].strip()
            if ("+" in self.name):
                self.element = self.name.split("+")[0].strip()
            elif ("-" in self.name):
                self.element = self.name.split("-")[0].strip()
            else:
                self.element = self.name

    def __repr__(self):
        mystr=""
        mystr = mystr + "\n" + "Element %s, oxidation state %i, coordination %i, spin state %s" % (self.element, self.oxstate, self.coordnumber, self.spinstate)
        mystr = mystr + "\n" + "Electron configuration %s" % self.elecconfig
        mystr = mystr + "\n" + "Crystal radius %3.3f, ionic radius %3.3f (Angstroms)" % (self.crystalrad, self.ionicrad)
        mystr = mystr + "\n" + "Notes %s, Z/IR %s" % (self.notes, self.zoverir)
        return mystr

def make_shannon_radii_dict(fname="shannon_radii.csv"):
    """Make dictionary of shannon radii, organized by element name
    """
    sfile=open(fname, 'rb')
    slines = sfile.readlines()
    sfile.close()
    ref1=slines.pop(0)
    ref2=slines.pop(0)
    headers=slines.pop(0)
    sdict=dict()
    for sline in slines:
        sentry = shannon_entry(sline)
        if not (sentry.element in sdict.keys()):
            sdict[sentry.element] = list()
        sdict[sentry.element].append(sentry)
    return sdict

def make_spin_dict(fname="shannon_radii_spins.csv"):
    """Make a dictionary of magnetic moments
    """
    sfile=open(fname, 'rb')
    slines = sfile.readlines()
    sfile.close()
    slines.pop(0)
    slines.pop(0)
    slines.pop(0)
    slines.pop(0)
    slines.pop(0)
    sdict=dict()
    for sline in slines:
        ssplit=sline.split(",")
        elem=ssplit[0]
        oxstate=int(ssplit[1])
        lowspin=ssplit[2]
        highspin=ssplit[3]
        if not (elem in sdict.keys()):
            sdict[elem] = dict()
        if not (oxstate in sdict[elem].keys()):
            sdict[elem][oxstate]=dict()
        try:
            sdict[elem][oxstate]["LS"] = float(lowspin)
        except ValueError:
            sdict[elem][oxstate]["LS"] = None
        try:
            sdict[elem][oxstate]["HS"] = float(highspin)
        except ValueError:
            sdict[elem][oxstate]["HS"] = None
    return sdict

#runs each time script is imported
shannon_radii_dict = make_shannon_radii_dict(shannon_radii_location)
spin_dict = make_spin_dict(spins_location)
def get_highest_cn_entries(sentrylist):
    """Get highest coordination number entries from entry list
    """
    maxcn=0
    maxlist=list()
    for sentryitem in sentrylist:
        if (sentryitem.coordnumber > maxcn):
            maxcn = sentryitem.coordnumber
    for sentryitem in sentrylist:
        if (sentryitem.coordnumber == maxcn):
            maxlist.append(sentryitem)
    return maxlist
    

def get_entries_from_shannon_dict(element="",ox="",cnlow=0, cnhigh=0,spinstate="",gethighestcn=0):
    """Get list of entries matching the criteria
        Args:
            element <str>: Element symbol, e.g. La
            ox <float>: Oxidation state; use "" for all oxidation states
            cnlow <int>: Low coordination number
                            For no lower bound, use ""
            cnhigh <int>: High coordination number
                            To restrict to a single value, set cnhigh to cnlow
                            For no upper bound, use ""
            spinstate <str>: Spin state (HS, LS); use "" for all spin states
            gethighestcn <int>: 0 - return all within range
                                1 - return only entries with the highest
                                    coordination numbers
    """
    element = element.title()
    if not (ox == ""):
        ox = float(ox)
    if not (cnlow == ""):
        cnlow = int(cnlow)
    if not (cnhigh == ""):
        cnhigh = int(cnhigh)
    spinstate = spinstate.upper()
    
    sentrylist=list()
    if not (element in shannon_radii_dict.keys()):
        return sentrylist
    for sentry in shannon_radii_dict[element]:
        remove=0
        if not (ox == ""):
            if not (ox == sentry.oxstate):
                remove = remove + 1
        scn = sentry.coordnumber
        if not (cnlow == ""):
            if (scn < cnlow):
                remove = remove + 1
        if not (cnhigh == ""):
            if (scn > cnhigh):
                remove = remove + 1
        if not (spinstate == ""):
            if not (sentry.spinstate == spinstate):
                remove = remove + 1
        if (remove == 0):
            sentrylist.append(sentry)
    if (gethighestcn == 1):
        sentrylist = get_highest_cn_entries(sentrylist)
    return sentrylist


def match_single_entry_from_element(element="",oxstate="", coordnum="", magmom="", verbose=0):
    """look up a shannon entry
        Args:
            element <str>: Element symbol, like La
            oxstate <int>: Oxidation state
            coordnum <int>: Coordination number
            magmom <float>: magnetic moment on ion; use "" to skip
    """
    initiallist = get_entries_from_shannon_dict(element, oxstate, coordnum, coordnum, "", 0)
    if len(initiallist) == 0:
        raise LookupError("No shannon entry found for %s oxidation %s coordination %s" % (element, oxstate, coordnum))
    spin_assigned=False
    if not (magmom == ""):
        if element in spin_dict.keys():
            if oxstate in spin_dict[element].keys():
                lowspin = spin_dict[element][oxstate]["LS"]
                highspin = spin_dict[element][oxstate]["HS"]
                difflow = np.abs(float(lowspin) - float(magmom))
                diffhigh = np.abs(float(highspin) - float(magmom))
                if (difflow < diffhigh):
                    spinstate="LS"
                else:
                    spinstate="HS"
                spin_assigned = True
                if (verbose == 1):
                    print "%s assigned for magmom of %3.3f (%3.0f, %3.0f)" % (spinstate, magmom, lowspin, highspin)
    if not spin_assigned:
        if not (len(initiallist) == 1):
            raise LookupError("No unique match found for %s oxidation %s coordination %s" % (element, oxstate, coordnum))
        else:
            return initiallist[0]
    else:
        for ientry in initiallist:
            if ientry.spinstate == spinstate:
                return ientry
    return None

def match_single_entry_from_element_highest_coordnum_match(element="",oxstate="", coordnum="", magmom="", verbose=0):
    """look up a shannon entry; if coordination number is not found,
        use the highest available match.
        Args:
            element <str>: Element symbol, like La
            oxstate <int>: Oxidation state
            coordnum <int>: Coordination number
            magmom <float>: magnetic moment on ion; use "" to skip
    """
    mycoord = coordnum
    mymatch=""
    found=False
    while (mycoord > 0):
        try:
            mymatch = match_single_entry_from_element(element, oxstate, mycoord, magmom, verbose)
            found=True
            return mymatch
        except LookupError:
            mycoord = mycoord - 1
    if not found:
        raise LookupError("No unique match found for %s oxidation %s coordination <= %s" % (element, oxstate, coordnum))
    return 

