#!/usr/bin/env python
#####################
# Tam Mayeshiba 2015-11-18
# Diffusion coefficient support
#####################
import os
import sys
import simpleplot
import numpy as np
import hocking_lstsq

def order_of_magnitude_change(temp=298.0,dact=0.1):
    """Order of magnitude change in diffusion coefficient
        that lowering the activation energy by 
        dact eV at temperature temp in K will produce.
        For D_vacancy, activation energy is migration energy.
        Args:
            temp <float>: temperature in K
            dact <float>: activation energy change in eV
    """
    myA=1.0 #arbitrary
    mytemp = float(temp)
    myQ = float(dact)
    boltzmann = 0.00008617 # eV/K
    myD_1 = myA * np.exp(-1.0 * myQ / (mytemp * boltzmann))
    myD_2 = myA * np.exp(-1.0 * 2.0 * myQ / (mytemp * boltzmann))
    oom1 = np.log10(myD_1)
    oom2 = np.log10(myD_2)
    myoom = oom1 - oom2
    return myoom

def slope_to_change(slope="",strain="",temp=""):
    """Order of magnitude change in diffusion coefficient
        for a given slope in activation energy with strain
        (migration barrier for D_v)
        at a given strain and temperature.
        Args:
            slope <float>: slope in activation energy as meV/% strain
            strain <float>: percent strain, where positive is tensile strain and
                            negative is compressive strain
            temp <float>: temperature in K
    """
    myslope = float(slope) / 1000.0 # now in eV/% strain
    myeVchange = float(strain) * myslope # eV difference
    mynum_dact = myeVchange / -0.1 # number of 0.1 eV decreases in dact
    myoom = order_of_magnitude_change(float(temp), 0.1) #oom change for 0.1 eV
    totaloom = mynum_dact * myoom
    return totaloom

def oom_to_slope(oom="",temp="",lattice_mismatch=""):
    """Convert order of magnitude change in diffusion coefficient
        into a slope value.
        Uses lattice mismatch as percent strain.
        Args:
            oom <float>: Order of magnitude change in D reported
            temp <float>: temperature in K
            lattice_mismatch <float>: lattice mismatch in percent
    """
    myoom = float(oom)
    mytemp = float(temp)
    mystrain = float(lattice_mismatch)
    myoom_pt1eV = order_of_magnitude_change(mytemp, 0.1)
    my_pt1eVs = myoom / myoom_pt1eV #number of 0.1 eV decreases
    total_meV_change = my_pt1eVs * -0.1 * 1000
    my_slope = total_meV_change / mystrain
    return my_slope


