#!/usr/bin/env python
# Tam Mayeshiba 2015-11-09
# cleanup from strain data
# Elastic constant functions, separated out from strain S12 script.
######
import os
import hocking_lstsq
import sys
import simpleplot
import numpy as np
import time

def bulkmod(pvec, vvec, label=""):
    """Calculate the bulk modulus and derivative of bulk modulus
        based on coefficients of a cubic fit to 
        V(P), as B0 = -V (partial P / partial V) | eval at P=0
        and using (partial P / partial V) = 1/(partial V / partial P).
        See Mayeshiba and Morgan, PCCP 2015
        Args:
            pvec <list or np.ndarray of float>: pressures in kbar
            vvec <list or np.ndarray of float>: volumes in cubic Angstroms
            label <str>: optional label
    """
    if type(pvec) == list:
        pvec = np.array(pvec, 'float')
    if type(vvec) == list:
        vvec = np.array(vvec, 'float')    
    print "Pvec, Vvec: ", pvec, vvec
    simpleplot.plot(pvec, vvec, "%s V(P)" % label)
    [betamat, ssq, betaerr] = hocking_lstsq.least_squares_regression(pvec, vvec, 3)
    coeffj=float(betamat[3])
    coeffk=float(betamat[2])
    coeffl=float(betamat[1])
    coeffm=float(betamat[0])
    stderrj=float(betaerr[3])
    stderrk=float(betaerr[2])
    stderrl=float(betaerr[1])
    stderrm=float(betaerr[0])
    bulkmod = -1.0*coeffm/coeffl #in kbar. TTM see strain paper
    bulkmoderr = bulkmod * np.sqrt(np.power((stderrm/coeffm),2) + np.power((stderrl/coeffl),2))
    derivbm = -1.0 + ((2.0 * coeffm * coeffk) / (coeffl * coeffl))
    #dbmgpa = dbm / 10.0
    #dbmeVA3 = dbmgpa / 160.2
    print "Bulk mod (kbar): %3.3f" % bulkmod
    print "Bulk mod err (kbar): %3.3f" % bulkmoderr
    print "Deriv of bulk mod (kbar): %3.3f" % derivbm
    retdesc = "Returning fit coefficients (intercept, x, x^2, x^3), S-squared, error in fit coefficients, bulk modulus in kbar, error in bulk modulus in kbar, derivative of bulk modulus in kbar"
    return [retdesc, betamat, ssq, betaerr, bulkmod, bulkmoderr, derivbm]


def poisson_from_biaxial(avec, cvec, label=""):
    """Calculate Poisson's ratio from biaxial strain responses.
        Args:
            avec <list or np.ndarray of float>: a-strain multipliers,
                e.g. "1.01" for 1% tensile strain and
                     "0.98" for 2% compressive strain
            cvec <list or np.ndarray of float>: c-strain multipliers
            label <str>: Optional label
        Because a-strains are 0 at 0 strain, c-strains will
            be normalized to their zero-strain values.
        Using a quadratic fit of c-strain as a function of a-strain,
            with the intercept set to zero,
            and strainxx = (1-nu) sigmaxx/E, where sigmaxx = E*strainxx,
            and strainzz = -2*nu*sigmaxx/E, 
            poisson's ratio is (strainc/straina) / ((strainc/straina) - 2);
            substituting the quadratic equation for strainc and
            taking the small-strain limit as straina goes to zero gives
            poisson's ratio v = f/(f-2) where f is the first-order coefficient 
            of the quadratic fit
    """
    if type(avec) == list:
        avec = np.array(avec, 'float')
    if type(cvec) == list:
        cvec = np.array(cvec, 'float')    
    print "Avec, Cvec: ", avec, cvec
    czero=""
    # Get normalization for c-strains
    for idx in range(0, len(avec)):
        if avec[idx] == 1:
            czero=cvec[idx]
    astrain=list()
    cstrain=list()
    for idx in range(0, len(avec)):
        astrain.append(avec[idx] - 1.0)
        cstrain.append(cvec[idx] - czero)
    astrainvec = np.array(astrain, 'float')
    cstrainvec = np.array(cstrain, 'float')
    simpleplot.plot(astrainvec, cstrainvec, "%s Normalized C strain(A strain)" % label)
    [betamat, ssq, betaerr] = hocking_lstsq.least_squares_regression(astrainvec, cstrainvec, 2, True) # zero the intercept
    coefff = betamat[0]
    stderrf = betaerr[0]
    poisson = coefff / (coefff - 2.0)
    poissonerr = poisson * np.sqrt(2.0) * (stderrf / coefff)
    print "Poisson's ratio: ", poisson
    print "Std error in poisson's ratio: ", poissonerr
    return [poisson, poissonerr]

def youngsmod(bulkmod, bulkmoderr, poisson, poissonerr, label=""):
    """Calculate Young's modulus from bulk modulus and poisson's ratio.
        Args:
            bulkmod <float>: Bulk modulus IN KBAR
            bulkmoderr <float>: Error in bulk modulus
            poisson <float>: Poisson's ratio
            poissonerr <float>: Error in poisson's ratio
            label <str>: Optional label
        Returns:
            young's modulus in KBAR
    """
    print label
    youngsmod = 3.0 * bulkmod * (1.0 - 2.0*poisson)
    errmodsq = 3.0 * 3.0 * bulkmoderr * bulkmoderr + \
                6.0 * 6.0 * bulkmod * poisson * bulkmod * poisson * \
                (np.power(bulkmoderr/bulkmod, 2) + \
                 np.power(poissonerr/poisson, 2))
    youngsmoderr = np.sqrt(errmodsq)
    return [youngsmod, youngsmoderr]
def strain_model_prefactor(poisson, poissonerr, youngsmod, youngsmoderr, label=""):
    """Get Schichtel elastic model prefactor.
        Args:
            poisson <float>: Poisson's ratio
            poissonerr <float>: Std error in poisson's ratio
            youngsmod <float>: Young's modulus IN KBAR
            youngsmoderr <float>: Std error in Young's modulus
            label <str>: Optional label
        Returns:
            prefactor in eV/cubic Angstroms
    """
    print label
    youngseVA3=""
    youngserreVA3=""
    if youngsmod < 20: 
        print "Assuming conversion of Young's modulus from kbar to"
        print "eV/A^3 has already been done."
        youngseVA3=youngsmod
        youngserreVA3=youngsmoderr
    else:
        youngseVA3 = youngsmod / 10.0 / 160.2
        youngserreVA3 = youngsmoderr / 10.0 / 160.2
    prefactor = -2.0 * youngseVA3 / (3.0 * (1.0 - poisson) )
    prefactorerr = prefactor * np.sqrt(np.power(youngserreVA3/youngseVA3,2) + \
            np.power(poissonerr/poisson,2))
    print "Strain model prefactor, eV/A^3: ", prefactor
    print "Error in prefactor: ", prefactorerr
    return [prefactor, prefactorerr]

def strain_model_slope(prefactor, prefactorerr, volume, volumeerr, label=""):
    """Get Schichtel strain model slope.
        Args:
            prefactor <float>: prefactor in eV/A^3
            prefactorerr <float>: error in prefactor
            volume <float>: volume in A^3
            volumeerr <float>: volume error
            label <str>: Optional label
    """
    slope = prefactor * volume
    slope = slope * 1000.0 # put in meV
    slope = slope / 100.0 # divide by 100 for %
    slopeerr = slope * np.sqrt(np.power(prefactorerr/prefactor,2.0) + np.power(volumeerr/volume, 2.0))
    print "Slope: ", slope
    print "Slope error: ", slopeerr
    return [slope, slopeerr]


def evaluate_birch_murnaghan_volume_guess(bulkmod="",derivbm="",pressure="",
        volatpress="",v0guess=""):
    """Given a (pressure, volume at pressure) point and a guess
        at the zero-pressure volume, evaluate that volume guess
        based on how closely the pressure that results from using the
        third-order Birch Murnaghan equation matches the given pressure.
        Args:
            bulkmod <float>: Bulk modulus IN KBAR
            derivbm <float>: Derivative of bulk modulus IN KBAR
            pressure <float>: Given pressure IN KBAR
            volatpress <float>: Volume associated with that pressure,
                in cubic Angstroms
            v0guess <float>: Guess for zero-pressure volume
    """
    vratio = float(v0guess)/float(volatpress)
    bulkmod = float(bulkmod)
    derivbm = float(derivbm)
    pressure = float(pressure)
    #Birch-Murnaghan equation
    peval = 3.0*bulkmod/2.0 * \
        (np.power(vratio, 7.0/3.0) - np.power(vratio, 5.0/3.0)) * \
        (1.0 + 0.75*(derivbm - 4.0)*(np.power(vratio, 2.0/3.0) - 1.0))
    pdiff = np.abs(peval - pressure)
    return pdiff

def get_volume_from_birch_murnaghan(bulkmod="", derivbm="", pressure="",
        volatpress="", volstart=300.0,volend=600.0,volstep=0.1):
    """Given a (pressure, volume at pressure) point, find the zero-pressure
        volume using the third-order Birch-Murnaghan equation.
        Args:
            bulkmod <float>: Bulk modulus IN KBAR
            derivbm <float>: Derivative of bulk modulus IN KBAR
            pressure <float>: Given pressure IN KBAR
            volatpress <float>: Volume associated with that pressure,
                in cubic Angstroms
            volstart <float>: Starting volume for guesses (default 300.0 A^3)
            volend <float>: Ending volume for guesses (default 600.0 A^3)
            volstep <float>: Step for volume guesses (default 0.1 A^3)
    """
    vguess = volstart
    lowest_p_diff = 10000.0
    best_v_guess = 0.0
    while (vguess <= volend):
        pdiff = evaluate_birch_murnaghan_volume_guess(bulkmod, derivbm, pressure, volatpress, vguess)
        if pdiff < lowest_p_diff:
            lowest_p_diff = pdiff
            best_v_guess = vguess
        vguess = vguess + volstep
    return best_v_guess
