#!/bin/env python
"""Strain energy fitting. TTM 3/12/14
Takes input.txt with columns: label, E0, supercell volume, strain fraction a, strain fraction b, strain fraction c
"""
import os
import sys
import numpy as np
import time
import hocking_lstsq

def least_squares_regression(xvec, yvec, order=1, zerointercept=False, verbose=1):
    """Least-squares regression from Hocking, 
        Methods and Applications of Linear Models: 
        Regression and the Analysis of Variance, 2nd Edition,
        John Wiley & Sons, Inc., 2003, Ch. 2
        Args:
            xvec <numpy array of float>: no-error inputs
            yvec <numpy array of float>: observed outputs
            order <int>: order of fit 
                            1 - linear (default)
                            2 - quadratic
                            3 - cubic
                            4 - quartic
                            (no other values supported)
            zerointercept <bool>: zero the intercept
            verbose <int>: verbosity of output 
                            1 - verbose (default)
                            0 - concise
    """
    print "---------------------"
    jvec = np.ones(len(xvec),float)
    if order == 1:
        if zerointercept:
            xmat = xvec.transpose()
        else:
            xmat = np.vstack([jvec,xvec]).transpose()
    elif order == 2:
        if zerointercept:
            xmat = np.vstack([xvec, xvec*xvec]).transpose()
        else:
            xmat = np.vstack([jvec,xvec,xvec*xvec]).transpose()
    elif order == 3:
        if zerointercept:
            xmat = np.vstack([xvec, xvec*xvec, np.power(xvec, 3)]).transpose()
        else:
            xmat = np.vstack([jvec,xvec,xvec*xvec, np.power(xvec, 3)]).transpose()
    elif order == 4:
        if zerointercept:
            xmat = np.vstack([xvec, xvec*xvec, np.power(xvec, 3), np.power(xvec,4)]).transpose()
        else:
            xmat = np.vstack([jvec,xvec,xvec*xvec,np.power(xvec,3),np.power(xvec,4)]).transpose()
    else:
        print "Order %s is not supported." % order
    xtrans = xmat.transpose()
    invxtx = np.linalg.inv(np.dot(xtrans, xmat))
    betamat = np.dot(np.dot(invxtx,xmat.transpose()),yvec.transpose())
    print "Beta matrix: ", betamat
    hatmat = np.dot(np.dot(xmat, invxtx),xtrans)
    yhat = np.dot(hatmat, yvec)
    rvec = yvec - yhat
    rss = np.dot(rvec.transpose(),rvec)
    iminush = np.eye(hatmat.shape[0]) - hatmat
    rank = np.rank(iminush)
    matrank = np.linalg.matrix_rank(iminush)
    print "Mat rank: ", matrank
    s_squared = rss/matrank
    print "S-squared = RSS/(N-rank(I-H)): ", s_squared
    print "Sqrt s-squared: ", np.sqrt(s_squared)
    stderrsq = np.diag(invxtx)*s_squared
    stderr = np.sqrt(stderrsq)
    print "Std errors in coefficients: ", stderr
    print "Returning: coeffs, s_squared, sterrs in coeffs"
    print "--------------------"
    return [betamat, s_squared, stderr]
