#!/usr/bin/env python
##
#@package   Pdirautorelax
#@ingroup   vasp
#@ingroup   python
#@brief     keep resubmitting a job until it no longer needs to be relaxed
#@author    Tam Mayeshiba
#@date      1/5/11: copied from Pqautorelax
#@date      2/16/11: qdel job if it has hung
#@date      2/16/11: add email notification if problem
#@date      2/16/11: if starting job already has CONTCAR,
#                    increase both relaxct and maxout
#@date      3/2/11: updating to use Pqsubretidnew instead of Pqueuesubretid
#@date      3/23/11: updating to accept both upper and lower case 't', 'p'
#@date      5/20/11 when looking to see if has already run, use a 
#                   status call rather than just CONTCAR presence
#@date      5/25/11 disable killing of hung jobs; send error and stop script,
#                   but keep jobs on queue - let user determine whether
#                   job is still healthy. User may have to create fake
#                   mon file in endpoint run's first directory.
###
def main(fullpath, numatoms=1, maxout=5, checksec=1800, checklag=1800, 
            encomp=0.001, entype='T', volcomp=1, voltype='P', emaddr=''):
    """
    Purpose: keep resubmitting a job until it no longer needs to be relaxed
       inputs:  
	        [$1] full path of folder of job to submit
                example: '/home/tam/rocksalt_rel1/NaClep1'
            [$2] number of atoms in system - default 1
                between 1 and 10000, inclusive
            [$3] maximum number of relaxations to do - default 5
                between 1 and 100, inclusive
            [$4] second between checks - default 1800
            [$5] acceptable number of seconds before job considered hung
                    - default 1800
            [$4] energy comparison number - default 0.001
            [$5] energy comparison type - default 'T' for threshhold
            [$6] volume comparison number - default 1
            [$7] volume comparison type - default 'P' for percent
            [$8] email address for error notification
       outputs:
            True - job has finished
            None - error
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import glob
    import Pqsubretidnew
    import Pdirmonitor
    import Xvaliddirectory
    import Xvalidinteger
    import Xtimestring
    import Xuniquename
    import Vrelax
    import Vokayvolume
    import Vokayenergy
    import Psendemail
    import Xvalidletter
    import Vcontinueprev
#   validation
#   #   validate path
    fullpath = Xvaliddirectory.main(fullpath)
    if (fullpath == None):
        print '(Exiting.)'
        return None
#   #   validate maxout
    maxout = Xvalidinteger.main(maxout, 1, 100)
    if (maxout == None):
        print '(Exiting.)'
        return None
#   #   validate numatoms
    numatoms = Xvalidinteger.main(numatoms, 1, 10000)
    if (numatoms == None):
        print '(Exiting.)'
        return None
#   #   validate checksec
#   #   #   handled by Pdirmonitor
#   #   validate lagsec
#   #   #   handled by Pdirmonitor
#   #   validate energy comp
#   #   #   handled by Vokayenergy
#   #   validate energy comp type
    entype = Xvalidletter.main(entype, ['t','p'], 1)
    if (entype == None):
        print '(Exiting.)'
        return None
#   #   validate volume comp
#   #   #   handled by Vokayvolume
#   #   validate volume comp type
    voltype = Xvalidletter.main(voltype, ['t','p'], 1)
    if (voltype == None):
        print '(Exiting.)'
        return None
#   #   #   handled by Vokayvolume
#   #   get submission script
    subsearch=[]
    subsearch = glob.glob(fullpath + '/' + '*.sh')
    subfile=""
    if len(subsearch) < 1:
        print 'No submission script detected in', fullpath
        print '(Exiting.)'
        return None
    if len(subsearch) > 1:
        print 'Warning: more than one submission script',
        print 'detected in', fullpath
        print 'Using first script only.'
        subsearch.sort()
    subfile = os.path.basename(subsearch[0])
#   #   run submission script for job
    oktostoprel=False
    volokay=False
    enokay=False
    jobid=""
    monitor=""
    rootpath=""
    rootname=""
    prevpath=""
    relaxct=1
    rootname = os.path.basename(fullpath)
    rootpath = os.path.dirname(fullpath)
    newpath=""
    newname=""
    relaxme=""
    startstatus=""
#   #   if has already been run, set up relaxation directory
    startstatus = Vcontinueprev.main(fullpath)
    if (startstatus == None):
        pass # no CONTCAR; just run this directory again
    elif (startstatus == True):
        pass # CONTCAR but unfinished; data files have been archived
             # and directory is ready to run with CONTCAR as POSCAR
    else:
        newname = Xuniquename.main(rootpath, rootname, 'D', '_rel', relaxct)
        relaxme = Vrelax.main(fullpath, rootpath, newname)
        if (relaxme == None):
            print 'Error relaxing', fullpath
            print '(Exiting.)'
            return None
        prevpath = fullpath
        fullpath = rootpath + '/' + newname
        relaxct = relaxct + 1
        maxout = maxout + 1    # had finished CONTCAR. Start as relax.
#   #   main loop
    while (oktostoprel == False):
        if (relaxct > maxout):
            print 'Maximum number of relaxations exceeded.'
            oktostoprel = None
            break
        jobid = Pqsubretidnew.main(fullpath, subfile)
        if (jobid == None):
            oktostoprel = None
            break
        monitor = Pdirmonitor.main(fullpath, checksec, checklag)
        if (monitor == None) or (monitor == False):
            oktostoprel = None
            break
        if (relaxct == 1):
            newname = Xuniquename.main(rootpath, rootname, 
                        'D', '_rel', relaxct)
            relaxme = Vrelax.main(fullpath, rootpath, newname)
            if (relaxme == None):
                print 'Error relaxing', fullpath
                print '(Exiting.)'
                return None
            prevpath = fullpath
            fullpath = rootpath + '/' + newname
        else:
            if (voltype == 't'):
                volokay = Vokayvolume.main(prevpath, fullpath, numatoms,
                            voltype, volcomp)
            else:
                volokay = Vokayvolume.main(prevpath, fullpath, numatoms,
                            voltype, "", volcomp)
            print 'Volume check:', volokay
            if (entype == 't'):
                enokay = Vokayenergy.main(prevpath, fullpath, numatoms,
                            entype, encomp)
            else:
                enokay = Vokayenergy.main(prevpath, fullpath, numatoms,
                            entype, "", encomp)
            print 'Energy check:', enokay
            oktostoprel = volokay and enokay
            if (volokay == None) or (enokay == None):
                oktostoprel == None
            print 'Okay to stop relaxing:', oktostoprel
            if (oktostoprel == False):
                newname = Xuniquename.main(rootpath, rootname, 
                            'D', '_rel', relaxct)
                relaxme = Vrelax.main(fullpath, rootpath, newname)
                if (relaxme == None):
                    print 'Error relaxing', fullpath
                    print '(Exiting.)'
                    return None
                prevpath = fullpath
                fullpath = rootpath + '/' + newname
        relaxct = relaxct + 1
    delproc=""
    strjobid=""
    if (oktostoprel == None):
        print 'There was an error in job', jobid
        if (emaddr != ""):
            Psendemail.main(emaddr,'Error in autorelaxation',
            'Error in job ' + jobid + ' related to: ' + fullpath)
        if (jobid != None):
            print 'Would have deleted job from queue, but'
            print 'this function has been disabled.'
            #strjobid = str(jobid)
            #delproc = subprocess.Popen(['qdel ' + strjobid], shell=True)
            #delproc.wait()
            #print 'Deleting job from queue.'
        print '(Exiting.)'
        return None
    else:
        print 'Job', jobid, 'finished relaxing around ', Xtimestring.main()
        return True
