#!//usr/bin/env python
##
#@package  Rdorb 
#@ingroup   
#@ingroup   python
#@brief     Rotations for d-orbitals
#@author    Tam Mayeshiba
#@date      2013-04-22
###
import numpy as np
import os

def get_occ_and_ev_lines(opath, printnow=False):
    """Get occupancies and eigenvectors lines from outcar.
        Use the last set of lines.

        opath: OUTCAR path
        printnow: set to True to print output

        Returns:
            Dictionary containing the
            last set of o= and v= (grep occupancies and eigenvectors)
            lines in the OUTCAR.
            The dictionary is organized as
            dict[atomnumber][linenumber] = line
    """
    if not os.path.exists(opath):
        print "No file at ", opath
        return
    myfile = open(opath, 'rb')
    mylines = myfile.readlines()
    myfile.close()
    okay = 0 #want to make sure whole file is transmitted.
    nionline = ""
    for line in mylines:
        if "User" in line:
            print "User time found. Continue.", line
            okay = 1
        if "NIONS" in line:
            nionline = line
    if okay == 0:
        print "Error reading all of OUTCAR."
        return
    nions = int(nionline.split()[-1])
    print "NIONS: ",nions
    atomindices=[]
    numlines = len(mylines)
    linect = 0
    while linect < numlines:
        if "atom =" in mylines[linect]:
            atomindices.append(linect)
        linect = linect + 1
    #print atomindices
    #print len(atomindices)
    lastset = atomindices[-1*nions:]
    occevlines_d = dict()
    for atomidx in lastset:
        linect = atomidx
        my_atom = int(mylines[linect].split()[2])
        #print "atom: ",str(my_atom)
        occevlines_d[my_atom]=dict()
        my_lval = int(mylines[linect].split()[-1])
        occlines = (my_lval*2+1)*2
        occup = my_lval*2 + 1
        #print "number of occ lines: ", occlines
        occline = 0
        myline=""
        while linect < numlines and occline < occlines:
            myline = mylines[linect]
            if "v = " in myline:
                occevlines_d[my_atom][linect] = myline
                occline = occline + 1
            linect = linect + 1
    if printnow:
        atomlist = occevlines_d.keys()
        atomlist.sort()
        print "Occupation and eigenvector lines for each atom:"
        for myatom in atomlist:
            print "Atom", myatom, ":"
            print_one_level_dictionary_vertical(occevlines_d[myatom])
    return occevlines_d




def make_occ_and_ev_dict(occevlines_d, printnow=False, removezeros=True):
    """Get occupancies and eigenvalues matrices from dictionary
        occevlines_d: dictionary of the occupation and eigenvector
                        lines from the OUTCAR, with keys as
                        atom numbers.
        printnow: set to True to print output
        removezeros: remove lines from dictionary if all orbital
                        values in that line are zero

        Returns:
            dictionary of occupancies and eigenvalues, indexed
            arbitrarily by line number (VASP sorts them high
            to low).
            Each atom gets keys 'spin1' and 'spin2'.
            Each spin gets keys 'eval' for eigenvalues or
                'evec' for eigenvectors.
            Each 'eval' and 'evec' key gets line number keys.
            Each line number under 'evec'
            gets additional keys corresponding to the
            real m number (e.g. little m = -2 for dxy in l=2)
            Ex: for atom 3, first occupancies and eigenvectors line,
            dict[3]['spin1']['eval'][0]=3
            dict[3]['spin2']['evec'][0][-2]=-0.4
            dict[3]['spin2']['evec'][0][-1]=-0.4
            Assume that if v = has 10 values, 5 are spin up, 5 are
            spin down, and so on, for real m = -l...0...l
    """
    occev_d=dict()
    atomlist = occevlines_d.keys()
    atomlist.sort()
    for myatom in atomlist:
        occev_d[myatom] = dict()
        occev_d[myatom]['spin1'] = dict()
        occev_d[myatom]['spin2'] = dict()
        occev_d[myatom]['spin1']['eval'] = dict()
        occev_d[myatom]['spin2']['eval'] = dict()
        occev_d[myatom]['spin1']['evec'] = dict()
        occev_d[myatom]['spin2']['evec'] = dict()
        linelist = occevlines_d[myatom].keys()
        linelist.sort()
        for lineno in linelist:
            myline = occevlines_d[myatom][lineno]
            eigval = complex(myline.split()[2])
            occev_d[myatom]['spin1']['eval'][lineno] = eigval
            occev_d[myatom]['spin1']['evec'][lineno] = dict()
            occev_d[myatom]['spin2']['eval'][lineno] = eigval
            occev_d[myatom]['spin2']['evec'][lineno] = dict()
            vstr = myline.split("v = ")[1].strip()
            vlist = vstr.split()
            occup = len(vlist)/2/2 #There is a second block of all zeros; ignore it?
            vct = 0
            myl = (occup - 1)/2
            orbct = -myl #start little m at -l
            while vct < occup:
                occev_d[myatom]['spin1']['evec'][lineno][orbct]=complex(vlist[vct])
                occev_d[myatom]['spin2']['evec'][lineno][orbct]=complex(vlist[vct + occup])
                vct = vct + 1
                orbct = orbct + 1
    if removezeros:
        nozerolines_d = dict(occev_d)
        for myatom in atomlist:
            for myspin in ['spin1','spin2']:
                linelist = occevlines_d[myatom].keys()
                linelist.sort()
                for lineno in linelist:
                    if sum(occev_d[myatom][myspin]['evec'][lineno].values()) == 0:
                        del nozerolines_d[myatom][myspin]['evec'][lineno]
                        del nozerolines_d[myatom][myspin]['eval'][lineno]
        occev_d_return = dict(nozerolines_d)
    else:
        occev_d_return = dict(occev_d)
    if printnow:
        for myatom in atomlist:
            print "ATOM ",str(myatom)
            for myspin in ['spin1','spin2']:
                print "Spin: ",myspin
                print "Eigenvectors:"
                print_two_level_dictionary(occev_d_return[myatom][myspin]['evec'])
                print "Eigenvalues:"
                print_one_level_dictionary(occev_d_return[myatom][myspin]['eval'])
    return occev_d_return

def make_column_vector_matrix(orbital_d, printnow=False):
    """Make a matrix of column vectors of values in an orbital
        dictionary, structured D[lineno][orbital].
        The columns are in order of line number, and the 
        orbitals are for example m=-l...0...l

        orbital_d: the orbital dictionary
        printnow: set to True to print output

        Returns:
            Numpy array
    """
    collist = orbital_d.keys()
    if len(collist) == 0:
        print "No columns."
        return
    collist.sort()
    rowlist = orbital_d[collist[0]].keys() 
    #Keys are assumed to be the same for each column.
    rowlist.sort()
    colct = 0
    vectormat = np.zeros([len(rowlist),len(collist)],complex)
    for mycol in collist:
        rowct = 0
        for myrow in rowlist:
            vectormat[rowct][colct] = orbital_d[mycol][myrow]
            rowct = rowct + 1
        colct = colct + 1
    if printnow:
        print_matrix(vectormat)
    return vectormat

def make_diagonal_matrix(eval_d, printnow=False):
    """Make a diagonal matrix from a dictionary of eigenvalues,
        set up like: D[lineno]=eigenvalue. Sort by line number.
        
        Returns: 
            Numpy array
    """
    collist = eval_d.keys()
    if len(collist) == 0:
        print "No columns."
        return
    collist.sort()
    colct = 0
    evalmat = np.zeros([len(collist),len(collist)],complex)
    for mycol in collist:
        evalmat[colct][colct] = eval_d[mycol]
        colct = colct + 1
    if printnow:
        print_matrix(evalmat)
    return evalmat

def get_occupations(evalmat, evecmat, printnow=False):
    """Get occupations from eigenvalue and eigenvector matrices.
        Get the magnitude of eigenvector coefficients 
        (coeff* x coeff) and multiply them by the corresponding
        eigenvalue (each column is an eigenvector).
        Then add up all the contributions in one row to get
        the contribution in an orbital.

        evalmat: eigenvalue diagonal matrix
        evecmat: eigenvector matrix, with eigenvectors in columns
        printnow: set to True to print output

        Returns:
            occ_d = occupation dictionary, as occ_d[orbital]=value
            occmat = occupation matrix per column
    """
    conjevecmat = np.conj(evecmat)
    sqmat = np.multiply(conjevecmat, evecmat)
    transposeocc = np.dot(evalmat, np.transpose(sqmat))
    occmat = np.transpose(transposeocc)
    occsum = np.sum(occmat,1) #sum across columns
    numorbs = len(occsum)
    myl = (numorbs - 1)/2
    occ_d=dict()
    for mym in range(-myl,myl+1):
        occ_d[mym] = occsum[mym + myl]
    if printnow:
        print "Printing occupation matrix:"
        print_matrix(occmat)
        print "Printing total occupation:"
        print_one_level_dictionary(occ_d)
    return [occ_d,occmat]

def get_original_matrix(evalmat, evecmat, printnow=False):
    """Get original matrix from set of eigenvalues and eigenvectors.
        The eigenvectors should be column vectors corresponding
        to the eigenvalues listed in order.
        [A-(lambda)I]v = 0
        evalmat: diagonal matrix of eigenvalues = D
        evecmat: matrix with each column being an eigenvector = P
        A = PD(P^-1)
        Rounds array to 4 decimal places, which is what vasp provides
    """
    my_p = evecmat
    my_d = evalmat
    my_pinv = np.linalg.pinv(evecmat)
    my_pd = np.dot(my_p,my_d)
    my_pdpinv = np.dot(my_pd, my_pinv)
    if printnow:
        print "P matrix :"
        print_matrix(my_p)
        print "D matrix :"
        print_matrix(my_d)
        print "PD matrix:"
        print_matrix(my_pd)
        print "Printing original matrix:"
        print_matrix(my_pdpinv)
    return np.round(my_pdpinv,4)
    
def make_orbdict_from_matrix(mymat, printnow=False):
    """Make an orbital dictionary list from a matrix
        with row indices 
        corresponding to m = -l...0...l in order, and
        arbitrary line numbers as columns.

        Returns:
            Dictionary of the form D[colnum][orbital] = value
    """
    numrows = np.size(mymat,0)
    numcols = np.size(mymat,1)
    myl = (numrows - 1)/2
    colct = 0
    mydict=dict()
    while colct < numcols:
        rowct = 0
        mydict[colct] = dict()
        while rowct < numrows:
            mydict[colct][rowct - myl] = mymat[rowct][colct]
            rowct = rowct + 1
        colct = colct + 1
    if printnow:
        print_two_level_dictionary(mydict)
    return mydict

def get_complex_from_real_string(ylmliststr):
    ylmdict = split_string_into_orbitals(ylmliststr)
    bigylmdict = get_complex_from_real(ylmdict,True,2)

def split_string_into_orbitals(ylmliststr, printnow=False):
    """Split a string into orbitals, in order, going from 
        -m...0...+m
            ylmliststr: string of orbital coefficients, in order
            printnow: set to True to print the dictionary.
        Return:
            ylmdict = dictionary of floats indexed by integer m
    """
    ylmsplit=ylmliststr.split()
    numorbs = len(ylmsplit)
    myl = (numorbs - 1)/2
    ylmvals=range(-myl,myl+1) #plus one stops at myl
    ylmdict=dict()
    #print "YLMVALS: ",ylmvals
    #print "YLMSPLIT: ",ylmsplit
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+myl])
    if printnow:
        print_one_level_dictionary(ylmdict)
    return ylmdict

def get_complex_from_real(ylmdict, printnow=False, myl=2):
    """Get complex orbitals from the real orbitals.
        ylmdict = dictionary of real orbital wavefunction coefficient
                    values, e.g. ylmdict[-2]=....
        l = quantum number l (only l=2 is supported)
        Returns:
            bigylmdict = dictionary of complex coefficients for
                        Y_lm
    """
    if not (myl == 2):
        print "Only l=2 is supported."
        return
    sqr2ov2 = np.sqrt(2)/2
    isqr2ov2 = np.complex(0,sqr2ov2)
    bigylmdict = dict()
    bigylmdict[0] = ylmdict[0]
    bigylmdict[2] = np.dot(sqr2ov2,ylmdict[2]) + np.dot(isqr2ov2,ylmdict[-2])
    bigylmdict[-2] = np.dot(sqr2ov2,ylmdict[2]) - np.dot(isqr2ov2,ylmdict[-2])
    bigylmdict[1] = -np.dot(sqr2ov2,ylmdict[1]) - np.dot(isqr2ov2,ylmdict[-1])
    bigylmdict[-1] = np.dot(sqr2ov2,ylmdict[1]) - np.dot(isqr2ov2,ylmdict[-1])
    if printnow:
        print_one_level_dictionary(bigylmdict)
    return bigylmdict


    


def get_complex_rotation_matrix(lval,gamma,beta,alpha,printnow=False):
    """
    Purpose:
        Output a numpy array for complex spherical harmonic rotations
        inputs:  
            [$1] lval: l for spherical harmonic Y_lm
            [$2] gamma: counterclockwise rotation in DEGREES around
                    the z-axis.
            (Yes, gamma comes first here. Apparently gamma is the
            first rotation in this paper. So we will feed in "alpha,
                beta, gamma" and the paper's formula will take our
                alpha to be their gamma, and our gamma to be their
                alpha.)
            [$4] beta: counterclockwise rotation in DEGREES around 
                    the x-axis rotated by gamma around z.
            [$5] alpha: counterclockwise rotation in DEGREES around 
                    the new z-axis, produced by the rotation of
                    gamma around z and then beta around x.
            [$6] printme: set to True to print the matrix nicely.
            example: getmatrix(2,10,-10,5)
        outputs: dictionary of rotation terms,
            D[orbital in old system (m')][orbital in new system (m)]
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   Ylm(theta', phi') = sum_m'[-l to l] of Ylm'(theta, phi) *
#                       D m',m,l (alpha, beta, gamma)
#   D m',m,l = exp(-im'alpha) * d m',m,l(beta) * exp(-im*gamma)
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!

#   # Validation
    myl = Xvalidinteger.main(lval,0,None)
    if myl == None:
        print lval, "is not a valid quantum number l (>= 0)"
        return None
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(alpha)*math.pi/180
    except ValueError, TypeError:
        print "Angle", alpha, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(gamma)*math.pi/180
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   Build dictionary        
    #D_mpr_m_l = np.zeros([2*l+1,2*l+1],complex)
    D_mpr_m_l = dict()
    for mpr in range(-myl,myl+1): #[-l...0...l]; +1 is because of python
        D_mpr_m_l[mpr]=dict()
        for mreg in range(-myl,myl+1):
            expa = cmath.exp(complex(0,-1*mpr*arad))
            expg = cmath.exp(complex(0,-1*mreg*grad))
            d_1 = math.factorial(myl+mpr)*math.factorial(myl-mpr)
            d_1 = d_1/(math.factorial(myl+mreg)*math.factorial(myl-mreg))
            d_1 = math.pow(d_1,0.5)
            d_2 = math.pow(-1,(mpr - mreg))
            ksum=0.0
            for myk in range(max(0,mreg-mpr),min(myl-mpr,myl+mreg)+1):
                kterm = math.pow(-1,myk)
                lchoose1 = math.factorial(myl+mreg)
                lchoose1 = lchoose1/math.factorial(myl+mreg - myk)
                lchoose1 = lchoose1/math.factorial(myk)
                lchoose2 = math.factorial(myl-mreg)
                lchoose2 = lchoose2/math.factorial(myl-mreg-(myl-mpr-myk))
                lchoose2 = lchoose2/math.factorial(myl-mpr-myk)
                costerm = math.cos(brad/2)
                costerm = math.pow(costerm,2*myl-mpr+mreg-2*myk)
                sinterm = math.sin(brad/2)
                sinterm = math.pow(sinterm, 2*myk-mreg+mpr)
                kterm = float(kterm) * float(lchoose1) * float(lchoose2) * float(costerm) * float(sinterm)
                ksum = ksum + kterm
            dterm = d_1 * d_2 * ksum
            D_mpr_m_l[mpr][mreg] = expa * dterm * expg
    if printnow:
        print_two_level_dictionary(D_mpr_m_l)
    return D_mpr_m_l

def get_real_from_complex(bigylmdict, printnow=False, myl=2):
    """Get real orbitals from complex orbitals.
        Only l=2 is supported.
    """
    if not (myl == 2):
        print "Only l=2 is supported."
        return
    ylmdict=dict()
    #1/(isqrt2) = (i/i)(1/isqrt2) = -i/sqrt2
    recipsqrt2 = np.power(np.power(2,0.5),-1)
    recipisqrt2 = np.complex(0,-recipsqrt2)
    for littlem in range(-myl,myl+1):
        absm = np.abs(littlem)
        if littlem < 0:
            ylmdict[littlem] = np.dot(np.power(-1,absm)*bigylmdict[absm] - bigylmdict[-absm],recipisqrt2)
        elif littlem == 0:
            ylmdict[littlem] = bigylmdict[absm]
        else:
            ylmdict[littlem] = np.dot(np.power(-1,absm)*bigylmdict[absm] + bigylmdict[-absm],recipsqrt2)
    if printnow:
        print_one_level_dictionary(ylmdict)
    return ylmdict


def rotate_complex_orbitals(alpha, beta, gamma, bigylmdict, printnow=False):
    """Rotate complex orbitals.
        alpha: counterclockwise rotation in degrees around z
        beta: counterclockwise rotation in degrees around new x
        gamma: counterclockwise rotation in degrees around new z
        bigylmdict: dictionary of complex orbital coefficients
        printnow: set to True to print the output
    """
    myl = (len(bigylmdict)-1)/2
    rotate_d = get_complex_rotation_matrix(myl, alpha, beta, gamma)
    # From Su and Coppens, Acta Cryst 1994
    #   Ylm(theta', phi') = sum_m'[-l to l] of Ylm'(theta, phi) *
    #                       D m',m,l (alpha, beta, gamma)
    #   D m',m,l = exp(-im'alpha) * d m',m,l(beta) * exp(-im*gamma)
    #   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
    #               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
    #               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
    #               x [sin(beta/2)]^(2k-m+m')
    #   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
    #   (a choose b) = a!/(a-b)!b!
    newbigylm_d = dict()
    for newm in range(-myl,myl+1):
        summpr = 0
        for mpr in range(-myl,myl+1):
            summpr = summpr + bigylmdict[mpr]*rotate_d[mpr][newm]
        newbigylm_d[newm] = summpr
    if printnow:
        print_one_level_dictionary(newbigylm_d)
    return newbigylm_d

def eig_matrix(mymat, printnow=False):
    """Get eigneeigenvalues and eigenvectors from numpy array.
    """
    #myevals, myevecs = np.linalg.eig(mymat)
    myevals, myevecs = np.linalg.eigh(mymat)
    myevals_diag = myevals * np.identity(len(myevals),complex)
    if printnow:
        print "Eigenvalues:"
        print_matrix(myevals_diag)
        print "Eigenvectors:"
        print_matrix(myevecs)
    return [myevals_diag, myevecs]

def print_one_level_dictionary(mydict):
    """Print a one-level dictionary whose indices are integers
        and whose values are floats.
    """
    key1list = mydict.keys()
    key1list.sort()
    print "keys: " + ' '.join(map(str,key1list))
    printstr=""
    for key1 in key1list:
        printseg = '%4.4f+%4.4fj   ' % (np.real(mydict[key1]),np.imag(mydict[key1]))
        printstr = printstr + printseg
    print printstr
    return

def print_one_level_dictionary_vertical(mydict):
    """Print a one-level dictionary whose indices are integers
        and whose values are floats or strings.
    """
    key1list = mydict.keys()
    key1list.sort()
    printstr=""
    for key1 in key1list:
        if type(mydict[key1]) == complex or type(mydict[key1]) == float:
            print "TYPE: ",type(mydict[key1])
            print '%8i:  %4.4f+%4.4fj   ' % (key1,np.real(mydict[key1]),np.imag(mydict[key1]))
        else:
            print '%8i: %s' % (key1,mydict[key1])
    return

def print_two_level_dictionary(mydict):
    """Print a two-level dictionary (which is like a matrix),
        whose indices are integers and whose values are floats.
    """
    key1list = mydict.keys()
    key1list.sort()
    firstline = True
    for key1 in key1list:
        key2list = mydict[key1].keys()
        key2list.sort()
        if firstline:
            print "               " + ' '.join(map(str,key2list)) 
            print "               ----------------------------"
        firstline = False
        printstr = '%2i : ' % key1
        for key2 in key2list:
            keyseg = '%4.4f+%4.4fj   ' % (np.real(mydict[key1][key2]),
                        np.imag(mydict[key1][key2]))
            printstr = printstr + keyseg
        print printstr
    return

def print_matrix(mymat):
    """Print a numpy array."""
    numrows = np.size(mymat,0)
    numcols = np.size(mymat,1)
    rowct = 0
    print "PRINTING MATRIX:"
    while rowct < numrows:
        printstr = ""
        colct = 0
        while colct < numcols:
            printstr = printstr + "%4.4f+%4.4fj" % (np.real(mymat[rowct,colct]),np.imag(mymat[rowct,colct])) + " "
            colct = colct + 1
        print printstr
        rowct = rowct + 1
    return

def rotate_occ_and_ev_through_complex(mypath,l=2,occatoms=[]):
    """Rotate occupancies by going through the complex orbitals
        first.
        mypath: directory
    """
    import Rcoords
    contcarfile = os.path.join(mypath, "CONTCAR")
    outcarfile = os.path.join(mypath, "OUTCAR")
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    #occatoms = [9] #TTM DEBUG
    coordset=Rcoords.get_coords(contcarfile, 0.05)
    oextend=Rcoords.extend_cell(coordset[2],0.2)
    octdict=Rcoords.build_octahedra(coordset[1],oextend,0.4)
    #parse occupation file
    occevlines_d = get_occ_and_ev_lines(outcarfile)
    occev_d = make_occ_and_ev_dict(occevlines_d)
    myprintlist=[]
    myprintlist.append("outcarfile: " + str(outcarfile))
    print "outcarfile: ",outcarfile
    myprintlist.append("contcarfile: " + str(contcarfile))
    print "contcarfile: ",contcarfile
    if l == 2:
        orbstr= "dxy dyz dz2 dxz dx2"
        print orbstr
        myprintlist.append(orbstr)
    occdict=dict()
    rotoccdict=dict()
    rotrloccdict=dict()
    evrotoccdict=dict()
    for atom in occatoms:
        occdict[atom]=""
        rotoccdict[atom]=""
        rotrloccdict[atom]=""
        evrotoccdict[atom]=""
        print ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
        print "ATOM: ",str(atom)
        getlines=Rcoords.get_lines_from_octahedron(octdict,atom)
        if not(len(getlines) > 0 and len(getlines[0]) > 0):
            print "Error getting coordinates!"
            return
        alphabetagamma = Rcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
        alp = alphabetagamma[0]
        bet = alphabetagamma[1]
        gam = alphabetagamma[2]
        #occupation dict is d[atom]['spin1']['evec'][lineno][orbital]
        # or d[atom]['spin1']['eval'][lineno]
        totaloccmat = np.zeros([(2*l+1),(2*l+1)],complex)
        for myspin in ['spin1','spin2']:
            print "============" + myspin + "============="
            evals = occev_d[atom][myspin]['eval']
            evecs = occev_d[atom][myspin]['evec']
            evecmat = make_column_vector_matrix(evecs, True)
            evalmat = make_diagonal_matrix(evals, True)
            print "------------No rotation:---------------- "
            [unrotocc,unrotoccmat] = get_occupations(evalmat, evecmat, True)
            print "---With rotation of complex orbitals:--- " 
            origmat = get_original_matrix(evalmat, evecmat, True)
            origorbs = make_orbdict_from_matrix(origmat,True)
            rotevecs=dict()
            for myline in origorbs.keys():
                complex_dict = get_complex_from_real(origorbs[myline])
                rot_dict = rotate_complex_orbitals(alp,bet,gam, complex_dict)
                real_back = get_real_from_complex(rot_dict)
                rotevecs[myline] = real_back
            rotmat = make_column_vector_matrix(rotevecs,True)
            [rotevalmat,rotevecmat] = eig_matrix(rotmat, True)
            [rotocc,rotoccmat] = get_occupations(rotevalmat, rotevecmat, True)
            print "---Real eigenvector rotation only, using unrotated eigenvalues:---- "
            evrotevecs = dict()
            for myline in evecs.keys():
                evrot_vec = rotatereal_dict(l,alp,bet,gam,evecs[myline])
                evrotevecs[myline] = evrot_vec
            evrotevecmat = make_column_vector_matrix(evrotevecs)
            [evrotocc,evrotoccmat] = get_occupations(evalmat,evrotevecmat,True)
            print "---Original matrix real rotation:--------------"
            rotrlevecs=dict()
            for myline in origorbs.keys():
                rot_dict = rotatereal_dict(l,alp,bet,gam, origorbs[myline])
                rotrlevecs[myline] = rot_dict
            rotrlmat = make_column_vector_matrix(rotrlevecs,True)
            [rotrlevalmat,rotrlevecmat] = eig_matrix(rotrlmat)
            [rotrlocc,rotrloccmat] = get_occupations(rotrlevalmat, rotrlevecmat, True)

            if occdict[atom] == "":
                occdict[atom] = dict(unrotocc)
                rotoccdict[atom] = dict(rotocc)
                rotrloccdict[atom] = dict(rotrlocc)
                evrotoccdict[atom] = dict(evrotocc)
            else:
                for orbkey in occdict[atom].keys():
                    print "orbkey:",orbkey
                    occdict[atom][orbkey] = occdict[atom][orbkey] + unrotocc[orbkey]
                    rotoccdict[atom][orbkey] = rotoccdict[atom][orbkey] + rotocc[orbkey]
                    rotrloccdict[atom][orbkey] = rotrloccdict[atom][orbkey] + rotrlocc[orbkey]
                    evrotoccdict[atom][orbkey] = evrotoccdict[atom][orbkey] + evrotocc[orbkey]

    print "RESULTS:"
    print "UNROTATED:"
    print_two_level_dictionary(occdict)
    print "ORIG MATRIX COMPLEX ROTATED:"
    print_two_level_dictionary(rotoccdict)
    print "ORIG MATRIX REAL ROTATED:"
    print_two_level_dictionary(rotrloccdict)
    print "EIGENVECTORS ROTATED WITH UNROTATED EIGENVALS."
    print_two_level_dictionary(evrotoccdict)
    return myprintlist

def getbetadict(l, beta):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$4] beta: counterclockwise rotation in DEGREES around y
            example: getbetadict(2,-10)
        outputs: dictionary[m'][m] of float
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   # Validation
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
#   #   Build array        
    d_mpr_m_l = dict()
    for mpr in range(-l,l+1): #[-l...0...l]; +1 is because of python
        d_mpr_m_l[mpr]=dict()
        for mreg in range(-l,l+1):
            d_1 = math.factorial(l+mpr)*math.factorial(l-mpr)
            d_1 = d_1/(math.factorial(l+mreg)*math.factorial(l-mreg))
            d_1 = math.pow(d_1,0.5)
            d_2 = math.pow(-1,(mpr - mreg))
            ksum=0.0
            for myk in range(max(0,mreg-mpr),min(l-mpr,l+mreg)+1):
                kterm = math.pow(-1,myk)
                lchoose1 = math.factorial(l+mreg)
                lchoose1 = lchoose1/math.factorial(l + mreg - myk)
                lchoose1 = lchoose1/math.factorial(myk)
                lchoose2 = math.factorial(l-mreg)
                lchoose2 = lchoose2/math.factorial(l-mreg-(l-mpr-myk))
                lchoose2 = lchoose2/math.factorial(l-mpr-myk)
                costerm = math.cos(brad/2)
                costerm = math.pow(costerm,2*l-mpr+mreg-2*myk)
                sinterm = math.sin(brad/2)
                sinterm = math.pow(sinterm, 2*myk-mreg+mpr)
                kterm = float(kterm) * float(lchoose1) * float(lchoose2) * float(costerm) * float(sinterm)
                ksum = ksum + kterm
            dterm = d_1 * d_2 * ksum
            d_mpr_m_l[mpr][mreg] = dterm
    return d_mpr_m_l


def rotatereal(l, alpha, beta, gamma, ylmliststr, printnow=False):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    #print "YLMVALS: ",ylmvals
    #print "YLMSPLIT: ",ylmsplit
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    if l==2 and printnow == True:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2]
        print "-------------------------------"
    return ylmreturn


def rotatereal_dict(l, gamma, beta, alpha, ylmdict, printnow=False):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] gamma: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            (Their gamma and alpha are switched from our
            alpha and gamma.)
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] alpha: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmdict: dictionary of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(alpha)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", alpha, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(gamma)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    if l==2 and printnow == True:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2]
        print "-------------------------------"
    return ylmreturn


def rotatereal_verbose(l, alpha, beta, gamma, ylmliststr):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        print "MRO :", mro
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    allsum=0.0
    for orbital in ylmreturn.keys():
        allsum = allsum + ylmreturn[orbital]
    if l==2:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2 sum"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2], allsum
        print "-------------------------------"
    return ylmreturn


def rotatereal_verbose_normalize(l, alpha, beta, gamma, ylmliststr):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        Normalizes so that the total
        post-rotation sum is the same as the pre-rotation sum.
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        print "MRO :", mro,":value: ",ylmdict[mro]
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    rotsum=0.0
    unrotsum=0.0
    for orbital in ylmreturn.keys():
        rotsum = rotsum + ylmreturn[orbital]
        unrotsum = unrotsum + ylmdict[orbital]
    print "ROT SUM:",rotsum
    print "UNROT SUM:",unrotsum
    #NEW: NORMALIZE
    for orbital in ylmreturn.keys():
        if not rotsum == 0:
            ylmreturn[orbital] = ylmreturn[orbital]*unrotsum/rotsum
        elif unrotsum == 0:
            ylmreturn[orbital] = 0
        else:
            print "Error: rotated sum is zero, but unrotated sum is not."
            return
    allsum=0.0
    for orbital in ylmreturn.keys():
        allsum = allsum + ylmreturn[orbital]
    if l==2:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2 sum"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2], allsum
        print "-------------------------------"
    return ylmreturn


