#!//usr/bin/env python
##
#@package  Rdrotate 
#@ingroup   
#@ingroup   python
#@brief     Rotations for d-orbitals
#@author    Tam Mayeshiba
#@date      2013-04-22
###
def getcomplexmatrix(l, m, alpha,beta,gamma):
    """
    Purpose:
        Output a numpy array for complex spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic Y_lm
            [$2] m: m for spherical harmonic Y_lm
            [$3] alpha: counterclockwise rotation in DEGREES around z
            [$4] beta: counterclockwise rotation in DEGREES around x
                    This is really rotation around [x-axis rotated by alpha 
                    around z].
            [$5] gamma: counterclockwise rotation in DEGREES around y
                    This is really rotation around other rotations, in plane
                    whose normal is new z after rotations alpha and beta.
            example: getmatrix(2,-2,10,-10,5)
        outputs: numpy array (2l+1 x 2l+1)
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   Ylm(theta', phi') = sum_m'[-l to l] of Ylm'(theta, phi) *
#                       D m',m,l (alpha, beta, gamma)
#   D m',m,l = exp(-im'alpha) * d m',m,l(beta) * exp(-im*gamma)
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!

#   # Validation
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    newm=0
    newm = Xvalidinteger.main(m, -1*l, l)
    if newm == None:
        print "Quantum number m must be within +/-", l
        return None
    m = newm
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(alpha)*math.pi/180
    except ValueError, TypeError:
        print "Angle", alpha, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(gamma)*math.pi/180
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   Build array        
    D_mpr_m_l = np.zeros([2*l+1,2*l+1],complex)
    for mpr in range(-l,l+1): #[-l...0...l]; +1 is because of python
        for mreg in range(-l,l+1):
            expa = cmath.exp(complex(0,-1*mpr*arad))
            expg = cmath.exp(complex(0,-1*mreg*grad))
            d_1 = math.factorial(l+mpr)*math.factorial(l-mpr)
            d_1 = d_1/(math.factorial(l+mreg)*math.factorial(l-mreg))
            d_1 = math.pow(d_1,0.5)
            d_2 = math.pow(-1,(mpr - mreg))
            ksum=0.0
            for myk in range(max(0,mreg-mpr),min(l-mpr,l+mreg)+1):
                kterm = math.pow(-1,myk)
                lchoose = math.factorial(l-mreg)
                lchoose = lchoose/math.factorial(l-mreg-(l-mpr-myk))
                lchoose = lchoose/math.factorial(l-mpr-myk)
                costerm = math.cos(brad/2)
                costerm = math.pow(costerm,2*l-mpr+mreg-2*myk)
                sinterm = math.sin(brad/2)
                sinterm = math.pow(sinterm, 2*myk-mreg+mpr)
                kterm = float(kterm) * float(lchoose) * float(costerm) * float(sinterm)
                ksum = ksum + kterm
            dterm = d_1 * d_2 * ksum
            mprtoidx = mpr + l
            mregtoidx = mreg + l
            D_mpr_m_l[mprtoidx, mregtoidx] = expa * dterm * expg
    return D_mpr_m_l

#   Ylm(theta', phi') = sum_m'[-l to l] of Ylm'(theta, phi) *
#                       D m',m,l (alpha, beta, gamma)
#   D m',m,l = exp(-im'alpha) * d m',m,l(beta) * exp(-im*gamma)
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!


    return

def getbetadict(l, beta):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$4] beta: counterclockwise rotation in DEGREES around y
            example: getbetadict(2,-10)
        outputs: dictionary[m'][m] of float
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
#   From Su and Coppens, Acta Cryst. 1994:
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   # Validation
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
#   #   Build array        
    d_mpr_m_l = dict()
    for mpr in range(-l,l+1): #[-l...0...l]; +1 is because of python
        d_mpr_m_l[mpr]=dict()
        for mreg in range(-l,l+1):
            d_1 = math.factorial(l+mpr)*math.factorial(l-mpr)
            d_1 = d_1/(math.factorial(l+mreg)*math.factorial(l-mreg))
            d_1 = math.pow(d_1,0.5)
            d_2 = math.pow(-1,(mpr - mreg))
            ksum=0.0
            for myk in range(max(0,mreg-mpr),min(l-mpr,l+mreg)+1):
                kterm = math.pow(-1,myk)
                lchoose1 = math.factorial(l+mreg)
                lchoose1 = lchoose1/math.factorial(l + mreg - myk)
                lchoose1 = lchoose1/math.factorial(myk)
                lchoose2 = math.factorial(l-mreg)
                lchoose2 = lchoose2/math.factorial(l-mreg-(l-mpr-myk))
                lchoose2 = lchoose2/math.factorial(l-mpr-myk)
                costerm = math.cos(brad/2)
                costerm = math.pow(costerm,2*l-mpr+mreg-2*myk)
                sinterm = math.sin(brad/2)
                sinterm = math.pow(sinterm, 2*myk-mreg+mpr)
                kterm = float(kterm) * float(lchoose1) * float(lchoose2) * float(costerm) * float(sinterm)
                ksum = ksum + kterm
            dterm = d_1 * d_2 * ksum
            d_mpr_m_l[mpr][mreg] = dterm
    return d_mpr_m_l


def rotatereal(l, alpha, beta, gamma, ylmliststr, printnow=False):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
    import Rdrotate
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    #print "YLMVALS: ",ylmvals
    #print "YLMSPLIT: ",ylmsplit
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = Rdrotate.getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    if l==2 and printnow == True:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2]
        print "-------------------------------"
    return ylmreturn


def rotatereal_verbose(l, alpha, beta, gamma, ylmliststr):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
    import Rdrotate
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = Rdrotate.getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        print "MRO :", mro
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    allsum=0.0
    for orbital in ylmreturn.keys():
        allsum = allsum + ylmreturn[orbital]
    if l==2:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2 sum"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2], allsum
        print "-------------------------------"
    return ylmreturn


def rotatereal_verbose_normalize(l, alpha, beta, gamma, ylmliststr):
    """
    Purpose:
        Output a numpy array for real spherical harmonic rotations
        Normalizes so that the total
        post-rotation sum is the same as the pre-rotation sum.
        inputs:  
            [$1] l: l for spherical harmonic y_lm
            [$2] alpha: counterclockwise rotation in DEGREES around
                z, resulting in axes x->N, y->y', z->z
            [$3] beta: counterclockwise rotation in DEGREES around
                N, resulting in axes N->N,y'->y'',z->Z
            [$4] gamma: counterclockwise rotation in DEGREES around
                Z, resulting in axes N->X, y''->Y, Z->Z
                So if beta is zero, gamma is really just around z.
            [$5] ylmliststr: string of y_lm (real spherical
                    harmonic) values, in order of increasing m
                    For d orbitals, m REAL of 
                    -2= dxy
                    -1= dyz
                    0= dz2
                    1= dxz
                    2= dx2-y2
            example: getbetadict(2,10,-10,5,"0.23 3.56 2.7 0.3 0.5")
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import math
    import cmath
    import numpy as np
    import Rdrotate
#   From Su and Coppens, Acta Cryst. 1994:
#   BUT NOTE THAT THEIR GAMMA IS WHAT WE DEFINED AS ALPHA ABOVE,
#   AND THEIR ALPHA IS WHAT WE DEFINED AS GAMMA.
#   ylm+(theta',phi')=(-1)^m*d_0ml(beta)cos(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m')d_m'ml*cos(m*gamma + m'*alpha)
#          + (-1)^m*d_-m'ml*cos(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*sin(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*sin(m*gamma-m'*alpha]*ylm'-
#   ylm-(theta',phi')=(-1)^(m+1)*d_0ml(beta)sin(m*gamma)*2^(1/2)*yl0
#   + sum(m'=1 to l) of
#       [(-1)^(m+m'+1)d_m'ml*sin(m*gamma + m'*alpha)
#          + (-1)^(m+1)*d_-m'ml*sin(m*gamma - m'*alpha)]*ylm'+
#       + [(-1)^(m+m')*d_m'ml*cos(m*gamma+m'*alpha) -
#           (-1)^m*d_-m'm*cos(m*gamma-m'*alpha]*ylm'-
#   yl0(theta',phi')=d_00l(beta)yl0(theta,phi)
#   + 2^(-1/2)* sum(m'=1 to l) of
#       [(-1)^m'*d_m'0l + d_-m'0l]*cos(m'*alpha)*ylm'+
#       [(-1)^m'*d_m'0l + d_-m'0l]*sin(m'*alpha)*ylm'-
#   d m',m,l = [(l+m')!(l-m')!/(l+m)!(l-m)!]^(1/2)
#               x (-1)^(m'-m) sum_k (-1)^k (l+m choose k)
#               x (l-m choose l-m'-k)[cos(beta/2)]^(2l-m'+m-2k)
#               x [sin(beta/2)]^(2k-m+m')
#   k is an integer max(0,m-m')<=k<=min(l-m',l+m)
#   (a choose b) = a!/(a-b)!b!
#   These REAL spherical harmonics are derived from the COMPLEX ones
#   by the following:
#   ylm+ = [(-1)^m*Y_l,m + Y_l,-m]/sqrt(2)
#   ylm- = [(-1)^m*Y_l,m - Y_l,-m]/[i*sqrt(2)]
#   yl0 = Y_l0
#   For d orbitals, 
#   y_2,0 = dz2
#   y_2,-2 = dxy
#   y_2,2 = dx2-y2
#   y_2,-1 = dyz
#   y_2,1 = dxz
#   # Validation
    ylmsplit=ylmliststr.split()
    ylmvals=range(-1*int(len(ylmsplit)/2),len(ylmsplit)/2+1)
    ylmdict=dict()
    for ylmval in ylmvals:
        ylmdict[ylmval] = float(ylmsplit[ylmval+l])
    newl=0
    newl = Xvalidinteger.main(l,0,None)
    if newl == None:
        print l, "is not a valid quantum number l (>= 0)"
        return None
    l = newl
    arad=0.0
    brad=0.0
    grad=0.0
    try:
        arad = float(gamma)*math.pi/180 #their alpha is our gamma
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
    try:
        brad = float(beta)*math.pi/180
    except ValueError, TypeError:
        print "Angle", beta, "cannot be parsed."
        return None
    try:
        grad = float(alpha)*math.pi/180 #their gamma is our alpha
    except ValueError, TypeError:
        print "Angle", gamma, "cannot be parsed."
        return None
#   #   get d_mpr_m_l array        
    dmprml = Rdrotate.getbetadict(l,beta)
    if dmprml == None:
        print "Error getting dictionary."
        return None
    ylmreturn=dict()
    for mro in range(-l,l+1): #[-l...0...l]; +1 is because of python
        newval=0.0
        zeroterm=0.0
        sumterm=0.0
        print "MRO :", mro,":value: ",ylmdict[mro]
        if mro > 0:
            zeroterm=math.pow(-1,mro)*dmprml[0][mro]
            zeroterm=zeroterm*math.cos(mro*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term1_1=term1_1*math.cos(mro*grad + mpr*arad)
                term1_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term1_2=term1_2*math.cos(mro*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mro+mpr)*dmprml[mpr][mro]
                term2_1=term2_1*math.sin(mro*grad + mpr*arad)
                term2_2=math.pow(-1,mro)*dmprml[-1*mpr][mro]
                term2_2=term2_2*math.sin(mro*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro < 0:
            mab = math.fabs(mro)
            zeroterm=math.pow(-1,mab+1)*dmprml[0][mab]
            zeroterm=zeroterm*math.sin(mab*grad)*math.pow(2,0.5)
            zeroterm=zeroterm*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mab+mpr+1)*dmprml[mpr][mab]
                term1_1=term1_1*math.sin(mab*grad + mpr*arad)
                term1_2=math.pow(-1,mab+1)*dmprml[-1*mpr][mab]
                term1_2=term1_2*math.sin(mab*grad - mpr*arad)
                term1=term1_1+term1_2
                term1=term1*ylmdict[mpr]
                term2_1=math.pow(-1,mab+mpr)*dmprml[mpr][mab]
                term2_1=term2_1*math.cos(mab*grad + mpr*arad)
                term2_2=math.pow(-1,mab)*dmprml[-1*mpr][mab]
                term2_2=term2_2*math.cos(mab*grad - mpr*arad)
                term2=term2_1-term2_2
                term2=term2*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + sumterm
        elif mro == 0:
            zeroterm=dmprml[0][mro]*ylmdict[0]
            print "    ZEROTERM: ",zeroterm
            for mpr in range(1, l+1): #[1,..l]; +1 for python
                term1_1=0.0
                term1_2=0.0
                term2_1=0.0
                term2_2=0.0
                term1=0.0
                term2=0.0
                term1_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term1_2=dmprml[-1*mpr][0]
                term1=term1_1+term1_2
                term1=term1*math.cos(mpr*arad)*ylmdict[mpr]
                term2_1=math.pow(-1,mpr)*dmprml[mpr][0]
                term2_2=dmprml[-1*mpr][0]
                term2=term2_1+term2_2 #this is a + because it is minus a negative sin term
                term2=term2*math.sin(mpr*arad)*ylmdict[-1*mpr]
                sumterm = sumterm + term1 + term2
                print "        MPR:", mpr, "term1, term2, sum: ", term1, term2, term1 + term2
            newval = zeroterm + math.pow(2,-0.5)*sumterm
        ylmreturn[mro]=newval
    rotsum=0.0
    unrotsum=0.0
    for orbital in ylmreturn.keys():
        rotsum = rotsum + ylmreturn[orbital]
        unrotsum = unrotsum + ylmdict[orbital]
    print "ROT SUM:",rotsum
    print "UNROT SUM:",unrotsum
    #NEW: NORMALIZE
    for orbital in ylmreturn.keys():
        if not rotsum == 0:
            ylmreturn[orbital] = ylmreturn[orbital]*unrotsum/rotsum
        elif unrotsum == 0:
            ylmreturn[orbital] = 0
        else:
            print "Error: rotated sum is zero, but unrotated sum is not."
            return
    allsum=0.0
    for orbital in ylmreturn.keys():
        allsum = allsum + ylmreturn[orbital]
    if l==2:
        print "----------------------"
        print "dxy dyz dz2 dxz dx2-y2 sum"
        print ylmreturn[-2], ylmreturn[-1], ylmreturn[0], ylmreturn[1], ylmreturn[2], allsum
        print "-------------------------------"
    return ylmreturn


