#!/bin/env/python
#Author: Tam Mayeshiba
#Date: 2013-04-18
import numpy as np
import math

def main(input_coords, theta, u, ploton=0):
    """Rotate coordinates around an axis through an angle
        input_coords = numpy array of x, y, z coordinates [x1 y1 z1; x2 y2 z2; etc.]
        angle = rotation angle in DEGREES, COUNTERCLOCKWISE
        u = list unit vector for rotation [ux, uy, uz]
        R = Icos(theta) + sin(theta)*(cross product matrix of u) + 
            (1-cos(theta))*(tensor product matrix of u)
        cross product matrix of u = [0 -uz uy; uz 0 -ux; -uy ux 0]
        tensor product matrix of u = [ux^2, uxuy, uxuz; uxuy, uy^2, uyuz; uxuz,
        uyuz, uz^2]
    """

    ux=float(u[0])
    uy=float(u[1])
    uz=float(u[2])
    thrad = theta*math.pi/180
    ucross = np.array([[0, -1*uz, uy],
                        [uz, 0, -1*ux],
                        [-1*uy, ux, 0]],float)
    utens = np.array([[np.power(ux,2), ux*uy, ux*uz],
                        [ ux*uy, np.power(uy,2), uy*uz],
                        [ux*uz, uy*uz, np.power(uz,2)]],float)
    Rmat = np.identity(3,float)*np.cos(thrad) + np.sin(thrad)*ucross + (1-np.cos(thrad))*utens
    rotated_coords = np.transpose(np.dot(Rmat,np.transpose(input_coords))); 
    #Rmat works on COLUMN vector of coords, [x; y; z]
    #The resulting column vector set of coordinates 
    #then gets transferred back into row vectors.

    if ploton==1:
        import matplotlib.pyplot as plt

        plt.plot(input_coords[:,0],input_coords[:,1],'rs')
        plt.plot(rotated_coords[:,0],rotated_coords[:,1],'b^')
        plt.xlabel('X (squares = original, triangles = rotated)')
        plt.ylabel('Y')
        plt.show()
    return rotated_coords

def rotate_octas(octdict, theta9x, theta9y, theta9z, theta13x,
        theta10z, theta12y):
    """Rotate octahedra.
        For eight linked octahedra there are only 6 unique tilts
        available. Octahedra along the same axis around which the
        tilt is being performed can have independent tilts.
        Octahedra along perpendicular axes must have equal and
        opposite tilts. (Glazer 1972)
        This function is for octahedra 
        in xz plane, lower left, counterclockwise 
        9, 13, 14, 10, back plane (higher y), lower left, ccwise
        11, 15, 16, 12
    """
    import Rrotateme
    ion=""
    octrotated=dict()
    for key in octdict.keys():
        octrotated[key]=dict()
    thetadict=dict()
    thetadict[9] =[ theta9x, theta9y, theta9z]
    thetadict[10]=[-1*theta9x, -1*theta9y, theta10z]
    thetadict[11]=[-1*theta9x, -1*theta12y, -1*theta9z]
    thetadict[12]=[theta9x, theta12y, -1*theta10z]
    thetadict[13]=[theta13x, -1*theta9y, -1*theta9z]
    thetadict[14]=[-1*theta13x, theta9y, -1*theta10z]
    thetadict[15]=[-1*theta13x, theta12y, theta9z]
    thetadict[16]=[theta13x, -1*theta12y, theta10z]
    for octkey in octdict.keys():
        for ionkey in octdict[octkey].keys():
            mycoord = np.array([octdict[octkey][ionkey]],float)
            bcoord = np.array([octdict[octkey][octkey]],float)
            mycoord = mycoord - bcoord #recenter around b-site cation
            myrotated = Rrotateme.main(mycoord, thetadict[octkey][0],
                            [1,0,0],0)
            myrotated = Rrotateme.main(myrotated,
                            thetadict[octkey][1],[0,1,0],0)
            myrotated = Rrotateme.main(myrotated, 
                            thetadict[octkey][2],[0,0,1],0)
            octrotated[octkey][ionkey]=myrotated + bcoord 

    for key, value in octrotated.iteritems():
        print key, ":", value
    return octrotated
def rotatestep(origname, perfname, myrange=2, workname=""):
    """Stepper rotation for pv only, 8 octahedra in cell.
        origname: original file name 
        perfname: perfect file name (want to match original)
    """
    import Rgetcoords
    import Rrotateme
    import os
    coordset_orig=Rgetcoords.main(origname, 0.05)
    coordset_perf=Rgetcoords.main(perfname, 0.05)
    distavgs=dict()
    diststds=dict()
    if workname=="":
        workname = os.path.dirname(perfname) + "/working_rotate"
    opstats = Rgetcoords.comparecoords(coordset_orig,coordset_perf)  
    #myrange = range(-20,21)
    perfoctdict=Rgetcoords.build_octahedra(coordset_perf[1],coordset_perf[2])
    myrange=range(-myrange,myrange+1)
    smallestbavg=100000
    smallestbavganglestr=""
    smallestbstd=100000
    smallestbstdanglestr=""
    for rot9x in myrange:
        for rot9y in myrange:
            for rot9z in myrange:
                for rot13x in myrange:
                    for rot10z in myrange:
                        for rot12y in myrange:
                            octrotated=Rrotateme.rotate_octas(perfoctdict, rot9x, rot9y, rot9z, rot13x, rot10z, rot12y)
                            Rgetcoords.print_back_out(perfname,
                                octrotated,workname,coordset_perf[0],
                                coordset_perf[1])
                            coordset_rot=Rgetcoords.main(workname,
                                    0.05)
                            newcompare=Rgetcoords.comparecoords(coordset_orig,coordset_rot)  
                            anglestr = str(rot9x) + " " + str(rot9y) + " " + str(rot9z) + " " +str(rot13x) + " " + str(rot10z) + " " + str(rot12y)
                            distavgs[anglestr]=newcompare[0]
                            diststds[anglestr]=newcompare[1]
                            if newcompare[0][2] < smallestbavg:
                                smallestbavg=newcompare[0][2]
                                smallestbavganglestr=anglestr
                            if newcompare[1][2] < smallestbstd:
                                smallestbstd=newcompare[1][2]
                                smallestbstdanglestr=anglestr


    return [distavgs,diststds, smallestbavg, smallestbavganglestr,
                smallestbstd, smallestbstdanglestr]
def rotatestep_smart(origname, perfname, myrange=10,difftol=0.001,workname=""):
    """Stepper rotation for pv only, 8 octahedra in cell.
        origname: original file name 
        perfname: perfect file name (want to match original)
    """
    import Rgetcoords
    import Rrotateme
    import os
    coordset_orig=Rgetcoords.main(origname, 0.05)
    coordset_perf=Rgetcoords.main(perfname, 0.05)
    distavgs=dict()
    diststds=dict()
    if workname=="":
        workname = os.path.dirname(perfname) + "/working_rotate"
    opstats = Rgetcoords.comparecoords(coordset_orig,coordset_perf)  
    #myrange = range(-20,21)
    perfoctdict=Rgetcoords.build_octahedra(coordset_perf[1],coordset_perf[2])
    myrange=range(-myrange,myrange+1)
    small9xavg=100000
    small9yavg=100000
    small9zavg=100000
    small13xavg=100000
    small10zavg=100000
    small12yavg=100000
    small9xangle=""
    small9yangle=""
    small9zangle=""
    small13xangle=""
    small10zangle=""
    small12yangle=""
    small9xstd=100000
    small9ystd=100000
    small9zstd=100000
    small13xstd=100000
    small10zstd=100000
    small12ystd=100000
    #octrotated=Rrotateme.rotate_octas(perfoctdict, rot9x, rot9y, rot9z, rot13x, rot10z, rot12y)
    for rot9x in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict, rot9x, 
                    0,0,0,0,0)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small9xavg - difftol:
            small9xavg=newcompare[0][2]
            small9xangle=rot9x
    for rot9y in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict, small9xangle,
                rot9y, 0, 0, 0, 0)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small9yavg - difftol:
            small9yavg=newcompare[0][2]
            small9yangle=rot9y
    for rot9z in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict, small9xangle,
                small9yangle, rot9z, 0, 0, 0)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small9zavg - difftol:
            small9zavg=newcompare[0][2]
            small9zangle=rot9z
    for rot13x in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict, small9xangle,
                small9yangle, small9zangle,
                rot13x, 0, 0)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small13xavg - difftol:
            small13xavg=newcompare[0][2]
            small13xangle=rot13x
    for rot10z in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict, small9xangle,
                        small9yangle, small9zangle, small13xangle,
                        rot10z, 0)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small10zavg - difftol:
            small10zavg=newcompare[0][2]
            small10zangle=rot10z
    for rot12y in myrange:
        octrotated=Rrotateme.rotate_octas(perfoctdict,small9xangle,
                small9yangle, small9zangle, small13xangle, small10zangle,
                rot12y)
        Rgetcoords.print_back_out(perfname, octrotated,workname,
                coordset_perf[0], coordset_perf[1])
        coordset_rot=Rgetcoords.main(workname, 0.05)
        newcompare=Rgetcoords.comparecoords(coordset_orig,
                coordset_rot)  
        if newcompare[0][2] < small12yavg - difftol:
            small12yavg=newcompare[0][2]
            small12yangle=rot12y
    octrotated=Rrotateme.rotate_octas(perfoctdict,small9xangle,
                small9yangle, small9zangle, small13xangle, small10zangle,
                small12yangle)
    Rgetcoords.print_back_out(origname, octrotated,workname,
                coordset_orig[0], coordset_orig[1])
    return [small9xavg, small9xangle, small9yavg, small9yangle, small9zavg, small9zangle, small13xavg, small13xangle, small10zavg, small10zangle, small12yavg, small12yangle]
def rotatestep_range(origname, perfname, 
        guess9x=-10,guess9y=-10,guess9z=10,
        guess13x=10,guess10z=-10,guess12y=10,
        myrange=1, workname=""):
    """Stepper rotation for pv only, 8 octahedra in cell.
        origname: original file name 
        perfname: perfect file name (want to match original)
    """
    import Rgetcoords
    import Rrotateme
    import os
    coordset_orig=Rgetcoords.main(origname, 0.05)
    coordset_perf=Rgetcoords.main(perfname, 0.05)
    distavgs=dict()
    diststds=dict()
    if workname=="":
        workname = os.path.dirname(perfname) + "/working_rotate"
    opstats = Rgetcoords.comparecoords(coordset_orig,coordset_perf)  
    perfoctdict=Rgetcoords.build_octahedra(coordset_perf[1],coordset_perf[2])
    smallestbavg=100000
    smallestbavganglestr=""
    smallestbstd=100000
    smallestbstdanglestr=""
    for rot9x in range(guess9x-myrange,guess9x+myrange+1):
        for rot9y in range(guess9y-myrange,guess9y+myrange+1):
            for rot9z in range(guess9z-myrange,guess9z+myrange+1): 
                for rot13x in range(guess13x-myrange,guess13x+myrange+1):
                    for rot10z in range(guess10z-myrange,guess10z+myrange+1):
                        for rot12y in range(guess12y-myrange,guess12y+myrange+1):
                            octrotated=Rrotateme.rotate_octas(perfoctdict, rot9x, rot9y, rot9z, rot13x, rot10z, rot12y)
                            Rgetcoords.print_back_out(origname,
                                octrotated,workname,coordset_orig[0],
                                coordset_orig[1])
                            coordset_rot=Rgetcoords.main(workname,
                                    0.05)
                            newcompare=Rgetcoords.comparecoords(coordset_orig,coordset_rot)  
                            anglestr = str(rot9x) + " " + str(rot9y) + " " + str(rot9z) + " " +str(rot13x) + " " + str(rot10z) + " " + str(rot12y)
                            distavgs[anglestr]=newcompare[0]
                            diststds[anglestr]=newcompare[1]
                            if newcompare[0][2] < smallestbavg:
                                smallestbavg=newcompare[0][2]
                                smallestbavganglestr=anglestr
                            if newcompare[1][2] < smallestbstd:
                                smallestbstd=newcompare[1][2]
                                smallestbstdanglestr=anglestr
    smallsplit=smallestbavganglestr.split()
    small9x=float(smallsplit[0])
    small9y=float(smallsplit[1])
    small9z=float(smallsplit[2])
    small13x=float(smallsplit[3])
    small10z=float(smallsplit[4])
    small12y=float(smallsplit[5])
    octrotated=Rrotateme.rotate_octas(perfoctdict, small9x,small9y,small9z,
        small13x,small10z,small12y)
    Rgetcoords.print_back_out(origname,
        octrotated,workname,coordset_orig[0],coordset_orig[1])
    return [smallestbavg, smallestbavganglestr,
                smallestbstd, smallestbstdanglestr]
