#!/bin/env python

import os
import numpy as np

def rotate_me(theta, evals, evecs, verbose=0):
    """Rotate ccwise through angle theta
        Args:
            theta <float>: Angle in degrees
            evals <str>: space-separated eigenvalues
            evecs <list of string>: comma-separated eigenvectors,
                                    with values separated by space
            verbose <int>: 0 (default) for normal or 1 for verbose
    """
    rotmat = get_rotation_matrix(theta, verbose)
    [evalmat, evecpmat]=parse_eigeninfo(evals, evecs, verbose)
    origocc = get_occ(evalmat, evecpmat, verbose)
    origrownorms = get_row_norms(evecpmat, verbose)
    origmat = get_original_matrix(evalmat, evecpmat, verbose)
    matrotated = rotate_mat(rotmat, origmat, verbose)
    [newevalmat, newevecpmat]=get_eigenvals_and_eigenvecs(matrotated,
                                            verbose)
    newocc = get_occ(newevalmat, newevecpmat, verbose)
    newrownorms = get_row_norms(newevecpmat, verbose)

def get_rotation_matrix(theta, verbose=0):
    """Get rotation matrix to rotate ccwise through angle theta in
        Cartesian x-y plane.
        Args:
            theta <float>: Angle in degrees
            verbose <int>: 0 (default) not verbose
                           1 = verbose
    """
    thetarad = theta * np.pi / 180
    myR = np.array([[np.cos(thetarad), -1*np.sin(thetarad)],
                    [np.sin(thetarad), np.cos(thetarad)]])
    if verbose == 1:
        print_mat("Rotation matrix: ", myR)
    return myR

def parse_eigeninfo(evals, evecs, verbose=0):
    """Parse eigenvalue and eigenvector info.
        Args:
            evals <str>: space-separated eigenvalues
            evecs <list of string>: comma-separated eigenvectors,
                                    with values separated by space
            verbose <int>: 0 (default) for normal or 1 for verbose
        Returns:
            [evalmat, evecpmat]
    """
    evallist=map(complex, evals.split())
    evalmat = np.diag(evallist)
    eveclist=evecs.split(',')
    evecpmat=None
    for evecstr in eveclist:
        evecrow = map(complex, evecstr.strip().split())
        eveccol = np.transpose([evecrow])
        eveccol = eveccol / np.linalg.norm(eveccol) #normalize
        if evecpmat==None:
            evecpmat = eveccol
        else:
            evecpmat = np.concatenate((evecpmat, eveccol),1)
    if verbose == 1:
        print_mat("evalmat:", evalmat)
        print_mat("evecpmat:", evecpmat)
    return [evalmat, evecpmat]
        

def get_original_matrix(evalmat, evecpmat, verbose=0):
    """Get the original matrix from eigenvalues and vectors.
        Original = PDP^-1
        Args:
            evalmat <numpy array of complex>: matrix of eigenvalues
                                            along the diagonal
            evecpmat <numpy array of complex>: corresponding matrix
                    of eigenvectors as columns ("P" matrix)
            verbose <int>: 1 for verbose, 0 otherwise

    """
    origmat = np.dot(evecpmat, np.dot(evalmat, np.linalg.inv(evecpmat)))
    if verbose == 1:
        print_mat("origmat: ", origmat)
    return origmat

def print_mat(header, mat):
    """Print matrix.
        Args:
            header <str>: header information to print
            mat <np.array>: matrix to print
    """
    print header
    nrows = np.size(mat, 0)
    ncols = np.size(mat, 1)
    for rct in range(0,nrows):
        printstr=""
        for cct in range(0,ncols):
            printstr = printstr + "%2.5f+%2.5fj" % (np.real(mat[rct][cct]), np.imag(mat[rct][cct])) + "  "
        print printstr
    print "Trace: %2.5f+%2.5fj" % (np.real(np.trace(mat)),np.imag(np.trace(mat)))
    return None

def rotate_mat(rotmat, origmat, verbose=0):
    """Rotate matrix.
        Args:
            rotmat <np.array of complex>: rotational matrix
            origmat <np.array of complex>: original matrix to rotate
            verbose <int>: 1 for verbose, 0 for non-verbose
        Returns:
            matrotated <np.array of complex>
    """
    matrotated = np.dot(rotmat, origmat)
    if verbose == 1:
        print_mat("Rotated matrix: ",matrotated)
    return matrotated

def get_eigenvals_and_eigenvecs(matrix, verbose=0):
    """Get eigenvalues and eigenvectors.
        Args:
            matrix <np.array of complex>: matrix
            verbose <int>: 1 for verbose, 0 for non-verbose
        Returns:
            [evalmat, evecpmat]
            evalmat <np.array of complex>: diagonal matrix of eigen-
                                            values
            evecpmat <np.array of complex>: matrix of eigenvectors as
                    column vectors
    """
    evals, evecpmat = np.linalg.eig(matrix)
    evalmat = np.diag(evals)
    if verbose == 1:
        print_mat("New eigenvalues:", evalmat)
        print_mat("New eigenvectors:", evecpmat)
    return [evalmat, evecpmat]

def get_occ(evalmat, evecpmat, verbose=0):
    """Get occupation.
        Args:
            evalmat <np.array of complex>: diagonal matrix of eigen-
                                            values
            evecpmat <np.array of complex>: matrix of eigenvectors as
                    column vectors
            verbose <int>: 1 for verbose, 0 for non-verbose
        Returns:
            occmat <np.array of complex>: matrix of occupations.
                                            Should be real.
    """
    evecpmat_sq = np.multiply(evecpmat, np.conj(evecpmat))
    nrows = np.size(evecpmat, 0)
    ncols = np.size(evecpmat, 1)
    occmat = np.zeros((nrows,1),complex)
    for rct in range(0,nrows):
        mysum=0
        for cct in range(0,ncols):
            eval = evalmat[cct][cct]
            mysum = mysum + evecpmat_sq[rct][cct]*eval
        occmat[rct][0]=mysum
    if verbose == 1:
        print_mat("Occupations: ", occmat)
    return occmat

def get_row_norms(evecpmat, verbose=0):
    """Get norms along rows. This corresponds to squares WITHOUT
        being multiplied by the respective eigenvalues.
        Not sure if this is useful.
        Args:
            evecpmat <np.array of complex>: matrix of eigenvectors as
                    column vectors
            verbose <int>: 1 for verbose, 0 for non-verbose
        Returns:
            rownorms <np.array of float>: column of row norms
    """
    evecpmat_sq = np.multiply(evecpmat, np.conj(evecpmat))
    nrows = np.size(evecpmat, 0)
    ncols = np.size(evecpmat, 1)
    rownorms = np.zeros((nrows,1),complex)
    for rct in range(0,nrows):
        mysum=0
        for cct in range(0,ncols):
            mysum = mysum + evecpmat_sq[rct][cct]
        rownorms[rct][0]=np.sqrt(mysum)
    if verbose == 1:
        print_mat("Row norms: ", rownorms)
    return rownorms
