#!/usr/bin/env python
##
#@package   Upathcalcs
#@ingroup   vasp
#@ingroup   python
#@brief     takes file of direct coordinates (fractional) and computes
#           either (1) the fractional path length formed by them
#           (2) the tetrahedral volume formed by them
#           (3) the triangular area formed by them
#           The coordinates should be space-delimited.
#@author    Tam Mayeshiba
#@date      2/20/12 updated to not use semicolons in instruction examples
###
def main(existdir, fname='coords1', whichcalc=1, setof=0):
    """
    Purpose: takes file of direct coordinates (fractional) and computes the
             path length formed by them. Coordinates should be space-
             delimited.
        inputs:  
            [$1] directory
            example: /home/tam/NaCl
            [$2] file name - default coords1
            example: coordlist
            [$3] which type of calculation:
                1 - fractional path length (default)
                2 - tetrahedral volume (needs groups of 4)
                3 - triangular area (needs groups of 3)
            [$4] group file into sets of this many
                default 0 - take all points as a path
                if this is a number > 0, remainders will not be processed 
                    (i.e. if setof=7 and 36 points are given, 5 sets will be
                    processed and 1 coordinate will be left unprocessed)
                if this is less than the number of coordinates in the file,
                    all coordinates will be used
        outputs:
            file of calculation outputs
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Upathcalcs
    #import pdb
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   file
    fullpath=""
    fullpath = os.path.join(existdir, fname)
    if not os.path.isfile(fullpath):
        print fname, 'is not a file in', existdir
        print '(Exiting.)'
        return None
#   #   setof
    setof = Xvalidinteger.main(setof, 0, 1000)
    if (setof == None):
        print '(Exiting.)'
        return None
#   #   whichcalc
    whichcalc = Xvalidinteger.main(whichcalc, 1, 3)
    if (whichcalc == None):
        print '(Exiting.)'
        return None
    if (whichcalc == 2) and (setof != 4):
        print 'Tetrahedral volume calculation needs groups of 4.'
        print 'Resetting group size to 4.'
        setof = 4
    if (whichcalc == 3) and (setof != 3):
        print 'Triangular area calculation needs groups of 3.'
        print 'Resetting group size to 3.'
        setof = 3
#   read coordinates from file
    #pdb.set_trace()
    bigmat=[]
    readme=""
    readln=""
    readme = open(fullpath, 'rb')
    readln = readme.readline()
    while(readln):
        readln = readln.strip()
        if (readln != ""):
            bigmat.append(readln)
        readln = readme.readline()
#   process matrix in groups
    bigmatlen=0
    numgroups=0
    groupct=0
    procmat=[]
    pathmat=[]
    ingroupct=0
    linect=0
    mypathcalc=""
    bigmatlen = len(bigmat)
    if (bigmatlen == 0):
        print 'No coordinates detected.'
        print '(Exiting.)'
        return None
    if (setof > bigmatlen) or (setof == 0):
        setof = bigmatlen
    numgroups = bigmatlen / setof
    if (numgroups == 0):
        print 'No groups in set.'
        print '(Exiting.)'
        return None
    while (groupct < numgroups):
        procmat = [] #reset processed matrix
        ingroupct = 0 #reset group count
        while (ingroupct < setof):
            procmat.append(bigmat[linect])
            linect = linect + 1
            ingroupct = ingroupct + 1
        if (whichcalc == 1):
            mypathcalc = Upathcalcs.pathlen(procmat)
        elif (whichcalc == 2):
            mypathcalc = Upathcalcs.tetrvol(procmat)
        elif (whichcalc == 3):
            mypathcalc = Upathcalcs.triarea(procmat)
        else:
            mypathcalc = None
        if (mypathcalc == None):
            print 'Set', groupct, 'returned with None.'
            pathmat.append('None\n')
        else:
            pathmat.append(mypathcalc)
        groupct = groupct + 1
    if (linect < bigmatlen):
        pathmat.append('Unused coordinates:\n')
        while (linect < bigmatlen):
            pathmat.append(bigmat[linect])
            linect = linect + 1
#   display
    if (whichcalc == 1):
        print 'Fractional path lengths:\n'
    elif (whichcalc == 2):
        print 'Tetrahedral volumes:\n'
    else:
        print 'Triangular areas:\n'
    pathitem=""
    for pathitem in pathmat:
        print pathitem
    return pathmat
#
#
def pathlen(cmat=[]):
    """
    Purpose: takes matrix of direct coordinates (fractional) and computes the
             path length formed by them. Coordinates should be space-
             delimited.
        inputs:  
            [$1] coordinate matrix
            example: [1 0 0, 0 0 1, 0 1 0]
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvector
#   Validation
    if (cmat == None) or (cmat == ""):
        print 'No matrix.'
        print '(Exiting.)'
        return None
    if len(cmat) == 0:
        print 'Matrix is empty.'
        print '(Exiting.)'
        return None
#   initialization
    numcoords=0
    coordct=0
    matvec=[]
    coordstr=""
    thiscoord=[]
    prevcoord=[]
    newvec=[]
#   get vectors each from previous
    numcoords = len(cmat)
    while (coordct < numcoords):
        coordstr = cmat[coordct]
        coordct = coordct + 1
        thiscoord = coordstr.split()
        if len(thiscoord) > 3:
            thiscoord = thiscoord[0:3] #strip off name label on line if any
        if (prevcoord == []):
            pass #skip first one
        else:
            newvec = Xvector.add(thiscoord, prevcoord, 1) #1 is subtract
            if (newvec == None):
                print 'Error subtracting', prevcoord, 'from', thiscoord
                print 'Skipping.'
            else:
                matvec.append(newvec)
        prevcoord = thiscoord
    if (matvec == []):
        print 'Matrix of vectors is empty.'
        print '(Exiting.)'
        return None
#   process magnitudes of vectors
    print matvec
    numvecs=0
    procct=0
    thislen=0
    totlen=0
    numvecs = len(matvec)
    if numvecs != len(cmat) - 1:
        print 'Warning: there is a wrong number of vectors.'
    while (procct < numvecs):
        thislen = Xvector.magnitude(matvec[procct])
        #print "This vector: ", matvec[procct]
        #print "This length: ", thislen
        totlen = totlen + thislen
        procct = procct + 1
    return totlen
#
#
def tetrvol(cmat=[]):
    """
    Purpose: takes matrix of 4 direct coordinates (fractional) and computes the
             tetrahedral volume formed by them. Coordinates should be space-
             delimited.
        inputs:  
            [$1] coordinate matrix
            example: [1 0 0, 0 0 1, 0 1 0]
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvector
#   Validation
    if (cmat == None) or (cmat == ""):
        print 'No matrix.'
        print '(Exiting.)'
        return None
    if len(cmat) == 0:
        print 'Matrix is empty.'
        print '(Exiting.)'
        return None
#   initialization
    coordct=0
    matvec=[]
    coordstr=""
    thiscoord=[]
    startcoord=[]
    newvec=[]
#   get 3 vectors relative to first vector
    if len(cmat) != 4:
        print 'Warning: only the first four coordinates will be used.'
    coordstr = cmat[0]
    startcoord = coordstr.split()
    coordct = coordct + 1
    while (coordct < 4):
        coordstr = cmat[coordct]
        coordct = coordct + 1
        thiscoord = coordstr.split()
        newvec = Xvector.add(thiscoord, startcoord, 1) #1 for subtraction
        if (newvec == None):
            print 'Error getting vector', coordct
            print '(Exiting.)'
            return None
        matvec.append(newvec)
    if (matvec == []):
        print 'Matrix of vectors is empty.'
        print '(Exiting.)'
        return None
#   process tetrahedral area
#   formula is volume = [c dot (a cross b) / 6]
    acrossb=[]
    cdotacrossb=[]
    tetrvol=0
    acrossb = Xvector.cross(matvec[0], matvec[1])
    if (acrossb == None):
        print 'Error crossing', matvec[0], 'and', matvec[1]
        print '(Exiting.)'
        return None
    cdotacrossb = Xvector.dot(matvec[2], acrossb)
    if (cdotacrossb == None):
        print 'Error in dot product of', matvec[2], 'and', acrossb
        print '(Exiting.)'
        return None
    tetrvol = abs(cdotacrossb) / 6
    return tetrvol
#
#
def triarea(cmat=[]):
    """
    Purpose: takes matrix of 3 direct coordinates (fractional) and computes the
             triangular area formed by them. Coordinates should be space-
             delimited.
        inputs:  
            [$1] coordinate matrix
            example: [1 0 0, 0 0 1, 0 1 0]
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvector
#   Validation
    if (cmat == None) or (cmat == ""):
        print 'No matrix.'
        print '(Exiting.)'
        return None
    if len(cmat) == 0:
        print 'Matrix is empty.'
        print '(Exiting.)'
        return None
#   initialization
    coordstr=""
    coord1=[]
    coord2=[]
    coord3=[]
    vec1=[]
    vec2=[]
    vec3=[]
#   get 2 vectors relative to first vector
    if len(cmat) != 3:
        print 'Warning: only the first three coordinates will be used.'
    coordstr = cmat[0]
    coord1 = coordstr.split()
    coordstr = cmat[1]
    coord2 = coordstr.split()
    coordstr = cmat[2]
    coord3 = coordstr.split()
    vec1 = Xvector.add(coord3, coord1, 1) # 1 for subtraction
    if (vec1 == None):
        print 'Error getting vector 1.'
        print '(Exiting.)'
        return None
    vec2 = Xvector.add(coord2, coord1, 1) # 1 for subtraction
    if (vec2 == None):
        print 'Error getting vector 2.'
        print '(Exiting.)'
        return None
#   process triangular area and angles
#   formula is (1/2) * magnitude(a cross b)
    acrossb=[]
    magacrossb=0
    area=0
    angle1=0
    angle2=0
    angle3=0
    retstr=""
    acrossb = Xvector.cross(vec1, vec2)
    if (acrossb == None):
        print 'Error crossing', vec1, 'and', vec2
        print '(Exiting.)'
        return None
    magacrossb = Xvector.magnitude(acrossb)
    if (magacrossb == None):
        print 'Error getting magnitude of cross product.'
        print '(Exiting.)'
        return None
    area = magacrossb / 2
    angle1 = Xvector.angle(vec1, vec2)
    retstr = 'Area: ' + str(area) + ' :Angle1: ' + str(angle1)
    return retstr
