#!/usr/bin/env python
##
#@package   Veptoneb
#@ingroup   vasp
#@ingroup   python
#@brief     take two endpoint top folders, create NEB folder and images
#@details   also copies first endpoint's POTCAR file, INCAR, and KPOINTS
#           but does not modify the INCAR at all.
#@author    Tam Mayeshiba
#@date      11/30/10     
#@date      1/13/11 rename as Vnebmake
#@date      3/1/11 add ability to use template directory, rename script
#@date      5/18/11 copied from Vnebmake; also REMOVE ability to create
#           sequential new directories. This is easily added to different
#           scripts with Xuniquename anyway.
#@date      6/30/11 redo jobstring mod call to work on Ranger
###
def main(ep1dir, ep2dir, inthisdir, nebdirname, templdir="", incdir="", 
            numim=1, spring=-5, climb=1, nelectadj=0):
    """
    Purpose: take two endpoint folders, match directories, create NEB
             folder and image from the endpoints' CONTCAR files
            also copies the first endpoint's POTCAR file over
        inputs:  
            [$1] endpoint 1 directory
                example: '/home/tam/rocksaltep1s/MgOep1_r3'
            [$2] second directory of endpoint directories
                example: '/home/tam/rocksaltep2s/MgOep2_r2'
            [$3] directory in which to create the neb folder
                example: '/home/tam/rocksaltnebs
            [$4] name for the neb directory
                example: 'MgOneb'
            [$5] template directory (default "" for none)
            [$6] directory for INCAR (default "" for none)
            [$7] number of images (default 1)
            [$3] spring constant (default -5)
            [$4] climbing NEB   
                1 - yes (default; adds LCLIMB tag to INCAR)
                0 - no
            [$5] adjust NELECT for endpoints and NEB (default 0)
                    negative number to remove electrons (positive vacancy)
                    positive number to add electrons (negative vacancy???)
        outputs:
            NEB directories
            returns the neb directory path of a successful creation
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xvalidname
    import Xvalidletter
    import Xgetfiles
    import Pmodfilenew
    import Vmodneb
#   validation
#   #   validate ep 1 directory
    ep1dir = Xvaliddirectory.main(ep1dir)
    if (ep1dir == None):
        print 'Error reading endpoint 1 directory.'
        print '(Exiting.)'
        return None
#   #   validate ep 2 collection directory
    ep2dir = Xvaliddirectory.main(ep2dir)
    if (ep2dir == None):
        print 'Error reading endpoint 2 directory.'
        print '(Exiting.)'
        return None
#   #   validate directory in which to create
    inthisdir = Xvaliddirectory.main(inthisdir)
    if (inthisdir == None):
        print 'Create valid directory first.'
        print '(Exiting.)'
        return None
#   #   validate directory name
    nebdirname = Xvalidname.main(nebdirname)
    if (nebdirname == None):
        return None
#   #   validate template directory
    if (templdir != ""):
        templdir = Xvaliddirectory.main(templdir)
        if (templdir == None):
            print '(Exiting.)'
            return None
#   #   validate INCAR directory
    if (incdir != ""):
        incdir = Xvaliddirectory.main(incdir)
        if (incdir == None):
            print '(Exiting.)'
            return None
#   #   numim
    numim = Xvalidinteger.main(numim, 1, None)
    if (numim == None):
        print '(Exiting.)'
        return None
#   #   spring
    spring = Xvalidinteger.main(spring, None, None)
    if (spring == None):
        print '(Exiting.)'
        return None
#   #   climb
    climb = Xvalidinteger.main(climb, 0, 1)
    if (climb == None):
        print '(Exiting.)'
        return None
#   #   NELECT adjustment
    nelectadj = Xvalidinteger.main(nelectadj, None, None)
    if (nelectadj == None):
        print '(Exiting.)'
        return None
#   make sure POSCAR and OSZICAR exist for endpoint 1
    if not os.path.isfile(ep1dir + '/' + 'CONTCAR'):
        print 'CONTCAR not found in', ep1dir
        print '(Exiting.)'
        return None
    if not os.path.isfile(ep1dir + '/' + 'OSZICAR'):
        print 'OSZICAR not found in', ep1dir
        print '(Exiting.)'
        return None
#   make sure POSCAR and OSZICAR exist for endpoint 2
    if not os.path.isfile(ep2dir + '/' + 'CONTCAR'):
        print 'CONTCAR not found in', ep2dir
        print '(Exiting.)'
        return None
    if not os.path.isfile(ep2dir + '/' + 'OSZICAR'):
        print 'OSZICAR not found in', ep2dir
        print '(Exiting.)'
        return None
#   create a new folder
    nebpath=""
    nebpath = inthisdir + '/' + nebdirname
    if os.path.exists(nebpath):
        print 'Directory', nebpath, 'already exists.'
        print '(Exiting.)'
        return None
    os.mkdir(nebpath)
#   copy in CONTCARs and one POTCAR, INCAR, KPOINTS
    niceme=""
    niceme = os.nice(19) # not sure if this works
    shutil.copy(ep1dir + '/CONTCAR', nebpath + '/CONTCARep1')
    shutil.copy(ep2dir + '/CONTCAR', nebpath + '/CONTCARep2')
    try:
        shutil.copy(ep1dir + '/KPOINTS', nebpath + '/KPOINTS')
    except (IOError):
        print 'Warning: KPOINTS does not exist in', ep1dir
    try:
        shutil.copy(ep1dir + '/POTCAR', nebpath + '/POTCAR')
    except (IOError):
        print 'Warning: POTCAR does not exist in', ep1dir
#   make images
    subconvint=""
    curpath=""
    curpath = os.getcwd()
    os.chdir(nebpath)
    convintstr='nice -n 19 convasp -intpol CONTCARep1 CONTCARep2'
    convintstr = convintstr + ' ' + str(numim) + ' n'
    subconvint = subprocess.Popen(convintstr, shell=True)
    subconvint.wait()
    os.chdir(curpath)
#   copy OSZICARs
    niceme = os.nice(19) #not sure if this works
    shutil.copy(ep1dir + '/OSZICAR', nebpath + '/00/OSZICAR')
    oszpath=""
    oszpath = numim + 1
    oszpath = str(oszpath)
    if (len(oszpath) == 1):
        oszpath = '0' + oszpath
    oszpath = '/' + oszpath + '/OSZICAR'
    shutil.copy(ep2dir + '/OSZICAR', nebpath + oszpath)
#   if template directory provided, copy files
    templfilelist=[]
    templfile=""
    jobstring=""
    jobstrmod=""
    if (templdir != ""):
        templfilelist = Xgetfiles.main(templdir, 1, 1)
        niceme = os.nice(19) #not sure if this works
        for templfile in templfilelist:
            shutil.copy(templfile, nebpath)
            if templfile[-3:] == '.sh':
                jobstring = '#PBS -N ' + nebdirname
                jobstrmod = Pmodfilenew.main(nebpath, 
                            os.path.basename(templfile),
                            'PBS -N', 'X', 'R', jobstring)
                if (jobstrmod == None):
                    jobstring = '#$ -N ' + nebdirname
                    jobstrmod = Pmodfilenew.main(nebpath,
                                os.path.basename(templfile),
                                '#$ -N', 'X', 'R', jobstring)
#   if INCAR directory provided, copy and modify INCAR
    incokay=1
    modinc=""
    if (incdir != ""):
        try:
            shutil.copy(incdir + '/INCAR', nebpath + '/INCAR')
        except (IOError):
            print 'Warning: INCAR does not exist in', incdir
            incokay=0
        if (incokay == 1):
            modinc = Vmodneb.main(nebpath, numim, spring, climb, nelectadj)
            if (modinc == None):
                print 'Error modifying INCAR files.'
                print '(Exiting.)'
                return None
#   Done 
    print 'Done.'
    return nebpath
