#!/usr/bin/env python
##
#@package   Vfccqcontrol_tom
#@ingroup   vasp
#@ingroup   python
#@brief     controls running of Hvac and Hmig in fcc metals (or other) for Q value determination
#@author    Tam Mayeshiba + Tom Angsten
#@date      7/27/11 created
###
def main():
    """
    Script must be activated in element's directory. This master directory must contain a directory entitled vac and a
    directory entitled neb.  Within each of these directories, there must exist two separate inputfiles, inputvac and
    inputneb, a directory entitled sub_script (only in neb directory), and the bulk directory ready to be run.  The same
    submission script (taken out of the subsequent sub_script folder in the neb directory) will be used for the eventual
    neb run.

    inputvac format:
    vacline = line number in POSCAR of the created vacancy
    encomp = (0.001) energy comparison value for Vokayenergy
    entype = (T) energy comparison type
    volcomp = (1) volume comparison value for Vokayvolume
    voltype = (P) volume comparison type
    emaddr = email for error reports
    checksec = number of seconds between checks
    lagsec = number of seconds until job is considered hung
    maxout = (5) maximum number of relaxations

    inputneb format:
    vac1line = first vacancy line number in POSCAR
    vac2line = second vacancy line number in POSCAR
    numim = number of images
    spring = sprint constant in INCAR
    climb = climb or no climb (1 for climb, 0 for no climb)
    encomp = energy comparison threshold
    entype = energy comparison type
    volcomp = volume comparison threshold
    voltype = volume comparison type
    emaddr = email address for error messages
    checksec = seconds between checks
    lagsec = seconds before job is considered hung
    maxout = maximum number of relaxtions before an error is produced
    """
#   Modules:
    import os
    import sys
    import time
    import Xinputmatch
    import Xvalidinteger
    import Vnumatoms
    import Pdirautorelaxwait
    import Vlastspecdir
    import Vmakeendpt
    import Vgetmon
    import Vrelax
    import Vmodstatic
    import Vgetstatus
    import Prunonewait
    import Veptoneb
    import Vgetnebstatus
    import Penergynebnew
    import Vnebstaticnew
    import synthep2_tom
    import hvfcalc_tom
    import util_tom
    import glob
    
#Retrieve vacancy input file parameters from inputvac file
    mydict = {}
    mycwd = os.getcwd()
    
    ifilename = 'vac/inputvac'
    catparams = 'vacline.encomp.entype.volcomp.voltype.emaddr.checksec.lagsec.maxout'
    
    mydict = Xinputmatch.main(mycwd + '/' + ifilename, catparams, 1)

    if (mydict == None):
        print 'Error getting parameters from input file.'
        print '(Exiting.)'
        return None
    
#   Validation
    checksec = mydict['checksec']
    checksec = Xvalidinteger.main(checksec, 10, 180000)
    if (checksec == None):
        print 'Seconds between check not within range.'
        print '(Exiting.)'
        return None
    lagsec = mydict['lagsec']
    lagsec = Xvalidinteger.main(lagsec, 300, 180000)
    if (lagsec == None):
        print 'Seconds to count job as hung not within range.'
        print '(Exiting.)'
        return None
#   get number of atoms
    bulkvacdir = mycwd + '/vac/bulk'
    numatomstr = Vnumatoms.main(bulkvacdir)
    if (numatomstr == None):
        print 'Number of atoms cannot be determined.'
        print '(Exiting.)'
        return None

    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot = numatomssplit[0]
#   get path above
    relaxdir = os.path.dirname(bulkvacdir)
#   Relax bulk
    runbulk=""
    print '---> Relax bulk in', bulkvacdir
    bulkmon=""
    bulkmon = Vgetmon.main(bulkvacdir)
    if (bulkmon == None) or ('True' not in bulkmon):
        runbulk = Pdirautorelaxwait.main(bulkvacdir,
            numatomsbulktot,
            mydict['maxout'], mydict['checksec'], mydict['lagsec'],
            mydict['encomp'], mydict['entype'], mydict['volcomp'],
            mydict['voltype'], mydict['emaddr'])
    bulkmon = Vgetmon.main(bulkvacdir)
    if (bulkmon == None) or ('True' not in bulkmon):
        print 'Error in bulk relaxation for', bulkvacdir
        return None
    print '---> Bulk', bulkvacdir, 'relaxed.'
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(bulkvacdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    print '---> Finished bulk relaxation: ', bulkfindir
        
#   bulk static run
    reltostat1=""
    bulkstat1dir=""
    modstat1=""
    runstat1=""
    bulkstat1dir = bulkfindir + '_static'
    if (os.path.exists(bulkstat1dir)):
        pass #bulk static directory has already been created
    else:
        reltostat1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(bulkstat1dir))
        modstat1 = Vmodstatic.main(bulkstat1dir)
        if (modstat1 == None):
            print 'Error modifying static bulk files.'
            return None
    print '---> Static bulk start in: ', bulkstat1dir

    #   run static
    #pdb.set_trace()
    if Vgetstatus.main(bulkstat1dir) == None:
        runstat1 = Prunonewait.main(bulkstat1dir, mydict['checksec'], mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', bulkstat1dir
            return None
    if Vgetstatus.main(bulkstat1dir) == None:
        print 'Error in static bulk run in: ', bulkstat1dir
        return None
    else:
        print '---> Static bulk finished in: ', bulkstat1dir

#   ENDPOINTS
#   num of atoms
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
#   endpoint 1
    reltoep1=""
    ep1dir=""
    makeep1=""
    relep1=""
    runep1=""
    ep1dir = bulkfindir + '_ep1'
    if (os.path.exists(ep1dir)):
        pass #ep 1 directory has already been created
    else:
        reltoep1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep1dir))
        makeep1 = Vmakeendpt.main(bulkfindir, ep1dir, mydict['vacline'],
                0, 0, 5)
        if (makeep1 == None):
            print 'Error modifying ep1 files.'
            return None
    print '---> Endpoint 1 start in: ', ep1dir
#   loop for endpoint 1
    ep1mon=""
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        runep1= Pdirautorelaxwait.main(ep1dir,numatomsforeps,mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'],mydict['voltype'], mydict['emaddr'])
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        print 'Error in ep1 relaxation for', ep1dir
        return None
    print '---> Endpoint 1', ep1dir, 'relaxed.'
    ep1findir=""
    ep1findir = Vlastspecdir.main(ep1dir, '_rel')
    if (ep1findir == None):
        print 'Error: no ep1 relaxation finish.'
        return None
    print '---> Finished ep1 relaxation: ', ep1findir

    
##   run static image on most relaxed bulk 

#    find most relaxed dir
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(bulkvacdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish for static usage.'
        return None
                            
#   #   STATIC ENDPOINTS
#   static endpoint 1
    reltostat1=""
    stat1dir=""
    modstat1=""
    runstat1=""
    stat1dir = ep1findir + '_static'
    if (os.path.exists(stat1dir)):
        pass #static ep1 directory has already been created
    else:
        reltostat1 = Vrelax.main(ep1findir, relaxdir, os.path.basename(stat1dir))
        modstat1 = Vmodstatic.main(stat1dir)
        if (modstat1 == None):
            print 'Error modifying static ep1 files.'
            return None
    print '---> Static ep1 start in: ', stat1dir
#   run static
    #pdb.set_trace()
    if Vgetstatus.main(stat1dir) == None:
        runstat1 = Prunonewait.main(stat1dir,mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', stat1dir
            return None
    if Vgetstatus.main(stat1dir) == None:
        print 'Error in static ep1 run in: ', stat1dir
        return None
    else:
        print '---> Static 1 endpoint finished in: ', stat1dir


    Hvac = hvfcalc_tom.main(bulkstat1dir,stat1dir,int(numatomsbulktot))

    print "Hvac: " + str(Hvac)

#   NEB phase begin *****NEB*******NEB**********NEB******NEB******NEB********NEB*******NEB*********NEB*******

#Retrieve vacancy input file parameters from inputvac file
    mydict = {}
    
    ifilename = 'neb/inputneb'
    catparams = 'vac1line.vac2line.numim.spring.climb.encomp.entype.volcomp.voltype.emaddr.checksec.lagsec.maxout'
    
    mydict = Xinputmatch.main(mycwd + '/' + ifilename, catparams, 1)

    if (mydict == None):
        print 'Error getting parameters from inputneb file.'
        print '(Exiting.)'
        return None
    
#   Validation
    checksec = mydict['checksec']
    checksec = Xvalidinteger.main(checksec, 10, 180000)
    if (checksec == None):
        print 'Seconds between neb check not within range.'
        print '(Exiting.)'
        return None
    lagsec = mydict['lagsec']
    lagsec = Xvalidinteger.main(lagsec, 300, 180000)
    if (lagsec == None):
        print 'Seconds to count neb job as hung not within range.'
        print '(Exiting.)'
        return None
#   get number of atoms
    bulkvacdir = mycwd + '/neb/bulk'
    numatomstr = Vnumatoms.main(bulkvacdir)
    if (numatomstr == None):
        print 'Number of atoms (neb) cannot be determined.'
        print '(Exiting.)'
        return None

    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of (neb) atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot = numatomssplit[0]
#   get path above
    relaxdir = os.path.dirname(bulkvacdir)
#   Relax bulk
    runbulk=""
    print '---> Relax bulk in', bulkvacdir
    bulkmon=""
    bulkmon = Vgetmon.main(bulkvacdir)
    if (bulkmon == None) or ('True' not in bulkmon):
        runbulk = Pdirautorelaxwait.main(bulkvacdir,numatomsbulktot,mydict['maxout'], mydict['checksec'], mydict['lagsec'],
            mydict['encomp'], mydict['entype'], mydict['volcomp'],mydict['voltype'], mydict['emaddr'])
    bulkmon = Vgetmon.main(bulkvacdir)
    
    if (bulkmon == None) or ('True' not in bulkmon):
        print 'Error in bulk relaxation for', bulkvacdir
        return None
    print '---> Bulk', bulkvacdir, 'relaxed.'
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(bulkvacdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    print '---> Finished bulk relaxation: ', bulkfindir

#   ENDPOINTS
#   num of atoms
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
#   endpoint 1
    reltoep1=""
    ep1dir=""
    makeep1=""
    relep1=""
    runep1=""
    ep1dir = bulkfindir + '_ep1'
    if (os.path.exists(ep1dir)):
        pass #ep 1 directory has already been created
    else:
        reltoep1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep1dir))
        makeep1 = Vmakeendpt.main(bulkfindir, ep1dir, mydict['vac1line'],0, 0, 5)
        if (makeep1 == None):
            print 'Error modifying ep1 files.'
            return None
    print '---> Endpoint 1 start in: ', ep1dir
#   loop for endpoint 1
    ep1mon=""
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        runep1= Pdirautorelaxwait.main(ep1dir,numatomsforeps,mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'],mydict['voltype'], mydict['emaddr'])
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        print 'Error in ep1 relaxation for', ep1dir
        return None
    print '---> Endpoint 1', ep1dir, 'relaxed.'
    ep1findir=""
    ep1findir = Vlastspecdir.main(ep1dir, '_rel')
    if (ep1findir == None):
        print 'Error: no ep1 relaxation finish.'
        return None
    print '---> Finished ep1 relaxation: ', ep1findir

#   #   STATIC ENDPOINTS
#   static endpoint 1
    reltostat1=""
    stat1dir=""
    modstat1=""
    runstat1=""
    stat1dir = ep1findir + '_static'
    if (os.path.exists(stat1dir)):
        pass #static ep1 directory has already been created
    else:
        reltostat1 = Vrelax.main(ep1findir, relaxdir, os.path.basename(stat1dir))
        modstat1 = Vmodstatic.main(stat1dir)
        if (modstat1 == None):
            print 'Error modifying static ep1 files.'
            return None
    print '---> Static ep1 start in: ', stat1dir
#   run static
    #pdb.set_trace()
    if Vgetstatus.main(stat1dir) == None:
        runstat1 = Prunonewait.main(stat1dir,mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', stat1dir
            return None
    if Vgetstatus.main(stat1dir) == None:
        print 'Error in static ep1 run in: ', stat1dir
        return None
    else:
        print '---> Static 1 endpoint finished in: ', stat1dir

#   static endpoint 2 (synthetically translated)

    ep2dir = bulkfindir + '_ep2'
    stat2dir = ep2dir + '_static'

    if(os.path.exists(stat2dir)):
        pass
    else:
        synthep2_tom.main(stat2dir,stat1dir)

#   NEB - nonstatic
    nebpath=""
    makeneb=""
    nebpath = mycwd + '/neb/' + os.path.basename(bulkfindir) + '_neb'
    modneb=""
    runneb=""
    if os.path.exists(nebpath):
        pass #neb already created
    else:
        makeneb = Veptoneb.main(stat1dir, stat2dir, mycwd + '/neb',os.path.basename(nebpath),
            mycwd + '/neb/sub_script',ep1findir,mydict['numim'], mydict['spring'],mydict['climb'],0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', stat2dir
            return None
        else:
            print '---> NEB created in', nebpath
    if (Vgetnebstatus.main(nebpath) != True):
        runneb = Prunonewait.main(nebpath, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath
            return None
    if (Vgetnebstatus.main(nebpath) != True):
        print 'NEB in', nebpath, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath

#   NEB static
    nebstatpath=""
    makestatneb=""
    nebstatpath = nebpath + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath,os.path.basename(nebstatpath))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath, 'in', nebstatpath
            return None
        else:
            print '---> Static tree created in', nebstatpath
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = int(mydict['numim'])
    
    while (statim <= numim):
        stattorun = nebstatpath + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath

    
#   NEB - check for local minimum
    nebimagedir = nebpath + "/01"
    nebpath2=""
    makeneb=""
    nebpath2 = mycwd + '/neb/' + os.path.basename(bulkfindir) + '_neb2'
    modneb=""
    runneb=""
    if os.path.exists(nebpath2):
        pass #neb check already created
    else:
        makeneb = Veptoneb.main(stat1dir, nebimagedir, mycwd + '/neb',os.path.basename(nebpath2),
            mycwd + '/neb/sub_script',ep1findir,mydict['numim'], mydict['spring'],0,0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', nebimagedir
            return None
        else:
            print '---> NEB created in', nebpath2
            
    if (Vgetnebstatus.main(nebpath2) != True):
        runneb = Prunonewait.main(nebpath2, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath2
            return None
    if (Vgetnebstatus.main(nebpath2) != True):
        print 'NEB in', nebpath2, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath2

    
#   NEB static check
    nebstatpath2=""
    makestatneb=""
    nebstatpath2 = nebpath2 + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath2):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath2,os.path.basename(nebstatpath2))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath2, 'in', nebstatpath2
            return None
        else:
            print '---> Static tree created in', nebstatpath2
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = 1
    
    while (statim <= numim):
        stattorun = nebstatpath2 + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath2


#   get migration barriers
    nonstatbarrier = Penergynebnew.main(nebpath, 0.01)
    statbarrier = Penergynebnew.main(nebstatpath, 1)

    nebstatenergy1 = util_tom.getEnergy(nebstatpath + "/static1/OSZICAR")
    nebstatenergy2 = util_tom.getEnergy(nebstatpath2 + "/static1/OSZICAR")

    print "\n\n+--------+-------------+---------++OUTPUT++------------+-------------+----------------+---------------+\n"
    
    print 'Nonstatic image migration barrier: ', nonstatbarrier
    print 'Static image migration barrier: ', statbarrier

    print '\n'
    print 'Image1 energy of standard neb        : ', str(nebstatenergy1), "eV"
    print 'Image2 energy of minimum checking neb: ', str(nebstatenergy2), "eV\n"

    if(nebstatenergy2 > nebstatenergy1):
        print "****WARNING**** : Static barrier of minimum check is higher than that of the first image neb barrier\n\n"

    statbarrier_num = util_tom.extractNumbers(statbarrier)[0]
    Hmig = statbarrier_num
    
    print "Hvac: " + str(Hvac) + " eV"
    print "Hmig: " + str(Hmig) + " eV"
    
    q = Hvac + Hmig
    print "Q: " + str(q) + ' eV\n'

    timepath1 = glob.glob(mycwd + "/neb/*ep1/OUTCAR")[0]
    time1 = util_tom.getTime(timepath1)
    timepath2 = glob.glob(mycwd + "/vac/*ep1/OUTCAR")[0]
    time2 = util_tom.getTime(timepath2)
    
    print "\ntime(vac ep1): " + str(time2)
    print "time(neb ep1): " + str(time1)

    Hmig = round(Hmig,6)
    Hvac = round(Hvac,6)
    q = round(q,6)
                
    print "\nwiki-output: " + "|| " + str(time2) + " || " + str(Hmig) + " || " + str(Hvac) + " || " + str(q) + " ||"
    print "\n"
    
def silent():
    """
    Purpose: run Vfccqcontrol_tom silently in the background
       outputs:
            None - error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xtimestring
    import Xuniquename
    import Pmakemonfile
#   validation will be handled by Pqautorelax
#   get cwd
    fullpath=""
    fullpath = os.getcwd()
#   set up output file
    filestr=""
    fileme=""
    filestr = Pmakemonfile.main(fullpath)
    if (filestr == None):
        print 'Could not create monitor file.'
        print '(Exiting.)'
        return None
    fileme = open(filestr, 'ab')
#   run process
    hidemon=""
    hidemon = subprocess.Popen(['Uwrapper.py', 'Vfccqcontrol_tom.main'],stdout=fileme, stderr=fileme)
    
    fileme.close()
    
    print hidemon
    return True

