#!/usr/bin/env python
##
#@package   Vhcpqcontrol_tom
#@ingroup   vasp
#@ingroup   python
#@brief     controls running of Hvac and Hmig in hcp metals for Q value determination (both jumps b and z are done)
#@author    Tam Mayeshiba + Tom Angsten
#@date      7/27/11 created
###
def main():
    """
    Script must be activated in element's directory. This master directory must contain a directory entitled b and a
    directory entitled z.  Within each of these directories, there must exist two separate inputfiles, inputb and
    inputz, a directory entitled sub_script, and the bulk directory ready to be run only in b.  The same submission script
    (taken out of the subsequent sub_script folder in the b or z directory) will be used for the eventual neb run.  This
    script assumes that both runs will be conducted at the same run parameters and will therefore copy portions of
    the b run into the z directory prior to the z run initiating (bulk to ep1_relx_static)

    inputb and inputz format:

    vac1line = first vacancy line number in POSCAR
    vac2line = second vacancy line number in POSCAR
    numim = number of images
    spring = spring constant in INCAR
    climb = climb or no climb (1 for climb, 0 for no climb)
    encomp = energy comparison threshold
    entype = energy comparison type
    volcomp = volume comparison threshold
    voltype = volume comparison type
    emaddr = email address for error messages
    checksec = seconds between checks
    lagsec = seconds before job is considered hung
    maxout = maximum number of relaxtions before an error is produced
    """
#   Modules:
    import os
    import sys
    import time
    import shutil
    import glob
    import Xinputmatch
    import Xvalidinteger
    import Vnumatoms
    import Pdirautorelaxwait
    import Vlastspecdir
    import Vmakeendpt
    import Vgetmon
    import Vrelax
    import Vmodstatic
    import Vgetstatus
    import Prunonewait
    import Veptoneb
    import Vgetnebstatus
    import Penergynebnew
    import Vnebstaticnew
    import synthep2_tom
    import hvfcalc_tom
    import util_tom
    

#Retrieve basal plane jump input file parameters from inputb file
    mydict = {}
    mycwd = os.getcwd()

    ifilename = 'b/inputb'
    catparams = 'vac1line.vac2line.numim.spring.climb.encomp.entype.volcomp.voltype.emaddr.checksec.lagsec.maxout'
    
    mydict = Xinputmatch.main(mycwd + '/' + ifilename, catparams, 1)

    if (mydict == None):
        print 'Error getting parameters from inputb file.'
        print '(Exiting.)'
        return None

#   Validation
    checksec = mydict['checksec']
    checksec = Xvalidinteger.main(checksec, 10, 180000)
    if (checksec == None):
        print 'Seconds between basal check not within range.'
        print '(Exiting.)'
        return None
    lagsec = mydict['lagsec']
    lagsec = Xvalidinteger.main(lagsec, 300, 180000)
    if (lagsec == None):
        print 'Seconds to count basal job as hung not within range.'
        print '(Exiting.)'
        return None
#   get number of atoms
    bulkvacdir = mycwd + '/b/bulk'
    numatomstr = Vnumatoms.main(bulkvacdir)
    if (numatomstr == None):
        print 'Number of atoms (basal) cannot be determined.'
        print '(Exiting.)'
        return None

    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of (basal) atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot = numatomssplit[0]
#   get path above
    relaxdir = os.path.dirname(bulkvacdir)
#   Relax bulk
    runbulk=""
    print '---> Relax bulk in', bulkvacdir
    bulkmon=""
    bulkmon = Vgetmon.main(bulkvacdir)
    if (bulkmon == None) or ('True' not in bulkmon):
        runbulk = Pdirautorelaxwait.main(bulkvacdir,numatomsbulktot,mydict['maxout'], mydict['checksec'], mydict['lagsec'],
            mydict['encomp'], mydict['entype'], mydict['volcomp'],mydict['voltype'], mydict['emaddr'])
    bulkmon = Vgetmon.main(bulkvacdir)
    
    if (bulkmon == None) or ('True' not in bulkmon):
        print 'Error in bulk relaxation for', bulkvacdir
        return None
    print '---> Bulk', bulkvacdir, 'relaxed.'
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(bulkvacdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    print '---> Finished bulk relaxation: ', bulkfindir
    
#   bulk static run
    reltostat1=""
    bulkstat1dir=""
    modstat1=""
    runstat1=""
    bulkstat1dir = bulkfindir + '_static'
    if (os.path.exists(bulkstat1dir)):
        pass #bulk static directory has already been created
    else:
        reltostat1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(bulkstat1dir))
        modstat1 = Vmodstatic.main(bulkstat1dir)
        if (modstat1 == None):
            print 'Error modifying static bulk files.'
            return None
    print '---> Static bulk start in: ', bulkstat1dir

#   run static
    if Vgetstatus.main(bulkstat1dir) == None:
        runstat1 = Prunonewait.main(bulkstat1dir, mydict['checksec'], mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', bulkstat1dir
            return None
    if Vgetstatus.main(bulkstat1dir) == None:
        print 'Error in static bulk run in: ', bulkstat1dir
        return None
    else:
        print '---> Static bulk finished in: ', bulkstat1dir

#   ENDPOINTS
#   num of atoms
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
#   endpoint 1
    reltoep1=""
    ep1dir=""
    makeep1=""
    relep1=""
    runep1=""
    ep1dir = bulkfindir + '_ep1'
    if (os.path.exists(ep1dir)):
        pass #ep 1 directory has already been created
    else:
        reltoep1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep1dir))
        makeep1 = Vmakeendpt.main(bulkfindir, ep1dir, mydict['vac1line'],0, 0, 5)
        if (makeep1 == None):
            print 'Error modifying ep1 files.'
            return None
    print '---> Endpoint 1 start in: ', ep1dir
#   loop for endpoint 1
    ep1mon=""
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        runep1= Pdirautorelaxwait.main(ep1dir,numatomsforeps,mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'],mydict['voltype'], mydict['emaddr'])
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        print 'Error in ep1 relaxation for', ep1dir
        return None
    print '---> Endpoint 1', ep1dir, 'relaxed.'
    ep1findir=""
    ep1findir = Vlastspecdir.main(ep1dir, '_rel')
    if (ep1findir == None):
        print 'Error: no ep1 relaxation finish.'
        return None
    print '---> Finished ep1 relaxation: ', ep1findir

#   #   STATIC ENDPOINTS
#   static endpoint 1
    reltostat1=""
    stat1dir=""
    modstat1=""
    runstat1=""
    stat1dir = ep1findir + '_static'
    if (os.path.exists(stat1dir)):
        pass #static ep1 directory has already been created
    else:
        reltostat1 = Vrelax.main(ep1findir, relaxdir, os.path.basename(stat1dir))
        modstat1 = Vmodstatic.main(stat1dir)
        if (modstat1 == None):
            print 'Error modifying static ep1 files.'
            return None
    print '---> Static ep1 start in: ', stat1dir
#   run static
    #pdb.set_trace()
    if Vgetstatus.main(stat1dir) == None:
        runstat1 = Prunonewait.main(stat1dir,mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', stat1dir
            return None
    if Vgetstatus.main(stat1dir) == None:
        print 'Error in static ep1 run in: ', stat1dir
        return None
    else:
        print '---> Static 1 endpoint finished in: ', stat1dir

    Hvac = hvfcalc_tom.main(bulkstat1dir,stat1dir,int(numatomsbulktot))
    print "Hvac: " + str(Hvac)

    #   endpoint 2
    reltoep2=""
    ep2dir=""
    makeep2=""
    relep2=""
    runep2=""
    ep2dir = bulkfindir + '_ep2'
    if (os.path.exists(ep2dir)):
        pass #ep 2 directory has already been created
    else:
        reltoep2 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep2dir))
        makeep2 = Vmakeendpt.main(bulkfindir, ep2dir, mydict['vac1line'],
                mydict['vac2line'], 0, 5)
        if (makeep2 == None):
            print 'Error modifying ep2 files.'
            return None
    print '---> Endpoint 2 start in: ', ep2dir
#   loop for endpoint 2
    ep2mon=""
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        runep2 = Pdirautorelaxwait.main(ep2dir, numatomsforeps, mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'], mydict['voltype'], mydict['emaddr'])
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        print 'Error in ep2 relaxation for', ep2dir
        return None
    print '---> Endpoint 2', ep2dir, 'relaxed.'
    ep2findir=""
    ep2findir = Vlastspecdir.main(ep2dir, '_rel')
    if (ep2findir == None):
        print 'Error: no ep2 relaxation finish.'
        return None
    print '---> Finished ep2 relaxation: ', ep2findir
    
    #   static endpoint 2
    reltostat2=""
    stat2dir=""
    modstat2=""
    runstat2=""
    stat2dir = ep2findir + '_static'
    if (os.path.exists(stat2dir)):
        pass #static ep2 directory has already been created
    else:
        reltostat2 = Vrelax.main(ep2findir, relaxdir, os.path.basename(stat2dir))
        modstat2 = Vmodstatic.main(stat2dir)
        if (modstat2 == None):
            print 'Error modifying static ep2 files.'
            return None
    print '---> Static ep2 start in: ', stat2dir
#   run static
    if Vgetstatus.main(stat2dir) == None:
        runstat2 = Prunonewait.main(stat2dir, mydict['checksec'], mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat2 == None):
            print 'Error running', stat2dir
            return None
    if Vgetstatus.main(stat2dir) == None:
        print 'Error in static ep2 run in: ', stat2dir
        return None
    else:
        print '---> Static 2 endpoint finished in: ', stat2dir

#   NEB - nonstatic
    nebpath=""
    makeneb=""
    nebpath = mycwd + '/b/' + os.path.basename(bulkfindir) + '_neb'
    modneb=""
    runneb=""
    if os.path.exists(nebpath):
        pass #neb already created
    else:
        makeneb = Veptoneb.main(stat1dir, stat2dir, mycwd + '/b',os.path.basename(nebpath),
            mycwd + '/b/sub_script',ep1findir,mydict['numim'], mydict['spring'],mydict['climb'],0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', stat2dir
            return None
        else:
            print '---> NEB created in', nebpath
    if (Vgetnebstatus.main(nebpath) != True):
        runneb = Prunonewait.main(nebpath, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath
            return None
    if (Vgetnebstatus.main(nebpath) != True):
        print 'NEB in', nebpath, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath

#   NEB static
    nebstatpath=""
    makestatneb=""
    nebstatpath = nebpath + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath,os.path.basename(nebstatpath))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath, 'in', nebstatpath
            return None
        else:
            print '---> Static tree created in', nebstatpath
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = int(mydict['numim'])
    
    while (statim <= numim):
        stattorun = nebstatpath + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath

    
#   NEB - check for local minimum
    nebimagedir = nebpath + "/01"
    nebpath2=""
    makeneb=""
    nebpath2 = mycwd + '/b/' + os.path.basename(bulkfindir) + '_neb2'
    modneb=""
    runneb=""
    if os.path.exists(nebpath2):
        pass #neb check already created
    else:
        makeneb = Veptoneb.main(stat1dir, nebimagedir, mycwd + '/b',os.path.basename(nebpath2),
            mycwd + '/b/sub_script',ep1findir,mydict['numim'], mydict['spring'],0,0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', nebimagedir
            return None
        else:
            print '---> NEB created in', nebpath2
            
    if (Vgetnebstatus.main(nebpath2) != True):
        runneb = Prunonewait.main(nebpath2, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath2
            return None
    if (Vgetnebstatus.main(nebpath2) != True):
        print 'NEB in', nebpath2, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath2

    
#   NEB static check
    nebstatpath2=""
    makestatneb=""
    nebstatpath2 = nebpath2 + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath2):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath2,os.path.basename(nebstatpath2))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath2, 'in', nebstatpath2
            return None
        else:
            print '---> Static tree created in', nebstatpath2
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = 1
    
    while (statim <= numim):
        stattorun = nebstatpath2 + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath2

#   get migration barriers
    b_nonstatbarrier = Penergynebnew.main(nebpath, 0.01)
    b_statbarrier = Penergynebnew.main(nebstatpath, 1)

    b_nebstatenergy1 = util_tom.getEnergy(nebstatpath + "/static1/OSZICAR")
    b_nebstatenergy2 = util_tom.getEnergy(nebstatpath2 + "/static1/OSZICAR")

    b_statbarrier_num = util_tom.extractNumbers(b_statbarrier)[0]
    b_Hmig = b_statbarrier_num
    
    b_q = Hvac + b_Hmig
    
    b_timepath = glob.glob(mycwd + "/b/*ep1/OUTCAR")[0]
    b_time = util_tom.getTime(b_timepath)

    
######################################z plane runs#######################################################

    #copy files from b into z
    b = mycwd + "/b/"
    z = mycwd + "/z/"

    if( len(glob.glob(z + '*ep1*')) == 0 ):
        shutil.copytree(b + "bulk",z + "bulk")
    
        bulks = glob.glob(b + 'bulk_rel?')

        for path in bulks:
            shutil.copytree(path,z + os.path.basename(path))

        endpoints = glob.glob(b + '*ep1*')

        for path in endpoints:
            shutil.copytree(path,z + os.path.basename(path))

        
#Retrieve basal plane jump input file parameters from inputb file
    
    ifilename = 'z/inputz'
    catparams = 'vac1line.vac2line.numim.spring.climb.encomp.entype.volcomp.voltype.emaddr.checksec.lagsec.maxout'
    
    mydict = Xinputmatch.main(mycwd + '/' + ifilename, catparams, 1)

    if (mydict == None):
        print 'Error getting parameters from inputz file.'
        print '(Exiting.)'
        return None

#   Validation
    checksec = mydict['checksec']
    checksec = Xvalidinteger.main(checksec, 10, 180000)
    if (checksec == None):
        print 'Seconds between z check not within range.'
        print '(Exiting.)'
        return None
    lagsec = mydict['lagsec']
    lagsec = Xvalidinteger.main(lagsec, 300, 180000)
    if (lagsec == None):
        print 'Seconds to count z job as hung not within range.'
        print '(Exiting.)'
        return None
#   get number of atoms
    bulkvacdir = mycwd + '/z/bulk'
    numatomstr = Vnumatoms.main(bulkvacdir)
    if (numatomstr == None):
        print 'Number of atoms (z) cannot be determined.'
        print '(Exiting.)'
        return None

    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of (z) atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot = numatomssplit[0]
#   get path above
    relaxdir = os.path.dirname(bulkvacdir)

    bulkfindir = Vlastspecdir.main(bulkvacdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
    
#   endpoint 2
    reltoep2=""
    ep2dir=""
    makeep2=""
    relep2=""
    runep2=""
    ep2dir = bulkfindir + '_ep2'
    if (os.path.exists(ep2dir)):
        pass #ep 2 directory has already been created
    else:
        reltoep2 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep2dir))
        makeep2 = Vmakeendpt.main(bulkfindir, ep2dir, mydict['vac1line'], mydict['vac2line'], 0, 5)
        if (makeep2 == None):
            print 'Error modifying ep2 files.'
            return None
    print '---> Endpoint 2 start in: ', ep2dir
#   loop for endpoint 2
    ep2mon=""
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        runep2 = Pdirautorelaxwait.main(ep2dir, numatomsforeps, mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'], mydict['voltype'], mydict['emaddr'])
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        print 'Error in ep2 relaxation for', ep2dir
        return None
    print '---> Endpoint 2', ep2dir, 'relaxed.'
    ep2findir=""
    ep2findir = Vlastspecdir.main(ep2dir, '_rel')
    if (ep2findir == None):
        print 'Error: no ep2 relaxation finish.'
        return None
    print '---> Finished ep2 relaxation: ', ep2findir
    
    #   static endpoint 2
    reltostat2=""
    stat2dir=""
    modstat2=""
    runstat2=""
    stat2dir = ep2findir + '_static'
    if (os.path.exists(stat2dir)):
        pass #static ep2 directory has already been created
    else:
        reltostat2 = Vrelax.main(ep2findir, relaxdir, os.path.basename(stat2dir))
        modstat2 = Vmodstatic.main(stat2dir)
        if (modstat2 == None):
            print 'Error modifying static ep2 files.'
            return None
    print '---> Static ep2 start in: ', stat2dir
#   run static
    if Vgetstatus.main(stat2dir) == None:
        runstat2 = Prunonewait.main(stat2dir, mydict['checksec'], mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat2 == None):
            print 'Error running', stat2dir
            return None
    if Vgetstatus.main(stat2dir) == None:
        print 'Error in static ep2 run in: ', stat2dir
        return None
    else:
        print '---> Static 2 endpoint finished in: ', stat2dir

#   NEB - nonstatic
    nebpath=""
    makeneb=""
    nebpath = mycwd + '/z/' + os.path.basename(bulkfindir) + '_neb'
    modneb=""
    runneb=""
    if os.path.exists(nebpath):
        pass #neb already created
    else:
        makeneb = Veptoneb.main(stat1dir, stat2dir, mycwd + '/z',os.path.basename(nebpath),
            mycwd + '/z/sub_script',ep1findir,mydict['numim'], mydict['spring'],mydict['climb'],0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', stat2dir
            return None
        else:
            print '---> NEB created in', nebpath
    if (Vgetnebstatus.main(nebpath) != True):
        runneb = Prunonewait.main(nebpath, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath
            return None
    if (Vgetnebstatus.main(nebpath) != True):
        print 'NEB in', nebpath, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath

#   NEB static
    nebstatpath=""
    makestatneb=""
    nebstatpath = nebpath + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath,os.path.basename(nebstatpath))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath, 'in', nebstatpath
            return None
        else:
            print '---> Static tree created in', nebstatpath
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = int(mydict['numim'])
    
    while (statim <= numim):
        stattorun = nebstatpath + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath

    
#   NEB - check for local minimum
    nebimagedir = nebpath + "/01"
    nebpath2=""
    makeneb=""
    nebpath2 = mycwd + '/z/' + os.path.basename(bulkfindir) + '_neb2'
    modneb=""
    runneb=""
    if os.path.exists(nebpath2):
        pass #neb check already created
    else:
        makeneb = Veptoneb.main(stat1dir, nebimagedir, mycwd + '/z',os.path.basename(nebpath2),
            mycwd + '/z/sub_script',ep1findir,mydict['numim'], mydict['spring'],0,0)
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', nebimagedir
            return None
        else:
            print '---> NEB created in', nebpath2
            
    if (Vgetnebstatus.main(nebpath2) != True):
        runneb = Prunonewait.main(nebpath2, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath2
            return None
    if (Vgetnebstatus.main(nebpath2) != True):
        print 'NEB in', nebpath2, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath2

    
#   NEB static check
    nebstatpath2=""
    makestatneb=""
    nebstatpath2 = nebpath2 + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath2):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath2,os.path.basename(nebstatpath2))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath2, 'in', nebstatpath2
            return None
        else:
            print '---> Static tree created in', nebstatpath2
#   run static images
    statim=1
    stattorun=""
    runstat=""
    numim = 1
    
    while (statim <= numim):
        stattorun = nebstatpath2 + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'],mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath2

#   get migration barriers
    z_nonstatbarrier = Penergynebnew.main(nebpath, 0.01)
    z_statbarrier = Penergynebnew.main(nebstatpath, 1)

    z_nebstatenergy1 = util_tom.getEnergy(nebstatpath + "/static1/OSZICAR")
    z_nebstatenergy2 = util_tom.getEnergy(nebstatpath2 + "/static1/OSZICAR")

    z_statbarrier_num = util_tom.extractNumbers(z_statbarrier)[0]
    z_Hmig = z_statbarrier_num
    z_q = Hvac + z_Hmig
    
    print "\n\n+--------+-------------+---------++OUTPUT++------------+-------------+----------------+---------------+\n"

    print "\n -------------Basal----------- \n"
    print 'Nonstatic image migration barrier: ', b_nonstatbarrier
    print 'Static image migration barrier: ', b_statbarrier
    print '\n'
    print 'Image1 energy of standard neb        : ', str(b_nebstatenergy1), "eV"
    print 'Image2 energy of minimum checking neb: ', str(b_nebstatenergy2), "eV\n"
    if(b_nebstatenergy2 > b_nebstatenergy1):
        print "****WARNING**** : Static barrier of minimum check is higher than that of the first image neb barrier\n\n"
    print "Basal Hvac: " + str(Hvac) + " eV"
    print "Basal Hmig: " + str(b_Hmig) + " eV"
    b_q = Hvac + b_Hmig
    print "Basal Q: " + str(b_q) + ' eV\n'
    print "\ntime(basal ep1): " + str(b_time)
    
    print "\n -------------Z-Jump----------- \n"
    print 'Nonstatic image migration barrier: ', z_nonstatbarrier
    print 'Static image migration barrier: ', z_statbarrier
    print '\n'
    print 'Image1 energy of standard neb        : ', str(z_nebstatenergy1), "eV"
    print 'Image2 energy of minimum checking neb: ', str(z_nebstatenergy2), "eV\n"
    if(z_nebstatenergy2 > z_nebstatenergy1):
        print "****WARNING**** : Static barrier of minimum check is higher than that of the first image neb barrier\n\n"
    print "Z Hvac: " + str(Hvac) + " eV"
    print "Z Hmig: " + str(z_Hmig) + " eV"
    print "Z Q: " + str(z_q) + ' eV\n'

    b_Hmig = round(b_Hmig,6)
    z_Hmig = round(z_Hmig,6)
    Hvac = round(Hvac,6)
    b_q = round(b_q,6)
    z_q = round(z_q,6)
    print "\nwiki || B || " + str(b_time) + " || " + str(b_Hmig) + " || " + str(Hvac) + " || " + str(b_q) + " ||"
    print "\nwiki || Z || " + str(b_time) + " || " + str(z_Hmig) + " || " + str(Hvac) + " || " + str(z_q) + " ||"
    print "\n"
        
def silent():
    """
    Purpose: run Vhcpqcontrol_tom silently in the background
       outputs:
            None - error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xtimestring
    import Xuniquename
    import Pmakemonfile
#   validation will be handled by Pqautorelax
#   get cwd
    fullpath=""
    fullpath = os.getcwd()
#   set up output file
    filestr=""
    fileme=""
    filestr = Pmakemonfile.main(fullpath)
    if (filestr == None):
        print 'Could not create monitor file.'
        print '(Exiting.)'
        return None
    fileme = open(filestr, 'ab')
#   run process
    hidemon=""
    hidemon = subprocess.Popen(['Uwrapper.py', 'Vhcpqcontrol_tom.main'],stdout=fileme, stderr=fileme)
    
    fileme.close()
    
    print hidemon
    return True
