#!/usr/bin/env python
##
#@package   Vocta 
#@ingroup   vasp
#@ingroup   python
#@brief     calculates octahedral tilts
#@author    Tam Mayeshiba
#@date      1/24/12
#@date      4/4/12 add angle-forming vector addition to output
###
def main(mydir, bsite=9, a1=18, a2=30, b1=19, b2=25, c1=17, c2=20, 
                fname='CONTCAR', debug=0):
    """
    Purpose: calculates octahedral tilts
        inputs:  
            [$1] directory
            example: /home/tam/NaCl
            [$2] B-site POSCAR position in the middle of the octahedron
            [$3 and $4] O nearest to lattice vector a
            [$5 and $6] O nearest to lattice vector b
            [$7 and $8] O nearest to lattice vector c
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvector
    import Xvaliddirectory
#   Validation
#   #  mydir
    mydir = Xvaliddirectory.main(mydir)
    if mydir == None:
        return None
#   #   
    try:
        bsite=int(bsite)
        a1=int(a1)
        a2=int(a2)
        b1=int(b1)
        b2=int(b2)
        c1=int(c1)
        c2=int(c2)
    except ValueError, TypeError:
        print 'One of your numbers is not an integer.'
        return None
#
    fpath = os.path.join(mydir, fname)
    if not os.path.isfile(fpath):
        print fname, 'not found in', mydir
        return None
#
    lines=[]
    readme = open(fpath, 'rb')
    lines = readme.readlines()
    readme.close()
    if len(lines) == 0:
        print 'Empty', fname
        return None
    latta=[]
    lattb=[]
    lattc=[]
    latta=lines[2].split()
    lattb=lines[3].split()
    lattc=lines[4].split()
    vaspadd=7 #vasp version 5, add 8 lines, index starts at 0
    try:
        bsitepos = lines[bsite + vaspadd].split()[0:3]
        a1pos = lines[a1 + vaspadd].split()[0:3]
        a2pos = lines[a2 + vaspadd].split()[0:3]
        b1pos = lines[b1 + vaspadd].split()[0:3]
        b2pos = lines[b2 + vaspadd].split()[0:3]
        c1pos = lines[c1 + vaspadd].split()[0:3]
        c2pos = lines[c2 + vaspadd].split()[0:3]
    except IndexError:
        print 'Index error in',fpath
        return None
    if len(bsitepos)< 3:
        print 'bsitepos does not have length 3'
        return None
    if len(a1pos) < 3 or len(a2pos) < 3 or len(b1pos) < 3 or len(b2pos) < 3:
        print 'a1, a2, b1, or b2 does not have 3 numbers'
        return None
    if len(c1pos) < 3 or len(c2pos) < 3:
        print 'c1 or c2 does not have 3 numbers'
        return None
    if a1pos[0] > bsitepos[0]:
        a1pos = normfrac(a1pos,0)
    if a2pos[0] < bsitepos[0]:
        a2pos = normfrac(a2pos,0)
    if b1pos[1] > bsitepos[1]:
        b1pos = normfrac(b1pos,1)
    if b2pos[1] < bsitepos[1]:
        b2pos = normfrac(b2pos,1)
    if c1pos[2] > bsitepos[2]:
        c1pos = normfrac(c1pos,2)
    if c2pos[2] < bsitepos[2]:
        c2pos = normfrac(c2pos,2)
    bsitecart = fractocart(bsitepos, latta, lattb, lattc)
    a1cart = fractocart(a1pos, latta, lattb, lattc)
    a2cart = fractocart(a2pos, latta, lattb, lattc)
    b1cart = fractocart(b1pos, latta, lattb, lattc)
    b2cart = fractocart(b2pos, latta, lattb, lattc)
    c1cart = fractocart(c1pos, latta, lattb, lattc)
    c2cart = fractocart(c2pos, latta, lattb, lattc)
    c2b = Xvector.add(c2cart, bsitecart, 1) #subtract
    c1b = Xvector.add(c1cart, bsitecart, 1) #subtract
    b2b = Xvector.add(b2cart, bsitecart, 1) #subtract
    b1b = Xvector.add(b1cart, bsitecart, 1) #subtract
    a2b = Xvector.add(a2cart, bsitecart, 1) #subtract
    a1b = Xvector.add(a1cart, bsitecart, 1) #subtract
    cangle = Xvector.angle(c2b, c1b)
    bangle = Xvector.angle(b2b, b1b)
    aangle = Xvector.angle(a2b, a1b)
    cline = Xvector.add(c2cart, c1cart, 1)
    bline = Xvector.add(b2cart, b1cart, 1)
    aline = Xvector.add(a2cart, a1cart, 1)
    acangle = Xvector.angle(cline, aline)
    bcangle = Xvector.angle(cline, bline)
    abangle = Xvector.angle(aline, bline)
    tilta = Xvector.angle(aline, latta)
    tiltb = Xvector.angle(bline, lattb)
    tiltc = Xvector.angle(cline, lattc)
    vsuma = Xvector.add(a2b, a1b)
    vsumb = Xvector.add(b2b, b1b)
    vsumc = Xvector.add(c2b, c1b)

    results=[]
    results=[mydir, ':', "how octahedral a-B-a b-B-b c-C-c aa-bb aa-cc bb-cc", 
                aangle, bangle, cangle, 
                abangle, acangle, bcangle,
            " how tilted alinewrta blinewrtb clinewrtc ", tilta, tiltb, tiltc,
            " vector sums for angles ",vsuma,vsumb,vsumc,"cvecs:",c2b, c1b]
    print results
    if debug:
        debug=[]
        debug = [bsitepos, a1pos, a2pos, b1pos, b2pos,
                c1pos, c2pos,
                latta, lattb, lattc,
                bsitecart, a1cart, a2cart, b1cart, b2cart,
                c1cart, c2cart,
                a1b, a2b, b1b, b2b, c1b, c2b,
                aline, bline, cline]
        print mydir, ':', debug
    return results


def normfrac(fracvec,pos):
    import os
    import sys
    import Xvector

    fvec=[]
    fvec = Xvector.valflt(fracvec)
    myflt=0
    newvec=[]
    ct=0
    while ct < 3:
        myflt = fvec[ct]
        if not ct==pos:
            newvec.append(myflt)
        else:
            if myflt > 0.75:
                newvec.append(1-myflt)
            elif myflt < 0.25:
                newvec.append(1+myflt)
            else:
                newvec.append(myflt)
        ct = ct + 1
    return newvec


def fractocart(fracvec,latt1,latt2,latt3):
    import os
    import sys
    import Xvector

    fvec=[]
    l1=[]
    l2=[]
    l3=[]

    fvec = Xvector.valflt(fracvec)
    l1 = Xvector.valflt(latt1)
    l2 = Xvector.valflt(latt2)
    l3 = Xvector.valflt(latt3)

    cart1=[]
    cart2=[]
    cart3=[]
    subtotal=[]
    total=[]

    cart1 = Xvector.scalprod(fvec[0],l1)
    cart2 = Xvector.scalprod(fvec[1],l2)
    cart3 = Xvector.scalprod(fvec[2],l3)
    
    subtotal=Xvector.add(cart1, cart2)
    total = Xvector.add(subtotal, cart3)

    return total
#
def loop(topdir, bsite=9, a1=18, a2=30, b1=19, b2=25, c1=17, c2=20, 
                fname='CONTCAR', debug=0):
    """
    Purpose: calculates octahedral tilts
        inputs:  
            [$1] directory
            example: /home/tam/NaClcollection
            [$2] B-site POSCAR position in the middle of the octahedron
            [$3 and $4] O nearest to lattice vector a
            [$5 and $6] O nearest to lattice vector b
            [$7 and $8] O nearest to lattice vector c
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Vocta
#   Validation
#   #  mydir
    topdir = Xvaliddirectory.main(topdir)
    if topdir == None:
        return None
#   #
    dirlist=[]
    dirlist = Xgetdirs.main(topdir,1,5)
    mydir=""
    myresult=""
    allres=[]
    for mydir in dirlist:
        myresult = Vocta.main(mydir, bsite, 
        a1, a2, b1, b2, c1, c2, fname, debug)
        allres.append(myresult)
    return allres


