#!/usr/bin/env python
##
#@package   Vvfe 
#@ingroup   vasp
#@ingroup   python
#@brief     vacancy formation energy for one oxygen vacancy in perovskite
#@author    Tam Mayeshiba
#@date      10/14/11, 8/18/14
###
def main(bulkstat, defstat, temp=1173, poxygen=0.1, numox=-1, charge=0):
    """
    Purpose: vfe for single oxygen vacancy in perovskite
        inputs:  
            [$1] bulk static directory
            [$2] defected static directory
            [$3] temperature (default 1173 K)
            [$4] oxygen partial pressure (default 0.1 atm)
            [$5] number of oxygens removed (default 1)
            [$6] supercell charge of defected cell (default 0)
                 positive number means electrons removed
            example:
        outputs:
    """
#   Modules:
    import os
    import sys
    import Vvfe
    import Xvaliddirectory
    import Xvalidinteger
    import Vgetenergy
#   Validation
#   #  
    bulkstat = Xvaliddirectory.main(bulkstat)
    if bulkstat == None:
        return None
    defstat = Xvaliddirectory.main(defstat)
    if defstat == None:
        return None
    tempflt=0
    try:
        tempflt = float(temp)
    except (ValueError, TypeError):
        print 'Temperature', temp, 'could not be converted to float.'
        return None
    poxygenflt=0
    try:
        poxygenflt = float(poxygen)
    except (ValueError, TypeError):
        print 'Oxygen partial pressure', poxygen
        print 'Could not be converted to float.'
        return None
#   #
    #Get corrected oxygen enthalpy
    hoxygen=""
    hoxygen = Vvfe.oenthalpy(tempflt, poxygenflt)
    if hoxygen == None:
        print 'Error getting oxygen corrected enthalpy.'
        print '(Exiting.)'
        return None
    #Get energies
    ebulk=0
    ebulkflt=0
    ebulk = Vgetenergy.main(bulkstat)
    if ebulk == None:
        print 'Error getting bulk energy.'
        print '(Exiting.)'
        return None
    try:
        ebulkflt = float(ebulk)
    except (ValueError, TypeError):
        print 'Bulk energy could not be converted to float.'
        print '(Exiting.)'
        return None
    edefected=0
    edefflt=0
    edefected = Vgetenergy.main(defstat)
    if edefected == None:
        print 'Error getting bulk energy.'
        print '(Exiting.)'
        return None
    try:
        edefflt = float(edefected)
    except (ValueError, TypeError):
        print 'Defected cell energy could not be converted to float.'
        print '(Exiting.)'
        return None
    # calculate VFE
    print bulkstat, ';', ebulkflt
    print defstat, ';', edefflt
    vfe=0
    vfe = edefflt - ebulkflt - (-1 * hoxygen/2)
    if charge == 0:
        return vfe
    else:
        import MAST
        from MAST.utility.defect_formation_energy.potential_alignment import PotentialAlignment as PA
        mypa=PA()
        perfect_stat=mypa.read_outcar(os.path.join(bulkstat,"OUTCAR"))
        defect_stat=mypa.read_outcar(os.path.join(defstat,"OUTCAR"))
        potalign=mypa.get_potential_alignment(perfect_stat,defect_stat)
        print "Potential alignment, mean all, Glen's code: ", potalign
        print "E-fermi from OUTCAR: ", perfect_stat[2]
        vfe = vfe + charge*(potalign + perfect_stat[2] + 0)
        return vfe

def oenthalpy(T=1173, P=0.1):
    """
    Based on
    Code from Open Systems Document: "A Tutorial on Calculating Formation
    Energies of Open Systems and Defects Using VASP": circulating in
    UW-Madison Computational Materials Group, 2007-2011. Contributors:
    Dane Morgan Yueh-Lin Lee Vasilios Vlahos
    David Shrader Shih-kang Lin Brian Puchala
    """
    import math
    print 'T =',T
    print 'P =',P
    #constants
    k_b= 0.00008617343 #Boltzmann constant [eV/K]
    eV = 96486 #1eV = 96486 J/mole
    bar=0.98692 #1 bar = 0.98692 atm
    #Temp & Pressure
    #T=input('Temperature(K)= ? : ')
    #P=input('P(O2) (atm)= ? : ')
    #VASP O2 energy (PAW-PBE O)
    #E_O2=-9.8644451 # eV

    #VASP O2 energy (PAW-PW91 O)
    E_O2=-9.0903605 # eV from Yueh-Lin's paper 2009

    # correction of O2 energy, from
    # Wang et al, PHYSICAL REVIEW B 73, 195107 (2006)
    #E_O2_corr=1.36 # eV
    E_O2_corr=0.33 #eV from Yueh-Lin's paper 2009
    t=T/1000
    # All parameters from NIST webbook:
    # http://webbook.nist.gov/cgi/cbook.cgi?ID=
    # C7782447&Units=SI&Type=JANAFG&Table=on#JANAFG
    #Entropy of oxygen gas (J/mol*K, 298~6000K): Shomate Equation
    S_O2=(29.659*math.log(t)+6.137261*t+(-1.186521)*math.pow(t,2)/2 +0.095780*math.pow(t,3)/3 -(-0.219663/(2*math.pow(t,2)))+237.9480)/eV
    S_O2_std= 205.0694/eV
    #( refernce to H_O2_298.15)
    H_O2_NIST = (29.659*t + 6.137261*math.pow(t,2)/2 -1.186521*math.pow(t,3)/3 + 0.09578*math.pow(t,4)/4+0.219663/t -9.861391)*1000/eV 
    # Gibbs energy of oxygen gas (G_O2 refernces to G_O2_298.15)
    G_O2 =(H_O2_NIST -T*S_O2+298.15*S_O2_std)+k_b*T*math.log(P/bar)
    #Enthalpy of oxygen gas
    H_O2=(E_O2+E_O2_corr)+G_O2+(-298.15*S_O2_std)+T*S_O2
    #The above equation is using G=H-TS to obtain H from G.
    #A plain way to get H_O2 is using NIST H_O2 equation
    # and VASP E_O2 plus Wang's correction, which will give
    # the same result.
    #H_O2=(E_O2+E_O2_corr)+ H_O2_NIST
    print H_O2
    print G_O2, H_O2_NIST,S_O2
    return H_O2
