// **************************************************
// CompareStrFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <algorithm>
#include "UsefulFuncs.h"
#include "constants.h"
#include "typedefs.h"
#include "PrintPlaton.h"
#include "RDFFuncs.h"
#include "CompareStrFuncs.h"


// **************************************************
// CmpStrDist
// **************************************************
// Compares the distances within rcut between str1 and str2.
// Assigns matrix of distances.  Each row is for a different
// pair type. Pair types are identified by pair i,j in row
// j+ntypes*i.  1<= i,j <=ntypes. I always take j>=i.
void CmpStrDist(const structure & str1, const structure & str2,
		const double & cutoff, matrix<double> & dist1,
		matrix<double> & dist2, matrix<double> & dist_diff,
		matrix<double> & dist_diff_n)
{
  matrix<atom> neigh_mat1;
  matrix<atom> neigh_mat2;
  GetStrNeighData(str1,cutoff,neigh_mat1);
  GetStrNeighData(str2,cutoff,neigh_mat2);
  int ntypes1=str1.GetNumEachType().size();
  int ntypes2=str2.GetNumEachType().size();
  int nprs1=(ntypes1*(ntypes1+1))/2;
  int nprs2=(ntypes2*(ntypes2+1))/2;
  vector<double> tmpvec;
  dist1=matrix<double> (nprs1,tmpvec);
  dist2=matrix<double> (nprs2,tmpvec);

  // Collect distances.
  // dist1
  for (int ia=0;ia<neigh_mat1.size();ia++){
    atom a = neigh_mat1[ia][0];
    for (int in=1;in<neigh_mat1[ia].size();in++){
      atom an = neigh_mat1[ia][in];
      int ta=a.GetType();
      int tna=an.GetType();
      int i=min(ta,tna);
      int j=max(ta,tna); 
      int id=j-i+ntypes1*i-max(0,i*(i-1)/2);
      dist1[id].push_back(AtomDist(a,an));
    } // in
  } // ia
  // dist2
  for (int ia=0;ia<neigh_mat2.size();ia++){
    atom a = neigh_mat2[ia][0];
    for (int in=1;in<neigh_mat2[ia].size();in++){
      atom an = neigh_mat2[ia][in];
      int ta=a.GetType();
      int tna=an.GetType();
      int i=min(ta,tna);
      int j=max(ta,tna);
      int id=j-i+ntypes2*i-max(0,i*(i-1)/2);
      dist2[id].push_back(AtomDist(a,an));
    } // in
  } // ia

  // Sort dist vectors.
  for (int ip=0;ip<nprs1;ip++){
    sort(dist1[ip].begin(),dist1[ip].end());  
  }
  for (int ip=0;ip<nprs2;ip++){
    sort(dist2[ip].begin(),dist2[ip].end());  
  }


  // tpx
  //  Vout(dist1[0],cout);
  //  Vout(dist2[0],cout);
  // Assign distance difference matrix
  int ntypes_min=min(ntypes1,ntypes2);
  int nprs_min=ntypes_min*(ntypes_min+1)/2;
  dist_diff = matrix<double> (nprs_min,tmpvec);
  dist_diff_n = matrix<double> (nprs_min,tmpvec);
  for (int i=0;i<ntypes_min;i++){
    for (int j=i;j<ntypes_min;j++){
      int id1=j-i+ntypes1*i-max(0,i*(i-1)/2);
      int id2=j-i+ntypes2*i-max(0,i*(i-1)/2);
      int idmin=j-i+ntypes_min*i-max(0,i*(i-1)/2);
      int num_dist=min(dist1[id1].size(),dist2[id2].size());
      for (int ip=0;ip<num_dist;ip++){
	double d1=dist1[id1][ip];
	double d2=dist2[id2][ip];
	dist_diff[idmin].push_back(d2-d1);
	dist_diff_n[idmin].push_back(2*(d2-d1)/(d1+d2));
      }// ip
    }// j
  }// i
} // end routine

// **************************************************
// PrintCmpStr
// **************************************************
// Prints out the comparison between two structures.
void PrintCmpStr(const structure & str1, const structure & str2,
		    const double & rcut, ostream & sout)
{
  sout.setf(ios::left, ios::adjustfield);
  sout.setf(ios::fixed, ios::floatfield);
  sout.precision(4);
  double cutoff=rcut;
  //  sout.width(10);
  // Rescale to avoid scale problems
  structure sstr1=str1;
  structure sstr2=str2;
  sstr1.ReScale(1.0);
  sstr2.ReScale(1.0);
  int nat1=sstr1.GetNumAtoms();
  int nat2=sstr2.GetNumAtoms();
  vector<int> num_each_type1=sstr1.GetNumEachType();
  vector<int> num_each_type2=sstr2.GetNumEachType();
  int ntyp_min;
  int ntyp_max;
  int nprs_min;
  int nprs_max;

  sout << "*************************  Structure Comparison *************************" << endl;

  // Nun atoms
  sout << "Number of atoms" << endl;
  sout << "  Num Atom Str1: " << nat1 << endl;
  sout << "  Num Atom Str2: " << nat2 << endl;
  sout << "  Num Atom %Err: " << 100*2*(nat2-nat1)/(nat2+nat1) << endl;

  // Num types
  int ntyp1=sstr1.GetNumEachType().size();
  int ntyp2=sstr2.GetNumEachType().size();
  sout << "Number of types" << endl;
  sout << "  Num Type Str1: " << ntyp1 << endl;
  sout << "  Num Type Str2: " << ntyp2 << endl;
  sout << "  Num Type Err : " << ntyp2-ntyp1 << endl;

  // Stoichiometry
  int wid=8;
  ntyp_max=max(ntyp1,ntyp2);
  vector<double> tmp_stoich1(ntyp_max,0);
  vector<double> tmp_stoich2(ntyp_max,0);
  sout << "Stoichiometry" << endl;
  sout << "  Type:        ";
  for (int it=0;it<ntyp_max;it++){
    sout << setw(wid) << it << " ";
  }
  sout << endl;
  sout << "  Stoich Str1: ";
  for (int it=0;it<ntyp1;it++){
    tmp_stoich1[it]=(double) num_each_type1[it]/ (double) nat1;
    sout << setw(wid) << tmp_stoich1[it] << " ";
  }
  sout << endl;
  sout << "  Stoich Str2: ";
  for (int it=0;it<ntyp2;it++){
    tmp_stoich2[it]=(double) num_each_type2[it]/ (double) nat2;
    sout << setw(wid) << tmp_stoich2[it] << " ";
  }
  sout << endl;
  sout << "  Stoich Diff: ";
  for (int it=0;it<ntyp_max;it++){
    sout << setw(wid) << tmp_stoich2[it]-tmp_stoich1[it] << " ";
  }
  sout << endl;

  // Volume
  double v1=GetVol(sstr1.GetLat())/nat1;
  double v2=GetVol(sstr2.GetLat())/nat2;
  sout << "Volume per atom" << endl;
  sout << "  Vol Str1: " << v1 << endl;
  sout << "  Vol Str2: " << v2 << endl;
  sout << "  Vol %Err: " << 100*2*(v2-v1)/(v2+v1) << endl;

  // lat par and angles
  vector<double> ldat1=Getabc_angles(sstr1.GetLat());
  vector<double> ldat2=Getabc_angles(sstr2.GetLat());
  ldat1=Sort_abc_angles(ldat1);  
  ldat2=Sort_abc_angles(ldat2);  
  vector<double> ldat_err(7,0.0);
  ldat_err[0]=100*2*(v2*nat2-v1*nat1)/(v2*nat2+v1*nat1);
  for (int i=0;i<6;i++){
    double tmp=(abs(ldat2[i])+abs(ldat1[i]));
    if (tmp>0){
      ldat_err[i+1]=100*2*(ldat2[i]-ldat1[i])/(ldat2[i]+ldat1[i]);
    }
    else{
      ldat_err[i+1]=0.0;
    }
  }
  sout << "Unit cell: tot_vol a b c alpha beta gamma" << endl;
  sout << "  Cell Str1: ";
  sout << v1*nat1 << " ";
  Vout(ldat1,sout);
  sout << "  Cell Str2: ";
  sout << v2*nat2 << " ";
  Vout(ldat2,sout);
  sout << "  Cell %Err: ";
  Vout(ldat_err,sout);


  // Volume adjusted distances
  double vavg=(v1+v2)/2.0;
  double rescale1=pow(vavg/v1,1.0/3.0);
  double rescale2=pow(vavg/v2,1.0/3.0);
  structure str1_newvol=sstr1;
  structure str2_newvol=sstr2;
  str1_newvol.SetScale(rescale1);
  str2_newvol.SetScale(rescale2);
  matrix<double> dist1;
  matrix<double> dist2;
  matrix<double> dist_diff;
  matrix<double> dist_diff_n;
  // If cutoff < 0 use default of 4*(avg rad of an atom).  Note
  // that in a rescaled str where each atom has vavg=1, rad=.6204.
  // In the true str, where each atom has vol vavg, rad=0.6204*vavg^(1/3).
  if (rcut<0) cutoff=4*0.6204*pow(vavg,1.0/3.0);
  CmpStrDist(str1_newvol,str2_newvol,cutoff,dist1,dist2,dist_diff,dist_diff_n);
  ntyp_min=min(ntyp1,ntyp2);
  nprs_min=dist_diff.size();
  sout << "Avg of Distance Magnitude Differences" << endl;
  sout << "  Rescaling vol/atom of Str1 and Str2 to the average vol/atom: " << vavg << endl;
  sout << "  Working with the all the types up to num types: " << ntyp_min << endl;
  sout << "  This corresponds to the following number of distinct pairs: " << nprs_min << endl;
  double tadiff=0;
  double tadiff_n=0;
  int tnn=0;
  for (int it1=0;it1<ntyp_min;it1++){
    for (int it2=it1;it2<ntyp_min;it2++){
      int id=it2-it1+ntyp_min*it1-max(0,it1*(it1-1)/2);
      int nn=dist_diff[id].size();      
      sout << "  P" << id << " Pair between types: " 
	   << it1 << " " << it2 <<endl; 
      sout << "    P" << id << " Comparing neighbors within cutoff (number): " 
	   << cutoff << " (" << nn << ")" <<  endl;
      double adiff=0;
      double adiff_n=0;
      double perc_err=0;
      for (int in=0;in<nn;in++){
	adiff=adiff+abs(dist_diff[id][in]);
	adiff_n=adiff_n+abs(dist_diff_n[id][in]);
      }
      tadiff=tadiff+adiff;
      tadiff_n=tadiff_n+adiff_n;
      if (nn>0){
	adiff=adiff/nn;
	adiff_n=adiff_n/nn;
      }
      else{
	adiff=0.0;
	adiff_n=0.0;
      }
      tnn=tnn+nn;
      sout << "    P" << id << " Avg Difference: " << adiff << endl;
      sout << "    P" << id << " Avg Diff/vol_avg^1/3 %: " 
	   << 100*adiff/pow(vavg,1.0/3.0) << endl;
      sout << "    P" << id << " Avg Difference/bond length %: " << 100*adiff_n << endl;
    }// it2
  }// it1
  if (tnn>0){
    tadiff=tadiff/tnn;
    tadiff_n=tadiff_n/tnn;
  }
  else{
    tadiff=0;
    tadiff_n=0;
  }
  sout << "  Totals for all pairs, compared by type, within cutoff (number): " 
	   << cutoff << " (" << tnn << ")" <<  endl;
  sout << "    Tot Avg Difference: " << tadiff << endl;
  sout << "    Tot Avg Diff/vol_avg^1/3 %: " 
       << 100*tadiff/pow(vavg,1.0/3.0) << endl;
  sout << "    Tot Avg Diff/bond_length %: " 
       << 100*tadiff_n << endl;

  /* THIS NEEDS WORK - need to sum the rdfsh over all atoms of one type,
     then output average numbers.  This is still not perfect, since you could
     get cancelling errors.  Could try to do site to site comparison.
  // Compare radial distribution functions
  ntyp_max=max(ntyp1,ntyp2);
  nprs_max=(ntyp_max+1)*(ntyp_max+2)/2;
  vector<double> tmpsh1(nprs_max,0);
  vector<double> tmpsh2(nprs_max,0);
  double bin_width=0.01;
  int nbins=Nint(cutoff/bin_width);
  int smooth_width=5;
  int nsh_max=2;
  matrix<double> rdf_all_1;
  matrix<double> rdf_all_2;
  GetRDF(sstr1,cutoff,nbins,rdf_all_1);
  GetRDF(sstr2,cutoff,nbins,rdf_all_2);
  matrix<double> rdf_all_1_sm=GetSmoothRDF(rdf_all_1,smooth_width);
  matrix<double> rdf_all_2_sm=GetSmoothRDF(rdf_all_2,smooth_width);
  // Get shells
  matrix<double> rdfsh_all_1;
  matrix<double> rdfsh_loc_1; // Radial location of rdf shells.
  matrix<double> rdfsh_all_2;
  matrix<double> rdfsh_loc_2; // Radial location of rdf shells.
  GetRDFShells(sstr1,cutoff,nbins,smooth_width,
	       rdf_all_1_sm,rdfsh_all_1,rdfsh_loc_1);
  GetRDFShells(sstr2,cutoff,nbins,smooth_width,
	       rdf_all_2_sm,rdfsh_all_2,rdfsh_loc_2);
  // print out position, number for each radial shell up to nsh_max.
  // Print out for each type of pair
  sout << "Radial shell comparison" << endl;
  for (int ish=0;ish<nsh_max;ish++){ // Radial shells loop.
    sout << "  Shell " << ish << endl;
    sout << "                    ";
    for (int it1=0;it1<ntyp_max;it1++){
      for (int it2=it1;it2<ntyp_max;it2++){
	sout << it1 << "-" << it2 << "  ";
      }
    }
    sout << endl;
    sout << "    Shell Occ Str1: ";
    for (int it1=0;it1<ntyp1;it1++){
      for (int it2=0;it2<ntyp1;it2++){
	int id=it2+(ntyp1+1)*it1;
	int idb=ntyp1*nat1+it2+(ntyp1+1)*it1;
	cout << id<<" "<<idb<< " "<<rdfsh_all_1.size()<<" "<<ntyp1<<" "<< nat1<< endl;
	tmpsh1[id]=rdfsh_all_1[ntyp1*nat1+id][ish];
	sout << tmpsh1[id] << " ";
      }
    }
    sout << endl;
  }    

  */

  // Number of each type up to rcut.
  nprs_max=ntyp_max*(ntyp_max+1)/2;
  vector<int> n1(nprs_max,0);
  vector<int> n2(nprs_max,0);
  sout << "Number neighbors of each type within cutoff for rescaled volume" << endl;
  sout << "  Cutoff: " << cutoff << endl;
  sout << "  Rescaling vol/atom of Str1 and Str2 to the average vol/atom: " << vavg << endl;
  sout << "           ";
  for (int it1=0;it1<ntyp_max;it1++){
    for (int it2=it1;it2<ntyp_max;it2++){
      sout << it1 << "-" << it2 << "  ";
    }
  }
  sout << endl;
  sout << "  NN Str1  ";
  for (int it1=0;it1<ntyp_max;it1++){
    for (int it2=it1;it2<ntyp_max;it2++){
      if (it1<ntyp1 && it2<ntyp1){
	int id1=it2-it1+ntyp1*it1-max(0,it1*(it1-1)/2);
	n1[id1]=dist1[id1].size();
	sout << setw(4) << n1[id1] << " ";
      }
    }
  }
  sout << endl;
  sout << "  NN Str2  ";
  for (int it1=0;it1<ntyp_max;it1++){
    for (int it2=it1;it2<ntyp_max;it2++){
      if (it1<ntyp2 && it2<ntyp2){
	int id2=it2-it1+ntyp2*it1-max(0,it1*(it1-1)/2);
	n2[id2]=dist2[id2].size();
	sout << setw(4) << n2[id2] << " ";
      }
    }
  }
  sout << endl;  
  sout << "  NN Err   ";
  for (int it1=0;it1<ntyp_max;it1++){
    for (int it2=it1;it2<ntyp_max;it2++){
      int id=it2-it1+ntyp_max*it1-max(0,it1*(it1-1)/2);
      sout << setw(4) << n2[id]-n1[id] << " ";
    }
  }
  sout << endl;  

  // Space group
  bool Platon_EQUAL=FALSE,Platon_EXACT=FALSE;
  double Platon_ang=1.0,Platon_d1=0.25,Platon_d2=0.25,Platon_d3=0.25;
  system("touch convasp_tmp_in"); // Create input file to connect to stream.
  ofstream outfile("convasp_tmp_out");
  ifstream infile("convasp_tmp_in");
  string spcgrp1a,spcgrp2a,spcgrp1b,spcgrp2b;
  PrintPlaton(str1,Platon_EQUAL,Platon_EXACT,Platon_ang,Platon_d1,Platon_d2,Platon_d3,outfile);
  system("cat convasp_tmp_out | platonSG > convasp_tmp_in");
  infile >> spcgrp1a >> spcgrp1b;
  // Puts file position pointers back to beginning.
  outfile.seekp(0); 
  infile.seekg(0); 
  PrintPlaton(sstr2,Platon_EQUAL,Platon_EXACT,Platon_ang,Platon_d1,Platon_d2,Platon_d3,outfile);
  system("cat convasp_tmp_out | platonSG > convasp_tmp_in");
  infile >> spcgrp2a >> spcgrp2b;
  system("/bin/rm convasp_tmp_out");
  system("/bin/rm convasp_tmp_in");
  sout << "Space groups " << endl;
  sout << "  SG Str1: " << spcgrp1a << " " << spcgrp1b << endl;
  sout << "  SG Str2: " << spcgrp2a << " " << spcgrp2b << endl;

  // Center of mass
  
} // end routine



// **************************************************
// PrintData1
// **************************************************
// Prints out the comparison between two structures.
void PrintData1(const structure & str1, const double & rcut, ostream & sout)
{
  sout.setf(ios::left, ios::adjustfield);
  sout.setf(ios::fixed, ios::floatfield);
  sout.precision(4);
  double cutoff=rcut;
  //  sout.width(10);
  // Rescale to avoid scale problems
  structure sstr1=str1;
  sstr1.ReScale(1.0);
  int nat1=sstr1.GetNumAtoms();
  int ntyp1=sstr1.GetNumEachType().size();
  vector<int> num_each_type1=sstr1.GetNumEachType();

  sout << "*************************  Structure Comparison *************************" << endl;

  // Nun atoms
  sout << "Number of atoms" << endl;
  sout << "  Num Atom Str1: " << nat1 << endl;

  // Num types
  sout << "Number of types" << endl;
  sout << "  Num Type Str1: " << ntyp1 << endl;

  // Stoichiometry
  int wid=8;
  vector<double> tmp_stoich1(ntyp1,0);
  sout << "Stoichiometry" << endl;
  sout << "  Type:        ";
  for (int it=0;it<ntyp1;it++){
    sout << setw(wid) << it << " ";
  }
  sout << endl;
  sout << "  Stoich Str1: ";
  for (int it=0;it<ntyp1;it++){
    tmp_stoich1[it]=(double) num_each_type1[it]/ (double) nat1;
    sout << setw(wid) << tmp_stoich1[it] << " ";
  }
  sout << endl;

  // Volume
  double v1=GetVol(sstr1.GetScaledLat())/nat1;
  sout << "Volume per atom" << endl;
  sout << "  Vol Str1: " << v1 << endl;

  // lat par and angles
  vector<double> ldat1=Getabc_angles(sstr1.GetLat());
  ldat1=Sort_abc_angles(ldat1);  
  sout << "Unit cell: tot_vol a b c alpha beta gamma" << endl;
  sout << "  Cell Str1: ";
  sout << v1*nat1 << " ";
  Vout(ldat1,sout);

  // Volume adjusted distances

  double rescale1=pow(1.0/v1,1.0/3.0);
  structure str1_newvol=sstr1;
  str1_newvol.SetScale(rescale1); // Sets vol/atom=1
  matrix<double> dist1;
  matrix<double> dist2;
  matrix<double> dist_diff;
  matrix<double> dist_diff_n;
  // If cutoff < 0 use default of 4*(avg rad of an atom).  Note
  // that in the rescaled str, where each atom has vavg=1, rad=.6204
  // In the true str, where each atom has vol = v1, rad=0.6204*v1^(1/3).
  if (rcut<0) cutoff=4*0.6204*pow(v1,1.0/3.0);
  CmpStrDist(str1_newvol,str1_newvol,cutoff*rescale1,dist1,dist2,dist_diff,dist_diff_n);
  int nprs1=dist1.size();
  sout << "Avg of Distance Magnitude Differences" << endl;
  sout << "  Rescaling vol/atom of Str1 to the average vol/atom (return to orig scale factor): " << GetVol(str1_newvol.GetScaledLat())/nat1 << " (" << 1/rescale1 << ")" << endl;
  sout << "  Working with the all the types up to num types (num pair types): " << ntyp1 << " (" << nprs1 << ")" << endl;
  sout << "  Including all pairs within cutoff: " << cutoff << endl;

  double tadiff=0;
  double tadiff_n=0;
  int tnn=0;
  for (int it1=0;it1<ntyp1;it1++){
    for (int it2=it1;it2<ntyp1;it2++){
      int id=it2-it1+ntyp1*it1-max(0,it1*(it1-1)/2);
      int nn=dist1[id].size();      
      sout << "  P" << id << "  " << nn << " Pairs between types: " 
	   << it1 << " " << it2 << "   "; 
      for (int in=0;in<nn;in++){
	sout << " " << dist1[id][in];
      }
      sout << endl;
    }// it2
  }// it1


//    // Number of each type up to rcut.
//    nprs_max=ntyp_max*(ntyp_max+1)/2;
//    vector<int> n1(nprs_max,0);
//    vector<int> n2(nprs_max,0);
//    sout << "Number neighbors of each type within cutoff for rescaled volume" << endl;
//    sout << "  Cutoff: " << cutoff << endl;
//    sout << "  Rescaling vol/atom of Str1 and Str2 to the average vol/atom: " << vavg << endl;
//    sout << "           ";
//    for (int it1=0;it1<ntyp_max;it1++){
//      for (int it2=it1;it2<ntyp_max;it2++){
//        sout << it1 << "-" << it2 << "  ";
//      }
//    }
//    sout << endl;
//    sout << "  NN Str1  ";
//    for (int it1=0;it1<ntyp_max;it1++){
//      for (int it2=it1;it2<ntyp_max;it2++){
//        if (it1<ntyp1 && it2<ntyp1){
//  	int id1=it2-it1+ntyp1*it1-max(0,it1*(it1-1)/2);
//  	n1[id1]=dist1[id1].size();
//  	sout << setw(4) << n1[id1] << " ";
//        }
//      }
//    }
//    sout << endl;
//    sout << "  NN Str2  ";
//    for (int it1=0;it1<ntyp_max;it1++){
//      for (int it2=it1;it2<ntyp_max;it2++){
//        if (it1<ntyp2 && it2<ntyp2){
//  	int id2=it2-it1+ntyp2*it1-max(0,it1*(it1-1)/2);
//  	n2[id2]=dist2[id2].size();
//  	sout << setw(4) << n2[id2] << " ";
//        }
//      }
//    }
//    sout << endl;  
//    sout << "  NN Err   ";
//    for (int it1=0;it1<ntyp_max;it1++){
//      for (int it2=it1;it2<ntyp_max;it2++){
//        int id=it2-it1+ntyp_max*it1-max(0,it1*(it1-1)/2);
//        sout << setw(4) << n2[id]-n1[id] << " ";
//      }
//    }
//    sout << endl;  

//    // Space group
//    bool Platon_EQUAL=FALSE,Platon_EXACT=FALSE;
//    double Platon_ang=1.0,Platon_d1=0.25,Platon_d2=0.25,Platon_d3=0.25;
//    system("touch convasp_tmp_in"); // Create input file to connect to stream.
//    ofstream outfile("convasp_tmp_out");
//    ifstream infile("convasp_tmp_in");
//    string spcgrp1a,spcgrp2a,spcgrp1b,spcgrp2b;
//    PrintPlaton(str1,Platon_EQUAL,Platon_EXACT,Platon_ang,Platon_d1,Platon_d2,Platon_d3,outfile);
//    system("cat convasp_tmp_out | platonSG > convasp_tmp_in");
//    infile >> spcgrp1a >> spcgrp1b;
//    // Puts file position pointers back to beginning.
//    outfile.seekp(0); 
//    infile.seekg(0); 
//    PrintPlaton(sstr2,Platon_EQUAL,Platon_EXACT,Platon_ang,Platon_d1,Platon_d2,Platon_d3,outfile);
//    system("cat convasp_tmp_out | platonSG > convasp_tmp_in");
//    infile >> spcgrp2a >> spcgrp2b;
//    system("/bin/rm convasp_tmp_out");
//    system("/bin/rm convasp_tmp_in");
//    sout << "Space groups " << endl;
//    sout << "  SG Str1: " << spcgrp1a << " " << spcgrp1b << endl;
//    sout << "  SG Str2: " << spcgrp2a << " " << spcgrp2b << endl;

  // Center of mass
  
} // end routine




