// **************************************************
// PrintMSI
// **************************************************
//  This funtion prints out structural data in a msi format (for cerius).

using namespace std;
#include <iostream>
#include <cmath>
#include "UsefulFuncs.h"
#include "PrintMSI.h"
#include <vector>

void PrintMSI(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();

  // Print out data in msi format

  // Tolerance
  double Tol=1e-10;

  // First we must determine the atomic numbers.
  vector<int> atomic_num(num_atoms);
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<at_name_vec.size();j++){
      if (names[i]==at_name_vec[j]) atomic_num[i]=at_num_vec[j];
    }
  }
  
//   Now we have to orient the lattice vectors for msi.
//   The desired orientation is obtained by
//   rotating c around Z into the XZ plane (R1a), then rotating c around
//   Y to align it with Z (R1b), then rotating b around Z (leaving c fixed)
//   until it is in the YZ plane (R2).  The total rotation needed is then
//   Rtot=R2*R1b*R1a.

  vector<double> v(3,0.0);
  vector<vector<double> > R1a(3,v);  
  vector<vector<double> > R1b(3,v);  
  vector<vector<double> > R2(3,v);  
  vector<vector<double> > Rtot(3,v);  
  vector<double> a(3);
  vector<double> b(3);
  vector<double> c(3);
  for (int i=0;i<3;i++){
    a[i]=lat[0][i];
    b[i]=lat[1][i];
    c[i]=lat[2][i];
  }
  double an=norm(a);
  double bn=norm(b);
  double cn=norm(c);
  double cxyn=sqrt(cn*cn-c[2]*c[2]);
  double bxyn=sqrt(bn*bn-b[2]*b[2]);
  double cosa=getcos(a,b);
  double cosb=getcos(a,c);
  double cosg=getcos(b,c);

// Build R1a.
  double cosp,sinp;
  if (cxyn>Tol && (abs(c[0])>Tol || abs(c[1])>Tol)){
    cosp=c[0]/cxyn;
    sinp=c[1]/cxyn;
  }
  else{
    cosp=1.0;
    sinp=0.0;
  }
  R1a[0][0]=cosp;
  R1a[0][1]=sinp;
  R1a[1][0]=-sinp;
  R1a[1][1]=cosp;
  R1a[2][2]=1.0;
// tpx
//  Mout(R1a,cout);
//  cout << "cosp, sinp" << cosp << " " << sinp << endl;
// Build R1b.
  double costh=c[2]/cn;
  double sinth=sqrt(1-costh*costh); // This is OK since sinth>0.
  R1b[0][0]=costh;
  R1b[0][2]=-sinth;
  R1b[2][0]=sinth;
  R1b[2][2]=costh;
  R1b[1][1]=1.0;
  vector<vector<double> > RR;
  RR=MMmult(R1b,R1a);
// Build R2.  
// Using rotated b vector, called bp, (bp=R1b*R1a*b) to define the angles.
  vector<double> bp(3);
  bp=MVmult(RR,b);
  double bpn=norm(bp);
  double bpxyn=sqrt(bpn*bpn-bp[2]*bp[2]);
  double cospp,sinpp;
  if (bpxyn>Tol && (b[0]>Tol || b[1]>Tol)){
    cospp=bp[1]/bpxyn;
    sinpp=bp[0]/bpxyn;
  }
  else {
    cospp=1.0;
    sinpp=0.0;
  }
  R2[0][0]=cospp;
  R2[0][1]=-sinpp;
  R2[1][0]=sinpp;
  R2[1][1]=cospp;
  R2[2][2]=1.0;
// Calculate final rotation matrix.
 // tpx
  for (int i=0;i<3;i++){
  for (int j=0;j<3;j++){
//    cout << "R2 " << R2[i][j] << endl;
  }
  }
  Rtot=MMmult(R2,RR);
  //tpx
  //Mout(Rtot,cout);
  //cout << "HERE " << cxyn << " " << cosa << " " << cosb << " " << cosg << " " << cosp << " " << sinp << endl; 
// Now rotate all the lattice vectors and the positions.
  vector<vector<double> > c2lat(3,v);
  vector<vector<double> > c2pos(num_atoms,v);
  v=MVmult(Rtot,a);
  for (int i=0;i<3;i++){
    c2lat[0][i]=v[i];
  }
  v=MVmult(Rtot,b);
  for (int i=0;i<3;i++){
    c2lat[1][i]=v[i];
  }
  v=MVmult(Rtot,c);
  for (int i=0;i<3;i++){
    c2lat[2][i]=v[i];
  }
  for (int i=0;i<num_atoms;i++){
    v=MVmult(Rtot,cpos[i]);
    for (int j=0;j<3;j++){
      c2pos[i][j]=v[j];
    }
  }

// Write output.
  // Title.
  cout << "# MSI CERIUS2 DataModel File Version 3 5" << endl;

  // Stuff.
  cout << "(1 Model" << endl;

  // Lattice vectors.
  cout << " (A D A3 (";
  for (int i=0;i<3;i++){
    cout << c2lat[0][i] << " ";
  }
  cout << "))" << endl;
  cout << " (A D B3 (";
  for (int i=0;i<3;i++){
    cout << c2lat[1][i] << " ";
  }
  cout << "))" << endl;
  cout << " (A D C3 (";
  for (int i=0;i<3;i++){
    cout << c2lat[2][i] << " ";
  }
  cout << "))" << endl;

  // Stuff.
    cout << " (A I Id 1)" << endl;
    cout << " (A C Label \"Model1\")" << endl;

  // The atoms.
    for (int i=0;i<num_atoms;i++){
      cout << " (" << i+2 << " Atom" << endl; 
      cout << "  (A C ACL \"" << atomic_num[i] << " " << names[i] << "\")" << endl; 
      cout << "  (A D XYZ (";
      for (int j=0;j<3;j++){
	cout << c2pos[i][j] << " ";
      }
      cout << "))" << endl;
      cout << "  (A I Id " << i+1 << ")" << endl;
      cout << " )" << endl;
    }
// Final parenthesis
  cout << ")" << endl;
}







