using namespace std;
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <algorithm>
#include "UsefulFuncs.h"
#include "constants.h"
#include "atom.h"
#include "RDFFuncs.h"

/*
**************************************************
Function GetRDF
**************************************************
  This function gets the radial distribution functions (RDF).
  The RDF for an atom is just a binned histogram of the 
  number of atoms (possibly of a given type) a distance r away.  
  The RDF for a type is just the average RDF's for all the atoms
  of that type.
  The rdf is stored as follows: I=(0,natoms-1),J,K=(0,ntypes)
  row: J+(ntypes+1)*I = atom I / Type J RDF.
     (if row: J=ntypes then the rdf is the atom I / All types RDF.)
  I don't think the following ever got coded.
  lines (ntypes+1)*natoms+K+(ntypes+1)*J = type J / type K average RDF.
     (if row: K=ntypes then the rdf is the type I / All types RDF.)
*/
void GetRDF(const structure & str, const double & rmax,
	    const int & nbins, matrix<double> & rdf_all)
{
  int natoms=str.GetNumAtoms();
  vector<int> num_each_type=str.GetNumEachType();
  int ntyp=num_each_type.size();
  rdf_all=matrix<double> ((ntyp+1)*natoms,nbins);
  matrix<atom> nmat;
  GetStrNeighData(str,rmax,nmat);
  for (int I1=0;I1<nmat.size();I1++){ // Each atom for which we find RDF.
    int I2=1;
    double dist=0;
    int s2=nmat[I1].size();
    atom a1=nmat[I1][0];
    // Check each atom neighbor for I1 in proper range.
    dist=AtomDist(a1,nmat[I1][I2]);
    while (dist<rmax && I2<s2){ 
      int ib=int((dist/rmax)*nbins); // first bin is [0,rmax/nbins).
      int J2=nmat[I1][I2].GetType();
      //      cout << I1 << " " << I2 << " " <<  J2 << " " << s2 << " " << dist << " " << ib << endl;
      rdf_all[(ntyp+1)*I1+J2][ib]++; // Does all binning for atom/type RDFs.
      I2++;
      if (I2<s2) dist=AtomDist(a1,nmat[I1][I2]);
    }  
    // Get sum over all types
    for (int ib=0;ib<nbins;ib++){
      for (int it=0;it<ntyp;it++){
	rdf_all[(ntyp+1)*I1+ntyp][ib]+=rdf_all[(ntyp+1)*I1+it][ib];
      }
    }
  } // I1 loop
}

/*
**************************************************
Function GetRDFShells
**************************************************
  This function gets the neighbor shells from the
  radial distribution functions.  Shells are found
  by the following method.  The code calculates
  the change in rdf from drdf=rdf[i+1]-rdf[i].  We 
  step through drdf starting adding up rdf into shell1
  while drdf>=0 and then while drdf<=0.  When drdf gets
  >=0 again we start a new shell until drdf passes 
  through <=0 and is >=0 again, at which point we again
  start a new shell.  In other words, we increment the
  shell we are adding to when drdf goes from >=0 to <0 to
  >=0 again.
*/
void GetRDFShells(const structure & str,const double & rmax,
		  const int & nbins,
		  const int & smooth_width,
		  const matrix<double> & rdf, 
		  matrix<double> & rdfsh,
		  matrix<double> & rdfsh_loc)
{
  double TOL=1e-5;
  int natom=str.GetNumAtoms();
  rdfsh = matrix<double> (rdf.size(),0);
  rdfsh_loc = matrix<double> (rdf.size(),0);
  double dr=(rmax/(double)nbins);
  for (int i=0;i<rdf.size();i++){

    /*
    // get smoothed rdf_sm 
    vector<double> rdf_sm(nbins,0.0);
    int cnt=0;
    for (int ib=0;ib<nbins;ib++){
      for (int ism=-(smooth_width-1)/2;ism<=(smooth_width-1)/2;ism++){
	int id=ib+ism;
	if (id>0 && id<nbins){
	  cnt++;
	  rdf_sm[ib]+=rdf[i][ib+ism];
	}
      }
      if (cnt>0) rdf_sm[ib]=rdf_sm[ib]/cnt;
      cnt=0;
    }
    rdf_sm=SmoothFunc(rdf[i],smooth_width);
    */

    // get drdf,ddrdf of rdf
    vector<double> drdf(nbins,0.0);
    vector<double> ddrdf(nbins,0.0);
    for (int ib=1;ib<nbins-1;ib++){
      drdf[ib]=(rdf[i][ib+1]-rdf[i][ib-1]);
    }
    for (int ib=1;ib<nbins-1;ib++){
      //      ddrdf[ib]=(rdf[i][ib+1]+rdf[i][ib-1]-2*rdf[i][ib]);
      ddrdf[ib]=(drdf[ib+1]-drdf[ib-1]);
    }

    // get all zeros of drdf with ddrf!=0 
    // -999 means nothing,-1 means ddrdf<0, 0 means ddrdf=0, +1 means ddrdf>0.
    vector<int> drdf_zeros(nbins-1,-999);
    for (int ib=1;ib<nbins-1;ib++){
      // If drdf==0
      if (drdf[ib]==0 && ddrdf[ib]<0) drdf_zeros[ib]=-1;
      if (drdf[ib]==0 && ddrdf[ib]==0) drdf_zeros[ib]=0;
      if (drdf[ib]==0 && ddrdf[ib]==1) drdf_zeros[ib]=1;
      // If drdf== is passing through 0 from +->- or -->+
      if (drdf[ib]!=0){
	if (drdf[ib-1]>0 && drdf[ib+1]<0) drdf_zeros[ib]=-1;
	if (drdf[ib-1]<0 && drdf[ib+1]>0) drdf_zeros[ib]=+1;
	// If drdf== is becoming or changinf from 0 through 
	// 0->+/- or +/-->0.
	if (drdf[ib-1]==0 && ddrdf[ib]<0) drdf_zeros[ib]=-1;
	if (drdf[ib-1]==0 && ddrdf[ib]==0) drdf_zeros[ib]=0;
	if (drdf[ib-1]==0 && ddrdf[ib]>0) drdf_zeros[ib]=1;
	if (drdf[ib+1]==0 && ddrdf[ib]<0) drdf_zeros[ib]=-1;
	if (drdf[ib+1]==0 && ddrdf[ib]==0) drdf_zeros[ib]=0;
	if (drdf[ib+1]==0 && ddrdf[ib]>0) drdf_zeros[ib]=1;
      }

      // tpx
      //      cout << "drdf_zeros " << ib << " " << rdf[i][ib] << " " << drdf[ib] << " " << ddrdf[ib] << " " << drdf_zeros[ib] << endl;
    }

    // get the actual shell atoms counts and avg radius.
    double shell=0;
    double rsh_avg=0;
    double state_dn=0;
    for (int ib=0;ib<nbins-2;ib++){
      //tpx
      //           cout << i << " " << ib << " " 
      //	         << rdf[i][ib] << " " << drdf[ib] << " " << shell << " " << endl;
      shell=shell+rdf[i][ib];
      double rad=(dr*(double)ib)+dr/2.0;
      rsh_avg=rsh_avg+rdf[i][ib]*rad;
      if (drdf_zeros[ib]==-1){ // Set state_dn
	state_dn=1;
      }
      if (state_dn && drdf_zeros[ib]==1){ // New shell
	if (shell>0) rsh_avg=rsh_avg/shell;
	rdfsh[i].push_back(shell);
	rdfsh_loc[i].push_back(rsh_avg);
	shell=0;
	rsh_avg=0;
	state_dn=0;
      }
    } // for ib

  } // for i
}

/*
**************************************************
Function PrintRDF
**************************************************
  This function prints out the RDF information.
*/
void PrintRDF(const structure & str, const double & rmax,
	      const int & nbins,
	      const int & smooth_width,
	      const matrix<double> & rdf_all, 
	      matrix<double> & rdfsh_all,
	      matrix<double> & rdfsh_loc,
	      ostream & outf)
{
  int natoms=str.GetNumAtoms();
  vector<int> types=str.GetTypes();
  vector<int> num_each_type=str.GetNumEachType();
  int ntypes=num_each_type.size();
  outf.setf(ios::left, ios::adjustfield);
  outf.setf(ios::fixed, ios::floatfield);
  int w1=5;
  matrix<double> rdfsh_bin(rdfsh_loc.size(),nbins,0.0);

  // Put rdfsh_all into bins so you can write out in same
  // way as rdf_all
  for (int i=0;i<rdfsh_loc.size();i++){
    for (int j=0;j<rdfsh_loc[i].size();j++){
      int ib=int((rdfsh_loc[i][j]/rmax)*nbins); // first bin is [0,rmax/nbins).
      rdfsh_bin[i][ib]+=rdfsh_all[i][j];
    }
  }

  // RDF
  outf<<"# Radial Distribution Function (RDF)"<< endl;
  outf<< "# rmax= " << rmax << "     nbins = " << nbins << "     sigma= " << smooth_width << endl;
  outf<< "# Format: bin  rad  type1:rdf shells  type2:rdf shells ... " << endl;
  double bw=rmax/(double)nbins;
  for (int ia=0;ia<natoms;ia++){
    outf << "# atom / type: " << ia+1 << " / " << types[ia]+1 << endl;
    outf << "# bin    rad ";
    outf << "       type" << 1;
    for (int it=1;it<ntypes;it++){
      outf << "          type" << it+1;
    }
    outf <<   "          typeAll";
    outf << endl;
    for (int ib=0;ib<nbins;ib++){
      outf << "  " << setw(w1) << ib+1;
      outf << "  " << ib*bw;
      outf << "  ";
      for (int it=0;it<ntypes;it++){
	outf <<" " << setw(w1) << rdf_all[(ntypes+1)*ia+it][ib];
	outf <<" " << setw(w1) << Nint(rdfsh_bin[(ntypes+1)*ia+it][ib]);
      }
      outf <<" " << setw(w1) << rdf_all[(ntypes+1)*ia+ntypes][ib];
      outf <<" " << setw(w1) << Nint(rdfsh_bin[(ntypes+1)*ia+ntypes][ib]);
      outf << endl;
    }
    
    // RDF SHELLS
    // Header line
    outf << "# atom / type: " << ia+1 << " / " << types[ia]+1 << endl;
    outf << "# NN   ";
    for (int it=0;it<ntypes;it++){
      outf << "  type" << it+1;
      outf << ":rad/size";
    }
    outf <<   "  typeAll";
    outf << ":rad/size";
    outf << endl;
    // Get nshmx for this atom
    int nshmx=0;
    for (int i=0;i<ntypes+1;i++){
      int temp=rdfsh_all[(ntypes+1)*ia+i].size();
      if (temp>nshmx) nshmx=temp;
    }
    // Output shell data
    for (int is=0;is<nshmx;is++){
      outf << "  " << setw(w1) << is+1;
      for (int it=0;it<ntypes+1;it++){
	int id=(ntypes+1)*ia+it;
	if (is<rdfsh_all[id].size()){
          outf << "  " << setw(w1+2) << setprecision(4) << rdfsh_loc[id][is]; 
	  outf << "  " << setw(w1) << Nint(rdfsh_all[id][is]);
	}
	else{
	  outf << "  " << setw(w1) << "NA";
	  outf << "  " << setw(w1) << "NA";
	}
      } // for it
      outf << endl;
    } // for is
  } // for ia

}


/*
**************************************************
Function CmpRDFShell
**************************************************
  This function compares the rdf shells of two structures.
  Two atoms similarity are based on the rms error between
  their first nsh nn shells (for every type).  Atoms must
  be the same type to even be compared.
  The results are given in:
  best_match: For each atom in str_A gives the best matching atom
              in str_B of the smae type.
  rms_mat:  Gives the rms error between every pair of atoms.
  For the best matches I start with the first atom of str_A and
  compare to all of B.  Each successive match for an atom of str_A
  is performed excluding the previously matched atoms of str_B.
  str_A and str_B must have the same number of each type of atom.
*/
void CmpRDFShells(const structure & str_A, const structure & str_B,
	     const matrix<double> & rdfsh_all_A,
	     const matrix<double> & rdfsh_all_B,
	     const int nsh, vector<int> & best_match,
	     matrix<double> & rms_mat)
{
  double TOL=1e-15;
  vector<int> netype_A=str_A.GetNumEachType();
  vector<int> netype_B=str_B.GetNumEachType();
  // Exit if A and B have different numbers of any types of atoms.
  if (!VVequal(netype_A,netype_B) || netype_A.size()!=netype_B.size()){
    cerr << "ERROR: in CmpRDFShell" << endl;
    cerr << "ERROR: structures A and B do not have the same "<< endl;
    cerr << "ERROR: number of each type of atom. "<< endl;
    cerr << "ERROR: Exiting" << endl;
    exit(1);
  }
  int nat=str_A.GetNumAtoms();
  int nt=netype_A.size();

  // Get rms_mat
  rms_mat = matrix<double> (nat,nat,-1);
  for (int iaA=0;iaA<nat;iaA++){
    for (int iaB=0;iaB<nat;iaB++){
      rms_mat[iaA][iaB]=RdfSh_RMS(iaA,iaB,nsh,nt,rdfsh_all_A,rdfsh_all_B);
    }
  }

  // Get best_match matrix.
  // For each atom in A check the rms error for all atoms of that 
  // type in B.  Then take the best one and then remove it from 
  // the list for further comparisons.
  best_match=vector<int> (nat,-1);
  vector<int> cand_Batoms(nat,0);
  for (int i=0;i<nat;i++) cand_Batoms[i]=i;
  for (int iaA=0;iaA<rms_mat.size();iaA++){
    double rms=-10;
    double trms=-10;
    int best_at_id=-1;
    int typeA=str_A.GetType(iaA);
    // Check each B atom and if it is the right type and lowest rms
    // then track its id.
    for (int ipB=0;ipB<cand_Batoms.size();ipB++){
      int iaB=cand_Batoms[ipB];
      int typeB=str_B.GetType(iaB);
      if (typeA==typeB){
	trms=rms_mat[iaA][iaB];
	if (rms<-1) rms=trms+1.0;
	if (trms<rms-TOL){
	  best_at_id=iaB;
	  rms=trms;
	}
      }
    }
    best_match[iaA]=best_at_id;
    // Remove best_at_id from future comparisons.
     cand_Batoms.erase(find(cand_Batoms.begin(),cand_Batoms.end(),best_at_id));
  }

}


/*
**************************************************
Function RdfSh_RMS
**************************************************
  This function compares the rdf shells of two 
  atoms for each type and returns the rms.  
*/
double RdfSh_RMS(const int iaA, const int iaB, const int nsh_max,
		 const int nt,
		 const matrix<double> & rdfsh_all_A,
		 const matrix<double> & rdfsh_all_B)

{
  double rms=0;
  int cnt=0;
  for (int it=0;it<nt;it++){
    int idA=(nt+1)*iaA+it;
    int idB=(nt+1)*iaB+it;
    int tempsh=min(rdfsh_all_A[idA].size(),rdfsh_all_B[idB].size());
    int nsh = min(tempsh,nsh_max);
    for (int ish=0;ish<nsh;ish++){
      cnt++;
      rms+=(rdfsh_all_A[idA][ish]-rdfsh_all_B[idB][ish])
	*(rdfsh_all_A[idA][ish]-rdfsh_all_B[idB][ish]);
      // tpx
      //      cout << "iaA,iaB,it,ish,cnt,rms " <<iaA<<" "<<iaB<<" "<<it<<" "<<ish<<" "<< cnt << " " << rms << endl;
    }
  }
  if (cnt>0) rms=sqrt(rms/cnt);
  return rms;
}


/*
**************************************************
Function PrintRDFCmp
**************************************************
  This function prints out the RDF shells comparison information.
*/
void PrintRDFCmp(const structure & str_A, const structure & str_B,
		 const double & rmax, const int nbins,
		 const double & smooth_width,const int nsh,
		 const matrix<double> & rdfsh_all_A,
		 const matrix<double> & rdfsh_all_B,
		 const vector<int> & best_match,
		 const matrix<double> & rms_mat, ostream & outf)
{
  int nat=str_A.GetNumAtoms();
  vector<int> num_each_type=str_A.GetNumEachType();
  int nt=num_each_type.size();
  int w1=4;
  outf.setf(ios::left, ios::adjustfield);
  outf.setf(ios::fixed, ios::floatfield);

  // Info on RDF calculation parameters.
  outf<<"# Comparison of structures based on radial distribution functions (RDFs)"<< endl;
  outf << "# rmax= " << rmax << "     nbins = " << nbins << endl;
  outf << "# sigma= " << smooth_width << "     nsh = " << nsh << endl;
  outf << endl;
  outf << "# The matrix of rms error between atoms in str A and str B" << endl;

  // The RMS matrix.

  // header line
  outf << setw(w1+1) << " ";
  for (int i=0;i<nat;i++){
    outf << " " << setw(w1) << i+1;
  }
  outf << endl;
  
  // The RMS matrix as a row/column matrix.
  for (int i=0;i<nat;i++){
    outf << setw(w1) << i+1 << " ";
    for (int j=0;j<nat;j++){
      outf << " " << setprecision(2) << rms_mat[i][j];
    }
    outf << endl;
  }

  // The fit data for each atom's best fit.
  double rms_tot=0;
  outf << endl;

  // Basic atoms and rms.
  for (int iA=0;iA<nat;iA++){
    int iB=best_match[iA];
    rms_tot+=rms_mat[iA][iB];
    outf << "A atom / Best matching B atom / RMS: " 
	 << iA+1 << " " << iB+1 << " " << rms_mat[iA][iB] << endl;

    // Detailed data on the A and B shells
    // Header line
    outf << "# CN ";
    outf << "   type" << 1;
    for (int it=1;it<nt;it++){
      outf << "             type" << it+1;
    }
    outf <<   "             typeAll";
    outf << endl;

    // Get nshmx for this atom
    int nshmx=0;
    int temp;
    for (int i=0;i<nt+1;i++){
      temp=rdfsh_all_A[(nt+1)*iA+i].size();
      if (temp>nshmx) nshmx=temp;
      temp=rdfsh_all_B[(nt+1)*iB+i].size();
      if (temp>nshmx) nshmx=temp;
    }

    // Output shell data
    for (int is=0;is<nshmx;is++){
      outf << "  " << setw(w1) << is+1;
      for (int it=0;it<nt+1;it++){
	int idA=(nt+1)*iA+it;
	int idB=(nt+1)*iB+it;
	if (is<rdfsh_all_A[idA].size()){
	  outf << " " << setw(w1) << Nint(rdfsh_all_A[idA][is]);
	}
	else{
	  outf << " " << setw(w1) << "NA";
	}
	if (is<rdfsh_all_B[idB].size()){
	  outf << " " << setw(w1) << Nint(rdfsh_all_B[idB][is]);
	}
	else{
	  outf << " " << setw(w1) << "NA";
	}
	if (is<rdfsh_all_A[idA].size() && is<rdfsh_all_B[idB].size()){
	  outf << " " << setw(w1) << Nint(rdfsh_all_A[idA][is]-rdfsh_all_B[idB][is]);
	}
	else{
	  outf << " " << setw(w1) << "NA";
	}
	outf << "   ";
      } // for it
      outf << endl;
    } // for is

  } // for iA

  // Total RMS
  outf << "RMS TOTAL: " << rms_tot/nat;

}





/*
**************************************************
Function GetSmoothRDF
**************************************************
  This function gets a smoothed RDF.
*/
matrix<double> GetSmoothRDF(const matrix<double> & rdf,const double & sigma)
{
  matrix<double> rdf_sm=rdf;
  for (int i=0;i<rdf.size();i++){
    rdf_sm[i]=SmoothFunc(rdf[i],sigma);
  }
  return rdf_sm;
}
