// **************************************************
// RayTraceFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <sstream>
#include "UsefulFuncs.h"
#include "rtparams.h"
#include "GetSuperCell.h"
#include "atom.h"
#include "RayTraceFuncs.h"
#include "constants.h"

/* 
**************************************************
GetDatFromOUTCAR
**************************************************
This gets the lattice vectors and num_each_type from an
OUTCAR file.
*/
void GetDatFromOutcar(vector<matrix<double> > & lat_vec, 
		      vector<int> & num_each_type, ifstream & outcar_inf)
{
  int first_lat=1;
  vector<string> a(3,"Z");
  lat_vec.clear();
  string sdum;
  while (outcar_inf >> a[2]){
    string key= (a[0]+" "+a[1]+" "+a[2]);
    if (key=="reciprocal lattice vectors"){
      matrix<double> lat(3,3);
      for (int ic=0;ic<3;ic++){
        outcar_inf >> lat[ic][0] >> lat[ic][1] >> lat[ic][2];
        outcar_inf >>sdum>>sdum>>sdum;
      }
      if (!first_lat){ // skip first lat.
        lat_vec.push_back(lat);
      }
      first_lat=0;
    }
    if (key=="ions per type"){
      string s;
      int id=0;
      getline(outcar_inf,s);
      GetNextVal(s,id).c_str(); // remove an "=" sign.
      while (id<s.size()){
        int n=atoi(GetNextVal(s,id).c_str());
        num_each_type.push_back(n);
      }
    }
    a[0]=a[1];
    a[1]=a[2];
  }
}

/* 
**************************************************
GetDatFromXDATCAR
**************************************************
This gets the dpos from XDATCAR.
*/
void GetDatFromXdatcar(vector<matrix<double> > & dpos_vec, 
		       ifstream & xdatcar_inf)
{
  dpos_vec.clear();
  string s;
  matrix<double> dpmat(0);
  vector<double> dpvec(3);
  // Read in header lines
  for (int il=0;il<6;il++){
    getline(xdatcar_inf,s);
  }
  // Read in all sets of dpos.
  int keep_reading=1;
  while (keep_reading){
    getline(xdatcar_inf,s);
    int id;
    string key;
    while (s.size()>1 && key!="Konfig="){
      id=0;
      for (int ic=0;ic<3;ic++){
        dpvec[ic]=atof(GetNextVal(s,id).c_str());
      }
      dpmat.push_back(dpvec);
      if (!getline(xdatcar_inf,s)) keep_reading=0;
      id=0;
      key=GetNextVal(s,id);
    }
    dpos_vec.push_back(dpmat);
    dpmat=matrix<double> (0);
  }
}

/* 
**************************************************
GetStrVecFromOUTCAR_XDATCAR
**************************************************
*/
vector<structure> GetStrVecFromOUTCAR_XDATCAR(ifstream & outcar_inf, ifstream & xdatcar_inf)
{
  vector<matrix<double> > dpos_vec;
  vector<matrix<double> > lat_vec;
  vector<int> num_each_type;
  GetDatFromOutcar(lat_vec,num_each_type,outcar_inf);
  GetDatFromXdatcar(dpos_vec,xdatcar_inf);
  if (lat_vec.size()!=dpos_vec.size()){
    cout << endl;
    cerr << "WARNING: RayTraceFuncs.cc/GetStrVecFromOUTCAR_XDATCAR" << endl;
      cerr << "WARNING: Number of lattice vector and positions of images are not equal." << endl;
      cerr << "WARNING: Number of lattices: " << lat_vec.size()<< endl;
      cerr << "WARNING: Number of positions: " << dpos_vec.size()<< endl;
      cerr << "WARNING: Resizing number of lattices to match number of positions and using last lattice to fill out vector if needed. " << endl;
      cout << endl;
    }
  if (lat_vec.size()>dpos_vec.size()) {lat_vec.resize(dpos_vec.size());} 
  if (lat_vec.size()<dpos_vec.size()) {
    matrix<double> tlat=lat_vec[lat_vec.size()-1];
    for (int i=lat_vec.size();i<dpos_vec.size();i++){
      lat_vec.push_back(tlat);
    }
  }
  vector<structure> str_vec;
  structure str;
  int nstr=lat_vec.size();
  for (int is=0;is<nstr;is++){
    int nat=dpos_vec[0].size();
    vector<string> names(nat,"H");
    vector<int> names_were_given(nat,0);
    str.SetLat(lat_vec[is]);
    str.SetNumEachType(num_each_type);
    str.SetAllAtomPos(dpos_vec[is],0);
    str.SetAllAtomNames(names);
    str.SetNamesWereGiven(names_were_given);
    str_vec.push_back(str);
  }
  return str_vec;
}

/* 
**************************************************
PrintStrVec
**************************************************
*/
void PrintStrVec(const vector<structure> & str_vec, ostream & outf)
{
  for (int ist=0;ist<str_vec.size()-1;ist++){
    outf << str_vec[ist];
    outf << endl;
  }
  outf << str_vec[str_vec.size()-1];
}

/* 
**************************************************
ReadInRTParams
**************************************************
  Reads the ray tracing parameters and stores them in the rtparam object.
  Input data is stored in rtinfile using tokens of the form
  TOKEN=value.  The input can have arbitrary spaces and blank lines.
  The input can also have arbitrary stuff after value as long as it 
  is separated by a space.  The value can sometimes be more than one
  field.  Comments are any sequence of text on a single line that 
  follows a "#".
*/

void ReadInRTParams(ifstream & rtinfile, rtparams & rtp)
{

// Read in all the tokens.
  string s,s_ns;
  vector<string> token_vec;
  vector<string> val_vec;
  while (!rtinfile.eof()){
    getline(rtinfile,s);
    s_ns=RemoveSpaces(s); // Get string with no spaces.
    if (s_ns.size()>0){ // Make sure line was not blank
      if (s_ns[0]!='#'){ // Exclude comment lines
	string token,sval;
	int id=s.find('=');
	if (id>=s.length()){
	  cout << "ERROR: The following token is incorrectly formatted: " << s << endl;
	  cout << "ERROR: Exiting" << endl;
	  exit(1);
	}
	token=s.substr(0,id);
	token=RemoveSpaces(token);
	int i_f=min(s.length(),s.find('#')); // End of string
	sval=s.substr(id+1,i_f-id-1);
	token_vec.push_back(token);
	val_vec.push_back(sval);
      } //if s_ns[0]!=#
    } // if s_ns.size>0

  } // while !rtinfile.eof

  // Read in values for variables in the rtparams object.
  int first_light = 1;
  vector<double> tmp;
  for (int i=0;i<token_vec.size();i++){
    int id=0;
    string tok=token_vec[i];
    string sval;
    double val;
    int found_token=0;
    if (tok=="RESX"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.resx=val;
      found_token=1;
    }// RESX
    if (tok=="RESY"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.resy=val;
      found_token=1;
    }// RESY
    if (tok=="VIEWDIR"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<3;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.viewdir[ic]=val;
      }
      found_token=1;
      rtp.viewdir_s=1;
    }// VIEWDIR
    if (tok=="UPDIR"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<3;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.updir[ic]=val;
      }
      found_token=1;
      rtp.updir_s=1;
    }// UPDIR
    if (tok=="ZOOM"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.zoom=val;
      found_token=1;
    }// ZOOM
    if (tok=="ASPECTRATIO"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.aspectratio=val;
      found_token=1;
    }// ASPECTRATIO
    if (tok=="ANTIALIASING"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.antialiasing=val;
      found_token=1;
    }// ANTIALIASING
    if (tok=="RAYDEPTH"){
      id=0;
      val=atof(GetNextVal(val_vec[i],id).c_str());
      rtp.raydepth=val;
      found_token=1;
    }// RAYDEPTH
    if (tok=="CENTER"){
      // Note that there are 6 values here.
      int id=0;
      for (int ic=0;ic<6;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.center_guide[ic]=val;
      }
      rtp.center_s=1;
      found_token=1;
    }// CENTER
    if (tok=="LIGHT"){
      /* This is a little confusing.  There can be multiple
	 LIGHT tokens.  Each time one is found a new LIGHT is
	 added to the rtparams object.  Note that there are 
	 7 values here.  3 for center, 1 for rad, 3 for color.
	 These values must be pushed back onto the correct vectors
	 and matrices.  However, for the first LIGHT token, one must
	 overwrite the default rather than add a new light.  
      */
      vector<double> lcen(3);
      double lrad;
      vector<double> lcolor(3);
      int id=0;
      for (int ic=0;ic<3;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	lcen[ic]=val;
      }
      val=atof(GetNextVal(val_vec[i],id).c_str());
      lrad=val;
      for (int ic=0;ic<3;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	lcolor[ic]=val;
      }
      if (first_light){
	first_light=0;
	rtp.lightcenter[0]=lcen;
	rtp.lightrad[0]=lrad;
	rtp.lightcolor[0]=lcolor;
      }
      else{
	rtp.lightcenter.push_back(lcen);
	rtp.lightrad.push_back(lrad);
	rtp.lightcolor.push_back(lcolor);
      }
      found_token=1;
    }// LIGHT
    if (tok=="BACKGROUND"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<3;ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.background[ic]=val;
      }
      found_token=1;
    }// BACKGROUND
    if (tok=="ATOMCOLOR"){
      // Note that there are 4 values here.
      int id=0;
      tmp = vector<double> (4);
      for (int ic=0;ic<tmp.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	tmp[ic]=val;
      }
      rtp.sphtex_color.push_back(tmp);
      found_token=1;
    }// ATOMCOLOR
    if (tok=="ATOMTEXTURE"){
      // Note that there are 5 values here.
      int id=0;
      tmp = vector<double> (5);
      for (int ic=0;ic<tmp.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	tmp[ic]=val;
      }
      rtp.sphtex_tex.push_back(tmp);
      found_token=1;
    }// ATOMTEXTURE
    if (tok=="ATOMRAD"){
      // Note that there are 2 values here.
      int id=0;
      tmp = vector<double> (2);
      for (int ic=0;ic<tmp.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.sph_rad.push_back(val);
      }
      found_token=1;
    }//ATOMRAD
    if (tok=="SHADING"){
      int id=0;
      rtp.shading = GetNextVal(val_vec[i],id);
      found_token=1;
    }// SHADING
    if (tok=="OUTFILE"){
      int id=0;
      rtp.outfile = GetNextVal(val_vec[i],id);
      found_token=1;
    }// OUTFILE
    if (tok=="SUPERCELL"){
      // Note that there are 9 values here.
      int id=0;
      for (int ic=0;ic<3;ic++){
	for (int jc=0;jc<3;jc++){
	  val=atof(GetNextVal(val_vec[i],id).c_str());
	  rtp.sc[ic][jc]=val;
	}
      }
      found_token=1;
      rtp.sc_s=1;
    }//SUPERCELL
    if (tok=="CALCTYPE"){
      int id=0;
      rtp.calc_type=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }//CALCTYPE
    if (tok=="INFILE"){
      int id=0;
      while (id<val_vec[i].size()){
	string s;
	s=GetNextVal(val_vec[i],id);
	if (s.size()>0) rtp.input_files.push_back(s);
      }
      found_token=1;
    }//INSERT_FILE
    if (tok=="INSERT_FILE"){
      int id=0;
      string s;
      s=GetNextVal(val_vec[i],id);
      rtp.insert_file=s;
      found_token=1;
    }//INSERT_FILE
    if (tok=="ROTATION"){
      int id=0;
      for (int ic=0;ic<6;ic++){
	string s=GetNextVal(val_vec[i],id);
	rtp.rotation[ic] = atof(s.c_str());
      //rtp.rotation[ic] = atof(GetNextVal(val_vec[ic],id).c_str());
      }
      found_token=1;
    }//ROTATION
    if (tok=="STRUCTURE_ORIGIN"){
      int id=0;
      for (int ic=0;ic<3;ic++){
	string s=GetNextVal(val_vec[i],id);
	rtp.struct_origin[ic] = atof(s.c_str());
      }
      rtp.struct_origin_s=1;
      found_token=1;
    }//STRUCTURE_ORIGIN
    if (tok=="PLANE"){
      int id=0;
      rtp.plane=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }//PLANE
    if (tok=="PLANECENTER"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<rtp.plane_center.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.plane_center[ic]=val;
      }
      found_token=1;
      rtp.updir_s=1;
    }// PLANECENTER
    if (tok=="PLANENORMAL"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<rtp.plane_normal.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.plane_normal[ic]=val;
      }
      found_token=1;
    }// PLANENORMAL
    if (tok=="PLANETEXTURE"){
      // Note that there are 4 values here.
      int id=0;
      for (int ic=0;ic<rtp.planetex_tex.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
	rtp.planetex_tex[ic]=val;
      }
      found_token=1;
    }// PLANETEXTURE
    if (tok=="PLANECOLOR"){
      // Note that there are 3 values here.
      int id=0;
      for (int ic=0;ic<rtp.plane_color.size();ic++){
	val=atof(GetNextVal(val_vec[i],id).c_str());
        rtp.plane_color[ic]=val;
      }
      found_token=1;
    }// PLANECOLOR


    if (!found_token){
      cerr << "ERROR: You have input a token " << tok <<endl;
      cerr << "ERROR: This token is not recognized. Exiting! " << endl;
      exit(1);
    }

  } // for i

}// end routine

/*
**************************************************
ReadInStrVec
**************************************************
*/
void ReadInStrVec(vector<structure> & str_vec, ifstream & strlist_inf)
{
//  int cnt=0;
  str_vec.clear();
  structure str;
  string dum;
  // Read in first structure.
  strlist_inf >> str;
//  cnt++;
//  cout << "Read in structure: " << cnt << endl;
  str_vec.push_back(str);
  // Read in all remaining structures.
  while (getline(strlist_inf,dum)){
    strlist_inf >> str;
//    cnt++;
//    cout << "Read in structure: " << cnt << endl;
    str_vec.push_back(str);
  }
}

/*
**************************************************
JoinStrVec
**************************************************
  Note that this can be made more memory efficient
quite easily if that is needed.
*/
void JoinStrVec(vector<structure> str_vec_1,
		vector<structure> str_vec_2,
		vector<structure> & str_vec_tot)
{
  //  cout << "In JOIN" << endl;
  //  cout << "strlist1 size " << str_vec_1.size();
  //  cout << "strlist2 size " << str_vec_2.size();
  int size1=str_vec_1.size();
  int size2=str_vec_2.size();
  if (size1<size2){// Pad str_vec_1
    structure str=str_vec_1[size1-1];
    for (int is=size1;is<size2;is++){
      str_vec_1.push_back(str);
    }
  }
  if (size2<size1){// Pad str_vec_2
    structure str=str_vec_2[size2-1];
    for (int is=size2;is<size1;is++){
      str_vec_2.push_back(str);
    }
  }
  for (int is=0;is<size1;is++){
    structure str1=str_vec_1[is];
    structure str2=str_vec_2[is];
    matrix<double> lat1=str1.GetLat();
    matrix<double> cpos2=str2.GetCpos();
    vector<int> num_each_type_1=str1.GetNumEachType();
    int num_types_1=num_each_type_1.size();
    vector<int> num_each_type_2=str2.GetNumEachType();
    int num_types_2=num_each_type_2.size();
    // Loop over str2 atoms, assign them a type and c/d positions in an atom.
    int cnt=0;
    for (int it=0;it<num_types_2;it++){
      for (int ia=0;ia<num_each_type_2[it];ia++){
	atom a;
	a.SetCpos(cpos2[cnt]);
	a.SetDpos(vecC2D(lat1,cpos2[cnt]));
	a.SetType(num_types_1+it);
	str1.AddAtom(a);
	cnt++;
      }
    }
    str_vec_tot.push_back(str1);
  }//is
}//end routine


/*
**************************************************
SetStrFromRTParams
**************************************************
  This assumes you have read in the rtparams with
SetRTParams.  It uses both a structure and the rtparams
and alters the structure based on rtparams.  Here the
structure is made into a supercell if needed and the 
struc_origin is set.
*/
void SetStrFromRTParams(structure & str, rtparams & rtp)
{
  // Make structral adjustments
  if (rtp.sc_s){
    // Make a supercell if it was set by user.  Note that this moves all atoms into the unit cell.
    str=GetSuperCell(rtp.sc,str);
    // Reset rtparams now that you have a supercell.
    SetRTParams(str,rtp);
  }
  str.SetOrigin(rtp.struct_origin);
}

/*
**************************************************
SuperCellStrVec
**************************************************
  Gets supercells for every strucutre in the structure
vector.
*/
void SuperCellStrVec(vector<structure> & str_vec, const matrix<double> & sc)
{
  for (int is=0;is<str_vec.size();is++){
    str_vec[is]=GetSuperCell(sc,str_vec[is]); // Makes a supercell.
  }
}

/* 
**************************************************
UpDateRTParams
*************************************************
  This simply makes changes to rtparams as the program steps
through different frames.  This is use for values that must
evolve during the movie (e.g., the center).
*/
void UpDateRTParams(rtparams & rtp, const int & istr, int nstr)
{
  // Update center
  // tpx tpx ??? (cut time to 1/2 movie for moving center by /2).
  //nstr=(int) (nstr/1.5);
  double r=0;
  if (nstr>1){
    r = (double)istr/(double)(nstr-1);
  }
  if (r>1) r=1;
  if (rtp.center_s){
    for (int ic=0;ic<3;ic++){
      rtp.center[ic]=rtp.center_guide[2*ic]+r*(rtp.center_guide[2*ic+1]-rtp.center_guide[2*ic]);
    }
  }
  //tpx
  cout << "CENTER " ;
  Vout(rtp.center,cout);

}


/* 
**************************************************
SetRTParams
**************************************************
  This assumes you have read in the rtparams with
SetRTParams.  It uses both a structure and the rtparams
and alters rtparams based on what has been read in and the
structure.  The rtparam defaults are set based on the 
ntypes and other structural characteristics.
*/
void SetRTParams(structure & str, rtparams & rtp)
{

  // Now set all defaults for rtp that 
  // use structural information.
  str.ReScale(1);
  matrix<double> lat=str.GetLat();
  matrix<double> cpos=str.GetCpos();
  int nat=cpos.size();

  // For all items that must be set for each type we do that here.
  // These include sphtex_* and sph_rad.
  // Note that we protect against reading in data for atom types that do not exist.
  // Also note that this makes of unique formats for the rtparam
  // data that only exist the when they are read in.  Therefore,
  // we must only do this the first time this set function is 
  // called.  This is controlled by the first_set parameter.
  if (rtp.first_set){
    int ntypes = str.GetNumEachType().size();
    matrix<double> tmp;
    int size;
    // Set sphtex_tex
    size=rtp.sphtex_tex_def.size();
    tmp=matrix<double> (ntypes,rtp.sphtex_tex_def);
    for (int i=0;i<rtp.sphtex_tex.size();i++){
      int id=(int)rtp.sphtex_tex[i][0]-1;
      for (int j=1;j<size+1;j++){
	if (id>=0 && id<ntypes) tmp[id][j-1]=rtp.sphtex_tex[i][j];
      }
    }
    rtp.sphtex_tex=tmp;
    // Set sphtex_color
    size=rtp.sphtex_color_def.size();
    tmp=matrix<double> (ntypes,rtp.sphtex_color_def);
    for (int i=0;i<rtp.sphtex_color.size();i++){
      int id=(int)rtp.sphtex_color[i][0]-1;
      for (int j=1;j<size+1;j++){
	if (id>=0 && id<ntypes) tmp[id][j-1]=rtp.sphtex_color[i][j];
      }
    }
    rtp.sphtex_color=tmp;
    // Set sphtex_phong
    size=rtp.sphtex_phong_def.size();
    tmp=matrix<double> (ntypes,rtp.sphtex_phong_def);
    for (int i=0;i<rtp.sphtex_phong.size();i++){
      int id=(int)rtp.sphtex_phong[i][0]-1;
      for (int j=1;j<size+1;j++){
	if (id>=0 && id<ntypes) tmp[id][j-1]=rtp.sphtex_phong[i][j];
      }
    }
    rtp.sphtex_phong=tmp;
    // Set sph_rad 
    size=1;
    vector<double> vtmp(ntypes,rtp.sph_rad_def);
    for (int i=0;i<rtp.sph_rad.size()/2;i++){
      int id=(int)rtp.sph_rad[2*i]-1;
      if (id>=0 && id<ntypes) vtmp[id]=rtp.sph_rad[2*i+1];
    }
    rtp.sph_rad=vtmp;
    // Set sphtex_names
    rtp.sphtex_names = vector<string> (ntypes,"txt_atom_type_");
    for (int it=0;it<ntypes;it++){
      ostringstream os;
      os << it+1 << ends;
      string s(os.str());
      rtp.sphtex_names[it]=rtp.sphtex_names[it]+s;
    }
    rtp.first_set=0;
  }// if first_set    
 
  //  Set view direction along the 0 axis.
  //  if (!rtp.viewdir_s) {rtp.viewdir=lat[0];}

  //  Set up direction along the 1 axis.
  //  if (!rtp.updir_s) {rtp.updir=lat[0];}

  /*
    Set up struct_origin, which is the point around which rotation
    will take place.  Here we default to the first moment of the atom
    positions.
  */
  if (! rtp.struct_origin_s){// If struct_orig was not set by user
    // rtp.struct_origin=SVprod(0.5,VVsum(lat[0],VVsum(lat[1],lat[2])));
    vector<double> mom1=str.GetMom1();
    rtp.struct_origin=mom1;
  }

  /*
    Set up center (where camera is located).
    Define the center to be displaced from the centroid along the view
    direction. Define center to be located a distance d=-2.0*R/tan(45)=-2.0*R
    from the centroid, where R is the the largest projected distance 
    of an atom from the centroid (first moment) in the image plane (the 
    plane defined to be perpendicular to the view direction and containing the 
    centroid).  So define 
    mom1=first moment
    vd=unit vector along view direction
    up=unit vector along up direction
    upp=unit vector consisting of projection of up vector into the image plane.
    vdp=unit vector in image plane perpendicular to vd and updir.

    Actually, instead of mom1 being the centroid just set it to the 
    struct_origin.  All else is the same.
  */
  if (! rtp.center_s){// If center was not set by user
    vector<double> mom1=rtp.struct_origin;
    vector<double> vd=SVprod(1.0/norm(rtp.viewdir),rtp.viewdir);
    vector<double> up=SVprod(1.0/norm(rtp.updir),rtp.updir);
    vector<double> upp;
    upp=VVsum(up,SVprod(-VVprod(up,vd),vd));
    upp=SVprod(1.0/norm(upp),upp);
    vector<double> vdp=VVcross(vd,upp);
    double proj=0;
    double max_iat=0;
    for (int iat=0;iat<nat;iat++){
      double p1=VVprod(VVdiff(cpos[iat],mom1),vdp);
      double p2=VVprod(VVdiff(cpos[iat],mom1),upp);
      double nproj=sqrt(p1*p1+p2*p2);
      if (nproj>proj){
	proj=nproj;
	max_iat=iat;
      }
    }
    double displ=-2.0*proj;
    rtp.center=VVsum(mom1,SVprod(displ,vd));
  }// if ! center_s
  else{ // if center is set by user
    for (int ic=0;ic<3;ic++){
      rtp.center[ic]=rtp.center_guide[2*ic];
    }
  }
}



// **************************************************
// GetRTDatFile
// **************************************************
void GetRTDatFile(structure str, const rtparams & rtp,
			ostringstream & rtdat_file)
{
  rtdat_file << "BEGIN_SCENE" << endl;
  rtdat_file << "  RESOLUTION " << rtp.resx << " " << rtp.resy << endl;
  /*
  string tga="tga";
  string outfile;
  outfile=rtp.outfile+"_"+tga;
  rtdat_file << "  OUTFILE " << outfile << endl;
  */
  rtdat_file << endl;
  rtdat_file << "CAMERA" << endl;
  rtdat_file << "  ZOOM " << rtp.zoom << endl;
  rtdat_file << "  ASPECTRATIO " << rtp.aspectratio <<  endl;
  rtdat_file << "  ANTIALIASING " << rtp.antialiasing << endl;
  rtdat_file << "  RAYDEPTH " << rtp.raydepth << endl;
  rtdat_file << "  CENTER " << rtp.center[0] << " " << rtp.center[1] << " " << rtp.center[2] << endl;
  rtdat_file << "  VIEWDIR " << rtp.viewdir[0] << " " << rtp.viewdir[1] << " " << rtp.viewdir[2] << endl;
  rtdat_file << "  UPDIR " << rtp.updir[0] << " " << rtp.updir[1] << " " << rtp.updir[2] << endl;
  rtdat_file <<  endl;
  rtdat_file << "END_CAMERA" << endl;
  rtdat_file <<  endl;
  rtdat_file << "BACKGROUND " << rtp.background[0] << " " << rtp.background[1] << " " << rtp.background[2] << endl;
  rtdat_file <<  endl;
  for (int il=0;il<rtp.lightcenter.size();il++){
    rtdat_file << "LIGHT ";
    rtdat_file << "CENTER " << rtp.lightcenter[il][0] << " " << rtp.lightcenter[il][1] << " " << rtp.lightcenter[il][2];
    rtdat_file << " RAD " << rtp.lightrad[il];
    rtdat_file << " COLOR " << rtp.lightcolor[il][0] << " " << rtp.lightcolor[il][1] << " " << rtp.lightcolor[il][2];
    rtdat_file <<  endl << endl;
  }
  for (int iat=0;iat<rtp.sphtex_names.size();iat++){
    rtdat_file << "TEXDEF " << rtp.sphtex_names[iat] << " ";
    rtdat_file << "AMBIENT " << rtp.sphtex_tex[iat][0] << " ";
    rtdat_file << "DIFFUSE " << rtp.sphtex_tex[iat][1] << " ";
    rtdat_file << "SPECULAR " << rtp.sphtex_tex[iat][2] << " ";
    rtdat_file << "OPACITY " << rtp.sphtex_tex[iat][3] << " ";
    rtdat_file << endl;
    rtdat_file << "PHONG PLASTIC " << rtp.sphtex_phong[iat][0] << " " << "PHONG_SIZE " << rtp.sphtex_phong[iat][1] << endl;
    rtdat_file << "  COLOR " << rtp.sphtex_color[iat][0] << " " << rtp.sphtex_color[iat][1] << " " << rtp.sphtex_color[iat][2] << endl;
    rtdat_file << "  TEXFUNC 0" << endl;
    rtdat_file << endl;
  }
    // Do atoms
  str.ReScale(1);
  matrix<double> cpos=str.GetCpos();
  vector<int> num_each_type=str.GetNumEachType();
  int nat=cpos.size();
  int ntype=num_each_type.size();
  int cnt=0;
  for (int it=0;it<ntype;it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      rtdat_file << "SPHERE CENTER " << cpos[cnt][0] << " " << cpos[cnt][1] << " " << cpos[cnt][2];
      rtdat_file << " RAD " << rtp.sph_rad[it];
      rtdat_file << " " << rtp.sphtex_names[it] << endl;
      rtdat_file << endl;
      cnt++;
    }
  }

  // Do plane
  if (rtp.plane==1){
    rtdat_file << "PLANE" << endl;
    rtdat_file << "  CENTER " << rtp.plane_center[0] << " " << rtp.plane_center[1] << " " << rtp.plane_center[2] << endl;
    rtdat_file << "  NORMAL " << rtp.plane_center[0] << " " << rtp.plane_center[1] << " " << rtp.plane_center[2] << endl;
    rtdat_file << "  TEXTURE " << endl; 
    rtdat_file << "    AMBIENT " << rtp.planetex_tex[0] << " ";
    rtdat_file << "DIFFUSE " << rtp.planetex_tex[1] << " ";
    rtdat_file << "SPECULAR " << rtp.planetex_tex[2] << " ";
    rtdat_file << "OPACITY " << rtp.planetex_tex[3] << " ";
    rtdat_file << "  COLOR " << rtp.plane_color[0] << " " << rtp.plane_color[1] << " " << rtp.plane_color[2] << endl;
    rtdat_file << "  TEXFUNC 0" << endl;
    cout << endl;
  }

  rtdat_file << "END_SCENE" << endl;
  rtdat_file << ends;
}


// **************************************************
// PrintRDatFile
// **************************************************
string PrintRTDatFile(ostringstream & rtdat_file, const rtparams & rt_params)
{  
  string filename = rt_params.outfile+".dat";
  ostringstream outss;
  outss << filename << ends;
  ofstream outf(outss.str().c_str());
  outf << rtdat_file.str();
  return filename;
} 


// **************************************************
// CreateRTtgaFile
// **************************************************
string CreateRTtgaFile(const string & datfile, const rtparams & rt_params)
{  
  // Run tachyon on datfile.
  ostringstream tachyon_stream;
  tachyon_stream << "tachyon " << "-" << rt_params.shading << " " << datfile << ends;
/*  char* tachyon_char = tachyon_stream.str().c_str();
  system(tachyon_char);*/
  system(tachyon_stream.str().c_str());
  // Move outfile.tga to user defined name.
  ostringstream mv_stream;
  string tgafile = rt_params.outfile+".tga";
  mv_stream << "mv outfile.tga " << tgafile << ends;
/*  char* mv_char = mv_stream.str().c_str();
  system(mv_char);*/
  system(mv_stream.str().c_str());
  return tgafile;
} 


// **************************************************
// CreateRTjpgFile
// **************************************************
string CreateRTjpgFile(const string & tgafile, const rtparams & rt_params)
{  
  // Run convert.
  ostringstream convert_stream;
  string jpgfile = rt_params.outfile+".jpg";
  convert_stream << "convert " << tgafile << " " << jpgfile << ends;
  system(convert_stream.str().c_str());
  return jpgfile;
} 

/*
**************************************************
GetRTencFile
**************************************************
*/
void GetRTencFile(const rtparams & rtp, const int nim,
                  ostringstream & os)
{
  os << "PATTERN         IBBPBBPBBPBBPBB" << endl;
  os << "OUTPUT          "<< rtp.outfile << ".mpg" << endl;
  os << "GOP_SIZE        16" << endl;
  os << "SLICES_PER_FRAME        1" << endl;
  os << "BASE_FILE_FORMAT        PPM" << endl;
  os << endl;
  os << "INPUT_CONVERT   djpeg *" << endl;
  os << "INPUT_DIR       ." << endl;
  os << "INPUT" << endl;
  os << rtp.outfile << "_*.jpg       [" << PaddedNumString(0,5) << "-" << nim-1 << "+1]" << endl; 
  os << "END_INPUT" << endl;
  os << endl;
  os << "IQSCALE         8" << endl;
  os << "PQSCALE         10" << endl;
  os << "BQSCALE         25" << endl;
  os << "PIXEL           HALF" << endl;
  os << "RANGE           10" << endl;
  os << "PSEARCH_ALG     LOGARITHMIC" << endl;
  os << "BSEARCH_ALG     CROSS2" << endl;
  os << "REFERENCE_FRAME DECODED" << endl;
  os << ends;
}


/*
**************************************************
PrintRTencFile
**************************************************
*/
string PrintRTencFile(const rtparams & rt_params, ostringstream & rtenc_file)
{
  string filename = rt_params.outfile+".enc";
  ostringstream outss;
  outss << filename << ends;
  ofstream outf(outss.str().c_str());
  outf << rtenc_file.str();
  return filename;
} 

/*
**************************************************
CreateRTmgpFile
**************************************************
*/
string CreateRTmpgFile(const rtparams & rt_params, const string & encfile)
{
  // Run mpeg_encoder on enc_file.
  ostringstream encoder_stream;
  encoder_stream << "mpeg_encode " << " " << encfile << ends;
/*  char* encoder_char = encoder_stream.str().c_str();
  system(encoder_char);*/
  system(encoder_stream.str().c_str());
  // Get outfile.mpg name
  string file;
  file = rt_params.outfile+".mpg";
  return file;

}





/* 
**************************************************
RayTraceManager
**************************************************
*/
void RayTraceManager(char * argv[])
{
  ifstream rtinfile(argv[2]); // File where RT params are input.
  InFileExistCheck("RayTraceFuncs.cc/RayTraceManager",argv[2],rtinfile,cerr);
  rtparams rtp; // Object that stores RT params.
  ReadInRTParams(rtinfile,rtp); // Sets RT params object from input.
  switch (rtp.calc_type){
  case 0:{ // A single file of structures.
    // Read in structure list.
    if (rtp.input_files.size()<1){
      cerr << "ERROR: RayTraceManager" << endl;
      cerr << "ERROR: You must specify a structure list file to open with the token INFILE." << endl;
      cerr << "ERROR: For example:  INFILE = STRLIST ." << endl;
      cerr << "ERROR: Exiting." << endl;
      exit(1);
    }
    ifstream strlist_inf(rtp.input_files[0].c_str());
    InFileExistCheck("RayTraceFuncs.cc/RayTraceManager",rtp.input_files[0].c_str(),strlist_inf,cerr);
    vector<structure> str_vec;
    cout << endl;
    cout << "Reading in structure list." << endl;
    cout << endl;
    ReadInStrVec(str_vec,strlist_inf);
    int nstr=str_vec.size();
    cout << "We are working with this many structures: " << nstr << endl;
    // Set rtparams.
    SetRTParams(str_vec[0],rtp);
    // Set structures from rtparams.
    for (int is=0;is<nstr;is++){
      SetStrFromRTParams(str_vec[is],rtp);
    }
    cout << endl;
    cout << "Here are the values of the ray trace parameters." << endl;
    rtp.Print();
    cout << endl;
    // Rotate str_vec
    RotateStrVec(str_vec,rtp.rotation);
    // Create dat,tga,jpg files.
    string datfile,tgafile,jpgfile;
    for (int is=0;is<nstr;is++){
      cout << endl;
      cout << "Creating dat/tga/jpg file num " << is+1 << " out of a total of " << nstr << endl;
      cout << endl;
      // Update rtparams
      UpDateRTParams(rtp,is,nstr);
      // Get dat file stringstream.
      ostringstream rtdat_file;
      structure str=str_vec[is];
      GetRTDatFile(str,rtp,rtdat_file); // Puts formatted .dat file into a stringstream.
      datfile=PrintRTDatFile(rtdat_file,rtp); // Prints outfile.dat file and returns name.
      tgafile=CreateRTtgaFile(datfile,rtp); // Creates outfile.tga file and returns name.
      jpgfile=CreateRTjpgFile(tgafile,rtp); // Creates outfile.jpg file and returns name.
      // Copy files to proper names (outfile.num.suffix).
      ostringstream mvcmd_dat;
      ostringstream mvcmd_tga;
      ostringstream mvcmd_jpg;
      string name;
      string num = PaddedNumString(is,5);
      string datname = (rtp.outfile+"_"+num+".dat");
      mvcmd_dat << "/bin/mv " << datfile << " " << datname << ends;
      system(mvcmd_dat.str().c_str());
      string tganame = (rtp.outfile+"_"+num+".tga");
      mvcmd_tga << "/bin/mv " << tgafile << " " << tganame << ends;
      system(mvcmd_tga.str().c_str());
      string jpgname = (rtp.outfile+"_"+num+".jpg");
      mvcmd_jpg << "/bin/mv " << jpgfile << " " << jpgname << ends;
      system(mvcmd_jpg.str().c_str());
      if (nstr>1){ // Remove dat,tga files if there are a lot of them.
	ostringstream rmcmd_dat;
	ostringstream rmcmd_tga;
	rmcmd_dat << "/bin/rm " << datname << ends;
	system(rmcmd_dat.str().c_str());
	//	rmcmd_tga << "/bin/rm " << tganame << ends;
	//	system(rmcmd_tga.str());
      }
    }
    // Get and print enc file for mpeg encoder.
    ostringstream rtenc_file;
    GetRTencFile(rtp,nstr,rtenc_file);
    string encfile;
    encfile=PrintRTencFile(rtp,rtenc_file);
    // Create mpg file with mpeg encoder.
    cout << endl;
    cout << "Creating mpg file " << rtp.outfile << ".mpg" << endl;
    cout << endl;
    string mpgfile;
    mpgfile=CreateRTmpgFile(rtp,encfile);
    break;
  }
  default:{
    cout << endl;
    cerr << "ERROR: RTManager" <<endl;
    cerr << "ERROR: You have set CALCTYPE= " << rtp.calc_type << endl;
    cerr << "ERROR: This does not correspond to any function. "<< endl;
    cerr << "ERROR: exiting" << endl;
    cout << endl;
    exit(1);
  }
  }//switch
}

/*
**************************************************
GetRotationMatrix
**************************************************
  This gets a rotation matrix from 3 angles assumed
to represent a rotation around x, then y, then z.
Angles are assumed to be in radians.
*/
matrix<double> GetRotationMatrix(const vector<double> & angles) 
{
  // Sin and cos.
  vector<double> sn(3,0.0);
  vector<double> cs(3,0.0);
  for (int ic=0;ic<3;ic++){
    sn[ic]=sin(angles[ic]);
    cs[ic]=cos(angles[ic]);
  }
  // Set rotation matrix (do x, then y, then z rotation).
  matrix<double> xm(3,3,0.0);
  matrix<double> ym(3,3,0.0);
  matrix<double> zm(3,3,0.0);

  xm[0][0]=1;
  xm[1][1]=cs[0];
  xm[1][2]=-sn[0];
  xm[2][1]=sn[0];
  xm[2][2]=cs[0];

  ym[0][0]=cs[1];
  ym[0][2]=sn[1];
  ym[1][1]=1;
  ym[2][0]=-sn[1];
  ym[2][2]=cs[1];

  zm[0][0]=cs[2];
  zm[0][1]=-sn[2];
  zm[1][0]=sn[2];
  zm[1][1]=cs[2];
  zm[2][2]=1;

  matrix<double> rm;
  rm=MMmult(zm,MMmult(ym,xm));
  return rm;
}

/*
**************************************************
RotateStrVec
**************************************************
  This rotates each structure in the str_vec.
The rotation goes from an initial to a final set of angles
in steps of the primitive rotation.
The primitive rotation is the rotation around x, y, then z 
by the amount of change given in rot divided by the number 
of structures - 1.  All frames are initially rotated 
according to the initial rotation angles.  The rotation is 
then done as one primitive rotation per frame.
*/
void RotateStrVec(vector<structure> & str_vec, const vector<double> & rot)
{
  // Get initial rotation matrix.
  vector<double> angles(3);
  for (int ic=0;ic<3;ic++){
    angles[ic]=rot[2*ic];
    angles[ic]=angles[ic]*TWOPI/360.0;
  }
  matrix<double> irm=GetRotationMatrix(angles);
  
  // get primitive rotation matrix.
  int s=str_vec.size()-1;
  if (s<1) s=1;
  for (int ic=0;ic<3;ic++){
    angles[ic]=(rot[2*ic+1]-rot[2*ic])/(s);
    angles[ic]=angles[ic]*TWOPI/360.0;
  }
  matrix<double> prm=GetRotationMatrix(angles);

  matrix<double> rm=irm;
  for (int is=0;is<str_vec.size();is++){
    str_vec[is].Rotate(rm);
    rm=MMmult(prm,rm);
  }
}

