// **************************************************
// CellReduceFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <algorithm>
#include "UsefulFuncs.h"
#include "GaussJordan.h"
#include "constants.h"
#include "CellReduceFuncs.h"



// **************************************************
// SDebug
// **************************************************
// Prints stuff for debugging
void sdebug(const int & step, const int & iter, 
	   const double & a, const double & b,
	   const double & c, const double & ksi,
	   const double & eta, const double & zeta,
	   ostream & sout)
{
  // tpx
//    sout << "Step: " << step << endl;
//    cout << "Iter: " << iter << endl;
//    cout << "a,b,c,ksi,eta,zeta: " <<a<<" "<<b<<" "<<c
//         <<" "<<ksi<<" "<<eta<<" "<<zeta<<endl;
//    cout << "sum a+b+c: " << a+b+c << endl;
}



// **************************************************
// GetNiggliCell
// **************************************************
// Calculates the reduced Niggli cell.  It is based on 
// a program of Eric Wu's.  Here is his + my documentation.
/* 
!! FUNCTION:
!! 
!!   Subroutine calculates the reduced cell (Niggli cell) for a given 
!!   primitive cell.  It also gives the transformation matrix to go from 
!!   the primitive cell to the Niggli cell.
!!
!! REFERENCES:
!!
!!    Y. Lepage, J. Appl. Cryst 15 255-229 (1982) and           
!!    Y. Lepage, J. Appl. Cryst 20 264-269 (1987)    
!!    International tables for crystallography Ch 5,9
!!    I Krivy, B. Gruber. Acta Cryst 1976 A32 297 (1975) 
!!    W. Clegg., Acta Cryst A37 913-915 (1981)
!!    A.D. Mighell J. Appl Cryst 9 491-498 (1976)  
!!
!! METHODOLOGY:
!!
!!   The Niggli cell is UNIQUE and reduced. A Niggli cell is a Buerger cell 
!!   (shortest lattice vectors) but  not necessarily the other way around, 
!!   since the Buerger cell is not unique.  This info is useful because with 
!!   the Niggli cell one knows instantly the type of bravais lattice 
!!   and how to go to the conventional cell if you apply the correct 
!!   algorithm.  There are 2 ways to do this.  The first is the straight 
!!   way -  see International Tables Ch. 9 .  The other involves counting 
!!   the # of 2fold axes and going from there, see the Lepage and Clegg 
!!   references.  This second method is more useful if you want to find     
!!   more than just bravais lattice (symmetry elements or space group 
!!   for example).  The subroutine assumes you input a primitive cell 
!!   and it spits out the Niggli cell and the transformation matrix   
!!   to go from the primitive cell to Niggli cell.   
!!   NB! There is a typo in step 3 of Krivy and Gruber that I corrected.
!!   [ (ksi*eta*ksi) should be (ksi*eta*zeta) in first part of Eq. 3. ]
!!    
!! INPUT:
!!    
!!    in_lat: a primitive set of lattice vectors, 
!!            in_lat(1,1)=a_x
!!            in_lat(1,2)=a_y 
!!            in_lat(2,1)=b_x  ...
!!
!! OUTPUT:
!! 
!!    niggli_lat - reduced (Niggli) cell, same format           
!!    P - Matrix to go from Primitive to Niggli cell    
!!    Q - P^-1;
!! 
!!    For more info of the definitions here see Int. Tables Cryst. ch.5
!!    Let nlat be the niggli lattice, where the first column is a, 
!!    second b, third c.  Let lat be the original cell equivalent.  
!!    The P matrix to transform cells is generally defined by 
!!                 nlat = lat * P
!!    We will follow this convention for defining P and Q.  
!!    Note that is the rest of the code, we define lattices with 
!!    rows equal to a, b, and c.  Therefore, under our conventional 
!!    definitions of nlat, call it nlat_c=nlat^T, and lat, call 
!!    it lat_c=lat^T, we have
!!                 nlat_c = P^T * lat_c
!!    where ^T means the transpose.
*/

void GetNiggliCell(const matrix<double> & in_lat, 
		   matrix<double> & niggli_lat, 
		   matrix<double> & P, matrix<double> & Q)
{

  double RENORM=1E+6;
  int MAXITER=100000;
  double TOL=1e-14;

  // Initialize matrices for tranformations (3x3).
  matrix<double> m1(3,3,0.0);
  m1[0][1]=1.0; m1[1][0]=1.0; m1[2][2]=-1.0;
  matrix<double> m2(3,3,0.0);
  m2[0][0]=-1.0; m2[1][2]=1.0; m2[2][1]=1.0;
  matrix<double> m3(3,3,0.0);
  // Els 1,1 2,2 3,3 are changed later for m3
  matrix<double> m4(3,3,0.0);
  // Els 1,1 2,2 3,3 are changed later for m4
  matrix<double> m5(3,3,0.0);
  m5[0][0]=1.0; m5[1][1]=1.0; m5[2][2]=1.0;
  // El 3,2 is changed later for m5
  matrix<double> m6(3,3,0.0);
  m6[0][0]=1.0; m6[1][1]=1.0; m6[2][2]=1.0;
  // El 3,1 is changed later for m6
  matrix<double> m7(3,3,0.0);
  m7[0][0]=1.0; m7[1][1]=1.0; m7[2][2]=1.0;
  // El 2,1 is changed later for m7
  matrix<double> m8(3,3,0.0);
  m8[0][0]=1.0; m8[1][1]=1.0; m8[0][2]=1.0; m8[1][2]=1.0; m8[2][2]=1.0;


  // Initialize a, b, c, ksi, eta, zeta
  vector<double> indat = Getabc_angles(in_lat);
  double a=indat[0]*indat[0];
  double b=indat[1]*indat[1];
  double c=indat[2]*indat[2];
  double ksi=2.0*indat[1]*indat[2]*cos(indat[3]);
  double eta=2.0*indat[0]*indat[2]*cos(indat[4]);
  double zeta=2.0*indat[0]*indat[1]*cos(indat[5]);
  // Round to RENORM decimal place to eliminate numerical errors
//    double a=Nint(indat[0]*indat[0]*RENORM);
//    double b=Nint(indat[1]*indat[1]*RENORM);
//    double c=Nint(indat[2]*indat[2]*RENORM);
//    double ksi=Nint(2.0*indat[1]*indat[2]*cos(indat[3])*RENORM);
//    double eta=Nint(2.0*indat[0]*indat[2]*cos(indat[4])*RENORM);
//    double zeta=Nint(2.0*indat[0]*indat[1]*cos(indat[5])*RENORM);

  // Dummy variables
  double temp,temp1,temp2,temp3;

  // Initialize tranformation matrix
  P = matrix<double> (3,3,0.0);
  P[0][0]=1; P[1][1]=1; P[2][2]=1;       

  int cnt=0;
  // tpx
  sdebug(0,0,a,b,c,ksi,eta,zeta,cout);

  // Start loop

 LoopHead:

    cnt++;
    if (cnt>MAXITER){
      cout << "ERROR: CellReduceFuncs/GetNiggliCell" << endl;
      cout << "ERROR: Number of interations greater the MAXITER = " << MAXITER << endl;
      cout << "ERROR: This seems like too many - there is probably some problem." << endl;
      cout << "ERROR: Exiting." << endl;
      exit(1);
    }

  // Step 1
    if (((a-b)>TOL) || ((abs(a-b)<TOL) && (abs(ksi)>abs(eta)) ) ) {
      temp=a;
      a=b;
      b=temp;
      temp=-ksi;
      ksi=-eta;
      eta=temp;
      P= MMmult(P,m1);
      // tpx
      sdebug(1,cnt,a,b,c,ksi,eta,zeta,cout);

    }
 
// Step 2
    if (((b-c)>TOL) || ((abs(b-c)<TOL) && (abs(eta)>abs(zeta)) ) ) {
      temp=b;
      b=c;
      c=temp;
      temp=-eta;
      eta=-zeta;
      zeta=temp;
      P=MMmult(P,m2);

      // tpx
      sdebug(2,cnt,a,b,c,ksi,eta,zeta,cout);

      goto LoopHead;
    }

// Step 3
    if ((ksi*eta*zeta)>0.0) {
      m3[0][0]=SignNoZero(ksi);
      m3[1][1]=SignNoZero(eta);
      m3[2][2]=SignNoZero(zeta);
      P=MMmult(P,m3);
      ksi=abs(ksi);
      eta=abs(eta);
      zeta=abs(zeta);

      // tpx
      sdebug(3,cnt,a,b,c,ksi,eta,zeta,cout);

    }

// Step 4 
    // Eric's original    if ((abs(ksi)<TOL) || (abs(eta)<TOL) || (abs(zeta)<TOL) || (ksi*eta*zeta<=0)) {
    if (ksi*eta*zeta<=0) {
      m4[0][0]=-(SignNoZero(ksi));
      m4[1][1]=-(SignNoZero(eta));
      m4[2][2]=-(SignNoZero(zeta));
      if (abs(ksi)<TOL) m4[0][0]=m4[1][1]*m4[2][2];
      if (abs(eta)<TOL) m4[1][1]=m4[0][0]*m4[2][2];
      if (abs(zeta)<TOL) m4[2][2]=m4[0][0]*m4[1][1];
      P=MMmult(P,m4);
      ksi=-abs(ksi);
      eta=-abs(eta);
      zeta=-abs(zeta);

      // tpx
      sdebug(4,cnt,a,b,c,ksi,eta,zeta,cout);

    }

// Step 5
    if (((abs(ksi)-b)>=TOL) || 
        ((abs(ksi-b)<TOL) && (2.0*eta<zeta)) || 
	((abs(ksi+b)<TOL) && (zeta<0)) ) { 
      m5[1][2]=-(SignNoZero(ksi));
      P=MMmult(P,m5);
      temp1=b+c-ksi*SignNoZero(ksi);
      temp2=eta-zeta*SignNoZero(ksi);
      temp3=ksi-2.0*b*SignNoZero(ksi);
      c=temp1;
      eta=temp2;
      ksi=temp3;

      // tpx
      sdebug(5,cnt,a,b,c,ksi,eta,zeta,cout);

      goto LoopHead;
    }


// Step 6
    if (((abs(eta)-a)>TOL) || ((abs(eta-a)<TOL) && (2.0*ksi<zeta)) || 
	((abs(eta+a)<TOL) && (zeta<0))) {
      m6[0][2]=-SignNoZero(eta);
      P=MMmult(P,m6);
      temp1=a+c-eta*SignNoZero(eta);
      temp2=ksi-zeta*SignNoZero(eta);
      temp3=eta-2.0*a*SignNoZero(eta);
      c=temp1;
      ksi=temp2;
      eta=temp3;

      // tpx
      sdebug(6,cnt,a,b,c,ksi,eta,zeta,cout);

    }

// Step 7
    if (((abs(zeta)-a)>TOL) || ((abs(zeta-a)<TOL) && (2.0*ksi<eta)) || 
	((abs(zeta+a)<TOL) && (eta<0))) {
      m7[0][1]=-SignNoZero(zeta);
      P=MMmult(P,m7);
      temp1=a+b-zeta*SignNoZero(zeta);
      temp2=ksi-eta*SignNoZero(zeta);
      temp3=zeta-2.0*a*SignNoZero(zeta);
      b=temp1;
      ksi=temp2;
      zeta=temp3;

      // tpx
      sdebug(7,cnt,a,b,c,ksi,eta,zeta,cout);

      goto LoopHead;
    }

// Step 8
    if ((ksi+eta+zeta+a+b<0) || 
	((ksi+eta+zeta+a+b<0) && (2.0*(a+eta)+zeta>0))) {
      P=MMmult(P,m8);
      temp1=a+b+c+ksi+eta+zeta;
      temp2=2.0*b+ksi+zeta;
      temp3=2.0*a+eta+zeta;
      c=temp1;
      ksi=temp2;
      eta=temp3; 

      // tpx
      sdebug(8,cnt,a,b,c,ksi,eta,zeta,cout);

      goto LoopHead;
    }

    // Renormalize back to regular cell (divide by RENORM)
    vector<double> outdat(6,0.0);
//      outdat[0]=sqrt(a/RENORM);
//      outdat[1]=sqrt(b/RENORM);
//      outdat[2]=sqrt(c/RENORM);
//      outdat[3]=acos(ksi/RENORM/2.0/outdat[1]/outdat[2]);
//      outdat[4]=acos(eta/RENORM/2.0/outdat[0]/outdat[2]);
//      outdat[5]=acos(zeta/RENORM/2.0/outdat[0]/outdat[1]);
    outdat[0]=sqrt(a);
    outdat[1]=sqrt(b);
    outdat[2]=sqrt(c);
    outdat[3]=acos(ksi/2.0/outdat[1]/outdat[2]);
    outdat[4]=acos(eta/2.0/outdat[0]/outdat[2]);
    outdat[5]=acos(zeta/2.0/outdat[0]/outdat[1]);
    
    // Get Niggli cell
    niggli_lat=MMmult(P.transpose(),in_lat);

    // Get Q
    Q = P;
    matrix<double> tmat(3,3,0.0);
    GaussJordan(Q,tmat); // Returns Q=P^-1.

    // Checks
    // Make sure that a,b,c,alpha,beta,gamma are the same from
    // direct calculation and from using P to get niggli_lat.
    vector<double> poutdat=Getabc_angles(niggli_lat);
    int flag=0;
    for (int i=0;i<6;i++){
      if (abs(poutdat[i]-outdat[i])>2*TOL) {flag=1;}
    }
    if (flag){
      cerr << "ERROR: CellReduceFuncs/GetNiggliCell" << endl;
      cerr << "ERROR: Lattice parameters/angles as calculated" << endl;
      cerr << "ERROR: with Niggli algorithm and P do not match." << endl;
      cerr << "ERROR: a,b,c,alpha,beta,gamma from direct algorithm: ";
      Vout(outdat,cerr);
      cerr << "ERROR: a,b,c,alpha,beta,gamma from P matrix: ";
      Vout(poutdat,cerr);
      cerr << "ERROR: Exiting." << endl;
      exit(1);
    }

}  


structure GetNiggliStr(const structure & in_str)
{
  matrix<double> niggli_lat(3,3);
  matrix<double> P(3,3);
  matrix<double> Q(3,3);
  structure sstr=in_str;
  double scale=sstr.GetScale();
  sstr.ReScale(1);
  GetNiggliCell(sstr.GetLat(),niggli_lat,P,Q);

  // Create new str same as before but with Niggli cell params.
  // Transform cell parameters by xyz_niggli = Q * xyz_original
  // (see ITC, ch.5).
  structure niggli_str=sstr;
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  int nat=dpos.size();
  matrix<double> dpos_niggli(nat,3);
  for (int ia=0;ia<nat;ia++){
    dpos_niggli[ia]=MVmult(Q,dpos[ia]);
  }

  // Set lattice params, atom positions, put in the 0th cell, and reset scale.
  niggli_str.SetLat(niggli_lat);
  niggli_str.SetAllAtomPos(dpos_niggli,0); // Sets the new cpos from the lat and dpos.
  niggli_str.PutInCell();
  niggli_str.ReScale(scale);
  return niggli_str;
}










