using namespace std;
#include <cmath>
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "UsefulFuncs.h"
#include "structure.h"
#include "atom.h"
#include "ChgFuncs.h"

void pd_params::Print(ostream & outf) const{
  cout << type << " # Type " << endl;
  cout << scale << " # scale " << endl;
  for (int ic=0;ic<3;ic++){
    for (int jc=0;jc<3;jc++){
      cout << pts[ic][jc] << " ";
    }
    cout << " # pts " << endl;
  }
  for (int ic=0;ic<3;ic++){
    for (int jc=0;jc<3;jc++){
      cout << dpts[ic][jc] << " ";
    }
    cout << " # dpts " << endl;
  }
  cout << orig_loc << " # origin_loc " << endl;
  cout << ortho << " # ortho " << endl;
}

// **************************************************
// ReadChg
// **************************************************
//  This funtion reads in the CHGCAR file from a file.
/*  Here is the format of a CHGCAR file (for v445 with 
    spin polarization from DEC linux).

    Note that for Intel linux vasp445 the newlines after 
    each chg density line seem to missing, however, the 
    code still reads this in correctly.

    newline
    Starts with a POSCAR file
    newline
    NX NY NZ
    NX*NY*NZ entries with a newline after each one (grouped by 5) (Tot dens = pUP+pDn)
    Natom lines, each with 4 fields, with a newline after each one
    Natom entries (grouped by 5).
    NX NY NZ
    NX*NY*NZ entries with a newline after each one (grouped by 5) (Diff dens = pUP-pDN)
    Natom lines, each with 4 fields, with a newline after each one
    Natom entries (grouped by 5). // I don't think these are actually in the CHGCAR.
*/ 
    
void ReadChg(structure & str,
	     vector<int> & ngrid,
	     vector<double> & chg_tot, 
	     vector<double> & chg_diff, 
	     istream & chgfile)
{
  // Read in CHGCAR
  // Read in intial POSCAR format at beginning of file
  chgfile >> str;
  // Read in the grid for total charge
  int npts;
  chgfile >> ngrid[0] >> ngrid[1] >> ngrid[2];
  npts=ngrid[0]*ngrid[1]*ngrid[2];
  // Read in the total charge
  chg_tot = vector<double> (npts,0.0);
  for (int i=0;i<npts;i++){
    chgfile >> chg_tot[i];
  }
  // Read in dummy fields
  // If first field is augmentation then read 4*natoms+natoms-1 more fields. For vasp 4.4.5.
  // If first field is not augmentation then read natoms-1 more fields. For vasp 4.4.1.
  string ds;
  int nf;
  int natoms=str.GetNumAtoms();
  chgfile >> ds;
  string keyword = "augmentation";
  if (ds==keyword) { // for version 4.4.5
    // Get first line for atom 0.
    chgfile >> ds >> ds >> ds;
    // Now read in any augmentation charges for atom 0.
    nf=atoi(ds.c_str());
    for (int i=0;i<nf;i++){
      chgfile >> ds;
    }
    // Then get remaining lines and aug. charges.
    for (int ia=1;ia<natoms;ia++){
      chgfile >> ds >> ds >> ds >> ds;
      nf=atoi(ds.c_str());
      for (int i=0;i<nf;i++){
	chgfile >> ds;
      }
    }
    // Then read final set of lines for aug. charges of size natoms.
    for (int i=0;i<natoms;i++){
      chgfile >> ds;
    }
  }
  else{ // for v.4.4.1
    for (int i=0;i<(natoms-1);i++){
      chgfile >> ds;
    }
  } 

  // Read in the grid for diff charge
  chgfile >> ngrid[0] >> ngrid[1] >> ngrid[2];
  npts=ngrid[0]*ngrid[1]*ngrid[2];
  // Read in the diff charge
  chg_diff = vector<double> (npts,0.0);
  for (int i=0;i<npts;i++){
    chgfile >> chg_diff[i];
  }
  // Read in 4*natoms=4*natoms dummy fields.  This is 
  // not necessary and the lines only exist in vasp 4.4.5.
  // Actually I don't think these final lines are in vasp445 
  // CHGCAR.
  /*
  for (int i=0;i<4*natoms;i++){
    chgfile >> ds;
  }
  */
} 



// **************************************************
// GetChgInt
// **************************************************
//  This funtion reads in the CHGCAR file from standard input
//  and returns a matrix of charge integrals in the Voronoi volume around
//  each atom and a vector of matrices which gives, for each atom, 
//  the integrated charges in a sphere vs. the radius of the sphere.
void GetChgInt(vector<matrix<double> > & rad_chg_int, 
	       matrix<double> & vor_chg_int,
	       structure & str,
	       vector<int> & ngrid,
	       vector<double> & chg_tot, 
	       vector<double> & chg_diff)
{
  // Loop over grid and bin charge densities.

  // Radial charge integration.
  // Integral over a sphere will be done by setting up NRBIN bins, each
  // of witdh DR, our to RMAX.  Then each charge at each point will be put is the
  // appropriate bin for each atom.  Note that the points and all possible relevant
  // images must be looped over.
  double DR=0.05; // width of radial bins.
  double RMAX=3; // Max distance to bin to.
  int NRBIN=(int)(RMAX/DR); // First bin is 0->DR, last bin is RMAX-DR->RMAX
  matrix<double> dum_mat(NRBIN,5,0.0);
  int natoms=str.GetNumAtoms();
  int npts=ngrid[0]*ngrid[1]*ngrid[2];
  rad_chg_int = vector<matrix<double> > (natoms,dum_mat);
  vector<int> ig(3);
  // Determine the number of images along each lattice vector which must be considered.
  // You will center the charge density on each atom so the max number of images along a 
  // given lattice param should be no more than however many lattice params are needed 
  // to make sure that you are more than RMAX from an atom along that lattice param direction.
  vector<int> imax(3);
  double scale=str.GetScale();
  matrix<double> lat=str.GetLat();
  for (int ic=0;ic<3;ic++){
    imax[ic]=(int)(RMAX/(scale*norm(lat[ic]))+1)*ngrid[ic];
  }
  double sumchg=0;
  // Atom loop
  matrix<double> at_dpos=str.GetDpos();
  for (int iat=0;iat<natoms;iat++){
    // Initialize radial positions in rad_chg_int    
    for (int ib=0;ib<NRBIN;ib++){
      rad_chg_int[iat][ib][0]=(ib+1)*DR;
    }
    // Find location of atom on grid.
    vector<int> at_grid_pos(3);
    for (int ic=0;ic<3;ic++){
      at_grid_pos[ic]=Nint(at_dpos[iat][ic]*ngrid[ic]);
    }
    // Grid loop: This loops over grid points shifted to center around grid point approximation to 
    // position of iat.  The ig values therefore need to be modified to be true grid points 
    // (true_ig) by undoing the shift (adding back the at_grid_pos).
    for (ig[0]=-imax[0];ig[0]<imax[0];ig[0]++){
      for (ig[1]=-imax[1];ig[1]<imax[1];ig[1]++){
	for (ig[2]=-imax[2];ig[2]<imax[2];ig[2]++){
	  // Get true grid points
	  vector<int> true_ig(3);
	  vector<double> chg_dpos(3);
	  for (int ic=0;ic<3;ic++){
	    true_ig[ic]=ig[ic]+at_grid_pos[ic];
            // Get true position of this grid point in direct coords.
	    chg_dpos[ic]=((float)true_ig[ic]/(float)ngrid[ic]);
            // Shift true grid point back into the 000 cell.
	    true_ig[ic]=true_ig[ic]-(int)((float)true_ig[ic]/(float)ngrid[ic])*ngrid[ic];
	    if (true_ig[ic]<0) true_ig[ic]=true_ig[ic]+ngrid[ic];
	  } // ic
	  // Get charge values for each grid position
	  int id=true_ig[0]+true_ig[1]*ngrid[0]+true_ig[2]*ngrid[0]*ngrid[1];
	  double chgtot=chg_tot[id];
	  double chgdiff=chg_diff[id];
	  double chgup=0.5*(chg_tot[id]+chg_diff[id]);
	  double chgdn=0.5*(chg_tot[id]-chg_diff[id]);
	  vector<double> dpos(3);
  	  // Get distance from iat to true grid point position.
          double dist=str.GetDistToAtomDir(chg_dpos,iat);
	  // Get bin for this atom and increment radial charge density.
	  int ibin=(int)(dist/DR);	  
	  if (ibin<NRBIN){
	    rad_chg_int[iat][ibin][1]=rad_chg_int[iat][ibin][1]+chgtot;
	    rad_chg_int[iat][ibin][2]=rad_chg_int[iat][ibin][2]+chgdiff;
	    rad_chg_int[iat][ibin][3]=rad_chg_int[iat][ibin][3]+chgup;
	    rad_chg_int[iat][ibin][4]=rad_chg_int[iat][ibin][4]+chgdn;
	  } // if ibin<NRBIN
	} // ig[2]
      } // ig[1]
    } // ig[0]
  } // iat

// Voronoi charge integration
  vor_chg_int = matrix<double> (natoms,4,0.0);
  for (ig[0]=0;ig[0]<ngrid[0];ig[0]++){
    for (ig[1]=0;ig[1]<ngrid[1];ig[1]++){
      for (ig[2]=0;ig[2]<ngrid[2];ig[2]++){
	// Get charge values for this grid position
	int id=ig[0]+ig[1]*ngrid[0]+ig[2]*ngrid[0]*ngrid[1];
	double chgtot=chg_tot[id];
	double chgdiff=chg_diff[id];
	double chgup=0.5*(chg_tot[id]+chg_diff[id]);
	double chgdn=0.5*(chg_tot[id]-chg_diff[id]);
	vector<double> dpos(3);
	// Set up grid positions in direct coordinates.
	for (int ic=0;ic<3;ic++){
	  dpos[ic]=((float)ig[ic])/(float)ngrid[ic];
	} // ic
	// Loop over atoms
        double dist;
	double min_dist=str.GetDistToAtomImageDir(dpos,0);
	int min_dist_id=0;
	for (int iat=0;iat<natoms;iat++){
  	  // Get "image" distance to each atom (distance to closest image)
          dist=str.GetDistToAtomImageDir(dpos,iat);
          // Keep track of which atom is the minimum "image" distance away
	  if (dist<min_dist){
            min_dist=dist;
	    min_dist_id=iat;
	  } // if dist<min_dist
	} // iat 
	vor_chg_int[min_dist_id][0]=vor_chg_int[min_dist_id][0]+chgtot;
	vor_chg_int[min_dist_id][1]=vor_chg_int[min_dist_id][1]+chgdiff;
	vor_chg_int[min_dist_id][2]=vor_chg_int[min_dist_id][2]+chgup;
	vor_chg_int[min_dist_id][3]=vor_chg_int[min_dist_id][3]+chgdn;
      } // ig[2]
    } // ig[1]
  } // ig[0]


// Normalize and turn rad_chg_int from chg in each radial shell to cumulative integral.
  for (int iat=0;iat<natoms;iat++){
    for (int i=0;i<4;i++){
     vor_chg_int[iat][i]=vor_chg_int[iat][i]/npts;
    }
    for (int i=1;i<5;i++){
      for (int ib=0;ib<NRBIN;ib++){
	rad_chg_int[iat][ib][i]=rad_chg_int[iat][ib][i]/npts;
	if (ib>0) rad_chg_int[iat][ib][i]=(rad_chg_int[iat][ib][i]+rad_chg_int[iat][ib-1][i]);
       } // ib
    } // for loop over tot,diff,up,dn
  } // iat

}



// **************************************************
// PrintChgInt
// **************************************************
//  This funtion prints out 3 lattice vectors in cartesian coordinates.

void PrintChgInt(vector<matrix<double> > & rad_chg_int, 
		 matrix<double> & vor_chg_int)
{  
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);

  // Output Voronoi data
  cout << "******************** Charge in Voronoi volume: Tot Diff Up Dn ********************" << endl;
  for (int iat=0;iat<vor_chg_int.size();iat++){
    cout << iat+1;
    for (int i=0;i<4;i++){
      cout << " " << vor_chg_int[iat][i];
    }
    cout << endl;
  }

  // Output Radial data
  cout << "******************** Charge in sphere: Tot Diff Up Dn ********************" << endl;
  for (int iat=0;iat<rad_chg_int.size();iat++){
    cout << "Atom " << iat+1 << endl;
    for (int ib=0;ib<rad_chg_int[iat].size();ib++){
      cout << "  " << rad_chg_int[iat][ib][0];
      for (int i=1;i<5;i++){
        cout << " " << rad_chg_int[iat][ib][i];
      }
      cout << endl;
    }
  }

} // end routine


// **************************************************
// ReadPlaneDensParams
// **************************************************
//  Reads in parameters.
/* Input file format:

   D # Coordinates for following points (Direct/Cartesian)
   scale # Scale factor - edges of plane get mult. by this (but not origin).
   x y z # origin point
   x y z # "X" axis
   x y z # "Y" axis
   Nx Ny # Number of X and Y grid points
   Middle # Location for origin (Middle/Corner).
   Ortho # Whether to use Y orthogonal to X (Ortho/Strict).
*/
void ReadPlaneDensParams(const structure & str, pd_params & pdp, istream & infile)
  // Note that this converts everything to direct coordinates.
{
  string s;
  double TOL=1e-7;
  pdp.pts = matrix<double> (3,3);
  pdp.dpts = matrix<double> (3,3);
  infile >> pdp.type;
  getline(infile,s);
  infile >> pdp.scale;
  getline(infile,s);
  for (int ic=0;ic<3;ic++){
    for (int jc=0;jc<3;jc++){
      infile >> pdp.pts[ic][jc];
    }
    getline(infile,s);
    // transform points to cart coordinates if necesary.
    if (pdp.type[0]=='D' || pdp.type[0]=='d') pdp.pts[ic]=vecD2C(str.GetLat(),pdp.pts[ic]);
  }
  infile >> pdp.Nx >> pdp.Ny;
  getline(infile,s);
  infile >> pdp.orig_loc;
  getline(infile,s);
  infile >> pdp.ortho;
  getline(infile,s);
  pdp.dpts[0]=pdp.pts[0];
  pdp.dpts[1]=VVdiff(pdp.pts[1],pdp.pts[0]);
  pdp.dpts[2]=VVdiff(pdp.pts[2],pdp.pts[0]);
  // Orthogonalize if necessary
  // Finds vector of same length as dpts[2] but perp. to dpts[1].
  if (pdp.ortho[0]=='O' || pdp.ortho[0]=='o'){
    double norm_orig=norm(pdp.dpts[2]);
    pdp.dpts[2]=CompAperpB(pdp.dpts[2],pdp.dpts[1]);
    double norm_new=norm(pdp.dpts[2]);
    if (norm_new<TOL){
      cout << "ERROR: ChgFuncs/ReadPlaneDensParams" << endl;
      cout << "Your two axis are parallel and no plane can be determined." << endl;
      cout << "Exiting." << endl;
      exit(1);
    }
    pdp.dpts[2]=SVprod(norm_orig/norm_new,pdp.dpts[2]);
  }
  // Shift origin to middle if necessary.
  // I always place dpts[0] at the corner.  So to put origin
  // at the middle shift dpts[0] by -0.5*(dpts[1]+dpts[2]).
  if (pdp.orig_loc[0]=='M' || pdp.orig_loc[0]=='m'){
    vector<double> shift = SVprod(0.5,VVsum(pdp.dpts[1],pdp.dpts[2]));
    pdp.dpts[0]=VVdiff(pdp.dpts[0],shift);
  }
  // Convert everything to direct coordinates
  for (int ic=0;ic<3;ic++){
    pdp.pts[ic]=vecC2D(str.GetLat(),pdp.pts[ic]);
    pdp.dpts[ic]=vecC2D(str.GetLat(),pdp.dpts[ic]);
    // rescale coordinates
    if (ic>0) pdp.dpts[ic]=SVprod(pdp.scale,pdp.dpts[ic]);
  }
}

// **************************************************
// GetPlaneDens
// **************************************************
/*
  Gets density in plane.  Work always within direct coordinates.
  The original densities are stored in chg_tot, chg_diff such that 
  if the direct coordinates of a grid point are i/Nx,j/Ny,k/Nz then
  the index for the charge at that point is id=i+j*Nx+k*Nx*Ny.  The
  charge in the plane will be stored the same way.  Let Npx,Npy =
  plane grid.  Then if the coordinates of a grid point in the plane are
  i/Npx, j/Npy then the index for the charge at that point in id=i+j*Npx.
  You need to be able to access charge densities and structural 
  parameters for this routine.
*/    
void GetPlaneDens(const pd_params & pdp, 
		  vector<double> & dens2d_tot,
		  vector<double> & dens2d_diff,
		  const structure & str,
		  const vector<int> & ngrid,
		  const vector<double> & chg_tot, 
		  const vector<double> & chg_diff)
{
  int Nx=pdp.Nx;
  int Ny=pdp.Ny;
  int npts=Nx*Ny;
  dens2d_tot=vector<double> (npts,0.0);
  dens2d_diff=vector<double> (npts,0.0);
  // Loop over each point in the 2d plane
  for (int i=0;i<Nx;i++){
    for (int j=0;j<Ny;j++){
      // Get id2d for this point.
      int id2d =i+Nx*j;
      // Get direct coordinates for this point.
      vector<double> pt(3,0.0);
      vector<double> Xpt(3,0.0);
      vector<double> Ypt(3,0.0);
      Xpt=SVprod(float(i)/float(Nx-1),pdp.dpts[1]);
      Ypt=SVprod(float(j)/float(Ny-1),pdp.dpts[2]);
      pt=VVsum(Xpt,Ypt);
      pt=VVsum(pdp.dpts[0],pt);
      // Get charge at this point.
      // Simply use value of charge at nearest point
      // on the 3d charge grid.
      int ii=Nint(ngrid[0]*pt[0])%ngrid[0];
      int jj=Nint(ngrid[1]*pt[1])%ngrid[1];
      int kk=Nint(ngrid[2]*pt[2])%ngrid[2];
      if (ii<0) ii=ii+ngrid[0];
      if (jj<0) jj=jj+ngrid[1];
      if (kk<0) kk=kk+ngrid[2];
      int id3d = ii+ngrid[0]*jj+ngrid[0]*ngrid[1]*kk;
      dens2d_tot[id2d]=chg_tot[id3d];
      dens2d_diff[id2d]=chg_diff[id3d];
    }
  }   
}  

// **************************************************
// PrintPlaneDens
// **************************************************
//  
void PrintPlaneDens(const pd_params & pdp, 
		    const vector<double> & dens2d_tot,
		    const vector<double> & dens2d_diff,
		    const structure & str)
{  
  ofstream outf_tot("dens2d.tot.out"); 
  ofstream outf_up("dens2d.up.out"); 
  ofstream outf_dn("dens2d.dn.out"); 
  ofstream outf_diff("dens2d.diff.out"); 

  int Nx=pdp.Nx;
  int Ny=pdp.Ny;
  for (int i=0;i<Nx;i++){
    for (int j=0;j<Ny;j++){
      int id=i+Nx*j;
      outf_tot << dens2d_tot[id] << " ";
      outf_diff << dens2d_diff[id] << " ";
      outf_up << (dens2d_tot[id]+dens2d_diff[id])/2.0 << " ";
      outf_dn << (dens2d_tot[id]-dens2d_diff[id])/2.0 << " ";
    }
    outf_tot << endl;
    outf_diff << endl;
    outf_up << endl;
    outf_dn << endl;
  }
}



// **************************************************
// PrintCHGCAR
// **************************************************
//  
void PrintCHGCAR(const structure & str,
		 const vector<int> & ngrid,
		 const vector<double> & chg_tot, 
		 const vector<double> & chg_diff)
{  
  ofstream outf("CHGCAR.diff.out"); 

  outf << str;
  outf << endl;
  outf << ngrid[0] << " " << ngrid[1] << " " << ngrid[2] << endl;
  int npts=ngrid[0]*ngrid[1]*ngrid[2];
  // chg_tot
  for (int i=1;i<=npts;i++){
    outf << chg_tot[i-1] << " ";
    if ((i%5)==0){
      outf << endl;
      outf << endl;
    }
  }
  if ((npts%5)!=0){
    outf << endl;
    outf << endl;
  }
  // Augmentation stuff for chg_tot
  // For vasp445 format
  int natoms=str.GetNumAtoms();
  for (int i=0;i<natoms;i++){
    outf << "augmentation occupancies " << i+1 << " " << "0" << endl;
    outf << endl;
  }
  for (int i=1;i<=natoms;i++){
    outf << "0.000E+0 ";
    if ((i%5)==0) outf << endl;
  }
  if ((npts%5)!=0){
    outf << endl;
  }
  outf << endl; // Puts space between last aug. line and chg_diff grid.
  // chg_diff
  outf << ngrid[0] << " " << ngrid[1] << " " << ngrid[2] << endl;
  for (int i=1;i<=npts;i++){
    outf << chg_diff[i-1] << " ";
    if ((i%5)==0){
      outf << endl;
  //    outf << endl;
    }
  }
  if ((npts%5)!=0){
    outf << endl;
    outf << endl;
  }
  // Augmentation stuff for chg_diff
  // For vasp445 format
  for (int i=0;i<natoms;i++){
    outf << "augmentation occupancies " << i << " " << "0" << endl;
    outf << endl;
  }
  /*
  for (int i=1;i<=natoms;i++){
    outf << "0.000E+0 ";
    if (i%5==0) outf << endl;
  }
  */
}



