// **************************************************
// GetKmesh
// **************************************************
//  This funtion returns a set of Monkhurst-Pack kpoint mesh values 
// (divisions along each reciprical lattice parameter)
// based on the desired kpt density and the lattice parameters.  The
// values assure the most even distribution of kpts along the lattice 
// params consistent with the kpt density.  Returns 4 kmesh vectors of length
// 3
//    1. Rounded to Nint
//    2. Exact
//    3. Rounded down
//    4. Rounded up

using namespace std;
#include <cmath>
#include <iostream>
#include "UsefulFuncs.h"

void GetKmesh(const structure & str, const double & kptdens, matrix<double> & kmesh)
{
/* Kmesh is chosen to be as uniform as possible along all directions.  This is done
  by weighting the kpoints along each rlv by the length of the projection of 
  the rlv on the unit normal to the plane formed by the other 2 vectors.
  Kmesh sets the density to kptdens kpts per inverse atom (=kpts/atom^-1=kpts*atom) 
  (ie, total # kpts = kptdens/natoms).  Maybe this should be per unit inverse vol but that 
  could change around a lot and be confusing.
*/
  
  structure sstr=str;
  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  int natoms=sstr.GetNumAtoms();
  matrix<double> lat=sstr.GetLat();
  matrix<double> rlat=RecipLat(lat);
  vector<double> latmag(6,0.0);
  vector<double> proj(3);
  for (int i=0;i<3;i++){
    vector<double> a = rlat[i];
    vector<double> b = rlat[(i+1)%3];
    vector<double> c = rlat[(i+2)%3];
    vector<double> bxc=VVcross(b,c);
    double nbxc=norm(bxc);
    proj[i]=abs(VVprod(a,bxc)/nbxc);
  }
  double normalizer = pow(kptdens/(proj[0]*proj[1]*proj[2]*natoms),1.0/3.0);
  vector<double> mesh = SVprod(normalizer,proj);
  // Exact values
  kmesh[1]=mesh;
  // Rounded to Nint
  for (int i=0;i<3;i++){
    kmesh[0][i]=Nint(kmesh[1][i]);
  }
  // Rounded down
  for (int i=0;i<3;i++){
    kmesh[2][i]=Nint(kmesh[1][i]-0.49999999);
  }
  // Rounded up
  for (int i=0;i<3;i++){
    kmesh[3][i]=Nint(kmesh[1][i]+0.49999999);
  }
}





