
using namespace std;
#include <vector>
#include <cmath>
#include <string>
#include "UsefulFuncs.h"
#include "matrix.h"

/*
**************************************************
Function GetSuperCell
**************************************************
This function finds a supercell. It uses the following
not very elegant algorithm.  Let mx[i](mn[i]) be the 
largest (smallest) multiple of the ith original 
lattice vector appearing in the supercell (ie, appearing 
along any edge of the supercell).  Then we will 
search all atoms in images spanning from mn to mx multiples
of the original unit cell and save the ones that are in
the new cell.  I think this should capture all atom images
in the new cell assuming the original atom is at the origin.
However, the original atom is inside the original unit cell
somewhere so it is within fractional coordinates 1 along each 
original lattice vector.  Therefore I increase the range of 
mx(mn) by 1 to account for this.  In addition, I increase the
range of mx(mn) by 1 to account for any edge effects. I 
am not 100% sure that this algorithm will always capture the 
atoms in the supercell so take care.
*/
structure GetSuperCell(const matrix<double> & sc, structure str){
  double tol=1e-12;
  double lb=0-tol;
  double ub=1-tol;
  // Set up mn,mx matrix with all the possible combinations of sc.
  matrix<double> mxmnmat(7,3);
  mxmnmat[0]=sc[0];
  mxmnmat[1]=sc[1];
  mxmnmat[2]=sc[2];
  mxmnmat[3]=VVsum(sc[0],sc[1]);
  mxmnmat[4]=VVsum(sc[0],sc[2]);
  mxmnmat[5]=VVsum(sc[1],sc[2]);
  mxmnmat[6]=VVsum(mxmnmat[3],sc[2]);
  vector<double> mn(3,0.0);
  vector<double> mx(3,0.0);
  for (int jc=0;jc<3;jc++){
    for (int i=0;i<7;i++){
      if (mxmnmat[i][jc]>mx[jc]) mx[jc]=Nint(mxmnmat[i][jc]);
      if (mxmnmat[i][jc]<mn[jc]) mn[jc]=Nint(mxmnmat[i][jc]);
    }
  }
  // Just to make sure "edge effects" and original position shifted
  // from origin don't cause problems.
  for (int ic=0;ic<3;ic++){
    mn[ic]=mn[ic]-2;
    mx[ic]=mx[ic]+2;
  }

  // Prepare to get new structure data.
  str.ReScale(1);
  str.PutInCell();
  int nat=str.GetNumAtoms();
  matrix<double> lat=str.GetScaledLat();
  matrix<double> nlat=MMmult(sc,lat);
  matrix<double> dpos=str.GetDpos();
  matrix<double> cpos=str.GetCpos();
  matrix<double> dpos_in_new_cell=dpos;
  matrix<double> ndpos;
  vector<string> names=str.GetNames();
  vector<string> nnames;
  vector<int> names_were_given=str.GetNamesWereGiven();
  vector<int> nnames_were_given;
  vector<int> num_each_type=str.GetNumEachType();
  int ntypes=num_each_type.size();
  vector<int> nnum_each_type(ntypes);
  /*
  // Shift atoms into new cell (Put cart in new direct, shift, then 
  // bring back to Cart then put in old direct coord.
  // Note that this shift creates wierd results if nlat are not
  // tranlation vectors of the cell (or if sc is not integers).
  // One could skip these lines but then one does not know how far from
  // the new unit cell the atoms are.
  for (int ia=0;ia<nat;ia++){
    vector<double> t1(3);
    t1=vecC2D(nlat,cpos[ia]);
	      // tpx
    //	      cout << "t1 init ";
    //      Vout(t1,cout);
    for (int ic=0;ic<3;ic++){
      t1[ic]=t1[ic]-int(t1[ic]);
      if (t1[ic]<0.0) t1[ic]=t1[ic]+1;
    }
    t1=vecD2C(nlat,t1);
    t1=vecC2D(lat,t1);
	      // tpx
    //	      cout << "t1 final ";
    //	      Vout(t1,cout);
    dpos_in_new_cell[ia]=t1;
  }
  */

  // Loop over all possible images to collect new basis atoms
  int iat=0;
  vector<int> id(3);
  for (int it=0;it<ntypes;it++){
    for (int jt=0;jt<num_each_type[it];jt++){
      for (id[0]=(int)mn[0];id[0]<=mx[0];id[0]++){
	for (id[1]=(int)mn[1];id[1]<=mx[1];id[1]++){
	  for (id[2]=(int)mn[2];id[2]<=mx[2];id[2]++){
	    vector<double> tdpos(3);
	    vector<double> tcpos(3);
	    // Get direct coordinate position of candidate image atom
	    for (int ic=0;ic<3;ic++){
	      //tdpos[ic]=dpos_in_new_cell[iat][ic]+id[ic];
	      tdpos[ic]=dpos[iat][ic]+id[ic];
	    }//ic
	      // tpx
	    //	      cout << "Id ";
	    //	      Vout(id,cout);
	    //	      cout << "Original direct ";
	    //	      Vout(tdpos,cout);
	    // Transform to Cartesian, then see if ndpos is in new supercell.
	    tcpos=vecD2C(lat,tdpos);
	    tdpos=vecC2D(nlat,tcpos);
	      // tpx
	    // cout << "New direct ";
	    // Vout(tdpos,cout);
	    int is_in_ncell=1;
	    for (int ic=0;ic<3;ic++){
	      if (tdpos[ic]<lb || tdpos[ic]>=ub) is_in_ncell=0;
	    }//ic
	      // tpx
	    //      cout << "Is in new cell ";
	    // cout<< is_in_ncell<< endl;
	    if (is_in_ncell){
	      ndpos.push_back(tdpos);
	      nnames.push_back(names[iat]);
	      nnum_each_type[it]++;
	      nnames_were_given.push_back(names_were_given[iat]);
  	    }
	  }//id0
	}//id1
      }//id2
      iat++;
    }//jt
  }//it
  str.SetLat(nlat);
  str.SetNumEachType(nnum_each_type);
  str.SetAllAtomPos(ndpos,0);
  str.SetAllAtomNames(nnames);
  str.SetNamesWereGiven(nnames_were_given);

  return str;
}


