using namespace std;
#include <cmath>
#include "UsefulFuncs.h"
#include "FittingFuncs.h"
#include "typedefs.h"

/*
**************************************************
Function SetSpline
**************************************************
  This function finds the second derivates so that
spline interpolations can be evaluated quickly.
From Numerical Recipes spline.  I have shifted all 
indices to go from 0->(n-1).
*/
void SetSpline(const vector<double> & x, const vector<double> & y, 
	       const double & yp1, const double & ypn, vector<double> & y2)
{
  int i,k;
  double p,qn,sig,un;
  int n=x.size();

  vector<double> u(n-1);
  y2=vector<double> (n,0.0);

  if (yp1 > 0.99e30)
    y2[0]=u[0]=0.0;
  else {
    y2[0] = -0.5;
    u[0]=(3.0/(x[2]-x[1]))*((y[1]-y[0])/(x[1]-x[0])-yp1);
  }
  for (i=1;i<=n-2;i++) {
    sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
    p=sig*y2[i-1]+2.0;
    y2[i]=(sig-1.0)/p;
    u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
  }
  if (ypn > 0.99e30)
    qn=un=0.0;
  else {
    qn=0.5;
    un=(3.0/(x[n-1]-x[n-2]))*(ypn-(y[n-1]-y[n-2])/(x[n-1]-x[n-2]));
  }
  y2[n-1]=(un-qn*u[n-2])/(qn*y2[n-2]+1.0);
  for (k=n-2;k>=0;k--){
    y2[k]=y2[k]*y2[k+1]+u[k];
  }
}

/*
**************************************************
Function GetSplineInt
**************************************************
  This function finds the value of a function y at a 
point x from a spline interpolation.  You must run 
SetSpline first. From Numerical Recipes splint.  
I have shifted all indices to go from 0->(n-1).
*/
void GetSplineInt(const vector<double> & xa, const vector<double> & ya, 
		  vector<double> & y2a, const double & x, double & y)
{
  void nrerror(char error_text[]);
  int klo,khi,k;
  double h,b,a;
  int n=xa.size();
  
  klo=0;
  khi=n-1;
  while (khi-klo > 1) {
    k=(khi+klo) >> 1;
    if (xa[k] > x) khi=k;
    else klo=k;
  }
  h=xa[khi]-xa[klo];
  if (h == 0.0) nrerror("Bad xa input to routine splint");
  a=(xa[khi]-x)/h;
  b=(x-xa[klo])/h;
  y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}

/*
**************************************************
Function PrintSpline
**************************************************
  This function prints out the original data and the
spline interpolation with keywords for grep.
*/
void PrintSpline(const vector<double> & x, const vector<double> & y, 
		 const int & npts, ostream & outf)
{
  outf.setf(ios::left, ios::adjustfield);
  outf.setf(ios::fixed, ios::floatfield);
  outf << "******************** Initial Data ********************" << endl;
  int ndat=x.size();
  for (int id=0;id<ndat;id++){
    outf<< x[id] << " " << y[id]<< "   " << "INPUT" << endl;
  }
  outf << "******************** Cubic Spline Interpolated Points ********************" << endl;
  if (npts==1){
    outf<< x[0] << " " << y[0] << "   " << "INTERPOLATED" << endl;
  }
  else{
    double yp1=0.0;
    double ypn=0.0;
    vector<double> y2;
    SetSpline(x,y,yp1,ypn,y2);
    double xrange=x[ndat-1]-x[0];
    double dx=xrange/(double)(npts-1);
    for (int ip=0;ip<npts;ip++){
      double xp=x[0]+(double)ip*dx;
      double yp;
      GetSplineInt(x,y,y2,xp,yp);
      outf << xp << " " << yp << "   " << "INTERPOLATED" << endl;
    }
  }
}






