// **************************************************
// GetPrim
// **************************************************
//  This funtion returns a structure object where everything 
// has been transformed into a primitive lattice.
/* Algorithm
     Construct a list of all candidate primitive lattice vectors.
     This list consists of the present lattice vectors and all basis
     vectors where translation by that basis vector leaves the all the 
     basis atom types unchanged.  Loop over all possible triads in this list.
     For each triad find the volume.  Store a list of all triads with the
     minimum volume.  Then select the triad for which the first triad vector 
     has the maximal projection onto the first lattice vector.  If there
     are more than one of these do the same for the second lattice vector,
     and then the third if needed.  Then just pick one if there are still
     multiple candidates.
*/

using namespace std;
#include <cmath>
#include <string>
#include <iostream>
#include "UsefulFuncs.h"
#include "GetPrim.h"

structure GetPrim(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;
 
  matrix<double> candidate_plvs;
  matrix<double> plat(3,3);
  vector<matrix<double> > plat_list;
  vector<double> displ_d(3,0.0);
  vector<double> displ_c(3,0.0);
  double tol=1e-11;

  // Get all the data from the structure for easy use.
  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  // Put everything in the unit cell.
  sstr.PutInCell();
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();
  vector<int> names_were_given;
  names_were_given=sstr.GetNamesWereGiven();
  vector<int> types;
  types=sstr.GetTypes();

  // Create list of candidate plvs.

  candidate_plvs=lat;

  for(int iat=1;iat<num_atoms;iat++){
    // Here we calculate all displacements from the first atom, i.e., the 
    // first atom is considered the origin.
    for(int ic=0;ic<3;ic++){
      displ_c[ic]=cpos[iat][ic]-cpos[0][ic];
    }
    if (IsTranslationVector(sstr,displ_c)){
      candidate_plvs.push_back(displ_c);
    } 
  }
  //  Now the candidate_plvs have been found and we must take all possible
  // traids of *distinct* vectors and get the smallest volume.
  int num_cand=candidate_plvs.size();
  double min_vol=GetVol(lat);
  for(int iu=0;iu<num_cand;iu++){
    for(int iv=0;iv<num_cand;iv++){
      for(int iw=0;iw<num_cand;iw++){
        plat[0]=candidate_plvs[iu];              
        plat[1]=candidate_plvs[iv];              
        plat[2]=candidate_plvs[iw];              
        double new_vol=GetVol(plat);
        if (new_vol<=min_vol && new_vol>tol) min_vol=new_vol;
      } //iw loop
    } //iv loop
  } //iu loop
  // Now that we know the min_vol we must go back through all the triads 
  // and store the set that have the min_vol.
  for(int iu=0;iu<num_cand;iu++){
    for(int iv=0;iv<num_cand;iv++){
      for(int iw=0;iw<num_cand;iw++){
        if (iu!=iw && iu!=iv && iv!=iw){
          plat[0]=candidate_plvs[iu];              
          plat[1]=candidate_plvs[iv];              
          plat[2]=candidate_plvs[iw];              
          double new_vol=GetVol(plat);
          if (fabs(new_vol-min_vol)<tol){
            // Add to plat_list
            plat_list.push_back(plat);
          } //if new_vol=min_vol
        } //if iu!=iv!=iw
      } //iw loop
    } //iv loop
  } //iu loop
  // Now we have a set of all possible primitive lattices
  // and we want to pick one that looks as much like the
  // original lattice as possible.
  // For now just pick the first one.
  if (plat_list.size()>0) plat=plat_list[0];
  // If volume did not reduce then just keep original lattice
  if (fabs(GetVol(lat)-min_vol)<tol){plat=lat;} 

  // Now we have the plvs for the new structure.  Now we must 
  // construct a new structure object with the plvs and return it.
  // This new structure object will be just like the old one but
  // with new lattice vectors and new basis atoms (which will be a 
  // subset of the original ones, chosen to have direct coordinates
  // less than 1).

  // To create new structure we must set, dpos (and then later cpos), types, 
  // num_each_type, names, names_were_given.  
  // To get new basis atoms, loop over all the original basis atoms,
  // get new direct coordinates, make all less than one, then take unique
  // values.

  matrix<double> new_dpos;
  vector<int> new_types;
  vector<int> new_num_each_type(1,1);
  vector<int> new_names_were_given;
  vector<double> v;
  vector<string> new_names;
  string stemp;
  int at_cnt=0;
  for (int it=0;it<num_each_type.size();it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      v=vecC2D(plat,cpos[at_cnt]);
      // Shift into cell
      for (int ic=0;ic<3;ic++){
	v[ic]=v[ic]-int(v[ic]); 
	if (v[ic]<0) v[ic]=v[ic]+1;
	if (fabs(v[ic]-1)<tol) v[ic]=0;
      }
      // Check if old basis atom is a new basis atom
      int is_new_basis_atom=1;
      for (int ib=0;ib<new_dpos.size();ib++){
	if (fabs(v[0]-new_dpos[ib][0])<tol 
	    && fabs(v[1]-new_dpos[ib][1])<tol 
	    && fabs(v[2]-new_dpos[ib][2])<tol
	    ) is_new_basis_atom=0;
      }
      if (is_new_basis_atom){
        // Creat new list of dpos
        new_dpos.push_back(v);
        // Create new list of types
        new_types.push_back(types[at_cnt]);
        // Create new list of names and names_were_given.
        new_names.push_back(names[at_cnt]);
        new_names_were_given.push_back(0);
        // If a name was given change new_names_were_given
        if (names_were_given[at_cnt]){
          new_names_were_given[new_names_were_given.size()-1]=1;
        }
      }
      at_cnt++;
    } 
  }
  // Here you must use list of new types to get new_num_each_type, and short_new_names.
  vector<string> short_new_names(1);
  short_new_names[0] = new_names[0];
  int old_val=new_types[0];
  int new_val;
  for (int i=1;i<new_types.size();i++){
    new_val=new_types[i];
    if (new_val==old_val){
      new_num_each_type[new_num_each_type.size()-1]++;
    }  
    else{
      new_num_each_type.push_back(1);
      short_new_names.push_back(stemp);
      short_new_names[short_new_names.size()-1]=new_names[i];
    }
    old_val=new_val;
  }

// Then copy everything into new structre.
  
  structure pstr=sstr;
  // Set lattice.
  pstr.SetLat(plat);
  // Set basis atoms
  pstr.SetAllAtomPos(new_dpos,0); // sets direct and cart
  // Set types
  pstr.SetTypes(new_types);
  // Set num_each_type
  pstr.SetNumEachType(new_num_each_type);
  // Set names (also sets names_were_given)
  pstr.SetNames(short_new_names);
  // Clear memory
  // rescale back to original scale.
  pstr.ReScale(str.GetScale());
  // Put everything in new primitive cell.
  pstr.PutInCell();

  return pstr;
}





