// **************************************************
// SumPDOSFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include "SumPDOSFuncs.h"
#include "UsefulFuncs.h"
#include "ProjFuncs.h"

// **************************************************
// ReadSumDOSParams
// **************************************************
/* Reads in parameters from PDOSParams_infile
   that decide what summations to perform over pdos.
   This routine makes use of the pdosdata object which 
   contains some parameters that are not used (parameters 
   for kpoints, bands, etc.)
*/
 
void ReadSumDOSParams(ifstream & infile, pdosdata & pdd){
  // Defaults (emin,emax,nbins,smooth_sigma,print_params,efermi)
  pdd.emin=0.0;
  pdd.emax=0.0;
  pdd.nbins=301;
  pdd.smooth_sigma=(pdd.emax-pdd.emin)/pdd.nbins;
  pdd.print_params=0;
  pdd.efermi=-999;
  pdd.spin=1; // 1: non spin polarized, 2 spin polarized.          
  pdd.nlm=9; // all the possible l and m:  9 for s,p,d,  16 for s,p,d,f 

// Read in all the tokens.
  string s,s_ns;
  vector<string> token_vec;
  vector<string> val_vec;
  while (!infile.eof()){
    getline(infile,s);
    s_ns=RemoveSpaces(s); // Get string with no spaces.
    if (s_ns.size()>0){ // Make sure line was not blank
      if (s_ns[0]!='#'){ // Exclude comment lines
        string token;
        string sval;
        int id=s.find('=');
        if (id>=s.length()){
          cout << "ERROR: The following token is incorrectly formatted: " << s << endl;
          cout << "ERROR: Exiting" << endl;
          exit(1);
        }
        token=s.substr(0,id);
        token=RemoveSpaces(token);
        int i_f=min(s.length(),s.find('#')); // End of string
        sval=s.substr(id+1,i_f-id-1);
        token_vec.push_back(token);
        val_vec.push_back(sval);
      } //if s_ns[0]!=#
    } // if s_ns.size>0
  } // while !infile.eof

  // Read in values for variables in the pdosdata object.
  vector<int> tmp;
  int atom_cnt=0;
  for (int i=0;i<token_vec.size();i++){
    int id=0;
    string tok=token_vec[i];
    string sval;
    double dval;
    int ival;
    int found_token=0;
    if (tok=="ATOMS"){
      id=0;
      atom_cnt++;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
        string s;
        s=GetNextVal(val_vec[i],id);
        if (s.size()>0){
          ival=atoi(s.c_str());
          tmp.push_back(ival);
        }
      }
      pdd.pdos_at.push_back(tmp);
    // add vector to pdos_lm
      tmp.clear();
      pdd.pdos_lm.push_back(tmp);
      found_token=1;
    }// ATOMS
    if (tok=="LMVALUES"){
      if (pdd.pdos_lm.size()!=atom_cnt || atom_cnt==0) AtomCntError(tok,pdd.pdos_lm.size(),atom_cnt);
      id=0;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
        string s;
        s=GetNextVal(val_vec[i],id);
        if (s.size()>0){
          ival=atoi(s.c_str());
          pdd.pdos_lm[pdd.pdos_lm.size()-1].push_back(ival);
        }
      }
      found_token=1;
    }// LMVALUES
    if (tok=="SMOOTH_SIGMA"){
      id=0;
      pdd.smooth_sigma=atof(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// SMOOTH_SIGMA
    if (tok=="PRINT_PARAMS"){
      id=0;
      pdd.print_params=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// PRINT_PARAMS
    if (tok=="EFERMI"){
      id=0;
      pdd.efermi=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// EFERMI
    if (tok=="SPIN"){
      id=0;
      pdd.spin=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// SPIN
    if (tok=="NLM"){
      id=0;
      pdd.nlm=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// NLM
    if (!found_token){
      cerr << "ERROR: You have input a token " << tok <<endl;
      cerr << "ERROR: This token is not recognized. Exiting! " << endl;
      exit(1);
    }
  } // for i over tokens

  // Check that all data is within bounds
  for (int i=0;i<pdd.pdos_at.size();i++){ // cases
    for (int ilm=0;ilm<pdd.pdos_lm[i].size();ilm++){
      int lmp=pdd.pdos_lm[i][ilm];
      if (lmp<1 || lmp>pdd.nlm){
        cerr << "ERROR: SumPDOSFuncs/ReadSumDOSParams" << endl;
        cerr << "ERROR: Error in case: " << i+1 << endl;
        cerr << "ERROR: You have entered too low or high an lm number in entry: " << ilm+1 << endl;
        cerr << "ERROR: Bad value is: " << lmp << endl;
        cerr << "ERROR: Exiting." << endl;
        exit(1);
      }
    }
  }
  
}// end routine

// **************************************************
// ReadSumDOSParams
// **************************************************
// Reads in pdos data from PDOSinfile.
void ReadInPDOSData(matrix<matrix<double> > & allpdos, pdosdata & pdd, 
		    ifstream & infile){
  string sdum;
  // Get natoms
  infile >> pdd.natoms;
  getline(infile,sdum);
  // Skip lines
  for (int i=0;i<4;i++){
    getline(infile,sdum);
  }
  // Get number of bins and E Fermi
  double tmpe;
  infile >> sdum >> sdum >> pdd.nbins >> tmpe;
  getline(infile,sdum);
  if (abs(pdd.efermi+999)<1e-6){ // Reset efermi if it has special -999 value
    pdd.efermi=tmpe;
  }
  // Skip total DOS
  for (int i=0;i<pdd.nbins;i++){
    getline(infile,sdum);
  }
  // Initialize size of allpdos[atoms][spin][lm][bin]
  matrix<double> tmp(pdd.nlm+1,pdd.nbins);
  allpdos = matrix<matrix<double> > (pdd.natoms,pdd.spin,tmp);
  // tpx 
  //cout << spin << " " << pdd.nlm << " " << pdd.natoms << " " << pdd.nbins << endl;
  // Loop over each atom
  for (int ia=0;ia<pdd.natoms;ia++){
    // Throw away first line since it is not DOS info
    getline(infile,sdum);
    // This is needed since getline just gets the end of the previous line when 
    // the previous line has been read in by >>.  Only for atom 0 is the
    // previous line read in by getline, so only one more getline is needed.
    if (ia>0){
      getline(infile,sdum);
    }
    // Loop over each bin
    for (int ib=0;ib<pdd.nbins;ib++){
      // Read in energy into first spin and nlm array elements.
      infile >> allpdos[ia][0][0][ib];
      // Loop over each lm
      for (int ilm=1;ilm<=pdd.nlm;ilm++){
	// Loop over each spin
	for (int is=0;is<pdd.spin;is++){
	  infile >> allpdos[ia][is][ilm][ib];
	  // tpx
	  /*
	  	  cout << "ia ib is ilm allpdos " << ia << " "
	  	       << ib << " " 
	  	       << is << " " 
	     << ilm << " " 
	     << allpdos[ia][is][ilm][ib] << endl;
	  */
	} // spin
      } // lm
    } // bin
  } // at
} // end routine

// **************************************************
// SumPDOS
// **************************************************
// Sums the PDOS accoring to the parameters specified.
void SumPDOS(const matrix<matrix<double> > & allpdos, pdosdata & pdd){
  if (pdd.spin==1){
    pdd.pdos = matrix<double> (pdd.nbins,3);
  }
  else{
    pdd.pdos = matrix<double> (pdd.nbins,7);
  }
  // Set energies
  for (int ib=0;ib<pdd.nbins;ib++){
    pdd.pdos[ib][0]=allpdos[0][0][0][ib]-pdd.efermi;
  }
  // Loop over flagged atoms and lm and sum
  for (int ic=0;ic<pdd.pdos_at.size();ic++){
    for (int ida=0;ida<pdd.pdos_at[ic].size();ida++){
      int ia=pdd.pdos_at[ic][ida]-1;
      // Error check 
      if (ia<0 || ia>=pdd.natoms){
        cerr << "ERROR: SumPDOSFuncs/SumPDOS" << endl;
	cerr << "ERROR: Error in case: " << ic+1 << endl;
        cerr << "ERROR: You have entered too low or high an atom number in entry: " << ida+1 << endl;
        cerr << "ERROR: Min/Max allowed values are: " << "1 / " << pdd.natoms << endl;
        cerr << "ERROR: Bad value is: " << ia+1 << endl;
        cerr << "ERROR: Exiting." << endl;
        exit(1);
      }
      // Loop over each lm
      for (int idlm=0;idlm<pdd.pdos_lm[ic].size();idlm++){
	int ilm=pdd.pdos_lm[ic][idlm];
	// Error check 
	if (ilm<1 || ilm>pdd.nlm){
	  cerr << "ERROR: SumPDOSFuncs/SumPDOS" << endl;
	  cerr << "ERROR: Error in case: " << ic+1 << endl;
	  cerr << "ERROR: You have entered too low or high an lm number in entry: " << idlm+1 << endl;
	  cerr << "ERROR: Min/Max allowed values are: " << "1 / " << pdd.nlm << endl;
	  cerr << "ERROR: Bad value is: " << ilm << endl;
	  cerr << "ERROR: Exiting." << endl;
	  exit(1);
	}
	// Loop over each bin
	for (int ib=0;ib<pdd.nbins;ib++){
	  // Do spin sum
	  double val;
	  // Note: allpdos[atoms][spin][lm][bin]
	  switch (pdd.spin){
	  case 1:{ // non spin polarized
	    pdd.pdos[ib][1]=pdd.pdos[ib][1]+allpdos[ia][0][ilm][ib];
	    break;
	  }
	  case 2:{ // spin polarized
	    pdd.pdos[ib][1]=pdd.pdos[ib][1]+allpdos[ia][0][ilm][ib];
	    pdd.pdos[ib][2]=pdd.pdos[ib][2]+allpdos[ia][1][ilm][ib];
	    break;
	  }
	  default: 
	  cerr << "ERROR: SumPDOSFuncs/SumPDOS" << endl;
	  cerr << "ERROR: Error in case: " << ic+1 << endl;
	  cerr << "ERROR: You have entered too low or high an spin number" << endl;
	  cerr << "ERROR: Min/Max allowed values are: " << "1 / 2" << endl;
	  cerr << "ERROR: Bad value is: " << pdd.spin << endl;
	  cerr << "ERROR: Exiting." << endl;
	  exit(1);
	  } // switch spin
	} // bin
      } // lm
    } // at
  } // case
  // Get cumulative and difference values
  double dE=pdd.pdos[1][0]-pdd.pdos[0][0];
    if (pdd.spin==1){
      pdd.pdos[0][2]=0;
      for (int ib=1;ib<pdd.nbins;ib++){
	pdd.pdos[ib][2]=pdd.pdos[ib-1][2]+pdd.pdos[ib][1]*dE;
      }
    }
    else{
      pdd.pdos[0][3]=pdd.pdos[0][1]-pdd.pdos[0][2];
      pdd.pdos[0][4]=0;
      pdd.pdos[0][5]=0;
      pdd.pdos[0][6]=0;
      for (int ib=1;ib<pdd.nbins;ib++){
	pdd.pdos[ib][3]=pdd.pdos[ib][1]-pdd.pdos[ib][2];
	pdd.pdos[ib][4]=pdd.pdos[ib-1][4]+pdd.pdos[ib][1]*dE;
	pdd.pdos[ib][5]=pdd.pdos[ib-1][5]+pdd.pdos[ib][2]*dE;
	pdd.pdos[ib][6]=pdd.pdos[ib-1][6]+pdd.pdos[ib][3]*dE;
      }
    }

}

// **************************************************
// PrintSumDOS
// **************************************************
// Prints out the summed pdos in user friendly format.
void PrintSumPDOS(ostream & cout, pdosdata & pdd){
  int spin=pdd.spin-1;
  pdd.PrintPDOS(cout,spin);
  if (pdd.print_params==1){
    projdata prd;
    vector<string> LMnames=prd.LMnames;
    pdd.PrintParams(cout,LMnames);
  }
}



