// To do:  ADD FORCES ???
// **************************************************
// EwaldFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <algorithm>
#include "UsefulFuncs.h"
#include "projdata.h"
#include "EwaldFuncs.h"
#include "constants.h"

// This converts the unit of z^2/r (z in e_electron and r in Ang to eV. 
// It is equal to  14.39965)
double CONV_FACT=(1.0E+10*E_ELECTRON/(4.0*PI*EPS_VACUUM));

// **************************************************
// Ewald
// **************************************************
// Calculates Ewald Sum.  Based on routine from Eric Wu.
/*
c     ******************************************************************
c     **                   Subroutine ewald                           **
c     **                                                              **
c     **   Subroutine ewald calculates the ewald sum in eV            **
c     **   Also, this calculates the ewald gradients in ev   since    **
c     **         it is just a small calculation to do- ewald scales   **
c     **         as N^2, extra force calculation  adds as  N          **
c     **                                                              **
c     **   NB if you want to convert forces to ev/A you need the      **
c     **   inverse of the lattice vectors matrix (A) then do a        **
c     **    Fx=ainvx*fx+ainvy*fy+ainvz*fz                             **
c     **                 where Fx is in ev/A fx is reduced            **
c     **   to get the components along each cooredinate (I think)     **
c     **                                                              **
c     **                                                              **
c     **   References:                                                **
c     **     www.dl.ac.uk/TCS/Software/DL_POLY/USRMAN/node62.html     **
c     **     www.ee.duke.edu/~ayt/ewaldpaper/node8.html               **
c     **     Moldy (molecular dynamics) manual                        **
c     **   http://www.earth.ox.ac.uk/~keith/moldy-manual/node11.html  **
c     **     Allen+tildesly p159 (for cubic)                          **
c     **     Kittel Appendix B                                        **
c     **                                                              **
c     **   Formula:                                                   **
c     **                                                              **
c     **   For the energy:                                            **
c     **      Etotal = Erecip + Ereal + Epoint                        **
c     **                                                              **
c     **      Epoint = [1/(4*pi*eo)]                                  **
c     **                   * sqrt(eta)/sqrt(pi) * sum(i) qi^2         **
c     **              +[1/(4*pi*e0)] * [pi/(2*vol*eta)]               **
c     **                   * [sum(i) qi]^2   (for charged cell)       **
c     **      Ereal = [1/(4*pi*e0)]* 0.5 *                            **
c     **                      sum(ij) (qi*qj*erfc(sqrt(eta)*rij)/rij  **       
c     **      Erecip = [1/(4*pi*e0)] *0.5* [4*pi/vol] *               **
c     **               sum(G) 1/G^2 * exp[-G^2/(4*eta)] *             **
c     **               |sum(i) (qi*exp(-iGri)|^2                      **
c     **                                                              **
c     **                                                              **
c     **  For the forces:                                             **
c     **      procedure: take the neg. derivatives.                   **
c     **                 The point is const,                          **
c     **                 and disappears.  So                          **
c     **                                                              **
c     **      Fj=  Fjrecip + Fjreal                                   **
c     **                                                              **
c     **      Fjxrecip = +[1/(4*pi*e0)] * [4*pi/vol] *                **
c     **                 qj*  sum(G) Gx * G*1/G^2 *                   ** 
c     **                  exp[-G^2/(4*eta)]*                          **
c     **                             [-sin(Grj)*sum(i) qicos(Gri)]    **
c     **                            +[cos(Grj)]*sum(i) qisin(Gri)]    **
c     **       to get this, recall exp(-iGr)=cos(Gr)+isin(Gr)         **
c     **       and put in Erecip.  Then take the derivatives of       **
c     **       cos^2(Gr) and sin^2(Gr)                                **
c     **                                                              **
c     **      Fjxreal =   -[1/(4*pi*e0)]  * qj *                      **
c     **                  Sum(i) qi/rij^3*                            **
c     **                         [2*sqrt(eta)*rij/sqrt(pi)            **
c     **                                  *exp(-eta*rij^2)            **
c     **                          +erfc(sqrt(eta)*rij)]*rxij          **
c     **                recall                                        **
c     **        erf(x)=2/sqrt(pi)integral(0,x)exp(-t^2)dt             **
c     **                                                              **
c     **                                                              **
c     **   input variables                                            **
c     **                                                              **
c     **     lat(3,3) real space lattice vectors input (ax ay az        **
c     **                                              bx by bz        **
c     **                                              cx cy cz)       **
c     **                                               but stored     **
c     **                                       ax bx cx...         .  **
c     **                                       as per fort convention **
c     **     rlat(3,3) recip space lattice vectors (see above)           **
c     **     natoms  number of atoms                                  **
c     **     ntype   # of types of atoms                              **
c     **     vol     cell volume (A^3)                                **
c     **     atdpos(natoms,3)   position of atoms in fractional    **
c     **     atchg(natoms)      nominal charge of atoms by TYPE    **
c     **     attyp(natoms)        type of atom (type of at 1, etc.   **
c     **                                                              **
c     **   output variables                                           **
c     **                                                              **
c     **     eewalde   ewald energy in   eV                            **
c     **     ewaldf(3,natoms)   negative derivative of ewald          **
c     **                        energy                                **
c     **                          aka ewald forces ev/A               **
c     ******************************************************************
*/
void Ewald(const structure & in_str, double & epoint, 
	   double & ereal, double & erecip, double & eewald,
	   double & eta, const double & SUMTOL)
{
  structure str=in_str;
  str.PutInCell();
  matrix<double> lat = str.GetScaledLat();
  matrix<double> rlat = RecipLat(lat);
  int natoms=str.GetNumAtoms();
  double vol=GetVol(lat);
  matrix<double> atdpos=str.GetDpos();
  vector<int> attyp=str.GetTypes();
  vector<string> names=str.GetNames();
  // Charges should be stored in names - turn into vector of doubles.
  vector<double> atchg(natoms);
  double totchg=0;
  for (int i=0;i<names.size();i++){
    atchg[i]=atof(names[i].c_str());
    totchg=totchg+atchg[i];
  }

  // Constants
  double TOL=1e-6;

  // Check chg neutral
  if (abs(totchg)>TOL){
    cerr << "WARNING: Cell is not neutral" << endl;
    cerr << "WARNING: Total charge = " << totchg << endl;
    cerr << "WARNING: Convasp will use jellium background to force neutrality."<< endl;
  }

  // Get eta if it was not input (it is <0 if it needs to be found).
  if (eta<=0) {eta=GetEta(natoms,vol);}
  // Constants
  double rteta=sqrt(eta);
  // Point energy
  epoint=GetPointEner(rteta,atchg,vol);
  // Recipricol energy
  erecip=GetRecipEner(eta,atchg,vol,rlat,atdpos,SUMTOL);
  // Real space energy
  ereal=GetRealEner(eta,atchg,vol,lat,atdpos,SUMTOL);
  // Sum all terms
  eewald=(ereal+erecip-epoint);
}


// **************************************************
// GetEta
// **************************************************
double GetEta(const int & natoms,const double & vol){
  double term=pow((5.5*natoms/vol/vol),1.0/6.0);
  double eta=sqrt(PI)/1.2*term;
  eta=eta*eta;
  return eta;
}

// **************************************************
// GetPointEner
// **************************************************
// Gets point energy.
double GetPointEner(const double & rteta, const vector<double> & atchg, 
		    const double & vol){
  double ept=0;
  double chg_cell_corr=0;
  for (int ia=0;ia<atchg.size();ia++){
    ept=ept+atchg[ia]*atchg[ia];
    chg_cell_corr=chg_cell_corr+atchg[ia];
  }
  ept=ept*rteta/RTPI;
  chg_cell_corr=chg_cell_corr*PI/(2.0*vol*rteta*rteta);
  ept=ept+chg_cell_corr;
  ept=CONV_FACT*ept;
  return ept;
}

// **************************************************
// GetRecipEner
// **************************************************
// Gets recipricol energy.
double GetRecipEner(const double & eta, const vector<double> & atchg,
		    const double & vol,
		    const matrix<double> & rlat, 
		    const matrix<double> & atdpos,
		    const double & SUMTOL)
{
  double TOL=1e-16;
  double log_eps=-30; // This assumes sf<
  double arg=0;
  double erecip=0;
  int gcnt=1;
  int gcont=1; // whether or not to add more shells.
  while(gcont){
    double maxterm=0;
    vector<int> ig(3);
    for (ig[0]=-gcnt;ig[0]<=gcnt;ig[0]++){
      for (ig[1]=-gcnt;ig[1]<=gcnt;ig[1]++){
	for (ig[2]=-gcnt;ig[2]<=gcnt;ig[2]++){
	  if ( (abs(ig[0])==gcnt) || (abs(ig[1])==gcnt) ||
	       (abs(ig[2])==gcnt) || gcnt==1){ // Just does new shells
	    vector<double> gvec=VMmult(ig,rlat);
	    double gsq=VVdot(gvec,gvec);
	    if (gsq>TOL){ // Avoid gsq=0.
	      // get structure factor
	      double sfr=0;
	      double sfi=0;
	      for (int ia=0;ia<atdpos.size();ia++){
		double exparg=TWOPI*VVprod(atdpos[ia],ig);
		sfr=sfr+atchg[ia]*cos(exparg);
		sfi=sfi+atchg[ia]*sin(exparg);
	      }
	      if (fabs(sfr)<1e-16){sfr=0.0;}
	      if (fabs(sfi)<1e-16){sfi=0.0;}
	      double sf=sfr*sfr+sfi*sfi;
	      // Get expval.  In order not to get floating point
	      // errors due to manipulating small numbers we must
              // not evaluate the exp when it produces irrelevantly
	      // small arguments.  This happens when 
	      // expval*sf << eps (where safe eps=1e-30), or equivalently,
	      // when ln(sf)-ln(g^2)-g^2/(4eta) < ln(eps)
	      arg=gsq/(4.0*eta);
	      double expval;
	      double term1=0;
	      if (sf>0){
		term1=log(sf)-log(gsq)-gsq/(4*eta);
	      }
	      if (term1<log_eps){ // Avoids some floating point exceptions.
		expval=0;
              }
	      else{
		expval=exp(-arg)/gsq;
	      }
	      double term=expval*sf;
	      //	      if (term<1e-16){term=0;}
	      erecip=erecip+term;
	      // Set maxterm to max of abs(term) and expval.
	      // The expval term is needed since in some shells
	      // term may be very small due to coincidently small sf,
	      // even before convergence.
	      if (fabs(term)>maxterm) {maxterm=fabs(term);}
	      if (expval>maxterm) {maxterm=expval;}
	      // ADD FORCES ???
	    } // If gsq>TOL
	  } // If to do only new shells (ig==gcnt)
	} // ig0
      } // ig1
    } // ig2
    gcnt++; // Add new shell
    if (maxterm<SUMTOL) {gcont=0;} // Do not add another shell
  } // while gcont
  erecip=erecip*0.5*4.0*PI/vol; // CGS units, convert to eV at end.
  erecip=CONV_FACT*erecip;
  // FORCES STUFF HERE ???				  
  return erecip;				  
} // End GetRecipEner


// **************************************************
// GetRealEner
// **************************************************
// Gets real space energy.
double GetRealEner(const double & eta, const vector<double> & atchg,
		   const double & vol,
		   const matrix<double> & lat, 
		   const matrix<double> & atdpos,
		   const double & SUMTOL)
{
  double ereal=0;
  double TOL=1e-5;
  double log_eps=-30;
  double erfcarg=0;
  double rteta=sqrt(eta);
  int nat=atdpos.size();
  int rcnt=1;
  int rcont=1;
  while(rcont){
    double maxterm=0;
    vector<int> ir(3);
    for (ir[0]=-rcnt;ir[0]<=rcnt;ir[0]++){
      for (ir[1]=-rcnt;ir[1]<=rcnt;ir[1]++){
	for (ir[2]=-rcnt;ir[2]<=rcnt;ir[2]++){
	  if ( (abs(ir[0])==rcnt) || (abs(ir[1])==rcnt) ||
	       (abs(ir[2])==rcnt) || rcnt==1){ // Just does new shells
	    for (int ia=0;ia<nat;ia++){ // All atoms in unit cell
	      for (int ja=0;ja<nat;ja++){ // All neighbors in cell given by ir
		// Get displacement between atoms ia(cell 0) and ja(cell ir).
		vector<double> disp=VVdiff(atdpos[ja],atdpos[ia]);
		disp=VVsum(disp,ir);
		disp=vecD2C(lat,disp);
		double dist=norm(disp);
		if (dist>TOL){ // Avoid atom dist to itself.
		  erfcarg=rteta*dist;
		  // Get erfcval.  In order not to get floating point
		  // errors due to manipulating small numbers we must
		  // not evaluate the erfc when it produces irrelevantly
		  // small arguments.  This happens when 
		  // erfc(x)*abs(q) << eps (where safe eps=1e-30), or equivalently,
		  // when ln(abs(q))-x^2-ln(x)-0.5*ln(pi) < ln(eps) (where I have
		  // used the asymptotic form erfc(x)~e^-x^2/(x*rt(pi)) 
		  // for large x.
		  double erfcval;
		  double term1=atchg[ia]*atchg[ja]/dist;
		  double term2=0;
		  if (abs(term1)>0){
		    term2=log(abs(term1))-erfcarg*erfcarg-log(erfcarg)-0.5*log(PI);
		  }
		  if (term2<log_eps){ // Avoids some floating point exceptions.
		    erfcval=0;
		  }
		  else{
		    erfcval=erfc(erfcarg);
		  }
		  // tpx
		  // cout << "term1 erfcval " << term1 << " " << erfcval << endl;
		  double term=term1*erfcval;
		  ereal=ereal+term;
		  if (fabs(term)>maxterm) {maxterm=fabs(term);}
		  // FORCES ???
		} // rdist>TOL
	      } // ja
	    } // ia
	  } // if to do only new shells (ir==rcnt)
	} // rg0
      } // rg1
    } // rg2
    rcnt++; // Add new shell
    if (maxterm<SUMTOL){rcont=0;} // Do not add another shell
  } // while rcont
  ereal=ereal*0.5; // CGS units, convert to eV at end.
  ereal=CONV_FACT*ereal;
  return ereal;				  
}  

// **************************************************
// PrintEwald
// **************************************************
void PrintEwald(const structure & in_str, double & epoint, 
		double & ereal, double & erecip, double & eewald,
		double & eta, const double & SUMTOL, ostream & outf)
{
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  matrix<double> lat=in_str.GetScaledLat();
  int nat = in_str.GetNumAtoms();
  double vol=GetVol(lat);
  double d=pow(vol,1.0/3.0)*1E-10; // In meters
  double eV2NoUnit=2*TWOPI*EPS_VACUUM*d/(E_ELECTRON*nat);
  int wid;
  int prec;
  wid=15;
  prec=6;
  outf << "*************************** Ewald Results ***********************" << endl;
  outf << "E(NoUnit)=PerAtom(eV)*4*pi*eps_0*vol^1/3/e_electron=Charge/Distance unit independent."<< endl;
  outf << "Using eta= " << setprecision(2*prec) << eta << endl;
  outf << "All real and k-space sums stop when maximum term in shell is less than SUMTOL= " << setiosflags(ios::scientific) << SUMTOL << endl;
  outf << "SUMTOL is the following percentage of your total Ewald energy: " << setiosflags(ios::scientific) << 100*SUMTOL/eewald << endl;
  cout.setf(ios::fixed, ios::floatfield);
  outf << "                         " 
       << setw(wid) << "Total(eV)"
       << setw(wid) << "Per atom(eV)"
       << setw(wid) << "Total(NoUnit)" << endl;
  outf << "Point Energy:            " 
       << " " << setw(wid) << setprecision(prec) << epoint 
       << " " << setw(wid) << setprecision(prec) << epoint/nat
       << " " << setw(wid) << setprecision(prec) << eV2NoUnit*epoint 
       << endl;
  outf << "Real Space Energy:       " 
       << " " << setw(wid) << setprecision(prec) << ereal 
       << " " << setw(wid) << setprecision(prec) << ereal/nat
       << " " << setw(wid) << setprecision(prec) << eV2NoUnit*ereal 
       << endl;
  outf << "Recipricol Space Energy: " 
       << " " << setw(wid) << setprecision(prec) << erecip 
       << " " << setw(wid) << setprecision(prec) << erecip/nat
       << " " << setw(wid) << setprecision(prec) << eV2NoUnit*erecip 
       << endl;
  outf << "Total Ewald Energy:      " 
       << " " << setw(wid) << setprecision(prec) << eewald 
       << " " << setw(wid) << setprecision(prec) << eewald/nat
       << " " << setw(wid) << setprecision(prec) << eV2NoUnit*eewald 
       << endl;
}

// **************************************************
// GetSreenedESEner
// **************************************************
// Gets electrostatic energy with screening (does sum in real space).
double ScreenedESEner(const structure & in_str, const double & Ks, 
			 const double & SUMTOL)
{
  structure str=in_str;
  str.PutInCell();
  matrix<double> lat = str.GetScaledLat();
  int natoms=str.GetNumAtoms();
  double vol=GetVol(lat);
  matrix<double> atdpos=str.GetDpos();
  vector<int> attyp=str.GetTypes();
  vector<string> names=str.GetNames();
  // Charges should be stored in names - turn into vector of doubles.
  vector<double> atchg(natoms);
  double totchg=0;
  for (int i=0;i<names.size();i++){
    atchg[i]=atof(names[i].c_str());
    totchg=totchg+atchg[i];
  }
  double ereal=0;
  double TOL=1e-5;
  int nat=atdpos.size();
  int rcnt=1;
  int rcont=1;
  while(rcont){
    double maxterm=0;
    vector<int> ir(3);
    for (ir[0]=-rcnt;ir[0]<=rcnt;ir[0]++){
      for (ir[1]=-rcnt;ir[1]<=rcnt;ir[1]++){
	for (ir[2]=-rcnt;ir[2]<=rcnt;ir[2]++){
	  if ( (abs(ir[0])==rcnt) || (abs(ir[1])==rcnt) ||
	       (abs(ir[2])==rcnt) || rcnt==1){ // Just does new shells
	    for (int ia=0;ia<nat;ia++){ // All atoms in unit cell
	      for (int ja=0;ja<nat;ja++){ // All neighbors in cell given by ir
		// Get displacement between atoms ia(cell 0) and ja(cell ir).
		vector<double> disp=VVdiff(atdpos[ja],atdpos[ia]);
		disp=VVsum(disp,ir);
		disp=vecD2C(lat,disp);
		double dist=norm(disp);
		if (dist>TOL){ // Avoid atom dist to itself.
		  double term=exp(-Ks*dist)*atchg[ia]*atchg[ja]/dist;
		  ereal=ereal+term;
		  if (fabs(term)>maxterm) {maxterm=fabs(term);}
		  // FORCES ???
		} // rdist>TOL
	      } // ja
	    } // ia
	  } // if to do only new shells (ir==rcnt)
	} // rg0
      } // rg1
    } // rg2
    rcnt++; // Add new shell
    if (maxterm<SUMTOL){rcont=0;} // Do not add another shell
  } // while rcont
  ereal=ereal*0.5; // CGS units, convert to eV at end.
  ereal=CONV_FACT*ereal;
  return ereal;				  
}  







