// **************************************************
// GetChgInt
// **************************************************
//  This funtion reads in the CHGCAR file from standard input
//  and returns a matrix of charge integrals in the Voronoi volume around
//  each atom and a vector of matrices which gives, for each atom, the integrated
//  charges in a sphere vs. the radius of the sphere.
/*  Here is the format of a CHGCAR file (for v445 with spin polarization from DEC linux).

    Note that for Intel linux vasp445 the newlines after each chg density line seem to missing,
    however, the code still reads this in correctly.

    newline
    Starts with a POSCAR file
    newline
    NX NY NZ
    NX*NY*NZ entries with a newline after each one (grouped by 5) (Tot dens = pUP+pDn)
    Natom lines, each with 4 fields, with a newline after each one
    Natom entries (grouped by 5).
    NX NY NZ
    NX*NY*NZ entries with a newline after each one (grouped by 5) (Diff dens = pUP-pDN)
    Natom lines, each with 4 fields, with a newline after each one
    Natom entries (grouped by 5).
*/ 
    
#include <math.h>
#include <string>
#include <vector>
#include "UsefulFuncs.h"
#include "GetChgInt.h"
#include "structure.h"
#include "structure.h"


void GetChgInt(vector<matrix<double> > & rad_chg_int, matrix<double> & vor_chg_int)
{
  // Read in CHGCAR
  // Read in intial POSCAR format at beginning of file
  structure str;
  cin >> str;
  // Read in the grid for total charge
  vector<int> ngrid(3);
  int npts;
  cin >> ngrid[0] >> ngrid[1] >> ngrid[2];
  npts=ngrid[0]*ngrid[1]*ngrid[2];
  // Read in the total charge
  vector<double> chg_tot(npts,0.0);
  for (int i=0;i<npts;i++){
    cin >> chg_tot[i];
  }
  // Read in dummy fields
  // If first field is augmentation then read 4*natoms+natoms-1 more fields. For vasp 4.4.5.
  // If first field is not augmentation then read natoms-1 more fields. For vasp 4.4.1.
  string ds;
  int natoms=str.GetNumAtoms();
  cin >> ds;
  string keyword = "augmentation";
  if (ds==keyword) {
    for (int i=0;i<(5*natoms-1);i++){
      cin >> ds;
    }
  }
  else{
    for (int i=0;i<(natoms-1);i++){
      cin >> ds;
    }
  } 

  // Read in the grid for diff charge
  cin >> ngrid[0] >> ngrid[1] >> ngrid[2];
  npts=ngrid[0]*ngrid[1]*ngrid[2];
  // Read in the diff charge
  vector<double> chg_diff(npts,0.0);
  for (int i=0;i<npts;i++){
    cin >> chg_diff[i];
  }
  // Read in 4*natoms=4*natoms dummy fields.  This is 
  // not necessary and the lines only exist in vasp 4.4.5.
  /*
  for (int i=0;i<4*natoms;i++){
    cin >> ds;
  }
  */

  // Loop over grid and bin charge densities.

  // Radial charge integration.
  // Integral over a sphere will be done by setting up NRBIN bins, each
  // of witdh DR, our to RMAX.  Then each charge at each point will be put is the
  // appropriate bin for each atom.  Note that the points and all possible relevant
  // images must be looped over.
  double DR=0.05; // width of radial bins.
  double RMAX=3; // Max distance to bin to.
  int NRBIN=(int)(RMAX/DR); // First bin is 0->DR, last bin is RMAX-DR->RMAX
  matrix<double> dum_mat(NRBIN,5,0.0);
  rad_chg_int = vector<matrix<double> > (natoms,dum_mat);
  vector<int> ig(3);
  // Determine the number of images along each lattice vector which must be considered.
  // You will center the charge density on each atom so the max number of images along a 
  // given lattice param should be no more than however many lattice params are needed 
  // to make sure that you are more than RMAX from an atom along that lattice param direction.
  vector<int> imax(3);
  double scale=str.GetScale();
  matrix<double> lat=str.GetLat();
  for (int ic=0;ic<3;ic++){
    imax[ic]=(int)(RMAX/(scale*norm(lat[ic]))+1)*ngrid[ic];
  }
  double sumchg=0;
  // Atom loop
  matrix<double> at_dpos=str.GetDpos();
  for (int iat=0;iat<natoms;iat++){
    // Initialize radial positions in rad_chg_int    
    for (int ib=0;ib<NRBIN;ib++){
      rad_chg_int[iat][ib][0]=(ib+1)*DR;
    }
    // Find location of atom on grid.
    vector<int> at_grid_pos(3);
    for (int ic=0;ic<3;ic++){
      at_grid_pos[ic]=Nint(at_dpos[iat][ic]*ngrid[ic]);
    }
    // Grid loop: This loops over grid points shifted to center around grid point approximation to 
    // position of iat.  The ig values therefore need to be modified to be true grid points 
    // (true_ig) by undoing the shift (adding back the at_grid_pos).
    for (ig[0]=-imax[0];ig[0]<imax[0];ig[0]++){
      for (ig[1]=-imax[1];ig[1]<imax[1];ig[1]++){
	for (ig[2]=-imax[2];ig[2]<imax[2];ig[2]++){
	  // Get true grid points
	  vector<int> true_ig(3);
	  vector<double> chg_dpos(3);
	  for (int ic=0;ic<3;ic++){
	    true_ig[ic]=ig[ic]+at_grid_pos[ic];
            // Get true position of this grid point in direct coords.
	    chg_dpos[ic]=((float)true_ig[ic]/(float)ngrid[ic]);
            // Shift true grid point back into the 000 cell.
	    true_ig[ic]=true_ig[ic]-(int)((float)true_ig[ic]/(float)ngrid[ic])*ngrid[ic];
	    if (true_ig[ic]<0) true_ig[ic]=true_ig[ic]+ngrid[ic];
	  } // ic
	  // Get charge values for each grid position
	  int id=true_ig[0]+true_ig[1]*ngrid[0]+true_ig[2]*ngrid[0]*ngrid[1];
	  double chgtot=chg_tot[id];
	  double chgdiff=chg_diff[id];
	  double chgup=0.5*(chg_tot[id]+chg_diff[id]);
	  double chgdn=0.5*(chg_tot[id]-chg_diff[id]);
	  vector<double> dpos(3);
  	  // Get distance from iat to true grid point position.
          double dist=str.GetDistToAtomDir(chg_dpos,iat);
	  // Get bin for this atom and increment radial charge density.
	  int ibin=(int)(dist/DR);	  
	  if (ibin<NRBIN){
	    rad_chg_int[iat][ibin][1]=rad_chg_int[iat][ibin][1]+chgtot;
	    rad_chg_int[iat][ibin][2]=rad_chg_int[iat][ibin][2]+chgdiff;
	    rad_chg_int[iat][ibin][3]=rad_chg_int[iat][ibin][3]+chgup;
	    rad_chg_int[iat][ibin][4]=rad_chg_int[iat][ibin][4]+chgdn;
	  } // if ibin<NRBIN
	} // ig[2]
      } // ig[1]
    } // ig[0]
  } // iat

// Voronoi charge integration
  vor_chg_int = matrix<double> (natoms,4,0.0);
  for (ig[0]=0;ig[0]<ngrid[0];ig[0]++){
    for (ig[1]=0;ig[1]<ngrid[1];ig[1]++){
      for (ig[2]=0;ig[2]<ngrid[2];ig[2]++){
	// Get charge values for this grid position
	int id=ig[0]+ig[1]*ngrid[0]+ig[2]*ngrid[0]*ngrid[1];
	double chgtot=chg_tot[id];
	double chgdiff=chg_diff[id];
	double chgup=0.5*(chg_tot[id]+chg_diff[id]);
	double chgdn=0.5*(chg_tot[id]-chg_diff[id]);
	vector<double> dpos(3);
	// Set up grid positions in direct coordinates.
	for (int ic=0;ic<3;ic++){
	  dpos[ic]=((float)ig[ic])/(float)ngrid[ic];
	} // ic
	// Loop over atoms
        double dist;
	double min_dist=str.GetDistToAtomImageDir(dpos,0);
	int min_dist_id=0;
	for (int iat=0;iat<natoms;iat++){
  	  // Get "image" distance to each atom (distance to closest image)
          dist=str.GetDistToAtomImageDir(dpos,iat);
          // Keep track of which atom is the minimum "image" distance away
	  if (dist<min_dist){
            min_dist=dist;
	    min_dist_id=iat;
	  } // if dist<min_dist
	} // iat 
	vor_chg_int[min_dist_id][0]=vor_chg_int[min_dist_id][0]+chgtot;
	vor_chg_int[min_dist_id][1]=vor_chg_int[min_dist_id][1]+chgdiff;
	vor_chg_int[min_dist_id][2]=vor_chg_int[min_dist_id][2]+chgup;
	vor_chg_int[min_dist_id][3]=vor_chg_int[min_dist_id][3]+chgdn;
      } // ig[2]
    } // ig[1]
  } // ig[0]


// Normalize and turn rad_chg_int from chg in each radial shell to cumulative integral.
  for (int iat=0;iat<natoms;iat++){
    for (int i=0;i<4;i++){
     vor_chg_int[iat][i]=vor_chg_int[iat][i]/npts;
    }
    for (int i=1;i<5;i++){
      for (int ib=0;ib<NRBIN;ib++){
	rad_chg_int[iat][ib][i]=rad_chg_int[iat][ib][i]/npts;
	if (ib>0) rad_chg_int[iat][ib][i]=(rad_chg_int[iat][ib][i]+rad_chg_int[iat][ib-1][i]);
       } // ib
    } // for loop over tot,diff,up,dn
  } // iat

}



