// poscar object. 

using namespace std;
#include <cmath>
#include <algorithm>
#include "constants.h"
#include <sstream>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <complex>
#include "UsefulFuncs.h"
#include "CellReduceFuncs.h"
#include "GaussJordan.h"

// **************************************************
// Function norm
// **************************************************
/*  This function finds the norm of a vector */
  double norm(const vector<double> & v)
{
  double sum=0;
  for (int i=0;i<v.size();i++){
    sum=sum+v[i]*v[i];
  }
  sum=sqrt(fabs(sum));
  return sum;
}

// **************************************************
// Function UnitVec
// **************************************************
/*  This function finds the unit vector along a vector */
  vector<double> UnitVec(const vector<double> & v)
{
  double n=norm(v);
  return SVprod(1/n,v);
}

// **************************************************
// Function getcos
// **************************************************
/*  This function finds the cosine of the angle between 
   two vectors. */
  double getcos(const vector<double> & a, const vector<double> & b)
{
  double sum=0;
  int size=min(a.size(),b.size());
  for (int i=0;i<size;i++){
    sum=sum+a[i]*b[i];
  }
  double na=norm(a);
  double nb=norm(b);
  assert(na>0 && nb>0);
  sum=sum/(na*nb);
  return sum;
}

// **************************************************
// Function MMmult
// **************************************************
/*  This function returns the product of two matrices.
   a=MxN, b=NxM. */
  vector<vector<double> > MMmult(const vector<vector<double> >& a, const vector<vector<double> > & b)
{
  int M=(a.size(),b[0].size());
  int N=min(a[0].size(),b.size());
  vector<double> v(M,0.0);
  vector<vector<double> > c(M,v);
  for (int i=0;i<M;i++){
    for (int j=0;j<M;j++){
      double sum=0.0;
      for (int k=0;k<N;k++){
	sum=sum+a[i][k]*b[k][j];
      }
      c[i][j]=sum;
    }
  }
  return c;
}

// **************************************************
// Function MVmult
// **************************************************
/*  This function returns the product of a matric and a vector A*v.
   A=MxN, v=Nx1. */
  vector<double> MVmult(const vector<vector<double> >& A, const vector<double> & v)
{
  int M=A.size();
  int N=min(A[0].size(),v.size());
  vector<double> u(M,0.0);
  for (int i=0;i<M;i++){
    double sum=0;
    for (int j=0;j<N;j++){
      sum=sum+A[i][j]*v[j];
    }
    u[i]=sum;
  }
  return u;
}

// **************************************************
// Function MVmult
// **************************************************
/*  This function returns the product of a matric and a vector A*v.
   A=MxN, v=Nx1. */
  vector<double> MVmult(const matrix<double> & A, const vector<double> & v)
{
  int M=A.size();
  int N=min(A[0].size(),v.size());
  vector<double> u(M,0.0);
  for (int i=0;i<M;i++){
    double sum=0;
    for (int j=0;j<N;j++){
      sum=sum+A[i][j]*v[j];
    }
    u[i]=sum;
  }
  return u;
}

// **************************************************
// Function VVcross
// **************************************************
/*  This function returns the cross product of two 3 dimensional vectors. */

  vector<double> VVcross(const vector<double> & a, const vector<double> & b)
{
  if (a.size()!=3 || b.size()!=3) return a; 
  vector<double> u(3);
  u[0]=a[1]*b[2]-a[2]*b[1];
  u[1]=-(a[0]*b[2]-a[2]*b[0]);
  u[2]=a[0]*b[1]-a[1]*b[0];
  return u;
}

// **************************************************
// Function VVdot
// **************************************************
/*  This function returns the dot product of two vectors. */
  double VVdot(const vector<double> & a, const vector<double> & b)
{
  double sum=0;
  int size=min(a.size(),b.size());
  for (int i=0;i<size;i++){
    sum=sum+a[i]*b[i];
  }
  return sum;
}

// **************************************************
// Function GetVol
// **************************************************
/*  This function returns the volume of the cell from the lattice parameters */
  double GetVol(const matrix<double> & lat)
{
  double vol;
  vector<double> u(3);
  u=VVcross(lat[0],lat[1]);
  vol=VVdot(u,lat[2]);
  return fabs(vol);
}

// **************************************************
// Function GetSignedVol
// **************************************************
/*  This function returns the volume of the cell from the lattice parameters */
  double GetSignedVol(const matrix<double> & lat)
{
  double vol;
  vector<double> u(3);
  u=VVcross(lat[1],lat[2]);
  vol=VVdot(lat[0],u);
  return vol;
}

// **************************************************
// Function Getabc_angles
// **************************************************
/*  This function returns a,b,c,alpha,beta,gamma for the cell 
  given the lattice vectors. */
  vector<double> Getabc_angles(const matrix<double> & lat)
{
  vector<double> data(6,0.0);
  data[0]=norm(lat[0]);
  data[1]=norm(lat[1]);
  data[2]=norm(lat[2]);
  data[3]=acos(getcos(lat[1],lat[2]));
  data[4]=acos(getcos(lat[0],lat[2]));
  data[5]=acos(getcos(lat[0],lat[1]));
  return data;
}

// **************************************************
// Function IsTranslationVector
// **************************************************
/*  This function returns 1 if the given vector is a translation
   vector of the given structure and 0 otherwise. Input translation
   vector is expectd to be in cartesian coordinates. */
  int IsTranslationVector(const structure & str, const vector<double> & ctvec)
{
  int IsTrans=0;
  int CntGoodTrans=0;
  int new_type=-1;
  int old_type=-1;
  double tol=1e-10;
  int num_atoms=str.GetNumAtoms();
  matrix<double> lat=str.GetLat();
  matrix<double> dpos=str.GetDpos();
  vector<double> tdpos(3,0.0);
  vector<int> types;
  types=str.GetTypes();
  vector<double> tvec(3);
  tvec=vecC2D(lat,ctvec); // Put Cartesian input into direct coords.
  for (int iat=0;iat<num_atoms;iat++){
    old_type=types[iat];
    // Get translated position and shift back into unit cell.
    for (int ic=0;ic<3;ic++){
      tdpos[ic]=dpos[iat][ic]+tvec[ic];
      tdpos[ic]=tdpos[ic]-int(tdpos[ic]);
      if (tdpos[ic]<0.0) tdpos[ic]=tdpos[ic]+1;
    }
    // Find closest atom in unit cell to translated position (should only
    // be one atom less than tol).  
    // Use the type of the atom at this positions to get new_type.
    for (int jat=0;jat<num_atoms;jat++){
      vector<double> diff(3);
      for (int ic=0;ic<3;ic++){
	diff[ic]=tdpos[ic]-dpos[jat][ic];
      }
      // If the translated atom maps onto another and its type is 
      // the same as the atom mapped onto then increment CntGoodTrans=1.  
      // Otherwise assume we either failed to map onto any atom or it 
      // had a different type and don't increment CntGoodTrans.
      if (norm(diff)<tol && types[jat]==old_type){CntGoodTrans++;}

// tpx
      /*
  cout << endl;
  cout << "Vector " << tvec[0]<<" "<<tvec[1]<<" "<<tvec[2]<<endl;
  cout << "IsTrans: " <<" iat: "<<iat<<" jat: "<<jat<<" oldtype: "<<old_type<<" new_type: "<<new_type<<endl;
  if (norm(diff)<tol) {
    cout << "  Checking these two types  " << endl;
    cout << "     iat " << iat << endl;
    cout << "     jat " << jat << endl;
    cout << "     tranlated iat pos " << tdpos[0]<<" "<<tdpos[1]<<" "<<tdpos[2]<<endl;
    cout << "     jat pos " << dpos[jat][0]<<" "<<dpos[jat][1]<<" "<<dpos[jat][2]<<endl;
    cout << "     diff " << diff[0]<<" "<<diff[1]<<" "<<diff[2]<<endl;
  }
  */

    } // For jat
  } // For iat
  // If we counted a good translation for every atoms then we have
  // a translation vector.
  if (CntGoodTrans==num_atoms) IsTrans=1;
  return IsTrans;
}

// **************************************************
// Function vecC2D
// **************************************************
/*  This function converts a vector in cartesian to direct */
  vector<double> vecC2D(const matrix<double> & lat, const vector<double> & vc)
{
  vector<double> vd(3,0.0);
  matrix<double> latT=lat.transpose();
  matrix<double> vcmat;
  vcmat.vec2mat(vc);
  matrix<double> b=vcmat.transpose();
  GaussJordan(latT,b);
  vd=b.transpose()[0];
  return vd;
}

// **************************************************
// Function vecD2C
// **************************************************
/*  This function converts a vector in direct to cartesian */
  vector<double> vecD2C(const matrix<double> & lat, const vector<double> & vd)
{
  vector<double> vc(3,0.0);
  for (int ic=0;ic<3;ic++){
    for (int jc=0;jc<3;jc++){
      vc[ic]=vc[ic]+vd[jc]*lat[jc][ic];
    }
  }
  return vc;
}

// **************************************************
// Function GetClat
// **************************************************
/*  This function returns cartesian lattice vectors for the cell given 
    a,b,c,alpha,beta,gamma. Assumes angles are in radians.
    This routine aligns a along +X, b in XY plane with
    angle gamma from a in +Y direction (bx=b.a_unit=b*cos[gamma], 
    by=sqrt(b^2-bx^2)=b*sin[gamma]), and c is then given by 
    cx=c.a_unit=c*cos[beta], 
    cy=c.by_unit=c*(cos[alpha]-cos[gamma]*cos[beta])/sin[gamma],
    cz=sqrt(c^2-cx^2-cy^2) */
  matrix<double> GetClat(const vector<double> & abc_angles)
{
  double TWOPI=2.0*3.14159265359;
  matrix<double> clat(3,3,0.0);
  double a = abc_angles[0];
  double b = abc_angles[1];
  double c = abc_angles[2];
  double al= abc_angles[3]; // angle from b to c
  double be= abc_angles[4]; // angle from a to c
  double ga= abc_angles[5]; // angle from a to b
  /*
  // Convert to radians.
  al=al*TWOPI/360.0;
  be=be*TWOPI/360.0;
  ga=ga*TWOPI/360.0;
  */
  clat[0][0]=a;
  clat[1][0]=b*cos(ga);
  clat[1][1]=b*sin(ga);
  clat[2][0]=c*cos(be);
  if (ga<0.00000001) {
    cout << "ERROR: The angle gamma from a to b is too small" << endl;
    cout << "ERROR: gamma = " << ga << endl;
    cout << "ERROR: STOPPING "<< endl;
    exit(1);
  }
  clat[2][1]=c*(cos(al)-cos(ga)*cos(be))/sin(ga);
  clat[2][2]=sqrt(fabs(c*c-clat[2][1]*clat[2][1]-clat[2][0]*clat[2][0]));
  return clat;
}


// **************************************************
// Function Nint
// **************************************************
//  This function returns the nearest integer to a number
int Nint(double x)
{
  int i;
  if (x>=0){
    i=(int)(x+0.5);
  }else{
    i=(int)(x-0.5);
  }
  return i;
}

// **************************************************
// Function AtomCDisp
// **************************************************
//  This function returns the Cartesian displacement vector at2-at1
// between two atoms.
vector<double> AtomCDisp(const atom & at1, const atom & at2)
{
  vector<double> pos1=at1.GetCpos();
  vector<double> pos2=at2.GetCpos();
  vector<double> diff(3);
  for (int i=0;i<3;i++){
    diff[i]=pos2[i]-pos1[i];
  }
  return diff;
}

// **************************************************
// Function AtomDist
// **************************************************
//  This function returns the distance between two atoms.
double AtomDist(const atom & at1, const atom & at2)
{
  return norm(AtomCDisp(at1,at2));
}

// **************************************************
// COMPARE_GetNeighData
// **************************************************
// For sort algorithm in GetNeighData
class compare_GetNeighData{
public:  
  int operator()(const atom & a, const atom & b)
  {
    double tol=1e-15;
    // Sort by distance
    if (fabs(a.GetDistFromOrigin()-b.GetDistFromOrigin())<=tol){
      // Sort by unit cell values
      if (a.GetName()==b.GetName()){
	vector<int> uca=a.GetUnitCell();
	vector<int> ucb=b.GetUnitCell();
	int va=100*uca[0]+10*uca[1]+1*uca[2];
	int vb=100*ucb[0]+10*ucb[1]+1*ucb[2];
	return va<vb;
      }
      else{
	return a.GetName()<b.GetName();
      }
    }
    else{
      return a.GetDistFromOrigin()<b.GetDistFromOrigin();
    }
    /*
    // Sort by name
    if (a.GetName()==b.GetName()){
      return a.GetDistFromOrigin()<b.GetDistFromOrigin();
    }
    else{
      return a.GetName()<b.GetName();
    }
    */
  }
};


// **************************************************
// Function GetStrNeighData
// **************************************************
/*  This function collects all the neighbor data out to some 
    cutoff and stores it for each atom in the structure.
    */
void GetStrNeighData(const structure & str, const double cutoff, 
		  matrix<atom> & neigh_mat)
{
  vector<atom> atom_vec;

  // Get data from str.
  // Set scale to 1 so you don't need to rescale coordinates.
  structure sstr=str;
  sstr.ReScale(1.0);
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  vector<int> types = sstr.GetTypes();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();

  // Create atom objects for each atom in structure.
  vector<int> uc(3,0);
  for (int iat=0;iat<num_atoms;iat++){
    atom a;
    a.SetName(names[iat]);
    a.SetNum(iat);
    a.SetUnitCell(uc);
    a.SetCpos(cpos[iat]);
    a.SetDpos(dpos[iat]);
    a.SetType(types[iat]);
    atom_vec.push_back(a);
  }
  double rmin=1e-6;
  GetNeighData(atom_vec,sstr,rmin,cutoff,neigh_mat);
}

// **************************************************
// Function GetGoodShellPoints
// **************************************************
/*  This function collects the information about atoms
    within a shell around a set of points.  The points are
    input in fractional coordinates.  The points with the
    correct shells (ns atoms between rmin and rmax) are
    stored, with their neighbors, as atoms in neigh_mat.
*/
void GetGoodShellPoints(matrix<double> & points, const structure & str, 
			const int & ns, const double & rmin, 
			const double & rmax, const string & sname,
			matrix<atom> & neigh_mat)
{
  structure sstr=str;
  sstr.ReScale(1.0);
  matrix<double> lat=sstr.GetLat();

  // define fake atoms for each point
  vector<atom> atvec;
  for (int ipt=0;ipt<points.size();ipt++){
    atom a;
    // Set positions, other defaults are fine for a.
    vector<double> dpt=points[ipt];
    a.SetDpos(dpt);
    a.SetCpos(vecD2C(lat,dpt));
    atvec.push_back(a);
  }

  // Send atoms to GetNeighData to get their neigh info.
  GetNeighData(atvec,str,rmin,rmax,neigh_mat);

  // Remove every point that does not have ns atoms in shell.
  matrix<atom> nnm;
  for (int ip=0;ip<neigh_mat.size();ip++){
    if (neigh_mat[ip].size()==(ns+1)){ // size is number of neighbors + the point.
      nnm.push_back(neigh_mat[ip]);
    }
  }
  // Remove every point that does not have neighbors with the required name.
  // If sname==NONE then keep all the points.
  matrix<atom> nnnm;
  string none="NONE";
  if (!(sname==none)){
    for (int ip=0;ip<nnm.size();ip++){
      int names_match=1;
      for (int in=1;in<nnm[ip].size();in++){
	if (!(sname==nnm[ip][in].GetName())){
	  names_match=0;
	}
      } // for in
      if (names_match) nnnm.push_back(nnm[ip]);
    } // for ip
    neigh_mat=nnnm;
  } // if sname
  else{
    neigh_mat=nnm;
  } // if sname
}

  

// **************************************************
// Function GetNeighData
// **************************************************
/*  This function collects all the neighbor data between
    rmina dn rmax and stores it for each atom in a vector of atom
    objects in order of increasing distance.  
*/
void GetNeighData(const vector<atom> & in_atom_vec, const structure & in_str, 
		  const double & rmin, const double & rmax, 
		  matrix<atom> & neigh_mat)
{
  vector<atom> neigh_vec;
  // Get data from str.
  structure sstr=in_str;
  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  // Use Niggli cell to avoid missing neighbors in angular cells. 
  // This involves converting your atoms into the 0th Niggli cell, 
  // keeping track of the unit cell you came from, and then shifting back 
  // at the end.  This shifting must be done for both atom_vec (at the
  // beginning) and neigh_mat (at the end).
  matrix<double> init_lat=sstr.GetLat();
  sstr=GetNiggliStr(sstr);  
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  vector<int> types = sstr.GetTypes();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
     
  //  Convert input atom_vec to Niggli lattice
  vector<atom> atom_vec=in_atom_vec;
  matrix<int> atom_shifts(atom_vec.size(),3,0);
  for (int ia=0;ia<atom_vec.size();ia++){
    atom a=atom_vec[ia];
    vector<double> p_cell0;
    GetUnitCellRep(a.GetCpos(),p_cell0,atom_shifts[ia],lat,1);
    atom_vec[ia].SetCpos(p_cell0);
    atom_vec[ia]=ConvertAtomToLat(atom_vec[ia],lat);
  }    

  /*
  Create vector of all atoms in all periodic images needed so that 
  for atoms in the unit cell every possible neighbor within rmax
  is included.  This can be done by looping over all atoms in 
  all unit cells such that if the unit cells are given by 
  n0*v0,n1*v1,n2*v2, then ni_max*vi>rmax for all i=0->2.  This is not
  rigorous but seems to work quite well.  It failed for very
  angular cells, but by using the Niggli reduced cell this pitfall
  seems to be avoided (but no proof, so be careful).
  */
  // Find imax
  vector<double> vec(3);
  vec[0]=rmax/norm(lat[0]);
  vec[1]=rmax/norm(lat[1]);
  vec[2]=rmax/norm(lat[2]);
  //  int imax=(int)max_element(vec.begin(),vec.end())+1; Doesn't work for some reason.
  sort(vec.begin(),vec.end());
  int imax=(int)vec[2]+1;
  vector<int> n(3);
  vector<atom> all_atom_vec;
  for (n[0]=-imax;n[0]<=imax;n[0]++){
    for (n[1]=-imax;n[1]<=imax;n[1]++){
      for (n[2]=-imax;n[2]<=imax;n[2]++){
	vector<double> tcpos(3);
	vector<double> tdpos(3);
	for (int iat=0;iat<num_atoms;iat++){
	  atom a;
	  a.SetName(names[iat]);
	  a.SetNum(iat);
	  a.SetUnitCell(n);
	  for (int ic=0;ic<3;ic++){
	    tcpos[ic]=cpos[iat][ic]
	      +n[0]*lat[0][ic]+n[1]*lat[1][ic]+n[2]*lat[2][ic];
	  }
	  tdpos=vecC2D(lat,tcpos);
	  a.SetCpos(tcpos);
	  a.SetDpos(tdpos);
	  a.SetType(types[iat]);
	  all_atom_vec.push_back(a);
	} // iat
      } // n2
    } // n1
  } // n0
  
  // Now build neighbors list for each atom on atom list.  Each
  // neighbor list will be row in the neigh_mat matrix.
  for (int ia1=0;ia1<atom_vec.size();ia1++){
    vector<double> corigin(3);
    atom at=atom_vec[ia1];
    corigin = at.GetCpos();
    at.SetCOrigin(corigin);
    vector<atom> neigh_vec;
    neigh_vec.push_back(at);
    for (int ia2=0;ia2<all_atom_vec.size();ia2++){
      double dist = AtomDist(at,all_atom_vec[ia2]);
      if (dist<=rmax && dist>=rmin){
	all_atom_vec[ia2].SetCOrigin(corigin);
	neigh_vec.push_back(all_atom_vec[ia2]);
      } // if
    } // ia2
    sort(neigh_vec.begin()+1,neigh_vec.end(),compare_GetNeighData());
    neigh_mat.push_back(neigh_vec);
  } // ia1

  //  Convert neigh_mat from Niggli cell to original lattice
  for (int ia=0;ia<neigh_mat.size();ia++){
    for (int ja=0;ja<neigh_mat[ia].size();ja++){
      vector<double> dpos(3);
      vector<double> cpos(3);
      atom a;
      a=neigh_mat[ia][ja];

      dpos=a.GetDpos();
      for (int ic=0;ic<3;ic++){
	dpos[ic]=dpos[ic]+atom_shifts[ia][ic];
      }
      cpos=vecD2C(lat,dpos);
      a.SetCpos(cpos);
      neigh_mat[ia][ja]=ConvertAtomToLat(a,init_lat);
    }
  }

}

/*
**************************************************
 Function SVprod
**************************************************
 This function returns the product of a scalar and a vector.
*/
 vector<double> SVprod(const double & s, const vector<double> & b)
{
  int size=b.size();
  vector<double> prod(size);
  for (int i=0;i<size;i++){
    prod[i]=s*b[i];
  }
  return prod;
}

/*
**************************************************
 Function VVsum
**************************************************
 This function returns the vector sum c=a+b.
*/
vector<double> VVsum(const vector<double> & a, const vector<double> & b)
{
  int size=min(a.size(),b.size());
  vector<double> c(size);
  for (int i=0;i<size;i++){
    c[i]=a[i]+b[i];
  }
  return c;
}

/*
**************************************************
 Function VVsum
**************************************************
 This function returns the vector sum c=a+b.
*/
vector<double> VVsum(const vector<double> & a, const vector<int> & b)
{
  int size=min(a.size(),b.size());
  vector<double> c(size);
  for (int i=0;i<size;i++){
    c[i]=a[i]+b[i];
  }
  return c;
}

/*
**************************************************
 Function VVdiff
**************************************************
 This function returns the vector c=a-b.
*/
vector<double> VVdiff(const vector<double> & a, const vector<double> & b)
{
  int size=min(a.size(),b.size());
  vector<double> c(size);
  for (int i=0;i<size;i++){
    c[i]=a[i]-b[i];
  }
  return c;
}

/*
**************************************************
 Function VVprod
**************************************************
 This function returns the scalar c=a*b.
*/
double VVprod(const vector<double> & a, const vector<double> & b)
{
  int size=min(a.size(),b.size());
  double c=0;
  for (int i=0;i<size;i++){
    c=c+a[i]*b[i];
  }
  return c;
}

/*
**************************************************
 Function VVprod
**************************************************
 This function returns the scalar c=a*b.
*/
double VVprod(const vector<double> & a, const vector<int> & b)
{
  int size=min(a.size(),b.size());
  double c=0;
  for (int i=0;i<size;i++){
    c=c+a[i]*b[i];
  }
  return c;
}

matrix<double> RecipLat(const matrix<double> & lat)
{
  matrix<double> rlat (3,3);
  double vol = GetSignedVol(lat);
  rlat[0]=SVprod(2.0*PI/vol,VVcross(lat[1],lat[2]));
  rlat[1]=SVprod(2.0*PI/vol,VVcross(lat[2],lat[0]));
  rlat[2]=SVprod(2.0*PI/vol,VVcross(lat[0],lat[1]));
  return rlat;
}

void GetXray(const structure & str, vector<double> & dist,
	     vector<double> & sf, const double & lambda,
	     vector<double> & scatt_fact, vector<double> & mass,
	     vector<double> & twoB_vec)
{
  // Get data from str.
  // Set scale to 1 so you don't need to rescale coordinates.
  structure sstr=str;
  sstr.ReScale(1.0);

  matrix<double> lat=sstr.GetLat();
  matrix<double> rlat=RecipLat(lat);
  int num_atoms=sstr.GetNumAtoms();
  vector<string> names=sstr.GetNames();
  matrix<double> dpos=sstr.GetDpos();

  // Set parameters for Debye Waller factors.
  double temp=300;
  double debye_temp=300;
  double dw,theta;

  // Get scattering factors and masses and Debye-Waller 2B values.
  double h=PLANKSCONSTANT_h;
  for (int i=0;i<num_atoms;i++){
    scatt_fact[i]=sstr.GetXrayScattFactor(names[i],lambda);
    mass[i]=sstr.GetAtomMass(names[i]);
    twoB_vec[i]=h*h*temp*12.0/(mass[i]*KBOLTZ*debye_temp*debye_temp);
  } 


 //  Get max h,k,l value that gives any scattering (theta<=90).
  int kmx0=(int) (4.0*PI/(norm(rlat[0])*lambda)+1);
  int kmx1=(int) (4.0*PI/(norm(rlat[1])*lambda)+1);
  int kmx2=(int) (4.0*PI/(norm(rlat[2])*lambda)+1);
  int kmx=max(kmx0,kmx1);
  kmx=max(kmx2,kmx);
  int len=2*kmx+1;
  int tlen=len*len*len;  
  double sfr,sfi;
  dist = vector<double> (tlen,0.0);
  sf = vector<double> (tlen,0.0);

  for (int i0=-kmx;i0<=kmx;i0++){
    for (int i1=-kmx;i1<=kmx;i1++){
      for (int i2=-kmx;i2<=kmx;i2++){
        int ii0=i0+kmx;
        int ii1=i1+kmx;
        int ii2=i2+kmx;
        int id=ii2+ii1*len+ii0*len*len;
        sfr=0.0;
        sfi=0.0;
        vector<double> rv(3,0.0);
        for (int ic=0;ic<3;ic++){
          rv[ic]=i0*rlat[0][ic]+i1*rlat[1][ic]+i2*rlat[2][ic];
        }
        double rvnorm=norm(rv);
        if (rvnorm>0) dist[id]=TWOPI/rvnorm;
        for (int iat=0;iat<num_atoms;iat++){
	  // Get Debye Waller factor.
	  dw=1;
	  if (dist[id]>0){
	    double term=lambda/(2.0*dist[id]);
	    if (term<=1){
	      theta=asin(term);
	      theta=theta*360.0/TWOPI; // rad->degrees
	      dw=DebyeWallerFactor(theta,lambda*1.0E-10,temp,debye_temp,mass[iat]);
	    }
	  }
          double gdotr=i0*dpos[iat][0]+i1*dpos[iat][1]+i2*dpos[iat][2];
          gdotr=TWOPI*gdotr;
          sfr=sfr+dw*scatt_fact[iat]*cos(gdotr);
          sfi=sfi-dw*scatt_fact[iat]*sin(gdotr);
        }
        sf[id]=sfr*sfr+sfi*sfi;
      } // i2
    } // i1
  } // i0

}

/*
**************************************************
 Function Vout
**************************************************
 This function outputs a vector.
*/
void Vout(const vector<double> & a, ostream & out)
{
  int size=a.size();
  for (int i=0;i<size;i++){
    out << a[i] << " ";
  }
  out << endl;
}

/*
**************************************************
 Function Vout
**************************************************
 This function outputs a vector.
*/
void Vout(const vector<int> & a, ostream & out)
{
  int size=a.size();
  for (int i=0;i<size;i++){
    out << a[i] << " ";
  }
  out << endl;
}

/*
**************************************************
 Function Vout
**************************************************
 This function outputs a vector.
*/
void Vout(const vector<string> & a, ostream & out)
{
  int size=a.size();
  for (int i=0;i<size;i++){
    out << a[i] << " ";
  }
  out << endl;
}

/*
**************************************************
 Function VVequal
**************************************************
 This function checks if two vectors are equal. Double.
*/
int VVequal(const vector<double> & a, const vector<double> & b)
{
  double tol=1e-15;
  int size=min(a.size(),b.size());
  int val=1;
  for (int i=0;i<size;i++){
    if (fabs(a[i]-b[i])>tol) val=0;
  }
  return val;
}

/*
**************************************************
 Function VVequal
**************************************************
 This function checks if two vectors are equal. Int.
*/
int VVequal(const vector<int> & a, const vector<int> & b)
{
  double tol=1e-15;
  int size=min(a.size(),b.size());
  int val=1;
  for (int i=0;i<size;i++){
    if (fabs((double)(a[i]-b[i]))>tol) val=0;
  }
  return val;
}


/*
**************************************************
 Function GetLTCell
**************************************************
 This function operates a linear transform on a cell.
*/
structure GetLTCell(const matrix<double> & lt, const structure & str)
{
  matrix<double> lat=str.GetLat();
  matrix<double> cpos=str.GetCpos();
  structure nstr=str;
  matrix<double> nlat=MMmult(lt,lat.transpose());
  matrix<double> ncpost=MMmult(lt,cpos.transpose());
  nstr.SetLat(nlat.transpose());
  nstr.SetAllAtomPos(ncpost.transpose(),1);
  return nstr;
}

/*
**************************************************
 Function GetLTFVCell
**************************************************
 This function operates a linear transform on a cell.  Rotates all vectors
by phi around nvec.  Formula from Goldstein, 1980 (Actually used version on 
Mathematic web site).
*/
structure GetLTFVCell(const vector<double> & nvec, const double phi, const structure & str)
{
  matrix<double> lat=str.GetLat();
  matrix<double> dpos=str.GetDpos();
  matrix<double> nlat(3,3,0.0);
  vector<double> nhat=UnitVec(nvec);
  double tmp;
  vector<double> v1(3);
  vector<double> v2(3);
  vector<double> v3(3);
  vector<double> r(3);
  for (int i=0;i<3;i++){
    r=lat[i];
    double nhatpr=VVdot(nhat,r);
    double cphi=cos(phi);
    double sphi=sin(phi);
    v1=SVprod(cphi,r);
    tmp=nhatpr*(1-cphi);
    v2=SVprod(tmp,nhat);
    v3=VVcross(r,nhat);
    v3=SVprod(sphi,v3);
    nlat[i]=VVsum(nlat[i],v1);
    nlat[i]=VVsum(nlat[i],v2);
    nlat[i]=VVsum(nlat[i],v3);
  }
  structure nstr=str;
  nstr.SetLat(nlat);
  nstr.SetAllAtomPos(dpos,0); // Will set cart coordinates w/new lattice params.
  return nstr;
}

// **************************************************
// Function MMmult
// **************************************************
/*  This function returns the product of two matrices.
   A=MxN, B=NxP. */
matrix<double> MMmult(const matrix<double> & A, const matrix<double> & B)
{
  int M=A.size();
  assert (M>0);
  int i=A[0].size();	
  int j=B.size();	
  //int N=min(A[0].size(),B.size());
  int N=min(i,j);
  assert (N>0);
  int P=B[0].size();
  matrix<double> AB(M,P);
  for (int i=0;i<M;i++){
    for (int j=0;j<P;j++){
      double sum=0;
      for (int k=0;k<N;k++){
        sum=sum+A[i][k]*B[k][j];
      }
      AB[i][j]=sum;
    }
  }
  return AB;
}

/*
**************************************************
 Function GetIntpolStr
**************************************************
 This function gets the structure that is linearly interpolated 
a fraction f of the way between strA and strB.  Interpolation 
nincludes lattice parameters.  The scale factors of the interpolated
structures are all set to 1.
*/
structure GetIntpolStr(structure strA, structure strB, const double & f,
		       const string & path_flag)
{
  strA.ReScale(1.0);
  strB.ReScale(1.0);
  // Get new lattice params.
  matrix<double> lati(3,3);
  matrix<double> latA=strA.GetLat();
  matrix<double> latB=strB.GetLat();
  for (int ic=0;ic<3;ic++){
    vector<double> dl=SVprod(f,VVdiff(latB[ic],latA[ic]));
    lati[ic]=VVsum(latA[ic],dl);
  }
  // Get new cart. coords.
  matrix<double> cposA=strA.GetCpos();
  matrix<double> cposB=strB.GetCpos();
  int size=min(cposA.size(),cposB.size());
  matrix<double> cposi(size,3);
  for (int iat=0;iat<size;iat++){
    vector<double> dp=VVdiff(cposB[iat],cposA[iat]);
    // If path_flag is n/N then the path is taken between
    // nearest images.  Otherwise path is between the atoms given.
    if (path_flag=="n" || path_flag=="N"){
      vector<double> ddp=vecC2D(lati,dp);
      for (int ic=0;ic<3;ic++){
	ddp[ic]=ddp[ic]-Nint(ddp[ic]);
      }
      dp=vecD2C(lati,ddp);
    }
    dp=SVprod(f,dp);
    cposi[iat]=VVsum(cposA[iat],dp);
  }
  structure stri=strA;
  stri.SetLat(lati);
  stri.SetAllAtomPos(cposi,1);
  return stri;
}

/*
**************************************************
 Function DebyeWallerFactor
**************************************************
 All data must be sent in SI units.
 This formula is from B.E. Warren, X-ray Diffraction, eq. 5.9.
 It is based on the Debye approximation and only holds for T>>TDebye.
 Note that this function returns the sqrt of what we usually call the 
 DW factor.  This is because this term multiplies the scattering 
 factors f, not f^2.  DW^2 would be the appropriate term to modulate 
 the intensity, but DW is appropriate to modulate the scattering factors 
 (see Warren,eq.3.24).
*/
double DebyeWallerFactor(const double & theta, const double & lambda,
			 const double & temp, const double & debye_temp,
			 const double & mass){
  double st=sin(theta);
  double h=PLANKSCONSTANT_h;
  double twoB=h*h*temp*12.0/(mass*KBOLTZ*debye_temp*debye_temp);
  double twoM=twoB*st*st/(lambda*lambda);
  double DWfact=exp(-1.0*twoM/2.0); // Use e^-M, not e^-2M.
  return DWfact;
}

/*
**************************************************
 Function RemoveSpaces
**************************************************
  Removes all spaces from a string.
*/
string RemoveSpaces(const string & s){
  string ss;
  for (int i=0;i<s.size();i++){
    if (s[i]!=' ') ss+=s[i];
  }
  return ss;
}

/*
**************************************************
 Function RemoveWhiteSpaces
**************************************************
  Removes all white spaces (spaces, tabs) from a string.
*/
string RemoveWhiteSpaces(const string & s){
  string ss;
  for (int i=0;i<s.size();i++){
    if (s[i]!=' ' & s[i]!='\t') ss+=s[i];
  }
  return ss;
}

int GetNumFields(const string & s){
  int nf=0;
  int in_a_field=0;
  for (int i=0;i<s.size();i++){
    if (!(in_a_field) && s[i]!=' '){
      in_a_field=1;
      nf++;
    }
    if (in_a_field && s[i]==' '){
      in_a_field=0;
    }
  }
  return nf;
}

string GetNextVal(const string & s, int & id){
  string ss;
  int i=id;
  while (s[i]==' '){i++;} // ignore leading spaces.
  while (i<s.size() && s[i]!=' '){ // pull out all text until next space.
    ss+=s[i];
    i++;
  }
  id=i;
  return ss;
}

string PaddedNumString(const int num, const int ndigits)
{
  ostringstream os;
  os << setw(ndigits) << setfill('0') << num << ends;
  return os.str();
}

void InFileExistCheck(const string & routine, const string & filename, 
		      ifstream & file_to_check, ostream & outf)
{
  if (!file_to_check){
    outf << "ERROR: in " << routine << endl;
    outf << "ERROR: Cannot open file: " << filename << endl;
    outf << "ERROR: Exiting" << endl;
    exit(1);
  }
}

/*
**************************************************
 Function GetMag
**************************************************
 This function returns the magnitude of a complex number.
*/
double GetMag(const complex<double> z)
{
   return sqrt(z.real()*z.real()+z.imag()*z.imag());
}


/*
**************************************************
 Function Normal
**************************************************
 This function returns the value of a normal distribution.
*/
double Normal(const double & x, const double & mu, const double & sigma){
  double tol=1e-12;
  // If sigma=0 return delta function
  if (abs(sigma)<tol){
    if (abs(x-mu)<tol){
      return 1;
    }
    else{
      return 0;
    }
  }
  double arg = -(x-mu)*(x-mu)/(2*sigma*sigma);
  return exp(arg)/(sqrt(TWOPI)*sigma);
}

/*
**************************************************
 Function CompAperpB
**************************************************
 This function returns the component of A perpendicular to B.
*/
vector<double> CompAperpB(const vector<double> & a, const vector<double> & b)
{
  double dp;
  dp=VVprod(a,b);
  double nb=norm(b);
  return VVdiff(a,SVprod(dp/(nb*nb),b));
}


/*
**************************************************
Function SmoothFunc
**************************************************
 This function smooths an input functions.
 Uses gaussian smoothing for now.
 Assumes function argument is integer given by vector id.
 Sigma is in units of function arguments (i.e., vector id).
*/
vector<double> SmoothFunc(const vector<double> & func, const double & sigma)
{
  if (sigma<1e-10) return func; 
  int nx=func.size();
  int range=(int) (5.0*sigma);
  // Get Gaussian weights
  vector<double> norm(nx,0.0);
  matrix<double> wt(nx,2*range+1,0.0);
  for (int ix=0;ix<nx;ix++){
    for (int i=-range;i<=range;i++){
      if ((ix+i)>=0 && (ix+i)<nx){
	wt[ix][i+range]=Normal((double)(ix+i),double(ix),sigma);
	norm[ix]=norm[ix]+wt[ix][i+range];
      }
    }
  }
  // Normalize to one
  for (int ix=0;ix<nx;ix++){
    for (int i=-range;i<=range;i++){
      wt[ix][i+range]=wt[ix][i+range]/norm[ix];
    }
  }

  vector<double> sfunc(nx,0.0);
  // Average in weighted nearby bins.
  for (int ix=0;ix<nx;ix++){
    double sf=0;
    for (int i=-range;i<=range;i++){
      if ((ix+i)>0 && (ix+i)<nx){
	sf+=wt[ix][i+range]*func[ix+i];
      }
    }
    sfunc[ix]=sf;
  }
  return sfunc;
}

/*
**************************************************
Function FindIfStringInStream
**************************************************
 This function returns true if string is in stream
 (on one line), or otherwise false. The search starts
 at the present file pointer location.  Note that this 
 does alter the input string, resetting the file pointer 
 to the input value at the end.
*/
int FindIfStringInStream(const string & key, istream & instream)
{
  // Get file pointer location at entry
  int loc=instream.tellg();
  int found_match=0;
  string s;
  getline(instream,s);
  int cont=0;
  if (getline(instream,s)) cont=1;
  while (cont){
    int id=s.find(key);
    if (id!=s.npos){ // Found key
      cont=0;
      found_match=1;
    }
    if (!getline(instream,s)) cont=0;
  }
  // Clear any fail bits associated with searching.
  instream.clear();
  // Set file pointer location to entry value
  instream.seekg(loc);
  return found_match;
}



// **************************************************
// Function VMmult
// **************************************************
/*  This function returns the product of a row vector and a matrix v*A.
   A=MxN, v=1xM. Return vector of length N. */
vector<double> VMmult(const vector<double> & v, const matrix<double> & A)
{
  int M=A.size();
  int N=min(A[0].size(),v.size());
  vector<double> u(M,0.0);
  for (int i=0;i<N;i++){
    double sum=0;
    for (int j=0;j<M;j++){
      sum=sum+v[j]*A[j][i];
    }
    u[i]=sum;
  }
  return u;
}


// **************************************************
// Function VMmult
// **************************************************
/*  This function returns the product of a row vector and a matrix v*A.
   A=MxN, v=1xM. Return vector of length N. */
vector<double> VMmult(const vector<int> & v, const matrix<double> & A)
{
  int M=A.size();
  int N=min(A[0].size(),v.size());
  vector<double> u(M,0.0);
  for (int i=0;i<N;i++){
    double sum=0;
    for (int j=0;j<M;j++){
      sum=sum+v[j]*A[j][i];
    }
    u[i]=sum;
  }
  return u;
}

// **************************************************
// Function Sign
// **************************************************
//  This function returns the sign of x, 0 when x=0;
int Sign(const double & x){
  if (x>0) return 1;
  if (x<0) return -1;
  if (x==0) return 0;
}

// **************************************************
// Function SignNoZero
// **************************************************
//  This function returns the sign of x, 1 when x=0;
int SignNoZero(const double & x){
  if (x>=0){
    return 1;
  }
  return -1;
}

/*
**************************************************
 Function Mout
**************************************************
 This function outputs a matrix.
*/
void Mout(const matrix<double> & m, ostream & out){
  for (int i=0;i<m.size();i++){
    for (int j=0;j<m[i].size();j++){
      out << m[i][j] << " ";
    }
    out << endl;
  }
}

/*
**************************************************
 Function Mout
**************************************************
 This function outputs a vector of vectors.
*/
void Mout(const vector<vector<double> > & m, ostream & out){
  for (int i=0;i<m.size();i++){
    for (int j=0;j<m[i].size();j++){
      out << m[i][j] << " ";
    }
    out << endl;
  }
}



/*
**************************************************
 Function GetUnitCellRep
**************************************************
 This function calculated the representation of a point
in terms of a position in cell 0 and its true unit cell.
Output position in cell 0 is given with same coordinate 
type (Cart or Direct) as input position.
Note that there is always an ambiguity of being in a given
cell with position 0, or one cell over with position 1.  This
is broken by forcing all cell positions to be at 0 if they 
are within TOL of 1.  This gives consistent cell image locations.
*/
void GetUnitCellRep(const vector<double> & in_pos, 
		    vector<double> & p_cell0, 
		    vector<int> & n_cell, 
		    const matrix<double> & lat, 
		    const int & coord_flag)
{
  double TOL=1e-11;
  vector<double> in_posd(3);
  p_cell0=vector<double> (3);
  n_cell=vector<int> (3);
  if (coord_flag==1){ // in_pos is in cartesian coords
    in_posd=vecC2D(lat,in_pos);
  }
  for (int ic=0;ic<3;ic++){
    int s;
    s=SignNoZero(in_posd[ic]);
    if (s>0){
      n_cell[ic]=int(in_posd[ic]);
    }
    else{
      n_cell[ic]=int(in_posd[ic])-1;
    }
    p_cell0[ic]=in_posd[ic]-n_cell[ic];
  // Boundary Correction.  p_cell0 is >=0 and <1.  
  // If p_cell0 is within TOL of 1
  // then set it to 0 and shift n_cell up by 1.
    if (abs(1-p_cell0[ic])<TOL){
      p_cell0[ic]=0;
      n_cell[ic]=n_cell[ic]+1;
    }
  }
  if (coord_flag==1){ // in_pos is in cartesian coords
    p_cell0=vecD2C(lat,p_cell0);
  }
}

/*
**************************************************
 Function ConvertAtomToLat
**************************************************
 This function makes sure that an atoms direct coords
and unit cell parameters are consistent with an input lattice.
*/
atom ConvertAtomToLat(const atom & in_at, const matrix<double> & lat){
  vector<double> cpos;
  vector<double> p_cell0;
  vector<int> n_cell;
  atom out_at=in_at;
  cpos=in_at.GetCpos();
  GetUnitCellRep(cpos,p_cell0,n_cell,lat,1);
  out_at.SetDpos(vecC2D(lat,cpos));
  out_at.SetUnitCell(n_cell);
  return out_at;
}


/*
**************************************************
 Function Sort_abc_angles
**************************************************
  This fucntions sorts lattice vectors to a>=b>=c and
arranges angles equivalently.
*/
vector<double> Sort_abc_angles(const vector<double> & abc_angles)
{
  vector<double> old = abc_angles;
  vector<double> newv = abc_angles;
  // if b>a then swap a/b: a=b,b=a,c=c,alpha=beta,beta=alpha,gamma=gamma.
  if (old[1]>old[0]){
    newv[0]=old[1];
    newv[1]=old[0];
    newv[3]=old[4];
    newv[4]=old[3];
    old=newv;
  }
  // if c>a then swap a/c: a=c,b=b,c=a,alpha=gamma,beta=beta,gamma=alpha.
  if (old[2]>old[0]){
    newv[0]=old[2];
    newv[2]=old[0];
    newv[3]=old[5];
    newv[5]=old[3];
    old=newv;
  }
  // if c>b then swap b/c: a=a,b=c,c=b,alpha=alpha,beta=gamma,gamma=beta.
  if (old[2]>old[1]){
    newv[1]=old[2];
    newv[2]=old[1];
    newv[4]=old[5];
    newv[5]=old[4];
    old=newv;
  }
  return newv;
}
