// **************************************************
// PrintShell
// **************************************************
//  This function prints out all points in unit cell with 
//  ns atoms between rmin and rmax.

using namespace std;
#include <cmath>
#include <string>
#include <iostream>
#include <fstream>
//#include <strstream.h>
#include <sstream>
#include <iomanip>
#include "UsefulFuncs.h"
#include "atom.h"
#include "PrintShell.h"
#include <vector>


/* 
**************************************************
GetAngleMat
**************************************************
  Given a point q and list of points, {pi} returns the matrix
M_ij = Angle from pi-q-pj.
*/
matrix<double> GetAngleMat(const vector<double> & q, 
			   const matrix<double> & p)
{
  double TOL=1e-6;
  int np=p.size();
  matrix<double> M(np,np);
  for (int i=0;i<np;i++){
    for (int j=0;j<np;j++){
      vector<double> a=VVdiff(p[i],q);
      vector<double> b=VVdiff(p[j],q);
      double na=norm(a);
      double nb=norm(b);
      double angle;
      double xcos=getcos(a,b);
      if (xcos>(1-TOL)){
	angle=0;
      } else{
	if (xcos<(-1+TOL)){
	  angle=180;
	} else{
	  angle = RAD2DEG*acos(xcos);
	}
      }
      M[i][j]=angle;
    }
  }
  return M;
}


/* 
**************************************************
new_point
**************************************************
  return true for a point with a different average
neighbor position.
*/
int new_point(const matrix<double> & avndpos, int na){
  for (int ia=0;ia<na;ia++){
    if (VVequal(avndpos[ia],avndpos[na])) return 0;
  }
  return 1;
}


void PrintShell(const structure & str, const int & ns,
		const double & rmin, const double & rmax,
		const string & sname, const int lin_dens)
{  
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  structure sstr=str;
  double scale=str.GetScale();
  sstr.ReScale(1);
  matrix<double> lat=sstr.GetLat();
  int coord_flag=sstr.GetCoordFlag();
  //  cout.setf(ios::fixed,ios::floatfield);
  //  cout.precision(10);
  matrix<atom> neigh_mat;
  matrix<double> points;
  for (int ix=0;ix<lin_dens;ix++){
    for (int iy=0;iy<lin_dens;iy++){
      for (int iz=0;iz<lin_dens;iz++){
	vector<double> pt(3);
	pt[0]=ix/(double)lin_dens;
	pt[1]=iy/(double)lin_dens;
	pt[2]=iz/(double)lin_dens;
	points.push_back(pt);
      }
    }
  }
  GetGoodShellPoints(points,str,ns,rmin,rmax,sname,neigh_mat);
  double tol=1e-15;

  // Get average dpos of neighbors of each point.
  matrix<double> avndpos;
  for (int ia=0;ia<neigh_mat.size();ia++){
    vector<double> av(3,0.0);
    for (int in=1;in<neigh_mat[ia].size();in++){
      atom a=neigh_mat[ia][in];
      vector<double> pos=a.GetDpos();
      av=VVsum(av,a.GetDpos());
    } // for in
    for (int ic=0;ic<3;ic++){
      av[ic]=av[ic]/(neigh_mat[ia].size()-1);
    } // for ic
    avndpos.push_back(av);
  }
  // Output header
  cout << "All atoms identified by:  ";
  cout << "[basis number]  [name]  [n1 n2 n3 (unit cell)]" << endl;
  cout << "Points must have " << ns << " atoms around them within a shell bounded by " << rmin << " and " << rmax << endl;
  cout << "Atoms must be of type: " << sname << ", ";
  cout << "Linear density of searched points is: " << lin_dens << endl;
  cout << "All coordinates are direct (cartesian) if the input POSCAR is direct (cartesian)." << endl;

  // Print out all points with unique environments.
  cout << endl;
  cout << "********** Unique environment points meeting shell constraints **********" << endl;
  int newcnt=0;
  matrix<double> all_new_pts;
  for (int ia=0;ia<neigh_mat.size();ia++){
    matrix<double> p; // For neighbors of averge point q.
    if (new_point(avndpos,ia)){
      newcnt++;
      atom a;
      vector<double> pt=avndpos[ia];
      all_new_pts.push_back(pt);
      a.SetDpos(pt);
      a.SetCpos(vecD2C(lat,pt));
      if (coord_flag==0){ // direct
	cout << setprecision(10) << setw(7) << ia+1 << "(" << newcnt << ")"
	     << "  UAvg "
	     << " " << setw(10) << pt[0]
	     << " " << setw(10) << pt[1]
	     << " " << setw(10) << pt[2] << endl;
      }
      if (coord_flag==1){ // cart
	vector<double> cv=vecD2C(lat,pt);
	cout << setprecision(10) << setw(7) << ia+1 << "(" << newcnt << ")"
	     << "  UAvg "
	     << " " << setw(10) << cv[0]
	     << " " << setw(10) << cv[1]
	     << " " << setw(10) << cv[2] << endl;
      }

      /*
      // Print out avg position of neighbors.
            cout << "        ";
           cout << setw(4) << "Avg ";
            cout << setw(4) << " ";
      if (coord_flag==0){ // direct
	cout << setprecision(10)<< setw(10) << avndpos[ia][0] << " " << setw(10) << avndpos[ia][1] << " " << setw(10) << avndpos[ia][2] << endl;
      }
      if (coord_flag==1){ // cart
	vector<double> cv=vecD2C(lat,avndpos[ia]);
	cout << setprecision(10)<< setw(10) << cv[0] << " " << setw(10) << cv[1] << " " << setw(10) << cv[2] << endl;
      }
      */
      // Print out neighbors.
      for (int in=1;in<neigh_mat[ia].size();in++){
	atom an = neigh_mat[ia][in];
	vector<int> uc(3);
	uc=an.GetUnitCell();
	cout << "        ";
	cout << setw(4) << an.GetNum()+1 << " ";
	cout << setw(4) << an.GetName().c_str() << "   ";
	cout << setw(3) << uc[0] << " " << setw(3) << uc[1] << " " << setw(3) << uc[2] << "   ";
	cout << setprecision(4) << AtomDist(a,an);
	cout << endl;
	p.push_back(an.GetCpos());
      } // in

      // Print out angles with neighbors.
      vector<double> q=vecD2C(lat,avndpos[ia]); 
      matrix<double> ang_mat=GetAngleMat(q,p);
      int np=p.size();
      cout << "        ";
      cout << "Angles"<<endl;
      cout << "           ";
      for (int i=0;i<np;i++){
	cout << " " << setw(6) << i+1;
      }
      cout << endl;
      for (int i=0;i<np;i++){
	cout << "        ";
	cout << setw(2) << i+1 ;
	for (int j=0;j<np;j++){
	  cout << " " << setprecision(2) << setw(6) << ang_mat[i][j];
	}
	cout << endl;
      }


    } // if newpoint
  } //ia

  // Print out all points.
  cout << endl;
  cout << "********** All points meeting shell constraints **********" << endl;
  for (int ia=0;ia<neigh_mat.size();ia++){
    atom a = neigh_mat[ia][0];
    vector<double> pt=a.GetDpos();
    if (coord_flag==0){ // direct
      cout << setprecision(10) << setw(7) << ia+1 
	   << " " << setw(10) << pt[0]
	   << " " << setw(10) << pt[1]
	   << " " << setw(10) << pt[2] << endl;
    }
    if (coord_flag==1){ // cart
      vector<double> cv=vecD2C(lat,pt);
      cout << setprecision(10) << setw(7) << ia+1 
	   << " " << setw(10) << cv[0]
	   << " " << setw(10) << cv[1]
	   << " " << setw(10) << cv[2] << endl;
    }
    for (int in=1;in<neigh_mat[ia].size();in++){
      atom an = neigh_mat[ia][in];
	vector<int> uc(3);
	uc=an.GetUnitCell();
	cout << "        ";
	cout << setw(4) << an.GetNum()+1 << " ";
	cout << setw(4) << an.GetName().c_str() << "   ";
	cout << setw(3) << uc[0] << " " << setw(3) << uc[1] << " " << setw(3) << uc[2] << "   ";
	cout << setprecision(4) << AtomDist(a,an);
	cout << endl;
    } // in
    // Print out avg position of neighbors.
	cout << "        ";
	cout << setw(4) << "Avg ";
	cout << setw(4) << " ";
	if (coord_flag==0){ // direct
	  cout << setprecision(10)<< setw(10) << avndpos[ia][0] << " " << setw(10) << avndpos[ia][1] << " " << setw(10) << avndpos[ia][2] << endl;
	}
	if (coord_flag==1){ // cart
	  vector<double> cv=vecD2C(lat,avndpos[ia]);
	  cout << setprecision(10)<< setw(10) << cv[0] << " " << setw(10) << cv[1] << " " << setw(10) << cv[2] << endl;
	}
  } // ia

  // WORKING HERE ???  Finish this and check consistency of 
  //  setting names, pos, nat, and particularly types - structure should
  //    keep all these consistent when any one is changed.
  //  Add new points to the POSCAR file and output.
  cout << endl;
  cout << "********** POSCAR file with unique environment points meeting shell constraints **********" << endl;
  // Number of atoms
  int nnat=all_new_pts.size();
  int nat=str.GetNumAtoms();
  int tnat=nat+nnat;
  // Number of types
  vector<int> num_each_type=str.GetNumEachType();
  vector<int> tnum_each_type=num_each_type;
  tnum_each_type.push_back(nnat);
  // Positions and names
  matrix<double> dpos=str.GetDpos();
  vector<string> names=str.GetNames();
  matrix<double> tdpos=dpos;
  vector<string> tnames=names;
  for (int i=0;i<nnat;i++){
    tdpos.push_back(all_new_pts[i]);
    ostringstream ostmp;
    ostmp<<i+1;
    string num=ostmp.str();
    string s1 = "SHELL";    
    string shell=s1+num;
    tnames.push_back(shell);
  }
  sstr.ReScale(scale);
  sstr.SetNumEachType(tnum_each_type);
  sstr.SetAllAtomPos(tdpos,0);
  sstr.SetAllAtomNames(tnames);
  cout << sstr;

} // end routine








