// **************************************************
// ProjFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <algorithm>
#include "UsefulFuncs.h"
#include "projdata.h"
#include "ProjFuncs.h"

// **************************************************
// ProcessProjection
// **************************************************
/* The projections might be a complex amplitude, a true
   real probability, or a probability with a complex
   phase.  Depending on which, you need to process 
   the projection differently.  Hence this routine, so
   all processing can be changed at once.
*/
complex<double> ProcessProjection(const complex<double> & proj){
  complex<double> p;
  p=proj;
  // For proj=probability with phase.
  // This allows for only positive probabilities (does not exactly match vasp output).
  // return sqrt(p*conj(p));
  // For proj=real probability with complex part always zero.
  // This allows for negative probabilities (does exactly match vasp output).
  return p;
}

// **************************************************
// ReadInProj
// **************************************************
/* Need to read everything in twice if it is spin polarized.
   rspin=1 for spin polarized, rspin=2 for non-spin polarized.
*/
void ReadInProj(projdata & pd)
{
  int have_aug;
  string s;
  char c;
  ifstream infile(pd.PROOUTinfile.c_str());
  InFileExistCheck("ReadInProj",pd.PROOUTinfile.c_str(),infile,cerr);
  string sdum;
  // Initial data
  infile >> sdum; // Title
  infile >> sdum >> sdum >> sdum >> pd.nkpts;
  infile >> sdum >> sdum >> sdum >> pd.nbands; 
  infile >> sdum >> sdum >> sdum >> pd.nions; 
  infile >> pd.ntypes >> pd.ntypes;
  pd.num_each_type=vector<int> (pd.ntypes);
  for (int it=0;it<pd.ntypes;it++){
    infile >> pd.num_each_type[it];
  }
  // For getting kpoint energies.
  int skip=2;
  if (pd.nions==1) skip=1;

  // Fermi weights (kpts (inner loop) and bands (outer loop)).
  pd.wfermi_u = matrix<double> (pd.nbands,pd.nkpts,0.0);
  for (int ib=0;ib<pd.nbands;ib++){
    for (int ik=0;ik<pd.nkpts;ik++){
      infile >> pd.wfermi_u[ib][ik];
      // tpx
      // cout<<"ib,ik,wfermi_u "<<ib<<" "<<ik<<" "<<pd.wfermi_u[ib][ik]<<endl;
    }
  }


  // Check to see if there are spd or spdf electrons (nl=3 or nl=4).
  if (FindIfStringInStream("s      p      d      f",infile)){
    pd.nl=4;
  }
  else{
    pd.nl=3;
  }
  if (pd.nl==3){
    pd.nlm=9; // for s,p,d orbitals
    pd.nlmtot=12; // for s,p,d orbitals + p,d,all totals
  }

  // Projections
  matrix<complex<double> > m(pd.nbands,pd.nlm,0.0);
  matrix<matrix<complex<double> > > mm(pd.nkpts,pd.nions,m);
  pd.pdat_u=vector<matrix<matrix<complex<double> > > > (pd.ntypes,mm);
  vector<double> rval(pd.nlm);
  vector<double> ival(pd.nlm);
  for (int it=0;it<pd.ntypes;it++){
    for (int ik=0;ik<pd.nkpts;ik++){
      for (int iat=0;iat<pd.num_each_type[it];iat++){
	for (int ib=0;ib<pd.nbands;ib++){
	  for (int ilm=0;ilm<pd.nlm;ilm++){
	    infile >> rval[ilm] >> ival[ilm];
	    complex<double> ctmp (rval[ilm],ival[ilm]);
	    pd.pdat_u[it][ik][iat][ib][ilm]=ctmp;
	    // tpx
	    //  cout << "PROJ ib,ilm " << ib << " " << ilm << " " <<  rval[ilm] << " " <<  ival[ilm] << " " << pd.pdat_u[it][ik][iat][ib][ilm] << endl;
 	  }
	}
      }
    }
  }

  // Augmented Projections
  getline(infile,s); // Gets final carriage return from previous line.
  getline(infile,s); // Gets "augmentation part"
  c = infile.peek();
  if (c=='#'){
    have_aug=0; // There are no augmented projections.
  }
  else{
    have_aug=1; // There are augmented projections.
  }
  if (have_aug){
    for (int ik=0;ik<pd.nkpts;ik++){
      for (int ib=0;ib<pd.nbands;ib++){
	for (int it=0;it<pd.ntypes;it++){
	  for (int iat=0;iat<pd.num_each_type[it];iat++){
	    for (int ilm=0;ilm<pd.nlm;ilm++){
	      infile >> rval[ilm];
              //tpx
	      // cout << "Rval " << rval[ilm] << " " << ilm  << endl;
	    }
	    for (int ilm=0;ilm<pd.nlm;ilm++){
	      pd.pdat_u[it][ik][iat][ib][ilm]=
		pd.pdat_u[it][ik][iat][ib][ilm]+rval[ilm];
	    }
	  }
	}
      }
    }
    getline(infile,s); // Gets final carriage return last line of augmentation charges.
  }//if have_aug

  // Kpt weights and values and energy vs. kpt,bnd
  pd.wkpt = vector<double> (pd.nkpts,0.0);
  pd.kpts = matrix<double> (pd.nkpts,3,0.0);
  pd.ener_k_b_u = matrix<double> (pd.nkpts,pd.nbands,0.0);
  getline(infile,s); // Gets line "# of k-points: ..."
  for (int ik=0;ik<pd.nkpts;ik++){
    getline(infile,s); // Gets blank line.
    infile >>sdum>>sdum>>sdum>>pd.kpts[ik][0]>>pd.kpts[ik][1]>>pd.kpts[ik][2]>>sdum>>sdum>>pd.wkpt[ik];
    getline(infile,s); // Gets final carriage return from previous line.
    getline(infile,s); // Gets line.
    for (int ib=0;ib<pd.nbands;ib++){
      infile >>sdum>>sdum>>sdum>>sdum>>pd.ener_k_b_u[ik][ib]>>sdum>>sdum>>sdum;
      for (int ii=0;ii<pd.nions+skip;ii++){
	// Gets lines (nbands*(nions+skip)).  Note that
	// the numer of blank lines varies in different vasp versions
	// so this loop does not count blank lines.
	getline(infile,s); 
	if (RemoveSpaces(s).size()==0) {
	  ii=ii-1; // Don't count blank lines.
	}
      }
    }
  }

  // Do again if spin polarized.  Look for another PROOUT title line.
  // At this point lines could be blank, end of file, or PROOUT.  We want
  // to keep reading if they are blank, stop if they are EOF, and do 
  // spin-polarized if they are PROOUT.
  infile >> s; // Skip all whitespace and get PROOUT title if it exists.
  if (RemoveSpaces(s)=="PROOUT"){
    // Set spin polarization dependent variables.
    pd.sp=1;
    pd.rspin=1;
    
    getline(infile,s); // Gets endline after PROOUT.
    getline(infile,s); // get line.
    getline(infile,s); // get line.
    
    // Fermi weights (kpts (inner loop) and bands (outer loop)).
    pd.wfermi_d = matrix<double> (pd.nbands,pd.nkpts,0.0);
    for (int ib=0;ib<pd.nbands;ib++){
      for (int ik=0;ik<pd.nkpts;ik++){
	infile >> pd.wfermi_d[ib][ik];
      }
    }

    // Projections
    matrix<complex<double> > m(pd.nbands,pd.nlm,0.0);
    matrix<matrix<complex<double> > > mm(pd.nkpts,pd.nions,m);
    pd.pdat_d=vector<matrix<matrix<complex<double> > > > (pd.ntypes,mm);
    vector<double> rval(pd.nlm);
    vector<double> ival(pd.nlm);
    for (int it=0;it<pd.ntypes;it++){
      for (int ik=0;ik<pd.nkpts;ik++){
	for (int iat=0;iat<pd.num_each_type[it];iat++){
	  for (int ib=0;ib<pd.nbands;ib++){
	    for (int ilm=0;ilm<pd.nlm;ilm++){
	      infile >> rval[ilm] >> ival[ilm];
	    }
	    for (int ilm=0;ilm<pd.nlm;ilm++){
	      complex<double> ctmp (rval[ilm],ival[ilm]);
	      pd.pdat_d[it][ik][iat][ib][ilm]=ctmp;
	    }
	  }
	}
      }
    }

    // Augmented Projections
    string s;
    getline(infile,s); // Gets final carriage return from previous line.
    getline(infile,s); // Gets "augmentation part"
    c = infile.peek();
    if (c=='#'){
      have_aug=0; // There are no augmented projections.
    }
    else{
      have_aug=1; // There are augmented projections.
    }
    if (have_aug){
      for (int ik=0;ik<pd.nkpts;ik++){
	for (int ib=0;ib<pd.nbands;ib++){
	  for (int it=0;it<pd.ntypes;it++){
	    for (int iat=0;iat<pd.num_each_type[it];iat++){
	      for (int ilm=0;ilm<pd.nlm;ilm++){
		infile >> rval[ilm];
	      }
	      for (int ilm=0;ilm<pd.nlm;ilm++){
		pd.pdat_d[it][ik][iat][ib][ilm]=
		  pd.pdat_d[it][ik][iat][ib][ilm]+rval[ilm];
	      }
	    }
	  }
	}
      }
      getline(infile,s); // Gets final carriage return last line of augmentation charges.
    }//if have_aug

  // Kpt weights and values and energy vs. kpt,bnd
  pd.ener_k_b_d = matrix<double> (pd.nkpts,pd.nbands,0.0);
  getline(infile,s); // Gets line "# of k-points: ..."
  for (int ik=0;ik<pd.nkpts;ik++){
    getline(infile,s); // Gets blank line.
    infile >>sdum>>sdum>>sdum>>pd.kpts[ik][0]>>pd.kpts[ik][1]>>pd.kpts[ik][2]>>sdum>>sdum>>pd.wkpt[ik];
    getline(infile,s); // Gets final carriage return from previous line.
    getline(infile,s); // Gets line.
    for (int ib=0;ib<pd.nbands;ib++){
      infile >>sdum>>sdum>>sdum>>sdum>>pd.ener_k_b_d[ik][ib]>>sdum>>sdum>>sdum;
      for (int ii=0;ii<pd.nions+skip;ii++){
	// Gets lines (nbands*(nions+skip)).  Note that
	// the numer of blank lines varies in different vasp versions
	// so this loop does not count blank lines.
	getline(infile,s); 
	if (RemoveSpaces(s).size()==0) {
	  ii=ii-1; // Don't count blank lines.
	}
      }
    }
  }

  }// if spin polarized

}// end


// **************************************************
// CalcNeatProj
// **************************************************
// Calculates all the basic occupancies.  If only_occ=true
// then it does this only for occupied states.  If only_occ=false
// then it does this for all states, which does not really 
// give occupations, but sets things up for PDOS calculations.
void CalcNeatProj(projdata & pd, int only_occ)
{
  int ioncnt;

  // Get occ_vs_ion_kpt_bnd_lm_(u,d)
  // Get occ_vs_ion_kpt_lm_(u,d)
  // Get occ_vs_ion_bnd_lm_(u,d)
  // Get occ_vs_ion_lm_(u,d)
  matrix<double> m1 (pd.nbands,pd.nlm_max,0.0);
  matrix<double> m2 (pd.nkpts,pd.nlm_max,0.0);
  matrix<double> m3 (pd.nbands,pd.nlm_max,0.0);
  pd.occ_vs_ion_kpt_bnd_lm_u = matrix<matrix<double> > (pd.nions,pd.nkpts,m1);
  pd.occ_vs_ion_kpt_lm_u = vector<matrix<double> > (pd.nions,m2);
  pd.occ_vs_ion_bnd_lm_u = vector<matrix<double> > (pd.nions,m3);
  pd.occ_vs_ion_lm_u = matrix<double> (pd.nions,pd.nlm_max,0.0);  
  if (pd.sp){
    pd.occ_vs_ion_kpt_bnd_lm_d = matrix<matrix<double> > (pd.nions,pd.nkpts,m1);
    pd.occ_vs_ion_kpt_lm_d = vector<matrix<double> > (pd.nions,m2);
    pd.occ_vs_ion_bnd_lm_d = vector<matrix<double> > (pd.nions,m3);
    pd.occ_vs_ion_lm_d = matrix<double> (pd.nions,pd.nlm_max,0.0);  
  }
  ioncnt=-1;
  for (int it=0;it<pd.ntypes;it++){
    for (int iat=0;iat<pd.num_each_type[it];iat++){
      ioncnt++;
      for (int ik=0;ik<pd.nkpts;ik++){
	for (int ib=0;ib<pd.nbands;ib++){
	  for (int ilm=0;ilm<pd.nlm;ilm++){
	    double temp;
	    temp=ProcessProjection(pd.pdat_u[it][ik][iat][ib][ilm]).real();
	    if (only_occ){
	      pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][ilm]=pd.rspin*pd.wfermi_u[ib][ik]*(temp);
	    }
	    else{
	      pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][ilm]=pd.rspin*(temp);
	    }
	    pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][ilm]=
	      pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][ilm]+
	      pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][ilm];
	    pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][ilm]=
	      pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][ilm]+
	      pd.wkpt[ik]*pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][ilm];
	    pd.occ_vs_ion_lm_u[ioncnt][ilm]=
	      pd.occ_vs_ion_lm_u[ioncnt][ilm]+
	      pd.wkpt[ik]*pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][ilm];
	    if (pd.sp){
	      temp=ProcessProjection(pd.pdat_d[it][ik][iat][ib][ilm]).real();
	      if (only_occ){
		pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][ilm]=pd.rspin*pd.wfermi_d[ib][ik]*(temp);
	      }
	      else{
		pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][ilm]=pd.rspin*(temp);
	      }
	      pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][ilm]=
		pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][ilm]+
		pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][ilm];
	      pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][ilm]=
		pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][ilm]+
		pd.wkpt[ik]*pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][ilm];
	      pd.occ_vs_ion_lm_d[ioncnt][ilm]=
		pd.occ_vs_ion_lm_d[ioncnt][ilm]+
		pd.wkpt[ik]*pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][ilm];
	    }
	  }
	}
      }
    }
  }

  // Get sums over m.
  // Get occ_vs_ion_l_(u,d)
  // Get occ_vs_ion_kpt_l_(u,d)
  // Get occ_vs_ion_bnd_l_(u,d)
  // Get occ_vs_ion_kpt_bnd_l_(u,d)
  // This assumes the usual spd orbitals.
  matrix<double> n1 (pd.nbands,pd.nl_max+1,0.0);
  matrix<double> n2 (pd.nkpts,pd.nl_max+1,0.0);
  matrix<double> n3 (pd.nbands,pd.nl_max+1,0.0);
  pd.occ_vs_ion_kpt_bnd_l_u = matrix<matrix<double> > (pd.nions,pd.nkpts,n1);
  pd.occ_vs_ion_kpt_l_u = vector<matrix<double> > (pd.nions,n2);
  pd.occ_vs_ion_bnd_l_u = vector<matrix<double> > (pd.nions,n3);
  pd.occ_vs_ion_l_u = matrix<double> (pd.nions,pd.nl_max+1,0.0);  
  if (pd.sp){
    pd.occ_vs_ion_kpt_bnd_l_d = matrix<matrix<double> > (pd.nions,pd.nkpts,n1);
    pd.occ_vs_ion_kpt_l_d = vector<matrix<double> > (pd.nions,n2);
    pd.occ_vs_ion_bnd_l_d = vector<matrix<double> > (pd.nions,n3);
    pd.occ_vs_ion_l_d = matrix<double> (pd.nions,pd.nl_max+1,0.0);  
  }

  ioncnt=-1;
  for (int it=0;it<pd.ntypes;it++){
    for (int iat=0;iat<pd.num_each_type[it];iat++){
      ioncnt++;
      for (int ik=0;ik<pd.nkpts;ik++){
	for (int ib=0;ib<pd.nbands;ib++){

	  // S
	  pd.occ_vs_ion_l_u[ioncnt][0]+=pd.occ_vs_ion_lm_u[ioncnt][0];
	  pd.occ_vs_ion_kpt_l_u[ioncnt][ik][0]+=pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][0];
	  pd.occ_vs_ion_bnd_l_u[ioncnt][ib][0]+=pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][0];
	  pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][0]+=pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][0];
	  if (pd.sp){
	    pd.occ_vs_ion_l_d[ioncnt][0]+=pd.occ_vs_ion_lm_d[ioncnt][0];
	    pd.occ_vs_ion_kpt_l_d[ioncnt][ik][0]+=pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][0];
	    pd.occ_vs_ion_bnd_l_d[ioncnt][ib][0]+=pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][0];
	    pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][0]+=pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][0];
	  }

	  // P
	  for (int ip=0;ip<3;ip++){
	    pd.occ_vs_ion_l_u[ioncnt][1]+=pd.occ_vs_ion_lm_u[ioncnt][1+ip];
	    pd.occ_vs_ion_kpt_l_u[ioncnt][ik][1]+=pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][1+ip];
	    pd.occ_vs_ion_bnd_l_u[ioncnt][ib][1]+=pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][1+ip];
	    pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][1]+=pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][1+ip];
	    if (pd.sp){
	      pd.occ_vs_ion_l_d[ioncnt][1]+=pd.occ_vs_ion_lm_d[ioncnt][1+ip];
	      pd.occ_vs_ion_kpt_l_d[ioncnt][ik][1]+=pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][1+ip];
	      pd.occ_vs_ion_bnd_l_d[ioncnt][ib][1]+=pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][1+ip];
	      pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][1]+=pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][1+ip];
	    }
	  }

	  // D
	  for (int id=0;id<5;id++){
	    pd.occ_vs_ion_l_u[ioncnt][2]+=pd.occ_vs_ion_lm_u[ioncnt][4+id];
	    pd.occ_vs_ion_kpt_l_u[ioncnt][ik][2]+=pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][4+id];
	    pd.occ_vs_ion_bnd_l_u[ioncnt][ib][2]+=pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][4+id];
	    pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][2]+=pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][4+id];
	    if (pd.sp){
	      pd.occ_vs_ion_l_d[ioncnt][2]+=pd.occ_vs_ion_lm_d[ioncnt][4+id];
	      pd.occ_vs_ion_kpt_l_d[ioncnt][ik][2]+=pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][4+id];
	      pd.occ_vs_ion_bnd_l_d[ioncnt][ib][2]+=pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][4+id];
	      pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][2]+=pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][4+id];
	    }
	  }

	  // F
	  for (int id=0;id<7;id++){
	    pd.occ_vs_ion_l_u[ioncnt][3]+=pd.occ_vs_ion_lm_u[ioncnt][9+id];
	    pd.occ_vs_ion_kpt_l_u[ioncnt][ik][3]+=pd.occ_vs_ion_kpt_lm_u[ioncnt][ik][9+id];
	    pd.occ_vs_ion_bnd_l_u[ioncnt][ib][3]+=pd.occ_vs_ion_bnd_lm_u[ioncnt][ib][9+id];
	    pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][3]+=pd.occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][9+id];
	    if (pd.sp){
	      pd.occ_vs_ion_l_d[ioncnt][3]+=pd.occ_vs_ion_lm_d[ioncnt][9+id];
	      pd.occ_vs_ion_kpt_l_d[ioncnt][ik][3]+=pd.occ_vs_ion_kpt_lm_d[ioncnt][ik][9+id];
	      pd.occ_vs_ion_bnd_l_d[ioncnt][ib][3]+=pd.occ_vs_ion_bnd_lm_d[ioncnt][ib][9+id];
	      pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][3]+=pd.occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][9+id];
	    }
	  }

	}//ib
      }//ik

      // Normalize sums over m for multiple counting.
      for (int il=0;il<pd.nl;il++){
	pd.occ_vs_ion_l_u[ioncnt][il]/=(pd.nbands*pd.nkpts);
	if (pd.sp)  pd.occ_vs_ion_l_d[ioncnt][il]/=(pd.nbands*pd.nkpts);
	for (int ik=0;ik<pd.nkpts;ik++){
	  pd.occ_vs_ion_kpt_l_u[ioncnt][ik][il]/=(pd.nbands);

	  if (pd.sp) pd.occ_vs_ion_kpt_l_d[ioncnt][ik][il]/=(pd.nbands);
	}
	for (int ib=0;ib<pd.nbands;ib++){
	  pd.occ_vs_ion_bnd_l_u[ioncnt][ib][il]/=(pd.nkpts);
	  if (pd.sp) pd.occ_vs_ion_bnd_l_d[ioncnt][ib][il]/=(pd.nkpts);
	}
      }


   // Calculate total sums over L.

      int iid=pd.nl_max;
      for (int id=0;id<pd.nl_max;id++){
	pd.occ_vs_ion_l_u[ioncnt][iid]+=pd.occ_vs_ion_l_u[ioncnt][id];	
	if (pd.sp) pd.occ_vs_ion_l_d[ioncnt][iid]+=pd.occ_vs_ion_l_d[ioncnt][id];
	for (int ik=0;ik<pd.nkpts;ik++){
	  pd.occ_vs_ion_kpt_l_u[ioncnt][ik][iid]+=pd.occ_vs_ion_kpt_l_u[ioncnt][ik][id];
	  if (pd.sp) pd.occ_vs_ion_kpt_l_d[ioncnt][ik][iid]+=pd.occ_vs_ion_kpt_l_d[ioncnt][ik][id];
	}
	for (int ib=0;ib<pd.nbands;ib++){
	  pd.occ_vs_ion_bnd_l_u[ioncnt][ib][iid]+=pd.occ_vs_ion_bnd_l_u[ioncnt][ib][id];
	  if (pd.sp) pd.occ_vs_ion_bnd_l_d[ioncnt][ib][iid]+=pd.occ_vs_ion_bnd_l_d[ioncnt][ib][id];
	}
	for (int ik=0;ik<pd.nkpts;ik++){
	  for (int ib=0;ib<pd.nbands;ib++){
	    pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][iid]+=pd.occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][id];
	    if (pd.sp) pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][iid]+=pd.occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][id];
	  }
	}
      }

    }// at
  }// type

  // Create single occ variable for all the l,m and sums over m together (for lmtot).
  // This way you can just loop over lm from 0 to lmtot-1.
  m1 = matrix<double> (pd.nbands,pd.nlmtot,0.0);
  pd.occ_vs_ion_kpt_bnd_lmtot_u = matrix<matrix<double> > (pd.nions,pd.nkpts,m1);
  if (pd.sp) pd.occ_vs_ion_kpt_bnd_lmtot_d = matrix<matrix<double> > (pd.nions,pd.nkpts,m1);
  for (int ii=0;ii<pd.nions;ii++){
    for (int ik=0;ik<pd.nkpts;ik++){
      for (int ib=0;ib<pd.nbands;ib++){
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][0]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][0];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][1]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][1];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][2]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][2];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][3]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][3];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][4]=pd.occ_vs_ion_kpt_bnd_l_u[ii][ik][ib][1];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][5]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][4];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][6]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][5];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][7]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][6];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][8]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][7];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][9]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][8];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][10]=pd.occ_vs_ion_kpt_bnd_l_u[ii][ik][ib][2];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][11]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][9];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][12]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][10];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][13]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][11];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][14]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][12];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][15]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][13];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][16]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][14];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][17]=pd.occ_vs_ion_kpt_bnd_lm_u[ii][ik][ib][15];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][18]=pd.occ_vs_ion_kpt_bnd_l_u[ii][ik][ib][3];
	pd.occ_vs_ion_kpt_bnd_lmtot_u[ii][ik][ib][19]=pd.occ_vs_ion_kpt_bnd_l_u[ii][ik][ib][4];
	if (pd.sp){
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][0]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][0];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][1]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][1];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][2]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][2];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][3]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][3];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][4]=pd.occ_vs_ion_kpt_bnd_l_d[ii][ik][ib][1];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][5]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][4];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][6]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][5];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][7]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][6];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][8]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][7];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][9]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][8];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][10]=pd.occ_vs_ion_kpt_bnd_l_d[ii][ik][ib][2];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][11]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][9];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][12]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][10];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][13]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][11];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][14]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][12];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][15]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][13];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][16]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][14];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][17]=pd.occ_vs_ion_kpt_bnd_lm_d[ii][ik][ib][15];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][18]=pd.occ_vs_ion_kpt_bnd_l_d[ii][ik][ib][3];
	  pd.occ_vs_ion_kpt_bnd_lmtot_d[ii][ik][ib][19]=pd.occ_vs_ion_kpt_bnd_l_d[ii][ik][ib][4];
	}
      }
    }
  }

}


// **************************************************
// PrintNeatProj
// **************************************************
void PrintNeatProj(projdata & pd)
{
    pd.Print(cout);
}


// **************************************************
// ReadInPDOSData
// **************************************************
// Reads in parameters that determine what partial DOS
//   will be calculated.

void ReadInPDOSData(const projdata & prd, pdosdata & pdd)
{

  ifstream infile(pdd.PDOSinfile.c_str());
  InFileExistCheck("ReadInPDOSData",pdd.PDOSinfile.c_str(),infile,cerr);

  // Defaults (emin,emax,nbins,smooth_sigma,print_params)
  double emn=prd.ener_k_b_u[0][0];
  double emx=emn;
  double e;
  for (int ik=0;ik<prd.nkpts;ik++){
    for (int ib=0;ib<prd.nbands;ib++){
      e = prd.ener_k_b_u[ik][ib];
      if (emn>e) emn=e;
      if (emx<e) emx=e;
      if (prd.sp) e = prd.ener_k_b_d[ik][ib];
      if (emn>e) emn=e;
      if (emx<e) emx=e;
    }
  }
  pdd.emin=emn-0.5;
  pdd.emax=emx+0.5;
  pdd.nbins=300;
  pdd.smooth_sigma=(pdd.emax-pdd.emin)/pdd.nbins;
  pdd.print_params=0;
	  
// Read in all the tokens.

  string s,s_ns;
  vector<string> token_vec;
  vector<string> val_vec;
  while (!infile.eof()){
    getline(infile,s);
    s_ns=RemoveSpaces(s); // Get string with no spaces.
    if (s_ns.size()>0){ // Make sure line was not blank
      if (s_ns[0]!='#'){ // Exclude comment lines
	string token;
	string sval;
	int id=s.find('=');
	if (id>=s.length()){
	  cout << "ERROR: The following token is incorrectly formatted: " << s << endl;
	  cout << "ERROR: Exiting" << endl;
	  exit(1);
	}
	token=s.substr(0,id);
	token=RemoveSpaces(token);
	int i_f=min(s.length(),s.find('#')); // End of string
	sval=s.substr(id+1,i_f-id-1);
	token_vec.push_back(token);
	val_vec.push_back(sval);
      } //if s_ns[0]!=#
    } // if s_ns.size>0

  } // while !infile.eof

  // Read in values for variables in the pdosdata object.
  vector<int> tmp;
  int atom_cnt=0;
  for (int i=0;i<token_vec.size();i++){
    int id=0;
    string tok=token_vec[i];
    string sval;
    double dval;
    int ival;
    int found_token=0;
    if (tok=="ATOMS"){
      id=0;
      atom_cnt++;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
	string s;
	s=GetNextVal(val_vec[i],id);
	if (s.size()>0){
	  ival=atoi(s.c_str());
	  tmp.push_back(ival);
	}
      }
      pdd.pdos_at.push_back(tmp);
      // add vector to pdos_k,pdos_b,pdos_lm
      tmp.clear();
      pdd.pdos_k.push_back(tmp);
      pdd.pdos_b.push_back(tmp);
      pdd.pdos_lm.push_back(tmp);
      found_token=1;
    }// ATOMS
    if (tok=="KPOINTS"){
      if (pdd.pdos_k.size()!=atom_cnt || atom_cnt==0) AtomCntError(tok,pdd.pdos_k.size(),atom_cnt);
      id=0;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
	string s;
	s=GetNextVal(val_vec[i],id);
	if (s.size()>0){
	  ival=atoi(s.c_str());
	  pdd.pdos_k[pdd.pdos_k.size()-1].push_back(ival);
	}
      }
      found_token=1;
    }// KPOINTS
    if (tok=="BANDS"){
      id=0;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
	string s;
	s=GetNextVal(val_vec[i],id);
	if (s.size()>0){
	  ival=atoi(s.c_str());
	  pdd.pdos_b[pdd.pdos_b.size()-1].push_back(ival);
	}
      }
      found_token=1;
    }// LMVALUES
    if (tok=="LMVALUES"){
      if (pdd.pdos_lm.size()!=atom_cnt || atom_cnt==0) AtomCntError(tok,pdd.pdos_lm.size(),atom_cnt);
      id=0;
      tmp.clear();
      // set up atoms matrix
      while (id<val_vec[i].size()){
	string s;
	s=GetNextVal(val_vec[i],id);
	if (s.size()>0){
	  ival=atoi(s.c_str());
	  pdd.pdos_lm[pdd.pdos_lm.size()-1].push_back(ival);
	}
      }
      found_token=1;
    }// LMVALUES
    if (tok=="EMIN"){
      id=0;
      pdd.emin=atof(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// EMIN
    if (tok=="EMAX"){
      id=0;
      pdd.emax=atof(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// EMAX
    if (tok=="NBINS"){
      id=0;
      pdd.nbins=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// NBIN
    if (tok=="SMOOTH_SIGMA"){
      id=0;
      pdd.smooth_sigma=atof(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// SMOOTH_SIGMA
    if (tok=="PRINT_PARAMS"){
      id=0;
      pdd.print_params=atoi(GetNextVal(val_vec[i],id).c_str());
      found_token=1;
    }// PRINT_PARAMS

    if (!found_token){
      cerr << "ERROR: You have input a token " << tok <<endl;
      cerr << "ERROR: This token is not recognized. Exiting! " << endl;
      exit(1);
    }

  } // for i

  // Defaults (all atoms, all kpts, all bands, lm = nlmtot = all l+m)
  vector<int> vv(4);
  vv[0]=prd.nions;
  vv[1]=prd.nkpts;
  vv[2]=prd.nbands;
  vv[3]=prd.nlmtot;
  int mx = *max_element(vv.begin(),vv.end());
  vector<int> cnt(mx);
  for (int i=1;i<=mx;i++){
    cnt[i-1]=i;
  }
  //  int mx = max(prd.nions,prd.nkpts,prd.nbands,prd.nlmtot);
  vector<int> atv(prd.nions);
  for (int i=0;i<pdd.pdos_at.size();i++){
    if (pdd.pdos_at[i].size()==0) pdd.pdos_at[i] = vector<int> (cnt.begin(),cnt.begin()+prd.nions);
    if (pdd.pdos_k[i].size()==0) pdd.pdos_k[i] = vector<int> (cnt.begin(),cnt.begin()+prd.nkpts);
    if (pdd.pdos_b[i].size()==0) pdd.pdos_b[i] = vector<int> (cnt.begin(),cnt.begin()+prd.nbands);
    if (pdd.pdos_lm[i].size()==0) pdd.pdos_lm[i] = vector<int> (1,prd.nlmtot);
  }


  // Check that all data is within bounds
  for (int i=0;i<pdd.pdos_at.size();i++){ // cases
    for (int ia=0;ia<pdd.pdos_at[i].size();ia++){
      int atp=pdd.pdos_at[i][ia];
      if (atp<1 || atp>prd.nions){
	cerr << "ERROR: ProjFuncs/ReadPDOSdat" << endl;
	cerr << "ERROR: Error in case: " << i+1 << endl;
	cerr << "ERROR: You have entered too low or high an atom number in entry: " << ia+1 << endl;
	cerr << "ERROR: Bad value is: " << atp << endl;
	cerr << "ERROR: Exiting." << endl;
	exit(1);
      }
    }
    for (int ik=0;ik<pdd.pdos_k[i].size();ik++){
      int kp=pdd.pdos_k[i][ik];
      if (kp<1 || kp>prd.nkpts){
	cerr << "ERROR: ProjFuncs/ReadPDOSdat" << endl;
	cerr << "ERROR: Error in case: " << i+1 << endl;
	cerr << "ERROR: You have entered too low or high a kpt number in entry: " << ik+1 << endl;
	cerr << "ERROR: Bad value is: " << kp << endl;
	cerr << "ERROR: Exiting." << endl;
	exit(1);
      }
    }
    for (int ib=0;ib<pdd.pdos_b[i].size();ib++){
      int bp=pdd.pdos_b[i][ib];
      if (bp<1 || bp>prd.nbands){
	cerr << "ERROR: ProjFuncs/ReadPDOSdat" << endl;
	cerr << "ERROR: Error in case: " << i+1 << endl;
	cerr << "ERROR: You have entered too low or high a bnd number in entry: " << ib+1 << endl;
	cerr << "ERROR: Bad value is: " << bp << endl;
	cerr << "ERROR: Exiting." << endl;
	exit(1);
      }
    }
    for (int ilm=0;ilm<pdd.pdos_lm[i].size();ilm++){
      int lmp=pdd.pdos_lm[i][ilm];
      if (lmp<1 || lmp>prd.nlmtot){
	cerr << "ERROR: ProjFuncs/ReadPDOSdat" << endl;
	cerr << "ERROR: Error in case: " << i+1 << endl;
	cerr << "ERROR: You have entered too low or high an lm number in entry: " << ilm+1 << endl;
	cerr << "ERROR: Bad value is: " << lmp << endl;
	cerr << "ERROR: Exiting." << endl;
	exit(1);
      }
    }
  }

}// end routine



// **************************************************
// CalcPDOS
// **************************************************
// Calculates the total PDOS based on the settings in pdd.

void CalcPDOS(const projdata & prd, pdosdata & pdd)
{
  double de=(pdd.emax-pdd.emin)/pdd.nbins;
  double e;
  int ibin;
  int sp_size=1;
  if (prd.sp) sp_size=3;
  pdd.pdos = matrix<double> (pdd.nbins,2*sp_size+1,0.0);
  for (int ib=0;ib<pdd.nbins;ib++){
    pdd.pdos[ib][0]=pdd.emin+(double)ib*de;
  }
  for (int i=0;i<pdd.pdos_at.size();i++){ // cases
    for (int ia=0;ia<pdd.pdos_at[i].size();ia++){
      int at=pdd.pdos_at[i][ia]-1;
      for (int ik=0;ik<pdd.pdos_k[i].size();ik++){
	int kp=pdd.pdos_k[i][ik]-1;
	for (int ib=0;ib<pdd.pdos_b[i].size();ib++){
	  int bp=pdd.pdos_b[i][ib]-1;
	  for (int ilm=0;ilm<pdd.pdos_lm[i].size();ilm++){
	    int lmp=pdd.pdos_lm[i][ilm]-1;
	    e = prd.ener_k_b_u[kp][bp];
	    ibin=int((e-pdd.emin)/de);
	    if (ibin>=0 && ibin<pdd.nbins){
	      pdd.pdos[ibin][1]+=prd.occ_vs_ion_kpt_bnd_lmtot_u[at][kp][bp][lmp]*prd.wkpt[kp];
	      if (prd.sp){
		e = prd.ener_k_b_d[kp][bp];
		ibin=int((e-pdd.emin)/de);
		pdd.pdos[ibin][2]+=prd.occ_vs_ion_kpt_bnd_lmtot_d[at][kp][bp][lmp]*prd.wkpt[kp];
	      }
	    }// ibin in range
	  }//ilm
	}//ik
      }//ib
    }//iat
  }//cases
  // Now do Up-Dn if spin polarized.
  if (prd.sp){
    for (int ibin=0;ibin<pdd.nbins;ibin++){ 
      pdd.pdos[ibin][3]=pdd.pdos[ibin][1]-pdd.pdos[ibin][2];
    }
  }

  // Normalize DOS to a density 
  for (int ib=0;ib<pdd.nbins;ib++){
    for (int i=1;i<sp_size+1;i++){
      pdd.pdos[ib][i]/=de;
    }
  }

  // Smooth DOS
  SmoothPDOS(prd,pdd);

  // Get integrated DOS
  for (int i=1;i<sp_size+1;i++){
    pdd.pdos[0][i+sp_size]=pdd.pdos[0][i]*de;
  }
  for (int ib=1;ib<pdd.nbins;ib++){
    for (int i=1;i<sp_size+1;i++){
      pdd.pdos[ib][i+sp_size]=pdd.pdos[ib-1][i+sp_size]+pdd.pdos[ib][i]*de;
    }
  }

}//end routine

// **************************************************
// SmoothPDOS
// **************************************************
// Smooths the total PDOS based on Gaussian smoothing.

void SmoothPDOS(const projdata & prd, pdosdata & pdd)
{
  double de=(pdd.emax-pdd.emin)/pdd.nbins;
  double e;
  int ibin;
  int sp_size=1;
  if (prd.sp) sp_size=3;

  // Get smoothed pdos.
  for (int is=1;is<=sp_size;is++){
    vector<double> tdos(pdd.nbins);
    for (int ib=0;ib<pdd.nbins;ib++){
      tdos[ib]=pdd.pdos[ib][is];
    }
    double sig=pdd.smooth_sigma/de;
    tdos=SmoothFunc(tdos,sig);
    for (int ib=0;ib<pdd.nbins;ib++){
      pdd.pdos[ib][is]=tdos[ib];
    }
  }

  /* Old smooth DOS calc - does not seem to work right!
  int range=(int)(5.0*pdd.smooth_sigma/de); // Uses gaussian out to 5 sigma.
  matrix<double> wt(pdd.nbins,2*range+1,0.0);

  // Get Gaussian weights
  vector<double> norm(pdd.nbins,0.0);
  for (int ib=0;ib<pdd.nbins;ib++){
    for (int i=-range;i<=range;i++){
      double x=(double)(i)*de+de/2.0;
      if ((ib+i)>=0 && (ib+i)<pdd.nbins){
	wt[ib][i+range]=Normal(x,de/2,pdd.smooth_sigma);
	norm[ib]=norm[ib]+wt[ib][i+range];
      }
    }
  }
  // Normalize to one
  for (int ib=0;ib<pdd.nbins;ib++){
    for (int i=-range;i<=range;i++){
      wt[ib][i+range]=wt[ib][i+range]/norm[ib];
    }
  }

  // Average in weighted nearby bins.
  for (int is=1;is<=sp_size;is++){
    for (int ib=0;ib<pdd.nbins;ib++){
      double tdos=0;
      for (int i=-range;i<=range;i++){
	if ((ib+i)>0 && (ib+i)<pdd.nbins){
	  tdos=tdos+wt[ib][i+range]*pdd.pdos[ib+i][is];
	}
      }
      pdd.pdos[ib][is]=tdos;
    }
  }

  */

}

// **************************************************
// AtomCntError
// **************************************************
// Smooths the total PDOS based on Gaussian smoothing.


void AtomCntError(const string & tok, const int tokcnt, const int atom_cnt)
{
  cerr << "ERROR: in AtomCntError " <<endl;
  cerr << "ERROR: The token " << tok << " has been defined (possibly by default) this many times: " << tokcnt << endl;
  cerr << "ERROR: You have used the token ATOMS this many times: " << atom_cnt << endl;
  cerr << "ERROR: These must be equal and >0 - the token ATOMS must precede each case and must occurr at least once. "<< endl;
  cerr << "ERROR: This error is probably due to your having forgot to use the ATOMS token. Please type ATOMS=*** as the first line for each case."<< endl;
  cerr << "ERROR: Exiting!" << endl;
  exit(1);
}
