import sys
import numpy as np

"""
TTM 2014-02-06

Citations:
1. 
Robin P. Sherman and Roger Grinter
Transformation matrices for the rotation of real p, d, and f atomic orbitals
Journal of Molecular Structure (Theochem), 135 (1986) 127-133

2.
x- and y-conventions, Wolfram Alpha, attributed to
H. Goldstein, Classical Mechanics, Reading, MA:
Addison-Wesley, 1980.

3.
Zhengwei Su and Philip Coppens
Rotation of Real Spherical Harmonics
Acta Cryst. (1994) A50 636-643
"""
class Rotator():
    """
    Attributes:
    self.angA <float>: Euler angle A
        For a right-hand coordinate system, rotate
        right-hand finger sweep with thumb along z axis
    self.angB <float>: Euler angle B
        For a right-hand coordinate system, rotate
        for x-convention: right-hand finger sweep with thumb along 'new x'
        for y-convention: right-hand finger sweep with thumb along 'new y'
    self.angC <float>: Euler angle C
        For a right-hand coordinate system, rotate
        right-hand finger sweep along 'new z'
    self.radA, B, and C: Euler angles in radians
    self.cosA, B, and C
    self.sinA, B, and C
    self.cos2A, B, and C
    self.sin2A, B, and C
    self.sinsqA, B, and C (sine squared)
    self.cossqA, B, and C
    self.zArotrhs:righthand sweep (counterclockwise) rotation around z-axis
                    by angle A
    self.yBrotrhs:right-hand sweep (ccwise) rotation around y-axis
                    by angle B
    self.xBrotrhs:right-hand sweep (ccwise) rotation around x-axis
                    by angle B
    self.zCrotrhs: righthand sweep (ccwise) rotation around new z-axis
                    by angle C
    self.rot_zxz: <np.matrix of float>: Euler rotation matrix,
                    x-convention (Goldstein)
    self.rot_zyz: <np.matrix of float>: Euler rotation matrix, 
                    y-convention (Goldstein)
    self.sherman_grinter_coord_rot: <np.matrix of float>: Euler rotation
                    matrix given by Sherman and Grinter
    self.su_coppens_coord_rot: <np.matrix of float>: Euler rotation
                    matrix given by Su and Coppens
    self.orig_coords <np.matrix of float, column vector>: 
                        original coordinates
    self.rotated_coords <np.matrix of float, column vector>: 
                    rotated coords
    self.orig_d_orbital_occs <np.matrix of float, column vector>:
        original d orbital occupations, in order dxy, dyz, dz2, dxz, dx2-y2
    self.rotated_d_orbital_occs <np.matrix of float, column vector>:
        rotated d orbital occupations, in order DXY, DYZ, DZ2, DXZ, DX2-Y2
    self.coords_or_orbitals = "coords" to rotate coords, "orbitals" 
        to rotate orbitals
    """
    def __init__(self, origstring,
                 angA, angB, angC,
                 deg_or_rad="degrees",
                 coords_or_orbitals="coords"):
        """Set up the coordinate or d-orbital rotator.
            Set up as my_rotator = Rotator(<args>) where arguments are
            as follows:
            Args:
                origstring <str>: String of coordinates or d-orbital occupations
                    ex. "0.5 1.0 3.3"
                angA <str or float>: Euler rotation angle A
                angB <str or float>: Euler rotation angle B
                angC <str or float>: Euler rotation angle C
                deg_or_rad <str>: "degrees" or "radians", for the units in which
                                    angA, angB, and angC are entered
                coords_or_orbitals <str>: "coords" or "orbitals" for
                        rotating coordinates or d-orbital occupations,
                        respectively
        """
        self.debug=False
        self.rotated_coords=""
        self.rotated_d_orbital_occs=""
        self.Rmatrix=""
        self.dorbR=""
        self.set_up_angles(angA, angB, angC, deg_or_rad)
        self.set_up_common_rotation_matrices()
        self.coords_or_orbitals = coords_or_orbitals
        mylist = origstring.split()
        if coords_or_orbitals == "coords":
            self.orig_coords = np.matrix(mylist, float).transpose()
        elif coords_or_orbitals == "orbitals":
            self.orig_d_orbital_occs = np.matrix(mylist, float).transpose()
            self.sqrt_d_orbital_occs = np.sqrt(self.orig_d_orbital_occs)
        return

    def do_rotations(self):
        """Do orbital or coordinate rotations."""
        if self.coords_or_orbitals == "coords":
            self.rotate_coords(self.rot_zyz, "rotate y-convention")
            retstr=""
            for idx in range(0,self.rotated_coords.shape[0]):
                retstr = retstr + "%3.3f " % self.rotated_coords[idx,0]
        elif self.coords_or_orbitals == "orbitals":

            comp_rot = self.get_complex_Y_orb_rot_mat()
            real_to_comp = self.turn_real_y_mat_into_complex_mat()
            comp_to_real = self.turn_complex_Y_mat_into_real_mat()
            #self.print_matrix(comp_rot, "Complex rotation")
            #self.print_matrix(real_to_comp, "Real to comp")
            #self.print_matrix(comp_to_real, "Comp to real")
            #self.print_matrix(real_to_comp.getI(), "Inverse of real to comp")
            #self.print_matrix(real_to_comp.getI()-comp_to_real, "Inv of real to comp minus comp to real")
            #print "Sum of real_to_comp.getI() - comp_to_real: ", (real_to_comp.getI()-comp_to_real).sum()
            #sg_as_written = self.get_sherman_and_grinter_orbital_rotation_matrix()
            #self.print_matrix(sg_as_written, "Sherman Grinter rotation matrix")
            #self.print_matrix(comp_to_real * comp_rot * real_to_comp, "comp_to_real * comp_rot * real_to_comp")
            #real_rot_mat = self.get_real_y_orb_rot_mat()
            #self.print_matrix(real_rot_mat, "real rot mat Su Coppens")
            #self.rotate_d_orbitals(self.dorbR, "Sherman Grinter rotation")
            #self.rotate_d_orbitals(comp_to_real * comp_rot * real_to_comp, "realtocomp, comprot, comptoreal")
            #self.rotate_d_orbitals(real_rot_mat, "real rot mat")
            #self.rotate_d_orbitals(comp_to_real * real_to_comp, "real to comp to real - no rotation")
            #sg_explicit_zxz = self.sherman_grinter_method_explicit_mat(self.rot_zxz)
            #rot2 = self.rotate_d_orbitals(sg_explicit_zxz, "Sherman Grinter zxz")
            sg_explicit = self.sherman_grinter_method_explicit_mat(self.rot_zyz)
            goodrot = self.rotate_d_orbitals(sg_explicit, "Sherman Grinter explicit matrix")
            goodrot = np.multiply(goodrot, goodrot)
            print goodrot.transpose()
            retstr = ""
            for gdx in range(0, goodrot.shape[0]):
                retstr = retstr + "%3.3f " % goodrot[gdx,0]
        if self.debug:
            self.print_me()
        return retstr
    def print_me(self):
        """Print output."""
        mydict = self.__dict__
        mykeys = mydict.keys()
        mykeys.sort()
        print "----------------------"
        for key in mykeys:
            value = mydict[key]
            if type(value) == np.matrix:
                self.print_matrix(value, key)
            else:
                print "%30s %s" % (key, value)
        return

    def print_matrix(self, mymatrix, matname=""):
        """Print a float matrix.
            Args:
                mymatrix <np.matrix of float>: matrix to print
                matname <str>: matrix name
        """
        if mymatrix=="":
            print "Matrix %s is empty." % matname
            return
        print "Printing matrix %s" % matname
        nrows = mymatrix.shape[0]
        ncols = mymatrix.shape[1]
        iscomplex = False
        if 'complex' in mymatrix.dtype.name:
            iscomplex = True
        for my_r in range(0, nrows):
            printstr=""
            rlist = mymatrix[my_r].tolist()[0]
            for my_c in range(0, ncols):
                if iscomplex:
                    printstr = printstr + "% 3.3f%+3.3fi " % (rlist[my_c].real,rlist[my_c].imag)
                else:
                    printstr = printstr + "% 3.3f " % rlist[my_c]
            print printstr
        return

    def set_up_angles(self, angA, angB, angC, deg_or_rad="degrees"):
        """Set up the angles and sines and cosines
            Args:
                angA <float>: Euler angle A
                angB <float>: Euler angle B
                angC <float>: Euler angle C
                deg_or_rad <str>: "degrees" - angles are entered in degrees
                               "radians" - angles are entered in radians
            Returns:
                True if successfully set the values
                self.radA,B,C, self.cosA,B,C, etc.
        """
        if not deg_or_rad in ["radians","degrees"]:
            raise ValueError("%s must be either 'degrees' or 'radians'" % deg_or_rad)
            return False
        self.angA = float(angA)
        self.angB = float(angB)
        self.angC = float(angC)
        if deg_or_rad=="degrees":
            self.radA = self.angA * np.pi/180
            self.radB = self.angB * np.pi/180
            self.radC = self.angC * np.pi/180
        else:
            self.radA = self.angA
            self.radB = self.angB
            self.radC = self.angC
        self.cosA = np.cos(self.radA)
        self.cosB = np.cos(self.radB)
        self.cosC = np.cos(self.radC)
        self.sinA = np.sin(self.radA)
        self.sinB = np.sin(self.radB)
        self.sinC = np.sin(self.radC)
        self.cos2A = np.cos(2*self.radA)
        self.cos2B = np.cos(2*self.radB)
        self.cos2C = np.cos(2*self.radC)
        self.sin2A = np.sin(2*self.radA)
        self.sin2B = np.sin(2*self.radB)
        self.sin2C = np.sin(2*self.radC)
        self.cossqA = self.cosA * self.cosA
        self.cossqB = self.cosB * self.cosB
        self.cossqC = self.cosC * self.cosC
        self.sinsqA = self.sinA * self.sinA
        self.sinsqB = self.sinB * self.sinB
        self.sinsqC = self.sinC * self.sinC

        if self.debug:
            print "-------------------------"
            print "Printing angle table:"
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Angles",self.angA, self.angB, self.angC)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Angles, radians", self.radA, self.radB, self.radC)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Cosines", self.cosA, self.cosB, self.cosC)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Sines", self.sinA, self.sinB, self.sinC)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Cosine 2*angles", self.cos2A, self.cos2B, self.cos2C)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Sine 2*angles", self.sin2A, self.sin2B, self.sin2C)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Cosine squares", self.cossqA, self.cossqB, self.cossqC)
            print "%30s; % 3.3f; % 3.3f; % 3.3f" % ("Sine squares", self.sinsqA, self.sinsqB, self.sinsqC)
            print "----------------------"
        return True

    
    
    def set_up_common_rotation_matrices(self):
        """Set up common rotation matrices.
            Args:
                None, but angles self.radA, etc. should have been set.
            Returns:
                Sets up self.rot_zxz, self.rot_zyz, self.zrotrhs, etc.
        """
        cosA = self.cosA
        cosB = self.cosB
        cosC = self.cosC
        sinA = self.sinA
        sinB = self.sinB
        sinC = self.sinC
        
        self.zArotrhs = np.matrix([[ cosA,  sinA,     0],
                                  [-sinA,  cosA,     0],
                                  [    0,     0,     1]], float)
        
        self.xBrotrhs = np.matrix([[    1,     0,     0],
                                   [    0,  cosB,  sinB],
                                   [    0, -sinB,  cosB]], float)
        
        self.yBrotrhs = np.matrix([[ cosB,     0,  sinB],
                                   [    0,     1,     0],
                                   [-sinB,     0,  cosB]], float)
        
        self.zCrotrhs = np.matrix([[ cosC,  sinC,     0],
                                   [-sinC,  cosC,     0],
                                   [    0,     0,     1]], float)

        self.yBrotlhs = np.matrix([[ cosB,     0, -sinB],
                                   [    0,     1,     0],
                                   [ sinB,     0,  cosB]], float)

        self.rot_zxz = self.zCrotrhs * (self.xBrotrhs * self.zArotrhs)
        self.rot_zyz = self.zCrotrhs * (self.yBrotrhs * self.zArotrhs)
        
        if self.debug:
            self.print_matrix(self.rot_zxz, "x-convention rotation matrix")
            self.print_matrix(self.rot_zyz, "y-convention rotation matrix")

    def set_up_specific_rotation_matrices(self):
        """Set up specific rotation matrices from papers.
        """
        cosA = self.cosA
        cosB = self.cosB
        cosC = self.cosC
        sinA = self.sinA
        sinB = self.sinB
        sinC = self.sinC
        el_00 = cosC*cosA - cosB*sinA*sinC
        el_01 = cosC*sinA + cosB*cosA*sinC
        el_02 = sinC*sinB
        el_10 = -sinC*cosA - cosB*sinA*cosC
        el_11 = -sinC*sinA + cosB*cosA*cosC
        el_12 = cosC*sinB
        el_20 = sinB*sinA
        el_21 = -sinB*cosA
        el_22 = cosB

        self.sherman_grinter_coord_rot=np.matrix([[el_00,el_01,el_02],
                                [el_10,el_11,el_12],
                                [el_20,el_21,el_22]],float)
        if self.debug:
            self.print_matrix(self.sherman_grinter_coord_rot, "Sherman Grinter coordinate rotation")
        
        fl_00 = cosA*cosB*cosC - sinA*sinC
        fl_01 = cosA*sinC + sinA*cosB*cosC
        fl_02 = -sinB*cosC
        fl_10 = -cosA*cosB*sinC - sinA*cosC
        fl_11 = cosA*cosC - sinA*cosB*sinC
        fl_12 = sinB*sinC
        fl_20 = cosA*sinB
        fl_21 = sinA*sinB
        fl_22 = cosB

        self.su_coppens_coord_rot = np.matrix([[fl_00,fl_01,fl_02],
                            [fl_10,fl_11,fl_12],
                            [fl_20,fl_21,fl_22]], float)

        if self.debug:
            self.print_matrix(self.su_coppens_coord_rot, "Su Coppens coordinate rotation")
            #self.print_matrix(self.zArotrhs*(self.yBrotrhs*self.zCrotrhs), "Su and Coppens rotate C angle first?")
            self.print_matrix(self.zCrotrhs*(self.yBrotlhs*self.zArotrhs), "Su and Coppens use y-convention but rotate B angle backwards?")
            #self.print_matrix(self.rot_zyz,"Su and Coppens use standard y-convention?")
        return

    def turn_complex_Y_mat_into_real_mat(self):
        """Turn complex Y matrix into a real y matrix
        Formulas in Su and Coppens. Also as
        Wikipedia http://en.wikipedia.org/wiki/Spherical_harmonics#Real_form 
        y_lm = (i/sqrt2)(Y_l,-|m| -(-1)^m Y_l|m|) if m < 0
               Y_l0 if m = 0
                (1/sqrt2)(Y_l,-|m| + (-1)^m Y_l|m|) if m > 0

        given all l = 2,
        y_n2 = (i/sqrt2)(Y_n2 - Y_p2)     =dxy
        y_n1 = (i/sqrt2)(Y_n1 + Y_p1)     =dyz
        y_p0 = Y_p0                       =dz2
        y_p1 = (1/sqrt2)(Y_n1 - Y_p1)     =dxz
        y_p2 = (1/sqrt2)(Y_n2 + Y_p2)     =dx2y2

        """
        sqrt2 = np.sqrt(2)
        isqrt2 = np.sqrt(2)*1j
        
        el_yn2_Yn2 = (1j/sqrt2)
        el_yn2_Yp2 = (-1j/sqrt2)
        el_yn1_Yn1 = (1j/sqrt2)
        el_yn1_Yp1 = (1j/sqrt2)
        el_yp0_Yp0 = 1
        el_yp1_Yn1 = (1/sqrt2)
        el_yp1_Yp1 = (-1/sqrt2)
        el_yp2_Yn2 = (1/sqrt2)
        el_yp2_Yp2 = (1/sqrt2)


        #el_yp0_Yp0 = 1
        #el_yp1_Yp1 = np.power(-1,1)*1/sqrt2
        #el_yp1_Yn1 = 1/sqrt2
        #el_yp2_Yp2 = np.power(-1,2)*1/sqrt2
        #el_yp2_Yn2 = 1/sqrt2
        #el_yn1_Yp1 = np.power(-1,1)*1/isqrt2
        #el_yn1_Yn1 = -1/isqrt2
        #el_yn2_Yp2 = np.power(-1,2)*1/isqrt2
        #el_yn2_Yn2 = -1/isqrt2
        comp_to_real = np.matrix([[el_yn2_Yn2,
                                  0,          0,          0, el_yn2_Yp2],
            [         0, el_yn1_Yn1,          0, el_yn1_Yp1,          0],
            [         0,          0, el_yp0_Yp0,          0,          0],
            [         0, el_yp1_Yn1,          0, el_yp1_Yp1,          0],
            [el_yp2_Yn2,          0,          0,          0, el_yp2_Yp2]],
            complex)
        if self.debug:
            self.print_matrix(comp_to_real, "transform complex to real, Su and Coppens")
        return comp_to_real

    def turn_real_y_mat_into_complex_mat(self):
        """Turn real y matrix into complex Y matrix
        Wikipedia http://en.wikipedia.org/wiki/Spherical_harmonics#Real_form 
        y_lm = (i/sqrt2)(Y_l,-|m| -(-1)^m Y_l|m|) if m < 0
               Y_l0 if m = 0
                (1/sqrt2)(Y_l,-|m| + (-1)^m Y_l|m|) if m > 0

        given all l = 2,
        y_n2 = (i/sqrt2)(Y_n2 - Y_p2)     =dxy
        y_n1 = (i/sqrt2)(Y_n1 + Y_p1)     =dyz
        y_p0 = Y_p0                       =dz2
        y_p1 = (1/sqrt2)(Y_n1 - Y_p1)     =dxz
        y_p2 = (1/sqrt2)(Y_n2 + Y_p2)     =dx2y2

        i*y_n2 = (-1/sqrt2)(Y_n2 - Y_p2) = (1/sqrt2)(-Y_n2 + Y_p2)
        i*y_n1 = (-1/sqrt2)(Y_n1 + Y_p1) = (1/sqrt2)(-Y_n1 - Y_p1)
        y_p2 + iy_n2 = (1/sqrt2)(2*Y_p2)
        Y_p2 = (sqrt2/2)(y_p2 + i*y_n2)
        y_p2 - iy_n2 = (1/sqrt2)(2*Y_n2)
        Y_n2 = (sqrt2/2)(y_p2 - i*y_n2)
        y_p1 + iy_n1 = (1/sqrt2)(-2*Y_p1)
        Y_p1 = (-sqrt2/2)(y_p1 + i*y_n1)
        y_p1 - iy_n1 = (1/sqrt2)(2*Y_n1)
        Y_n1 = (sqrt2/2)(y_p1 - i*y_n1)

        #Treatment in Su and Coppens seems incorrect: (i/2)^(1/2)
        # seems like it should really be i*sqrt(0.5)
        # which is equivalent to -1/(i*sqrt(2))
        #dz2 = Y_0, l=2 = y_p0
        #dyz = isqrt(1/2)[Y_n1 + Y_p1] = [-Y_n1-Y_p1]/isqrt2=y_n1
        #dxz = sqrt(1/2)[Y_n1 - Y_p1] = y_p1
        #dx2y2 = sqrt(1/2)[Y_n2 + Y_p2] = y_p2
        #dxy = isqrt(1/2)[Y_n2 - Y_p2] =[-Y_n2 +Y_p2]/isqrt2=y_n2
        isqrt2*y_n1 = -Y_n1 - Y_p1
        sqrt2 *y_p1 = Y_n1  - Y_p1
        -2Y_p1 = isqrt2*y_n1 + sqrt2*y_p1
        Y_p1 = -(isqrt2/2)y_n1 - (sqrt2/2)y_p1
        Y_n1 = sqrt2*y_p1 + (sqrt2/2)y_p1 + (isqrt2/2)y_n1
        isqrt2*y_n2 = -Y_n2 + Y_p2
        sqrt2*y_p2 = Y_n2 + Y_p2
        2*Y_p2 = isqrt2*y_n2 + sqrt2*y_p2
        Y_p2 = (isqrt2/2)y_n2 + (sqrt2/2)y_p2
        Y_n2 = (isqrt2/2)y_n2 + (sqrt2/2)y_p2 - (isqrt2)y_n2
        """
        sqrt2 = np.sqrt(2)
        #Y_n2 = (sqrt2/2)(y_p2 - i*y_n2)
        #Y_n1 = (sqrt2/2)(y_p1 - i*y_n1)
        #Y_p1 = (-sqrt2/2)(y_p1 + i*y_n1)
        #Y_p2 = (sqrt2/2)(y_p2 + i*y_n2)
        
        el_Yn2_yp2 = (sqrt2/2)
        el_Yn2_yn2 = (-1j*sqrt2/2)
        el_Yn1_yp1 = (sqrt2/2)
        el_Yn1_yn1 = (-1j*sqrt2/2)
        el_Yp0_yp0 = 1
        el_Yp1_yp1 = (-sqrt2/2)
        el_Yp1_yn1 = (-1j*sqrt2/2)
        el_Yp2_yp2 = (sqrt2/2)
        el_Yp2_yn2 = (1j*sqrt2/2)
        
        #el_Yp1_yn1 = -(isqrt2/2)
        #el_Yp1_yp1 = -(sqrt2/2)
        #el_Yn1_yn1 = (isqrt2/2)
        #el_Yn1_yp1 = (3*sqrt2/2)
        #el_Yp2_yn2 = (isqrt2/2)
        #el_Yp2_yp2 = (sqrt2/2)
        #el_Yn2_yn2 = (isqrt2/2) - isqrt2
        #el_Yn2_yp2 = (sqrt2/2)

        real_to_comp = np.matrix([[el_Yn2_yn2,
                                  0,          0,          0, el_Yn2_yp2],
            [         0, el_Yn1_yn1,          0, el_Yn1_yp1,          0],
            [         0,          0, el_Yp0_yp0,          0,          0],
            [         0, el_Yp1_yn1,          0, el_Yp1_yp1,          0],
            [el_Yp2_yn2,          0,          0,          0, el_Yp2_yp2]],
                    complex)


        if self.debug:
            self.print_matrix(real_to_comp, "transform real to complex")
        return real_to_comp

    def get_real_y_orb_rot_mat(self):
        """Get real Y rotation omatrix as described in Su and Coppens.
        """
        myl=2
        alpha=self.radA
        beta=self.radB
        gamma=self.radC
        sqrt2 = np.sqrt(2)
        tarm=2
        curm=0
        el_yp2_yp0 = np.power(-1,tarm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma)*sqrt2
        curm=1
        el_yp2_yp1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) + np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=2
        el_yp2_yp2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) + np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=1
        el_yp2_yn1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) - np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=2
        el_yp2_yn2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) - np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        tarm=1
        curm=0
        el_yp1_yp0 = np.power(-1,tarm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma)*sqrt2
        curm=1
        el_yp1_yp1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) + np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=2
        el_yp1_yp2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) + np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=1
        el_yp1_yn1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) - np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=2
        el_yp1_yn2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) - np.power(-1, tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        tarm=2
        curm=0
        el_yn2_yp0 = np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma)*sqrt2
        curm=2
        el_yn2_yp2 = np.power(-1,tarm+curm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) + np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=1
        el_yn2_yp1 = np.power(-1,tarm+curm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) + np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=2
        el_yn2_yn2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) - np.power(-1,tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=1
        el_yn2_yn1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) - np.power(-1,tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        tarm=1
        curm=0
        el_yn1_yp0 = np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma)*sqrt2
        curm=2
        el_yn1_yp2 = np.power(-1,tarm+curm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) + np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=1
        el_yn1_yp1 = np.power(-1,tarm+curm+1)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(tarm*gamma + curm*alpha) + np.power(-1,tarm+1)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(tarm*gamma - curm*alpha)
        curm=2
        el_yn1_yn2 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) - np.power(-1,tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        curm=1
        el_yn1_yn1 = np.power(-1,tarm+curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(tarm*gamma + curm*alpha) - np.power(-1,tarm)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(tarm*gamma - curm*alpha)
        tarm=0
        curm=0
        el_yp0_yp0 = self.get_wigner_smalld(tarm,curm,myl,beta)
        curm=2
        el_yp0_yp2 = (1/sqrt2)*np.power(-1,curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(curm*alpha) + (1/sqrt2)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(curm*alpha)
        curm=1
        el_yp0_yp1 = (1/sqrt2)*np.power(-1,curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.cos(curm*alpha) + (1/sqrt2)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.cos(curm*alpha)
        curm=2
        el_yp0_yn2 = (1/sqrt2)*np.power(-1,curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(curm*alpha) + (1/sqrt2)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(curm*alpha)
        curm=1
        el_yp0_yn1 = (1/sqrt2)*np.power(-1,curm)*self.get_wigner_smalld(tarm,curm,myl,beta)*np.sin(curm*alpha) + (1/sqrt2)*self.get_wigner_smalld(tarm,-curm,myl,beta)*np.sin(curm*alpha)

        real_rot_mat = np.matrix([[el_yn2_yn2,
                      el_yn2_yn1, el_yn2_yp0, el_yn2_yp1, el_yn2_yp2],
         [el_yn1_yn2, el_yn1_yn1, el_yn1_yp0, el_yn1_yp1, el_yn1_yp2],
         [el_yp0_yn2, el_yp0_yn1, el_yp0_yp0, el_yp0_yp1, el_yp0_yp2],
         [el_yp1_yn2, el_yp1_yn1, el_yp1_yp0, el_yp1_yp1, el_yp1_yp2],
         [el_yp2_yn2, el_yp2_yn1, el_yp2_yp0, el_yp2_yp1, 
                    el_yp2_yp2]], complex)
        #normalizations L/M to get d instead of y:
        norp0 = 0.65553
        norp1 = 0.68647 # which also equals nor_n1
        norp2 = 0.68647 # which also equals nor_n2
        norn2 = 0.68647
        norn1 = 0.68647
        nor_mat = np.matrix([[1,
                         1, norn2/norp0, 1, 1],
                     [1, 1, norn1/norp0, 1, 1],
                     [1, 1, norp0/norp0, 1, 1],
                     [1, 1, norp1/norp0, 1, 1],
                     [1, 1, norp2/norp0, 1, 1]], complex)

        #element-by-element multiplication
        real_rot_mat_for_d = np.multiply(real_rot_mat, nor_mat) 



        return real_rot_mat_for_d

    def get_complex_Y_orb_rot_mat(self):
        """Get complex Y rotation matrix as described in
            Su and Coppens.
                    Be careful here. They use primes for the rotated
                    theta and phi, but the NOT prime m is the 
                    complex m value for the rotated system.
                    So, target m's are "m" while current m's are "m'".
        """
        complex_mat = np.matrix(np.zeros([5,5]),complex)
        myl = 2
        tarm_idx = 0
        for tarm in [-2,-1,0,1,2]:
            curm_idx = 0
            for curm in [-2,-1,0,1,2]:
                complex_mat[tarm_idx, curm_idx] = self.get_wigner_bigd(tarm,
                        curm, myl, self.radA, self.radB, self.radC)
                curm_idx = curm_idx + 1
            tarm_idx = tarm_idx + 1
        if self.debug:
            self.print_matrix(complex_mat, "complex rotation matrix")
        return complex_mat

    def get_wigner_bigd(self,tarm, curm, myl, alpha, beta, gamma):
        """Get Wigner matrix D(l)_m'm as described in Su and Coppens.
            Args:
                tarm <int>: target m value in rotated system, m
                    Be careful here. They use primes for the rotated
                    theta and phi, but the NOT prime m is the 
                    complex m value for the rotated system.
                curm <int>: current quantum number m value, m'
                myl <int>: current quantum number l value, l
                alpha <float>: Euler angle A, in radians
                beta <float>: Euler angle B, in radians
                gammaa <float>: Euler angle C, in radians
            Returns:
                <float>: value for D(l)_m'm
        """
        #may be incorrect in numpy: 
        #alphaterm = np.exp(np.complex(0,-1*curm*alpha))
        #gammaterm = np.exp(np.complex(0,-1*tarm*gamma))
        alphaterm = np.cos(curm*alpha)-1j*np.sin(curm*alpha)
        gammaterm = np.cos(tarm*gamma)-1j*np.sin(tarm*gamma)
        smalld = self.get_wigner_smalld(tarm, curm, myl, beta)
        return alphaterm*smalld*gammaterm

    def get_wigner_smalld(self,tarm,curm,myl,beta):
        """Get Wigner matrix d(l)_m'm as described in Su and Coppens.
            Args:
                tarm <int>: target m value in rotated system, m
                curm <int>: current quantum number m value, m'
                myl <int>: current quantum number l value, l
                beta <float>: Euler angle B, in radians
            Returns:
                <float>: value for d(l)_m'm
        """
        #return self.look_up_wigner_smalld(tarm,curm)
        prefact = self.get_wigner_smalld_prefactor(tarm, curm, myl)
        posneg = np.power(-1,(curm - tarm))
        krange = self.get_wigner_smalld_krange(tarm, curm, myl)
        ksum = 0
        for kval in krange:
            kposneg = np.power(-1, kval)
            par1 = self.from_topval_choose_botval(myl+tarm, kval)
            par2 = self.from_topval_choose_botval(myl-tarm, myl-curm-kval)
            costerm = np.power(np.cos(0.5*beta),2*myl-curm+tarm-2*kval)
            sinterm = np.power(np.sin(0.5*beta),2*kval-tarm+curm)
            kterm = kposneg*par1*par2*costerm*sinterm
            ksum = ksum + kterm
        return prefact*posneg*ksum

    def get_wigner_smalld_prefactor(self, tarm, curm, myl):
        """Get prefactor for d(l)_m'm as described in Su and Coppens.
            Args:
                tarm <int>: target m value in rotated system, m
                curm <int>: current quantum number m value, m'
                myl <int>: current quantum number l value, l
            Returns:
                <float>: prefactor with the factorials
        """
        pc1 = np.math.factorial(myl + curm)
        pc2 = np.math.factorial(myl - curm)
        pc3 = np.math.factorial(myl + tarm)
        pc4 = np.math.factorial(myl - tarm)
        divme = (pc1*pc2)/(pc3*pc4)
        return np.sqrt(divme)


    def get_wigner_smalld_krange(self,tarm,curm,myl):
        """Get range of k for d(l)_m'm as described in Su and Coppens.
            Args:
                tarm <int>: target m value in rotated system, m
                curm <int>: current quantum number m value, m'
                myl <int>: current quantum number l value, l
            Returns:
                <list of int>: k range for d(l)_m'm
        """
        lowerbound = np.max([0, tarm-curm])
        upperbound = np.min([myl-curm, myl+tarm])
        return range(lowerbound, upperbound+1)
        

    def from_topval_choose_botval(self,topval,botval):
        """Get topvalue choose bottomvalue:
            Args:
                topval <int>: top value, to choose from
                botval <int>: bottom value, number to choose
            Returns:
                topval!/[(topval-botval)!botval!]
        """
        top1 = np.math.factorial(topval)
        bot1 = np.math.factorial(topval - botval)
        bot2 = np.math.factorial(botval)
        return (top1/(bot1*bot2))
    def look_up_wigner_smalld(self, tarm, curm):
        """Look up wigner small d from Wikipedia table
        """
        cosB = self.cosB
        sinB = self.sinB
        sinsqB = self.sinsqB
        cossqB = self.cossqB
        sin2B = self.sin2B
        #d_currentm_targetm
        d_p2_p2 = 0.25*np.power((1+cosB),2)
        d_p2_p1 = -0.5*sinB*(1+cosB)
        d_p2_p0 = np.sqrt(0.375)*sinsqB
        d_p2_n1 = -0.5*sinB*(1-cosB)
        d_p2_n2 = 0.25*np.power((1-cosB),2)
        
        d_p1_p2 = -1*d_p2_p1
        d_p1_p1 = 0.5*(2*cossqB+cosB-1)
        d_p1_p0 = -np.sqrt(0.375)*sin2B
        d_p1_n1 = 0.5*(-2*cossqB+cosB+1)
        d_p1_n2 = d_p2_n1
        
        d_p0_p2 = 1*d_p2_p1
        d_p0_p1 = -1*d_p1_p0
        d_p0_p0 = 0.5*(3*cossqB-1)
        d_p0_n1 = d_p1_p0
        d_p0_n2 = d_p2_p0

        d_n1_p2 = -1*d_p2_n1
        d_n1_p1 = 1*d_p1_n1
        d_n1_p0 = -1*d_p0_n1
        d_n1_n1 = d_p1_p1
        d_n1_n2 = d_p2_p1
        
        d_n2_p2 = 1*d_p2_n2
        d_n2_p1 = -1*d_p1_n2
        d_n2_p0 = 1*d_p0_n2
        d_n2_n1 = -1*d_n1_n2
        d_n2_n2 = d_p2_p2

        wigner_lookup=np.matrix([[d_n2_n2,
                     d_n2_n1, d_n2_p0, d_n2_p1, d_n2_p2],
          [ d_n1_n2, d_n1_n1, d_n1_p0, d_n1_p1, d_n1_p2],
          [ d_p0_n2, d_p0_n1, d_p0_p0, d_p0_p1, d_p0_p2],
          [ d_p1_n2, d_p1_n1, d_p1_p0, d_p1_p1, d_p1_p2],
          [ d_p2_n2, d_p2_n1, d_p2_p0, d_p2_p1, d_p2_p2]],
          complex)
        return wigner_lookup[curm, tarm]


    def rotate_coords(self, rot_mat, rotlabel="coordinate rotation",verbose=False):
        """Rotate coordinates by the specified rotation matrix
            Args:
                rot_mat <np.matrix of float>: Rotation matrix to use
                rotlabel <str>: Label for rotation
                verbose <bool>: False - do not print
                               True - print
            Returns:
                Rotated coordinates <np.matrix>:
                [[coordX],
                [coordY],
                [coordZ]]
                Sets self.rotated_coords
        """
        if self.debug:
            verbose = True
        if verbose:
            self.print_matrix(rot_mat, rotlabel)
            self.check_for_orthonormal_matrix(rot_mat, rotlabel)
        self.rotated_coords = rot_mat * self.orig_coords
        if verbose:
            self.print_matrix(self.orig_coords, "original coords")
            self.print_matrix(self.rotated_coords, "rotated coords")
        return self.rotated_coords

    def get_sherman_and_grinter_orbital_rotation_matrix(self):
        """Get Sherman and Grinter orbital rotation matrix
        """
        #short form for ease in formula writing
        cosA = self.cosA
        cosB = self.cosB
        cosC = self.cosC
        sinA = self.sinA
        sinB = self.sinB
        sinC = self.sinC
        cos2A = self.cos2A
        cos2B = self.cos2B
        cos2C = self.cos2C
        sin2A = self.sin2A
        sin2B = self.sin2B
        sin2C = self.sin2C
        cossqA = self.cossqA
        cossqB = self.cossqB
        cossqC = self.cossqC
        sinsqA = self.sinsqA
        sinsqB = self.sinsqB
        sinsqC = self.sinsqC
        sqrt3over2 = np.sqrt(3)/2

        #given in the primary section
        el_X2Y2_x2y2 = 0.5*cos2A*(1+cossqB)*cos2C - sin2A*cosB*sin2C
        el_X2Y2_z2 = -sqrt3over2*sinsqB*cos2C
        el_X2Y2_xz = 0.5*sinA*sin2B*cos2C + cosA*sinB*sin2C
        #converted using rule i, cosNA -> sinNA, sinNA -> -cosNA
        #el_X2Y2_x2y2 = 0.5*cos2A*(1+cossqB)*cos2C - sin2A*cosB*sin2C
        el_X2Y2_xy = 0.5*sin2A*(1+cossqB)*cos2C - -cos2A*cosB*sin2C
        #el_X2Y2_xz = 0.5*sinA*sin2B*cos2C + cosA*sinB*sin2C
        el_X2Y2_yz = 0.5*-cosA*sin2B*cos2C + sinA*sinB*sin2C

        #converted using rule ii, cosNC -> -sinNC, sinNC -> cosNC
        #el_X2Y2_x2y2 = 0.5*cos2A*(1+cossqB)*cos2C - sin2A*cosB*sin2C
        el_XY_x2y2 = 0.5*cos2A*(1+cossqB)*-sin2C - sin2A*cosB*cos2C
        #el_X2Y2_z2 = -sqrt3over2*sinsqB*cos2C
        el_XY_z2 = -sqrt3over2*sinsqB*-sin2C
        #el_X2Y2_xz = 0.5*sinA*sin2B*cos2C + cosA*sinB*sin2C
        el_XY_xz = 0.5*sinA*sin2B*-sin2C + cosA*sinB*cos2C
        #el_X2Y2_xy = 0.5*sin2A*(1+cossqB)*cos2C - -cos2A*cosB*sin2C
        el_XY_xy = 0.5*sin2A*(1+cossqB)*-sin2C - -cos2A*cosB*cos2C
        #el_X2Y2_yz = 0.5*-cosA*sin2B*cos2C + sinA*sinB*sin2C
        el_XY_yz = 0.5*-cosA*sin2B*-sin2C + sinA*sinB*cos2C

        #given in the primary section
        el_Z2_z2 = cossqB - 0.5*sinsqB
        el_Z2_xz = sqrt3over2*sinA*sin2B
        #converted using rule iii, cosNC -> cosNA, cosNA -> cosNC,
        #                          sinNC -> -sinNA, sinNA -> -sinNC
        #el_XY_z2 = -sqrt3over2*sinsqB*-sin2C
        el_Z2_xy = -sqrt3over2*sinsqB*sin2A
        #el_X2Y2_z2 = -sqrt3over2*sinsqB*cos2C
        el_Z2_x2y2 = -sqrt3over2*sinsqB*cos2A
        #converted using rule i, cosNA -> sinNA, sinNA -> -cosNA
        #el_Z2_xz = sqrt3over2*sinA*sin2B
        el_Z2_yz = sqrt3over2*-cosA*sin2B

        #given in the primary section
        el_XZ_xz = cosA*cosB*cosC - sinA*cos2B*sinC
        #converted using rule iii, cosNC -> cosNA, cosNA -> cosNC,
        #                          sinNC -> -sinNA, sinNA -> -sinNC
        #el_XY_xz = 0.5*sinA*sin2B*-sin2C + cosA*sinB*cos2C
        el_XZ_xy = 0.5*-sinC*sin2B*sin2A + cosC*sinB*cos2A
        #el_X2Y2_xz = 0.5*sinA*sin2B*cos2C + cosA*sinB*sin2C
        el_XZ_x2y2 = 0.5*-sinC*sin2B*cos2A + cosC*sinB*-sin2A
        #el_Z2_xz = sqrt3over2*sinA*sin2B
        el_XZ_z2 = sqrt3over2*-sinC*sin2B
        #converted using rule i, cosNA -> sinNA, sinNA -> -cosNA
        #el_XZ_xz = cosA*cosB*cosC - sinA*cos2B*sinC
        el_XZ_yz = sinA*cosB*cosC - -cosA*cos2B*sinC

        #converted using rule ii, cosNC -> -sinNC, sinNC -> cosNC
        #el_XZ_xz = cosA*cosB*cosC - sinA*cos2B*sinC
        el_YZ_xz = cosA*cosB*-sinC - sinA*cos2B*cosC
        #el_XZ_xy = 0.5*-sinC*sin2B*sin2A + cosC*sinB*cos2A
        el_YZ_xy = 0.5*-cosC*sin2B*sin2A + -sinC*sinB*cos2A
        #el_XZ_x2y2 = 0.5*-sinC*sin2B*cos2A + cosC*sinB*-sin2A
        el_YZ_x2y2 = 0.5*-cosC*sin2B*cos2A + -sinC*sinB*-sin2A
        #el_XZ_z2 = sqrt3over2*-sinC*sin2B
        el_YZ_z2 = sqrt3over2*-cosC*sin2B
        #el_XZ_yz = sinA*cosB*cosC - -cosA*cos2B*sinC
        el_YZ_yz = sinA*cosB*-sinC - -cosA*cos2B*cosC

        sg_as_written_mat =np.matrix([[el_XY_xy,
                      el_XY_yz,  el_XY_z2,  el_XY_xz,  el_XY_x2y2],
          [el_YZ_xy,  el_YZ_yz,  el_YZ_z2,  el_YZ_xz,  el_YZ_x2y2],
          [el_Z2_xy,  el_Z2_yz,  el_Z2_z2,  el_Z2_xz,  el_Z2_x2y2],
          [el_XZ_xy,  el_XZ_yz,  el_XZ_z2,  el_XZ_xz,  el_XZ_x2y2],
        [el_X2Y2_xy,el_X2Y2_yz,el_X2Y2_z2,el_X2Y2_xz,el_X2Y2_x2y2]],
                              float)
        return sg_as_written_mat

    def rotate_d_orbitals(self, rot_mat, rotlabel="d orbital rotation"):
        """Rotate the d orbitals.
            Args:
                rot_mat <np.matrix>: Rotation matrix
                rotlabel <str>: Label for rotation
            Return:
                numpy matrix of rotated occupations, in the same order
                sets self.rotated_d_orbital_occs and self.dorbR
        """
        print "-----------------------------"
        print "BEGINNING NEW ROTATION"
        print "-----------------------------"
        self.print_matrix(rot_mat, rotlabel)
        self.check_for_orthonormal_matrix(rot_mat, rotlabel)

        #self.rotated_d_orbital_occs = rot_mat * self.orig_d_orbital_occs
        self.rotated_d_orbital_occs = rot_mat * self.sqrt_d_orbital_occs
        self.print_matrix(self.sqrt_d_orbital_occs, "sqrt occs")
        sumorig = self.orig_d_orbital_occs.sum()
        print "SUM squared: % 3.3f%+3.3f" % (sumorig.real, sumorig.imag)
        self.print_matrix(self.rotated_d_orbital_occs, "rotated occs")
        self.squared_rotated_d_orbital_occs = np.multiply(self.rotated_d_orbital_occs,self.rotated_d_orbital_occs.conjugate())
        sumrot = self.squared_rotated_d_orbital_occs.sum()
        #sumrotunsq = self.rotated_d_orbital_occs.sum()
        print "SUM of squared: % 3.3f%+3.3fi" % (sumrot.real, sumrot.imag)
        #print "SUM notsquared: % 3.3f%+3.3fi" % (sumrotunsq.real, sumrotunsq.imag)
        return self.rotated_d_orbital_occs

    def check_for_orthonormal_matrix(self, mymatrix, matname):
        """Check to see if the rotation matrices are orthonormal
            Args:
                mymatrix <np.matrix of float>: matrix to check
                matname <str>: matrix name
                self.Rmatrix or self.dorbR should have been set
        """
        print "-------------------"
        print "Checking to see if %s matrix is orthonormal:" % matname
        orthothresh=1e-5
        nrows = mymatrix.shape[0]
        ncols = mymatrix.shape[1]
        if not (nrows == ncols):
            print "Orthonormal checker only works for square matrices."
            print "-----------------"
            return
        qtq = mymatrix.transpose()*mymatrix
        self.print_matrix(qtq, "transpose multiplied by self")
        ortho_low = qtq >= np.eye(nrows) - orthothresh
        #print ortho_low
        ortho_high = qtq <= np.eye(nrows) + orthothresh
        #print ortho_high
        if (ortho_low.sum()==qtq.size) and (ortho_high.sum() == qtq.size):
            print "Matrix is orthogonal."
        else:
            print "WARNING! Matrix is not orthogonal."
        if (qtq.sum()-orthothresh>nrows) or (qtq.sum()+orthothresh<nrows):
            #self.print_matrix(qtq.sum(0), "column sums (should be 1's)")
            #self.print_matrix(qtq.sum(1), "row sums (should be all 1's)")
            print "WARNING! Matrix is not normal."
        else:
            print "Matrix is normal."
        print "-----------------"
        return

    def sherman_grinter_method_explicit_mat(self, rotmat):
        """Sherman Grinter method matrices.
            Args:
                rotmat <np.matrix of float>: Rotation matrix R
                    for coordinates
            Returns:
                <np.matrix of float>: Rotation matrix for
                    d-orbitals, explicit (W*M*inverse(W))

            t here stands for transpose. In general,
            column vectors are used.
            
            t([dz2,dxy,dyz,dx2,dxz]) = cWt([x2,y2,z2,xy,yz,xz])
            where c = real = 1/r2 (here 2 = squared)
            and W is as below, because of the normalization
            coefficients given in Sherman and Grinter.

            Now, captials mean rotated. So the rotated 
            orbitals
            t([DZ2,DXY,DYZ,DX2,DXZ]) = CWt([X2,Y2,Z2,XY,YZ,XZ])
            where C = real = 1/R2, W is as above, and 
            X2 is (rotated x)^2, etc.
            C = c = 1/r^2 since the rotation does not affect r^2
            t([X2,Y2,Z2,XY,YZ,XZ]) = Mt([x2,y2,z2,xy,yz,xz])
            where M is created using t([X,Y,Z]) = Rt([x,y,z]).
            R is the rotation matrix. If R is given by:
            [[m n p],
             [q r s],
             [t u v]]
            then big X = mx + ny + pz
            big X^2 = (mx + ny + pz) = mxmx + mxny + mxpz +
                                       nymx + nyny + nypz + 
                                       pzmx + pzny + pzpz
                = m2x2 + n2y2 + p2z2 + 2mnxy + 2npyz + 2mpxz
            so for example, M_X^2 row (first row)= 
                [m2 n2 p2 2mn 2np 2mp]

            [little d]=cW[little x2 etc.]
            (1/c)[little d] = W[little x2 etc.]
            inverse(W)(1/c)[little d] = inverse(W)(W)[little x2]
            c is real, so:
            (1/c)inverse(W)[little d] = [little x2 etc.]
            [Big D] = CW[Big X2 etc.] = cW[Big X2 etc.]
            [Big X2 etc.] = M[little x2 etc.]
            [Big D] = cWM(1/c)inverse(W)[little d]
            [Big D] = WMinverse(W)[little d]
        """
        rt5ov4pi = np.sqrt(5.0/(4*np.pi))
        rt3 = np.sqrt(3.0)
        # dz2 = (5/16pi)^(1/2)*([3z2-r2]/r2) 
        #         where 3z2-r2 = 2z2-x2-y2, and 1/r2 is covered
        #         by little c
        # dxy = (15/4pi)^(1/2)*(xy/r^2)
        # dyz = (15/4pi)^(1/2)*(yz/r^2)
        # dx2 = (15/16pi)^(1/2)*([x2-y2]/r2)
        # dxz = (15/4pi)^(1/2)*(xz/r^2)
        # Wmat has rows going down in order dz2,dxy,dyz,dx2,dxz
        # and columns in order x2 y2 z2 xy yz xz
        Wmat = np.matrix([[-0.5*rt5ov4pi,
                  -0.5*rt5ov4pi, rt5ov4pi, 0, 0, 0],
            [0,0,0,rt3*rt5ov4pi,0,0],
            [0,0,0,0,rt3*rt5ov4pi,0],
            [0.5*rt3*rt5ov4pi,-0.5*rt3*rt5ov4pi,0,0,0,0],
            [0,0,0,0,0,rt3*rt5ov4pi]], float)

        Xx = rotmat[0,0]
        Xy = rotmat[0,1]
        Xz = rotmat[0,2]
        Yx = rotmat[1,0]
        Yy = rotmat[1,1]
        Yz = rotmat[1,2]
        Zx = rotmat[2,0]
        Zy = rotmat[2,1]
        Zz = rotmat[2,2]


        Mmat = np.matrix([[Xx*Xx,
                Xy*Xy,Xz*Xz,2*Xx*Xy,2*Xy*Xz,2*Xx*Xz],
            [Yx*Yx, Yy*Yy, Yz*Yz, 2*Yx*Yy, 2*Yy*Yz, 2*Yx*Yz],
            [Zx*Zx, Zy*Zy, Zz*Zz, 2*Zx*Zy, 2*Zy*Zz, 2*Zx*Zz],
            [Xx*Yx, Xy*Yy, Xz*Yz, 
                Xx*Yy + Yx*Xy, Xy*Yz + Yy*Xz, Xx*Yz + Yx*Xz],
            [Yx*Zx, Yy*Zy, Yz*Zz,
                Yx*Zy + Zx*Yy, Yy*Zz + Zy*Yz, Yx*Zz + Zx*Yz],
            [Xx*Zx, Xy*Zy, Xz*Zz,
                Xx*Zy + Zx*Xy, Xy*Zz + Zy*Xz, Xx*Zz + Zx*Xz]],
                float)

        #self.print_matrix(Mmat, "Mmat for SG explicit, only")
        #self.print_matrix(Mmat*Wmat.getI(), "Mmat*inv(Wmat)")
        return Wmat*Mmat*Wmat.getI()


class EulerDetermination():
    """Determine Euler angles for a vector"""
    def __init__(self, vecstr):
        """
            Args:
                vecstr <str>: String of x, y, and z components
                                of a vector whose magnitude is 1
        """
        vsplit = vecstr.strip().split()
        self.vx = float(vsplit[0])
        self.vy = float(vsplit[1])
        self.vz = float(vsplit[2])
        self.vec = np.matrix([[self.vx],[self.vy],[self.vz]], float)
        return

    def step_rotation(self, thresh=0.01, step=0.5, beta_as_arccos_z=True, rhs_small_rot=True):
        """Rotate through Euler angles in steps.
            Although inefficient, this method allows several
            arbitrary Euler rotation definitions to be used.

            Args:
                thresh <float>: Threshold for declaring a match
                step <float>: Degree step
                beta_as_arccos_z <bool>: Take Beta to be the
                        arccos of the z-coordinate if the
                        vector is primarily along z. This 
                        calculation is suitable for zxz or zyz
                        rotation conventions; default True
                    The vector
                    [0,0,Z] = R[0,0,1] where the Z_z element of
                    R is cosB in these rotation conventions.
                    [0,0,1] is the rotation vector used for
                    self.vec primarily along z.
                rhs_small_rot <bool>: Inputs are using a right-hand
                        coordinate system, and angles are
                        assured to be small; default True

        """
        print "Original vector to match: %3.3f, %3.3f, %3.3f" % (self.vx, self.vy, self.vz)
        maxabsidx = np.argmax(np.absolute(self.vec))
        matchstr=""
        if maxabsidx == 0:
            matchstr="1 0 0"
        elif maxabsidx == 1:
            matchstr="0 1 0"
        elif maxabsidx == 2:
            matchstr="0 0 1"
        print "Candidate angles, newvector, norm of difference"
        if rhs_small_rot:
            myrange = np.arange(-45,45,step)
        else:
            myrange=np.arange(-180,180,step)

        if beta_as_arccos_z and matchstr=="0 0 1":
            bval = np.arccos(self.vz)*180/np.pi
            if rhs_small_rot:
                if bval > 45:
                    bval = bval - 90
            brange = [bval]
        else:
            if rhs_small_rot:
                brange = np.arange(-45,45,step)
            else:
                brange = np.arange(-90,90,step)
        myct=0
        for angA in myrange:
            for angB in brange:
                for angC in myrange:
                    myrotator = Rotator(matchstr, angA,angB,angC, "degrees","coords")
                    newvec = myrotator.rotate_coords(myrotator.rot_zyz) #using zyz convention
                    if np.linalg.norm(self.vec - newvec) < thresh:
                        print "%3.3f, %3.3f, %3.3f: %3.3f, %3.3f, %3.3f: %3.8f" % (angA, angB, angC, newvec[0,0],newvec[1,0],newvec[2,0], np.linalg.norm(self.vec - newvec))
                    myct = myct + 1
                    if np.mod(myct,1000) == 0:
                        print "--------%3.3f %3.3f %3.3f--------" % (angA,angB,angC)
                    
        return


class OctaCollection():
    """
        self.path <str>: path to POSCAR or CONTCAR file
        self.adict <dict of A-site coordinate float np.array>
        self.bdict <dict of B-site coordinate float np.array>
        self.odict <dict of O-site coordinate float np.array>
        self.octdict <dict of octahedra, organized by B position>
    """
    def __init__(self, mypath):
        self.path = mypath
        self.adict=dict()
        self.bdict=dict()
        self.odict=dict()
    def get_rotations(mypath,l=2,occatoms=[]):
        """Get octahedral rotations.
            mypath: directory
        """
        mypath = self.path
        import Rcoords
        contcarfile = os.path.join(mypath, "CONTCAR")
        outcarfile = os.path.join(mypath, "OUTCAR")
        if not os.path.isfile(contcarfile):
            print "No such file at ",contcarfile
            return
        if not os.path.isfile(outcarfile):
            print "No such file at ", outcarfile
            return
        if len(occatoms) == 0:
            occatoms = range(9,17)
        #occatoms = [9] #TTM DEBUG
        coordset=Rcoords.get_coords(contcarfile, 0.05)
        oextend=Rcoords.extend_cell(coordset[2],0.2)
        octdict=Rcoords.build_octahedra(coordset[1],oextend,0.4)
        #parse occupation file
        occevlines_d = get_occ_and_ev_lines(outcarfile)
        occev_d = make_occ_and_ev_dict(occevlines_d)
        myprintlist=[]
        myprintlist.append("outcarfile: " + str(outcarfile))
        print "outcarfile: ",outcarfile
        myprintlist.append("contcarfile: " + str(contcarfile))
        print "contcarfile: ",contcarfile
        if l == 2:
            orbstr= "dxy dyz dz2 dxz dx2"
            print orbstr
            myprintlist.append(orbstr)
        occdict=dict()
        rotoccdict=dict()
        rotrloccdict=dict()
        evrotoccdict=dict()
        for atom in occatoms:
            occdict[atom]=""
            rotoccdict[atom]=""
            rotrloccdict[atom]=""
            evrotoccdict[atom]=""
            print ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
            print "ATOM: ",str(atom)
            getlines=Rcoords.get_lines_from_octahedron(octdict,atom)
            if not(len(getlines) > 0 and len(getlines[0]) > 0):
                print "Error getting coordinates!"
                return
            alphabetagamma = Rcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])

    def get_coords(self,tolerance=0.0):
        """Get coordinates from POSCAR at self.path
            Args:
                tolerance <float>: edge tolerance for 
                        adding/subtracting back to 
                        periodic fraction (e.g. if this is 0.05, 
                        then since 0.97 is within 0.05 of 1, 
                        0.97 will become -0.03
            assumes A B O ordering (e.g. 8 8 24)
            Returns:
                sets self.adict, self.bdict, self.odict
        """
        myfile=open(self.path,'rb')
        mylines = myfile.readlines()
        myfile.close()
        myspecs=mylines[6].split()
        #numa=int(myspecs[0])
        #numb=int(myspecs[1])
        #numo=int(myspecs[2])
        #ASSUMES THAT this is an A B O3 type perovskite and the A
        #are listed first, then the B, then the O.
        if len(myspecs) == 3:
            numo = int(myspecs[2])
        elif len(myspecs) == 4:
            numo = int(myspecs[3])
        if numo%2 == 1: 
            ocalc = numo + 1 #assume there is a vacancy!
        else:
            ocalc = numo
        numa = ocalc/3
        numb = ocalc/3
        numatoms = numa + numb + numo
        atomlines=mylines[8:numatoms+8]
        onesplit=[]
        onecoord=[]
        atomcoords=[]
        coord=""
        fltcoord=0.0
        for oneatom in atomlines:
            onecoord=[]
            onesplit = oneatom.split()[:3] #only the first 3 values
            for coord in onesplit:
                fltcoord = float(coord)
                if fltcoord > 1 - tolerance:
                    onecoord.append(fltcoord - 1)
                elif fltcoord < 0 - tolerance:
                    onecoord.append(fltcoord + 1)
                else:
                    onecoord.append(fltcoord)
            atomcoords.append(np.array(onecoord, float).transpose())
        adict=dict()
        bdict=dict()
        odict=dict()
        for count in range(1,numa+1):
            adict[count]=atomcoords[count-1]
        for count in range(numa+1, numa+numb+1):
            bdict[count]=atomcoords[count-1]
        for count in range(numa+numb+1, numatoms+1):
            odict[count]=atomcoords[count-1]
        print adict
        print bdict
        print odict
        self.adict = adict
        self.bdict = bdict
        self.odict = odict
        return [adict, bdict, odict]
    
    def build_bob_cube_small(self, fracbond=0.5):
        """Build a small B-O-B cube (for 40-atom cell).
            Args:
                fracbond <float>: fractional bond length
                                  allowed
            Returns:
                edges of the cube, in B-O-B indices,
                and the actual nonlinear angle of each
                cube edge, in degrees:
                B-O-B angle
        """
        bobdict=dict()
        ocoord=""
        bcoord=""
        dist=0.0
        for okey in self.odict.keys():
            ocoord=self.odict[okey]
            bpair=list()
            bvecs=list()
            for bkey in self.bdict.keys():
                bcoord=self.bdict[bkey]
                dist=np.linalg.norm(bcoord-ocoord)
                if dist <= fracbond:
                    bvec = np.array(bcoord-ocoord,float)
                    bpair.append(bkey)
                    bvecs.append(bvec)
            if len(bpair) == 2:
                bobdict[okey]=dict()
                norm0 = np.linalg.norm(bvecs[0])
                norm1 = np.linalg.norm(bvecs[1])
                bobdot = np.dot(bvecs[0],bvecs[1])/norm0/norm1
                bobangle = np.arccos(bobdot)
                bobequidist = float(norm0)/float(norm1)
                if np.isnan(bobangle):
                    if (bvecs[0]==bvecs[1]).all():
                        bobangle=0.0
                    else:
                        bobangle=np.pi
                bobangle = 180*bobangle/np.pi
                bpair.append(bobangle)
                bpair.append(bobequidist)
                bobdict[okey]=list(bpair)
        print bobdict
        allangles=list()
        alldists=list()
        nine30thirteenequi=0
        print "B-O-B:BOBangle,OBnorm/BOnorm"
        for key in bobdict.keys():
            print "%2i-%2i-%2i:%3.3f,%3.3f" % (bobdict[key][0],key,bobdict[key][1],bobdict[key][2],bobdict[key][3])
            allangles.append(bobdict[key][2])
            if bobdict[key][3] > 1:
                alldists.append(float(1)/bobdict[key][3])
            else:
                alldists.append(bobdict[key][3])
            if (bobdict[key][0]==9) and (key==30) and (bobdict[key][1]==13):
                nine30thirteenequi=bobdict[key][3]
        numedges = len(allangles)
        allangles = np.array(allangles, float)
        alldists = np.array(alldists,float)
        avgangle = np.mean(allangles)
        stdangle = np.std(allangles)
        maxangle = max(allangles)
        minangle = min(allangles)
        avgequidist = np.mean(alldists)
        stdevequidist = np.std(alldists)
        bobdict["mean"]=avgangle
        bobdict["stdev"]=stdangle
        bobdict["numedges"]=numedges
        bobdict["max"]=maxangle
        bobdict["min"]=minangle
        bobdict["equidistmean"]=avgequidist
        bobdict["equidiststd"]=stdevequidist
        bobdict["nine30thirteenequi"]=nine30thirteenequi
        print "    mean:%3.3f" % avgangle
        print "   stdev:%3.3f" % stdangle
        print "     max:%3.3f" % maxangle
        print "     min:%3.3f" % minangle
        print "numedges:%7i" % numedges
        print "equidist:%3.3f" % avgequidist
        print " stdequi:%3.3f" % stdevequidist
        print " 9-30-13:%3.3f" % nine30thirteenequi
        print ""
        print "path:%s:mean:%3.3f:stdev:%3.3f:max:%3.3f:min:%3.3f:numedges:%i:equidistmean:%3.3f:equidiststd:%3.3f:nine30thirteenequi:%3.3f" % (self.path, avgangle,stdangle,maxangle,minangle,numedges,avgequidist,stdevequidist,nine30thirteenequi)
        return bobdict
        
                
    
    def build_octahedra(self, fracbond=0.5):
        """Build octahedra from the coordinate lists.
            Args:
                fracbond <float>: fractional bond length
                                  allowed
        """
        octdict=dict()
        ocoord=""
        bcoord=""
        dist=0.0
        for bkey in self.bdict.keys():
            octdict[bkey]=dict()
            octdict[bkey][bkey]=self.bdict[bkey]
            bcoord=self.bdict[bkey]
            for okey in self.odict.keys():
                ocoord=self.odict[okey]
                dist = np.linalg.norm(bcoord-ocoord)
                if dist <= fracbond:
                    octdict[bkey][okey]=self.odict[okey]
        for key in octdict.keys():
            print key, ":", octdict[key].keys()
        #print octdict
        self.octdict = octdict
        return octdict
    def get_lines_from_octahedron(self, whichoct, tolerance=0.01):
        """Get lines from an octahedron; that is, the atoms which
            will be used to determine the x, y, and z axes of the 
            octahedron.
            Args:
                whichoct <int>: B-site cation number for
                                which octahedron
                toerance <float>: tolerance for maximum
                                  deviation to figure out if
                                  the coordinate is mostly
                                  along x, y, or z
        """
        bdict=self.octdict[whichoct]
        centerdict=dict() #center around B-site cation
        coordb = bdict[whichoct]
        for key in bdict.keys():
            coordo = bdict[key]
            centerdict[key] = coordo - coordb
        print "Centerdict: ",centerdict
        pairdict=dict()
        pairdict['x'] = list()
        pairdict['y'] = list()
        pairdict['z'] = list()
        for key in centerdict.keys():
            if key == whichoct:
                pass #Skip the B-site for now
            else:
                maxabsidx = np.argmax(np.absolute(centerdict[key]))
                if np.absolute(centerdict[key][maxabsidx]) < tolerance:
                    pass
                else:
                    if maxabsidx == 0:
                        pairdict['x'].append(key) #largest val is in x
                    elif maxabsidx == 1:
                        pairdict['y'].append(key)
                    elif maxabsidx == 2:
                        pairdict['z'].append(key)
        for pairdir in ['x','y','z']:
            if len(pairdict[pairdir]) < 2:
                pairdict[pairdir].append(whichoct)
            if len(pairdict[pairdir]) > 2:
                print "Problem with pairdict ",pairdir,": > 2 values"
        print pairdict
        return [pairdict, centerdict]
    def extend_cell(self, tol=0.2, printnow=True):
        """Extend a dictionary (odict) so that all octahedra are full.
            Cases to extend:
               -0.03 should extend to 0.97 (smaller than 0)
               0.03 should extend to 1.03
               0.97 should extend to -0.03
               1.03 should extend to 0.03 (greater than 1)
               And do this for *each* direction.
        """
        mydict=dict(self.odict)
        for key in mydict.keys():
            for mydir in [0,1,2]:
                newval=list(mydict[key])
                makenew = 0
                myval = mydict[key][mydir]
                if myval < 0:
                    newval[mydir] = myval + 1
                    mydict[key+1000000+10000*mydir]=list(newval)
                elif myval < tol:
                    newval[mydir] = myval + 1
                    mydict[key+1000000+10000*mydir]=list(newval)
                elif myval > (1-tol):
                    newval[mydir] = myval - 1
                    mydict[key+1000000+10000*mydir]=list(newval)
                elif myval > 1:
                    newval[mydir] = myval - 1
                    mydict[key+1000000+10000*mydir]=list(newval)
        if printnow:
            print "Extended dict:"
            for key in mydict.keys():
                print key, ":", mydict[key]
        self.odict = mydict
        return mydict
    def get_euler_angles_from_octahedron(self, pairdict, centerdict):
        """Get Euler tilts from octahderon.
            [$1] dictionary of pairs along closest axes ['x']=[20,21], etc.
            [$2] dictionary of coordinates, centered so B-cation is at (0,0,0)
        """
        # alpha = arccos(-Z2/sqrt(1-(Z3)^2))
        # beta = arccos(Z3)
        # gamma = arccos(Y3/sqrt(1-(Z3)^2))
        # Let's assume these octahedra have perfectly orthogonal axes.
        # Make long vectors out of each pair. One pair in a defected cell
        # might be made up of the center cation and an anion.
        mycoordsX = centerdict[pairdict['x'][0]] - centerdict[pairdict['x'][1]]
        mycoordsY = centerdict[pairdict['y'][0]] - centerdict[pairdict['y'][1]]
        mycoordsZ = centerdict[pairdict['z'][0]] - centerdict[pairdict['z'][1]]
        if mycoordsX[0] < 0: #ensure a right-hand coordinate system
            mycoordsX = -1*mycoordsX
        if mycoordsY[1] < 0:
            mycoordsY = -1*mycoordsY
        if mycoordsZ[2] < 0:
            mycoordsZ = -1*mycoordsZ
        print "UNNORMALIZED: ",mycoordsX,mycoordsY,mycoordsZ
        mycoordsX = mycoordsX/np.linalg.norm(mycoordsX) #normalize to 1
        mycoordsY = mycoordsY/np.linalg.norm(mycoordsY)
        mycoordsZ = mycoordsZ/np.linalg.norm(mycoordsZ)
        #NEW: Round these to 4 decimal places
        #mycoordsX = np.round(mycoordsX,4)
        #mycoordsY = np.round(mycoordsY,4)
        #mycoordsZ = np.round(mycoordsZ,4)
        print "OCT COORDS: ", mycoordsX, mycoordsY, mycoordsZ
        myZ3 = mycoordsZ[2]
        myZ2 = mycoordsZ[1]
        myZ1 = mycoordsZ[0]
        myY3 = mycoordsY[2]
        myX3 = mycoordsX[2]
        print "X Y CROSS: ", np.cross(mycoordsX, mycoordsY)
        #myalpha = np.arccos(-1*myZ2/np.sqrt(1-(np.power(myZ3,2))))*180/np.pi
        mybeta = np.arccos(myZ3)*180/np.pi
        #mygamma = np.arccos(myY3/np.sqrt(1-(np.power(myZ3,2))))*180/np.pi
        #print "As-is alpha, beta, gamma: ",myalpha, mybeta, mygamma
        #alpha, gamma angles are too large.
        #alpha = atan2(Z1, -Z2)
        #gamma = atan2(X3, Y3)
        if np.absolute(myZ2) < 0.00001:
            newalpha = np.arctan2(myZ1, myZ2)
        else:
            newalpha = np.arctan2(myZ1, -1*myZ2)
        newgamma = np.arctan2(myX3, myY3)
        #print "My alpha, beta, gamma: ",newalpha, mybeta, newgamma
        #newalpha, newgamma angles are too small now. Forgot to turn
        # these back into degrees! No wonder.
        newalpha = newalpha*180/np.pi
        newgamma = newgamma*180/np.pi
        #But now they are too large again.
        print "My alpha, beta, gamma: ",newalpha, mybeta, newgamma
        normangles=[]
        myangles=[newalpha, mybeta, newgamma]
        #for angle in myangles:
        #    if angle > 90:
        #        normangles.append(angle-180)
        #    elif angle > 45:
        #        normangles.append(angle-90)
        #    else:
        #        normangles.append(angle)
        #print "Normalized alpha, beta, gamma: ", normangles
        #return normangles
        return myangles











#if len(sys.argv) == 1:

    #Step rotation example
    #mye = EulerDetermination("0.534522 0.801784 0.267261")
    #mye.step_rotation()
    
    #Rotate coords check
    #myr = Rotator("3 4 5","30","20","40","degrees","coords")
    
    #Rotate orbitals example
    #myr = Rotator("1 1 1 1 1","30","20","40","degrees","orbitals")
    #returnstr = myr.do_rotations()
    #print returnstr
#elif sys.argv[1] == "help":
#    print Rotator.__init__.__doc__
#else:
    #myr = Rotator(sys.argv[1],sys.argv[2],sys.argv[3],
    #                        sys.argv[4],sys.argv[5],sys.argv[6])
    #returnstr = myr.do_rotations()
#    print returnstr



