#!/bin/env python
"""Slope fitting. TTM 04/07/14
Takes input.txt with columns: 
system, label, neblabel, imagenumber, E0 energy, supercell magnetic moment, strainlabel, strainpercent
image numbers must be in order already, 0 1 2 3 4.
"""
import os
import sys
import numpy as np
import time
import hocking_lstsq

class BarrierSet():
    """Migration barrier set
        Attributes:
            self.symbol <str>: symbol or label for strain set
            self.energies <dict>: supercell E0 energies
                self.energies[ip or oop][strain][imnum] = float(energy)
            self.barriers <dict of np.array of float>: migration barriers
                self.barriers[ip or oop] = np.array of float
            self.strains <dict of np.array of float>: strains
                self.strains[ip or oop] = np.array of float, corresponding
                                            to the barriers
            self.slopes <dict of float>
                self.slopes[ip or oop] = float(slope)
            self.errors <dict of float>
                self.errors[ip or oop] = float(error)
    """
    def __init__(self, lineblock):
        """
            Args:
                lineblock <list of str>: List of strings
        """
        headerline = lineblock[0]
        numlines = len(lineblock)
        self.energies = dict()
        for lct in range(1, numlines):
            lline = lineblock[lct].split()
            neblabel = lline[2]
            imnum = int(lline[3])
            energy = float(lline[4])
            strperc = float(lline[7])
            if not neblabel in self.energies.keys():
                self.energies[neblabel]=dict()
            if not strperc in self.energies[neblabel].keys():
                self.energies[neblabel][strperc]=dict()
            self.energies[neblabel][strperc][imnum] = energy
        self.symbol = lline[0]
        self.set_up_barriers()
        self.fit_barriers_vs_strains()

    def set_up_barriers(self):
        """Set up migration barrier versus strain dictionaries."""
        self.barriers = dict()
        self.strains = dict()
        for neblabel in self.energies.keys():
            barrierlist = list()
            strainlist = list()
            for strainperc in self.energies[neblabel].keys():
                strainlist.append(strainperc)
                ep1energy = self.energies[neblabel][strainperc][0]
                maxenergy = max(self.energies[neblabel][strainperc].values())
                mybarrier = maxenergy - ep1energy
                barrierlist.append(mybarrier)
            self.barriers[neblabel] = np.array(barrierlist, float)
            self.strains[neblabel] = np.array(strainlist, float)
        print self.barriers
        print self.strains
    
    def fit_barriers_vs_strains(self):
        """Fit migration barrier versus strain."""
        self.slopes=dict()
        self.errors=dict()
        for neblabel in self.barriers.keys():
            print "Hocking lstsq linear"
            hocking=hocking_lstsq.least_squares_regression(self.strains[neblabel], self.barriers[neblabel], 1, False)
            print hocking
            print "numpy lstsq linear"
            numpyfit = np.polyfit(self.strains[neblabel], self.barriers[neblabel], 1, None, True)
            print numpyfit
            sum_of_strains_minus_means_squared = np.sum(np.power(self.strains[neblabel]-np.mean(self.strains[neblabel]),2))
            one_over_n_minus_2 = 1.0/(len(self.strains[neblabel])-2)
            homemade_slope_error = np.sqrt(one_over_n_minus_2 * numpyfit[1][0]/sum_of_strains_minus_means_squared)
            print "Handcalc error in slope: ", homemade_slope_error
            self.slopes[neblabel] = hocking[0][1]
            self.errors[neblabel] = hocking[2][1]

myinput = sys.argv[1]
ifile = open(myinput, 'rb')
ifilelines = ifile.readlines()
ifile.close()
ichunk=list()
isym=""
headerline = ifilelines[0]
outputs=list()
ipstr="neb_vac1-ipv_q=p2_stat"
oopstr="neb_vac1-opv_q=p2_stat"
for ilct in range(1, len(ifilelines)):
    linenow = ifilelines[ilct]
    symnow = linenow.split()[0]
    if isym == "":
        isym = symnow
        ichunk.append(headerline)
        ichunk.append(linenow)
    elif not (symnow == isym):
        print "Chunk finished:"
        #for cline in ichunk:
        #    print cline
        newclass = BarrierSet(ichunk)
        outputstr = "%2s %3.5f %3.5f %3.5f %3.5f" % (newclass.symbol, newclass.slopes[ipstr],newclass.errors[ipstr],newclass.slopes[oopstr],newclass.errors[oopstr])
        outputs.append(outputstr)
        ichunk=list()
        isym=symnow
        ichunk.append(headerline)
        ichunk.append(linenow)
    else:
        ichunk.append(linenow)
print "Chunk finished:"
#for cline in ichunk:
#    print cline
newclass = BarrierSet(ichunk) #Do last chunk
outputstr = "%2s %3.5f %3.5f %3.5f %3.5f" % (newclass.symbol, newclass.slopes[ipstr],newclass.errors[ipstr],newclass.slopes[oopstr],newclass.errors[oopstr])
#outputstr = "%2s %3.12f %3.3f %3.12f %3.3f %3.12f" % (newclass.symbol, newclass.errorincstrain, newclass.poisson, newclass.errorinpoisson, newclass.youngs, newclass.errorinyoungs)
outputs.append(outputstr)
print "Symbol    IP_slope    IP_error    OOP_slope     OOP_error"
for oline in outputs:
    print oline
