#!/bin/env python
"""Strain energy fitting. TTM 3/12/14
Takes input.txt with columns: label, E0, supercell volume, strain fraction a, strain fraction b, strain fraction c
"""
import os
import sys
import numpy as np
import time
import hocking_lstsq

class UndefectedStrainSet():
    """Undefected strain set
        Attributes:
            self.symbol <str>: symbol or label for strain set
            self.bulkmod <float>: bulk modulus in eV/A^3
            self.fracsa <np.array of float>: strain fractions, 
                                original
                                strain a (strain b identical)
            self.fracsc <np.array of float>: strain fractions, 
                                                response c
            self.strainsa <np.array of float>: strain in a 
                = delta a / a = (fracsa - 1)a/a = (fracsa - 1)
            self.strainsc <np.array of float>: strain in c
                = delta c / c = (fracsc - zero strain frac c)
            self.vols <np.array of float>: supercell volumes
            self.energies <np.array of float>: supercell E0 energies
            self.strainenergies <np.array of float>: strain energies
            self.zerodx <int>: zero-strain index in arrays
            self.quad_strainsc <np.array of float>: c-strains
                    derived from the quadratic fit
            self.errorincstrain <float>: Error in c-strain from
                    c-strain versus a-strain quadratic fit
            self.poisson <float>: Poisson's ratio
            self.errorinpoisson <float>: Error in Poisson's ratio
            self.youngs <float>: Young's modulus, eV/A^3
            self.errorinyoungs <float>: Error in Young's modulus
            self.bulkmod <float>: Bulk modulus, eV/A^3
            self.errorinbulkmod <float>: Error in Bulk modulus
            self.strain_energy_model <float>: Strain-energy model
                outputs, given poisson and Young's modulus
    """
    def __init__(self, lineblock):
        """
            Args:
                lineblock <list of str>: List of strings
        """
        headerline = lineblock[0]
        numlines = len(lineblock) - 1 #remove header line
        self.vols = np.zeros(numlines)
        self.energies = np.zeros(numlines)
        self.fracsa = np.zeros(numlines)
        self.fracsc = np.zeros(numlines)
        self.zerodx=""
        for lct in range(0, numlines):
            lline = lineblock[lct + 1].split()
            self.energies[lct] = float(lline[1])
            self.vols[lct] = float(lline[2])
            self.fracsa[lct] = float(lline[3])
            if self.fracsa[lct] == 1:
                self.zerodx = lct
            self.fracsc[lct] = float(lline[5])
        self.bulkmod = float(lline[6])
        self.errorinbulkmod = float(lline[7])
        self.symbol = lline[0]
        if self.zerodx == "":
            print "Cannot find zero-strain index! Exiting."
            return
        self.set_up_strains_and_strain_energies()
        self.fit_strainc_vs_straina()
        #self.calculate_nu_from_fit()
        self.calculate_Y()
        #self.fit_for_Y()
        self.get_strain_energy_model()

    def set_up_strains_and_strain_energies(self):
        """Set up strains from the fractions."""
        self.strainsa = self.fracsa - 1
        self.strainsc = self.fracsc - self.fracsc[self.zerodx]
        zeroedenergies = self.energies - self.energies[self.zerodx]
        self.strainenergies = zeroedenergies/self.vols
        return

    def fit_strainc_vs_straina(self):
        """Fit strain c versus strain a and get the norm,
            to use as the error in strain c
        """
        #print self.strainsa
        #print self.strainsc
        #Polyfit with intercept set to zero:
        #Anne Archibald through
        #http://mail.scipy.org/pipermail/numpy-discussion/2008-June/034912.html
        print "Hocking lstsq"
        hocking=hocking_lstsq.least_squares_regression(self.strainsa, self.strainsc, 2, True)
        print "numpy lstsq"
        strainsamat = np.power.outer(self.strainsa,[2,1])
        strainc_vs_straina = np.linalg.lstsq(strainsamat, self.strainsc)
        #strainc_vs_straina = np.polyfit(self.strainsa, self.strainsc, 2, rcond=None, full=True)
        coeffs=strainc_vs_straina[0]
        residualsquaresum = 0
        self.quad_strainsc = np.zeros(len(self.strainsa),float)
        for sct in range(0, len(self.strainsa)):
            straina = self.strainsa[sct]
            predicted_strainc = coeffs[0]*straina*straina + coeffs[1]*straina
            self.quad_strainsc[sct] = predicted_strainc
            residual = self.strainsc[sct] - predicted_strainc
            residualsq = residual*residual
            residualsquaresum = residualsquaresum + residualsq
        residualsquaresum = residualsquaresum / (len(self.strainsa)-2)
        errorinc = np.sqrt(residualsquaresum)
        print "Error in c strain: %3.12f " % errorinc
        self.errorincstrain = np.sqrt(hocking[1])
        hockingcoeffs = hocking[0]
        myb = hockingcoeffs[0]
        nufromb = myb/(myb-2)
        print "My nu, calculated only from b coefficient: %3.5f" % nufromb
        err_in_b = hocking[2][0]
        err_in_nufromb = nufromb*np.fabs(err_in_b/myb)*np.sqrt(2)
        print "Error in nu: %3.12f" % err_in_nufromb
        print strainc_vs_straina
        self.poisson = nufromb
        self.errorinpoisson = err_in_nufromb
    def calculate_nu_from_fit(self):
        """Calculate Poisson's ratio nu for plane stress case, using
            nu = (strain_c/strain_a)/[(strain_c/strain_a) - 2]
            Linearly-fit the return function (which may have
            some asymptotic values; remove the worst ones) 
            and take the value at
            strain_a = 0 (intercept value)
        """
        raise NotImplementedError
        mynus = list()
        strainsa = list()
        for sct in range(0, len(self.strainsa)):
            if sct == self.zerodx:
                pass
            else:
                ratio = self.quad_strainsc[sct]/self.strainsa[sct]
                poisson = ratio/(ratio-2)
                mynus.append(poisson)
                strainsa.append(self.strainsa[sct])
        goodnus = np.array(mynus, float)
        goodstrains = np.array(strainsa, float)
        nufit = np.polyfit(goodstrains, goodnus, 1, rcond=None, full=True)
        mynu = nufit[0][1] #intercept
        errorinpoisson = mynu*np.sqrt(2*np.power(self.errorincstrain/0.02,2))
        print "My nu: %3.3f" % mynu
        #print "My nu: %3.3f error %3.5f" % (mynu, errorinpoisson)
        self.poisson = mynu
        self.errorinpoisson = errorinpoisson 
    def calculate_Y(self):
        self.youngs = 3*self.bulkmod*(1-2*self.poisson)
        print "My Y: %3.3f" % (self.youngs)
        err_in_youngs_squared = 9*np.power(self.errorinbulkmod,2)+36*np.power(self.bulkmod*self.poisson,2)*(np.power(self.errorinbulkmod/self.bulkmod,2)+np.power(self.errorinpoisson/self.poisson,2))
        err_in_youngs = np.sqrt(err_in_youngs_squared)
        self.errorinyoungs = err_in_youngs
        print "Error in Y: %3.12f" % self.errorinyoungs

    def fit_for_Y(self):
        """Fit for Young's modulus:
            strain energy = 0.5*lambda*(sum of strains, squared) +
                            G*(sum of squared strains)
            where lambda = nu*Y/[(1+nu)(1-2*nu)]
            G = Y/[2*(1+nu)]
            strain energy = Y(A(epssumsq) + B(epssqsum))
        """
        epssumsq = self.strainsa + self.strainsa + self.strainsc
        epssumsq = epssumsq * epssumsq
        epssqsum = (self.strainsa*self.strainsa) + (self.strainsa*self.strainsa) + (self.strainsc*self.strainsc)
        coeffA = 0.5*self.poisson/((1+self.poisson)*(1-2*self.poisson))
        coeffB = 1/(2*(1+self.poisson))
        Ycoeffs = (coeffA*epssumsq + coeffB*epssqsum)
        print "Ycoeffs: ", Ycoeffs
        print "epssumsq: ", epssumsq
        print "epssqsum: ", epssqsum
        myYs = self.strainenergies/((coeffA*epssumsq) + (coeffB*epssqsum))
        print "strainenergies: ", self.strainenergies
        print "my Y's: ", myYs
        nonasymptotics=list()
        strainsa=list()
        for sct in range(0, len(self.strainsa)):
            if sct == self.zerodx:
                pass
            else:
                nonasymptotics.append(myYs[sct])
                strainsa.append(self.strainsa[sct])
        goodYs = np.array(nonasymptotics,float)
        goodstrains = np.array(strainsa, float)
        Yfit = np.polyfit(goodstrains, goodYs, 1, rcond=None, full=True)
        yval = Yfit[0][1] #intercept
        erry = yval*np.sqrt(6*np.power(self.errorinpoisson/self.poisson,2))
        print "my Y: %3.3f error %3.5f" % (yval, erry)
        self.youngs = yval
        self.errorinyoungs = erry
    def get_strain_energy_model(self):
        """
            strain energy = 0.5*lambda*(sum of strains, squared) +
                            G*(sum of squared strains)
            where lambda = nu*Y/[(1+nu)(1-2*nu)]
            G = Y/[2*(1+nu)]
        """
        mylambda = self.poisson*self.youngs/((1+self.poisson)*(1-2*self.poisson))
        myG = self.youngs/(2*(1+self.poisson))
        epssumsq = self.strainsa + self.strainsa + self.strainsc
        epssumsq = epssumsq * epssumsq
        epssqsum = (self.strainsa*self.strainsa) + (self.strainsa*self.strainsa) + (self.strainsc*self.strainsc)
        self.strain_energy_model = 0.5*mylambda*epssumsq + myG*epssqsum
        print "Strain energies: ", self.strainenergies
        print "Strain energy model using calc nu and Y: ", self.strain_energy_model
        print "Strain_a     Strain_energy     Strain_energy_model"
        for sct in range(0, len(self.strainsa)):
            print "%1s %3.3f %3.12f %3.12f" % (self.symbol, self.strainsa[sct],self.strainenergies[sct],self.strain_energy_model[sct])



myinput = sys.argv[1]
ifile = open(myinput, 'rb')
ifilelines = ifile.readlines()
ifile.close()
ichunk=list()
isym=""
headerline = ifilelines[0]
outputs=list()
for ilct in range(1, len(ifilelines)):
    linenow = ifilelines[ilct]
    symnow = linenow.split()[0]
    if isym == "":
        isym = symnow
        ichunk.append(headerline)
        ichunk.append(linenow)
    elif not (symnow == isym):
        print "Chunk finished:"
        for cline in ichunk:
            print cline
        newclass = UndefectedStrainSet(ichunk)
        outputstr = "%2s %3.12f %3.3f %3.12f %3.3f %3.12f" % (newclass.symbol, newclass.errorincstrain, newclass.poisson, newclass.errorinpoisson, newclass.youngs, newclass.errorinyoungs)
        outputs.append(outputstr)
        ichunk=list()
        isym=symnow
        ichunk.append(headerline)
        ichunk.append(linenow)
    else:
        ichunk.append(linenow)
print "Chunk finished:"
for cline in ichunk:
    print cline
newclass = UndefectedStrainSet(ichunk) #Do last chunk
outputstr = "%2s %3.12f %3.3f %3.12f %3.3f %3.12f" % (newclass.symbol, newclass.errorincstrain, newclass.poisson, newclass.errorinpoisson, newclass.youngs, newclass.errorinyoungs)
outputs.append(outputstr)
print "Symbol    Error_in_c_strain    Poisson    Error_in_poisson    Young's_modulus   Error_in_Youngs_modulus"
for oline in outputs:
    print oline

print "Bulk modulus fitting"
for symbol in ['c','f','g','m','n','r','s','t','v']:
    fname ="//home/tam/pstrtests/stretch/birchmurn/pvfitting/vp%s" % symbol
    openf = open(fname,'rb')
    flines = openf.readlines()
    openf.close()
    listx=list()
    listy=list()
    for lct in range(0, len(flines)):
        if lct == 0:
            pass
        else:
            lsplit = flines[lct].strip().split()
            listx.append(float(lsplit[0]))
            listy.append(float(lsplit[1]))
    xvec = np.array(listx, float)
    yvec = np.array(listy, float)
    bmfit = hocking_lstsq.least_squares_regression(xvec, yvec, 3)
    print "%s %s" % (symbol, bmfit)
